﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     chat_Api.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/chat/chat_definitions.h>


namespace nn
{
namespace pia
{
namespace chat
{


/*!
@brief  Determines whether the <tt>chat</tt> module is initialized.

@details  This function is thread-safe.

@return  Returns <tt>true</tt> if it is initialized, and <tt>false</tt> otherwise.
@see  Initialize, Finalize
*/
bool IsInitialized(void);


/*!
@brief  Initializes the <tt>chat</tt> module.

@return  Returns a <tt>Result</tt> value indicating success if initialization succeeds. You must make sure that the implementation of this function in your application does not return any errors.
@retval ResultAlreadyInitialized  Indicates that the chat module is already initialized. Programming error. Fix your program so that this error is not returned.
@retval ResultInvalidState  <tt>PiaCommon</tt> has not been initialized. Programming error. Fix your program so that this error is not returned.
@see  IsInitialized, Finalize
*/
nn::Result Initialize(void);


/*!
@brief  Finalizes the <tt>chat</tt> module.
This function does nothing and returns if it is called before the <tt>Initialize</tt> function has been called.

@see  IsInitialized, Initialize

*/
void Finalize(void);


/*!
@brief  Declares the start of setup.

@details  Call this function before creating a singleton in the <tt>chat</tt> module.
Always call this function and <tt>@ref EndSetup</tt> even if your application does not create any singletons in the <tt>chat</tt> module.

@return  Returns a <tt>Result</tt> value indicating success if called at the right time. You must make sure that the implementation of this function in your application does not return any errors.
@retval ResultNotInitialized  Indicates that the <tt>chat::Initialize</tt> function has not been called. Programming error. Fix your program so that this error is not returned.
@retval ResultInvalidState  Indicates that the function was called at the wrong time. Programming error. Fix your program so that this error is not returned.
@see  EndSetup

*/
nn::Result BeginSetup(void);


/*!
@brief  Declares the end of setup.

@details  Call this function after creation of a singleton in the <tt>chat</tt> module is complete.
Calling this function configures memory to be used by the <tt>chat</tt> module singletons, and optimizes memory management within the library.
Always call <tt>@ref BeginSetup</tt> and this function even if your application does not create any singletons in the <tt>chat</tt> module.

@return  Returns a <tt>Result</tt> value indicating success if called at the right time. You must make sure that the implementation of this function in your application does not return any errors.
@retval ResultNotInitialized  Indicates that the <tt>chat::Initialize</tt> function has not been called. Programming error. Fix your program so that this error is not returned.
@retval ResultInvalidState  Indicates that the function was called at the wrong time. Programming error. Fix your program so that this error is not returned.
@see  BeginSetup


*/
nn::Result EndSetup(void);


/*!
@cond PRIVATE
@brief  Indicates whether an application is executing code between calls to the <tt>BeginSetup()</tt> and <tt>EndSetup()</tt> functions.

@details  This function is thread-safe.
*/
bool IsInSetupMode(void);
//! @endcond

u32 ConvertID_Pia2VCT(u32);
u32 ConvertID_VCT2Pia(u32);
}
}
}  // end of namespace nn::pia::chat
