﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     clone_CloneProfilerBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/clone/clone_definitions.h>


namespace nn
{
namespace pia
{
namespace clone
{


/*!
@brief  This base class manages <tt>PiaClone</tt> profiling.

@date 2013-11-25  Initial version.
*/
class CloneProfilerBase
{
public:
/*!
@brief  Prints information that is useful for debugging.

@param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> type.
*/
    void Trace(u64 flag) const;


    //! @cond PRIVATE

public:
    void Reset();

    bool IsProfileLatest() const
    {
        return (m_BufferSize > 0);
    }

    void ProfileByStationIndex(u32 dispatchCount, StationIndex stationId, u32 size);
    void ProfileByStationBmp(u32 dispatchCount, u32 stationBmp, u32 size);

    u32 GetTotalCount(StationIndex stationId) const;
    u32 GetTotalSize(StationIndex stationId) const;
    u32 GetLatestCount(StationIndex stationId, u32 dispatchCount) const;
    u32 GetLatestSize(StationIndex stationId, u32 dispatchCount) const;

protected:
    CloneProfilerBase(u32* paDispatchCount, u8* paCount, u16* paSize, u16* paTotalCount, u32* paTotalSize, u32 stationNum, u32 bufferSize);

private:
    u32 GetBufferIdx(u32 stationId, u32 idx) const
    {
        return stationId + idx * m_StationNum;
    }

    void AssignBuffer(u32 dispatchCount);

private:
    u32* m_paDispatchCount;
    u8* m_paCount;
    u16* m_paSize;
    u16* m_paTotalCount;
    u32* m_paTotalSize;

    u32 m_StationNum;
    u32 m_BufferSize;

    u32 m_CurrentIdx;


    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
