﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     clone_SequentialSharingClone.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/clone/clone_definitions.h>
#include <pia/clone/clone_SharingClone.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
@brief  This class is a derivative of <tt>CloneBase</tt> and provides functionality for bidirectional sending and receiving. It is capable of unlimited sending.

@attention  <tt>EventCloneElement</tt> and <tt>reckoning::ReckoningCloneElement</tt> objects cannot be registered with <tt>SequentialSharingClone</tt>.

@date 2013-09-25  Initial version.
*/
class SequentialSharingClone : public SharingClone
{
public:
/*!
@brief  Instantiates the object with default parameters (default constructor).
*/
    SequentialSharingClone();


/*!
@brief  Destroys the object (destructor).
*/
    virtual ~SequentialSharingClone();


/*!
@brief  Prints information that is useful for debugging.

@param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> type.
*/
    virtual void Trace(u64 flag) const;


    //! @cond PRIVATE

public:
    static Type GetTypeStatic()
    {
        return TYPE_SEQUENTIAL;
    }
    static Key ToKey(Id id)
    {
        return CloneBase::ToKey(TYPE_SEQUENTIAL, STATION_INDEX_INVALID, id);
    }
    virtual Type GetTypeVirtual() const
    {
        return GetTypeStatic();
    }

public:
    virtual bool IsReadyToSetValue() const;

protected:
public:
    template <int TEST_ID, typename Arg>
    static void Test(const SequentialSharingClone& obj, Arg* pArg = NULL);

    //! @endcond
};
}
}
}  // end of namespace nn::pia::clone
