﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     clone_SerializePolicyDefinition.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/clone/clone_definitions.h>
#include <pia/clone/clone_StraightSerializePolicy.h>
#include <pia/clone/clone_ReverseSerializePolicy.h>

namespace nn
{
namespace pia
{
namespace clone
{


/*!
@brief  Represents a serialization algorithm based on network byte order.
@details  Use this class when you want to communicate with the network byte order regardless of the hardware byte order.
@tparam Type  Specifies the type of the data to be serialized.
@date 2013-07-18  Initial version.
*/
template <typename Type>
class NetworkByteOrderSerializePolicy
#if NN_PIA_ENDIAN_BIG
    : public StraightSerializePolicy<Type>
#else
    : public ReverseSerializePolicy<Type>
#endif
{
};


/*!
@brief  Represents a serialization algorithm based on host byte order.
@details  This algorithm is fast because byte order conversion is not needed, but you cannot communicate between hardware with different host byte orders.
@tparam Type  Specifies the type of the data to be serialized.
@date 2013-07-18  Initial version.
*/
template <typename Type>
class HostByteOrderSerializePolicy : public StraightSerializePolicy<Type>
{
};


/*!
@brief  Represents a serialization algorithm based on little endian.
@details  Use this class when you want to communicate with little endian regardless of the hardware byte order.
@tparam Type  Specifies the type of the data to be serialized.
@date 2013-07-18  Initial version.
*/
template <typename Type>
class LittleEndianSerializePolicy
#if NN_PIA_ENDIAN_BIG
    : public ReverseSerializePolicy<Type>
#else
    : public StraightSerializePolicy<Type>
#endif
{
};
}
}
}  // end of namespace nn::pia::clone
