﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     clone_VoidMemberConcealer.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/clone/clone_definitions.h>


namespace nn
{
namespace pia
{
namespace clone
{

/*
n1589:memo
This is a wrapper template for not using extra memory when <tt>void</tt> is set to <tt>T2</tt>.
*/

//! @cond PRIVATE
template <typename T1, typename T2>
struct VoidMemberConcealer
{
public:
    VoidMemberConcealer()
      : m_T1(), m_T2()
    {
    }
    T1* GetT1Ptr()
    {
        return &m_T1;
    }
    T2* GetT2Ptr()
    {
        return &m_T2;
    }

private:
    T1 m_T1;
    T2 m_T2;
};

template <typename T1>
struct VoidMemberConcealer<T1, void>
{
public:
    VoidMemberConcealer()
      : m_T1()
    {
    }
    T1* GetT1Ptr()
    {
        return &m_T1;
    }
    void* GetT2Ptr()
    {
        return NULL;
    }

private:
    T1 m_T1;
};
//! @endcond
}
}
}  // end of namespace nn::pia::clone
