﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_CriticalSection.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/definitions.h>
#include <pia/common/common_RootObject.h>

#if NN_PIA_CAFE
#include <sdk_ver.h>

#if CAFE_OS_SDK_VERSION >= 21002
#define NN_PIA_USE_FAST_MUTEX 1
#else
#define NN_PIA_USE_FAST_MUTEX 0
#endif
#endif

#define PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE NN_PIA_UNDER_DEVELOP


namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
@brief  Wrapper for <tt>CriticalSection</tt>.
*/
class CriticalSection : public common::RootObject
{
public:
/*!
@brief  Constructor.
@param[in] lockOrder  Specifies the lock order. Specify the value of <tt>CriticalSectionLockChecker::LockOrder</tt>.
*/
    explicit CriticalSection(int lockOrder = -1);


/*!
@brief  Destructor.
*/
    ~CriticalSection();


/*!
@brief  Locks the critical section.
*/
    void Lock();


/*!
@brief  Tries to lock the critical section.
@return  Returns <tt>true</tt> if successful.
*/
    bool TryLock();


/*!
@brief  Unlocks the critical section.
*/
    void Unlock();


/*!
@brief  Prints information useful for debugging.

@param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> type.
*/
    void Trace(u64 flag) const;


/*!
@brief  Gets the lock order.
*/
#if PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE
    int GetLockOrder() const
    {
        return m_LockOrder;
    }
#endif


private:
#if NN_PIA_CTR
    nn::os::CriticalSection m_CriticalSection;
#elif NN_PIA_WIN || NN_PIA_NIN_WIN
    CRITICAL_SECTION m_CriticalSection;
#elif NN_PIA_CAFE
#if NN_PIA_USE_FAST_MUTEX
    OSFastMutex m_FastMutex;
#else
    OSMutex m_Mutex;
#endif
#else
#error  "Invalid platform"
#endif

#if PIA_CRITICAL_SECTION_LOCK_CHECKER_ENABLE
    int m_LockOrder;
#endif
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
