﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_Crypto.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
@brief  This class handles encryption and decryption.

@date 2013-03-06  Initial version.
*/
class Crypto
{
public:
/*!
@brief  Indicates the type of encryption algorithm.
*/
    enum Mode
    {
        MODE_NOTHING = 0,  //!< None
        MODE_AES_128       //!< AES-128
    };

    struct Setting
    {
        Mode mode;              //!< The encryption algorithm.
        const void* cpKeyData;  //!< The address of the encryption key data.
        size_t keySize;         //!< The size of the encryption key.
    };

/*!
@brief  Encrypts data using the specified settings.
@return
*/
    static nn::Result Encrypt(void* pDst, const void* pSrc, size_t srcSize, const Setting& setting);

/*!
@brief  Decrypts data using the specified settings.
@return
*/
    static nn::Result Decrypt(void* pDst, const void* pSrc, size_t srcSize, const Setting& setting);

/*!
@brief  Returns the block size according to the type of encryption algorithm.
@return
*/
    static size_t GetBlockSize(Mode mode);

private:
};
}
}
}  // end of namespace nn::pia::common

//! @endcond
