﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_CryptoSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>
#include <pia/common/common_Crypto.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
@brief  A structure for encryption settings.

@date 2013-12-04  Initial version.
*/
struct CryptoSetting
{
public:
/*!
@brief  Instantiates the object with default parameters (default constructor).
@details  The encryption algorithm is initialized to no encryption, and the encryption key is initialized to zero.
*/
    CryptoSetting();


public:
/*!
@brief  Indicates the type of encryption algorithm.
*/
    enum Mode
    {
        MODE_NOTHING = Crypto::MODE_NOTHING,  //!< None
        MODE_AES_128 = Crypto::MODE_AES_128   //!< AES-128
    };

/*!
@brief  Specifies the key size.
*/
    static const size_t KEY_SIZE = 16;

/*!
@brief  The type of encryption algorithm.
*/
    Mode m_Mode;

/*!
@brief  The encryption key.
@details  It does not need to be set if there is no encryption (<tt>MODE_NOTHING</tt>).
*/
    u8 m_Key[KEY_SIZE];
};
}
}
}  // end of namespace nn::pia::common
