﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_EmptyStream.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
@brief  This class uses the same interface as <tt>StringStream</tt>. However, it performs no substantial operations. It is intended to prevent warnings.
*/
class EmptyStream
{
public:
    EmptyStream(void)
    {
    }
    EmptyStream(u64)
    {
    }  // Dummy constructor used only for receiving flags.
    ~EmptyStream(void)
    {
    }

    u32 GetLength() const
    {
        return 0;
    }
    u32 GetSize() const
    {
        return 0;
    }
    const char* CStr() const
    {
        return NULL;
    }

    EmptyStream& operator<<(const char*)
    {
        return *this;
    }
    EmptyStream& operator<<(const void*)
    {
        return *this;
    }
    EmptyStream& operator<<(bool)
    {
        return *this;
    }
    EmptyStream& operator<<(f64)
    {
        return *this;
    }
    EmptyStream& operator<<(f32)
    {
        return *this;
    }

    EmptyStream& operator<<(u8)
    {
        return *this;
    }
    EmptyStream& operator<<(s8)
    {
        return *this;
    }
    EmptyStream& operator<<(u16)
    {
        return *this;
    }
    EmptyStream& operator<<(s16)
    {
        return *this;
    }
    EmptyStream& operator<<(u32)
    {
        return *this;
    }
    EmptyStream& operator<<(s32)
    {
        return *this;
    }

    EmptyStream& operator<<(u64)
    {
        return *this;
    }
    EmptyStream& operator<<(s64)
    {
        return *this;
    }

    EmptyStream& operator<<(const EmptyStream&)
    {
        return *this;
    }

    // Manipulators.
    EmptyStream& operator<<(EmptyStream& (*)(EmptyStream&))
    {
        return *this;
    }

    EmptyStream& Manip_hex()
    {
        return *this;
    }
    EmptyStream& Manip_dec()
    {
        return *this;
    }
    EmptyStream& Manip_zerohex()
    {
        return *this;
    }


    EmptyStream& Manip_showbase()
    {
        return *this;
    }
    EmptyStream& Manip_noshowbase()
    {
        return *this;
    }
    EmptyStream& Manip_boolalpha()
    {
        return *this;
    }
    EmptyStream& Manip_noboolalpha()
    {
        return *this;
    }

    const char& operator[](u32) const;  // Not implemented.

private:
    EmptyStream(const EmptyStream&);
    EmptyStream& operator=(const EmptyStream&);
};


inline EmptyStream& hex(EmptyStream& s)
{
    return s;
}
inline EmptyStream& dec(EmptyStream& s)
{
    return s;
}
inline EmptyStream& endl(EmptyStream& s)
{
    return s;
}
inline EmptyStream& showbase(EmptyStream& s)
{
    return s;
}
inline EmptyStream& noshowbase(EmptyStream& s)
{
    return s;
}
inline EmptyStream& boolalpha(EmptyStream& s)
{
    return s;
}
inline EmptyStream& noboolalpha(EmptyStream& s)
{
    return s;
}

inline EmptyStream& zerohex(EmptyStream& s)
{
    return s;
}

//! @endcond
}
}
}  // end of namespace nn::pia::common
