﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_FixedObjList.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>
#include <pia/common/common_ObjList.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
@brief  Represents an <tt>ObjList</tt> object that includes an internal buffer.

@param T  Specifies the type of object being managed.
@param N  Specifies the maximum number of elements that can be stored in the list.
*/
template <typename T, u32 N>
class FixedObjList : public nn::pia::common::ObjList<T>
{
public:
/**
@brief  Instantiates the object with default parameters (default constructor).
*/
    FixedObjList()
      : ObjList<T>()
    {
        SetBuffer(N, mBuffer);
    }

private:
    // Make <tt>SetBuffer</tt> unusable from the outside.
    void SetBuffer(u32 limitNum, void* pBuffer)
    {
        ObjList<T>::SetBuffer(limitNum, pBuffer);
    }

private:
    u8 mBuffer[ObjList<T>::c_NodeSize * N];
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
