﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_IPacketInput.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>

namespace nn
{
namespace pia
{
namespace common
{

class Packet;

/*!
@cond PRIVATE
@brief  This class is an interface for reading from \c PacketOld objects.
*/
class IPacketInput
{
public:
/*!
@brief  Default constructor.
*/
    IPacketInput(void)
    {
    }


/*!
@brief  Destructor.
*/
    virtual ~IPacketInput(void)
    {
    }


/*!
@brief  Interface for reading.

@param[out] pPacketData  Pointer to a \c Packet instance.
@return  If the read is successful, returns a \c Result value for which the \c IsSuccess function will return \c true.
@retval ResultInvalidArgument  One or more arguments is invalid (such as a \c NULL pointer was passed).
@retval ResultInvalidState  The timing by which this function has been called is incorrect (for example, initialization is not finished).
@retval ResultNoData  Indicates that no data was received (no data arrived).
@retval ResultNetworkConnectionIsLost  The network connection was lost.
*/
    virtual nn::Result Read(nn::pia::common::Packet* pPacketData) = 0;

/*!
@brief  A reader interface.

@param[out] pPacket  Pointer to a \c PacketOld instance.
@return  If the read operation succeeds, returns a <tt>Result</tt> value for which the <tt>IsSuccess</tt> function will return <tt>true</tt>.
@retval ResultInvalidArgument  An argument is incorrect (for example, a NULL pointer was passed in).
@retval ResultInvalidState  The timing by which this function has been called is incorrect (for example, initialization is not finished).
@retval ResultNoData  No data has been received (no data has arrived).
@retval ResultNetworkConnectionIsLost  The network connection was lost.
*/
    //virtual nn::Result Read(nn::pia::common::PacketOld*) {PIA_RETURN_RESULT(ResultNotImplemented);}
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
