﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_IPacketOutput.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>

#include <pia/common/private/common_Packet.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
@brief  This class is an interface for writing to \c PacketOld objects.
*/
class IPacketOutput
{
public:
/*!
@brief  Default constructor.
*/
    IPacketOutput(void)
    {
    }


/*!
@brief  Destructor.
*/
    virtual ~IPacketOutput(void)
    {
    }

    virtual nn::Result OnStationConnectionEvent() = 0;

/*!
@brief  A writing interface.

@param[in] packetData  The \c Packet object to write.
@return  If the write is successful, returns a \c Result value for which the \c IsSuccess function will return \c true.
@retval ResultInvalidArgument  There is an error in one or more arguments (such as there is something wrong with the packet destination).
@retval ResultInvalidState  The timing by which this function has been called is incorrect (for example, initialization is not finished).
@retval ResultBufferIsFull  The send failed because the buffer is momentarily full or CPU capacity is maxed out.
@retval ResultNetworkConnectionIsLost  The network connection was lost.
*/
    virtual nn::Result Write(const nn::pia::common::Packet& packetData) = 0;

/*!
@brief  Writer interface.

@param[in] packetData  The \c PacketOld object to write.
@return  If the write succeeds, returns a <tt>Result</tt> value for which the <tt>IsSuccess</tt> function will return <tt>true</tt>.
@retval ResultInvalidArgument  One or more arguments is invalid (for example, there is something wrong with the packet recipient).
@retval ResultInvalidState  The timing by which this function has been called is incorrect (for example, initialization is not finished).
@retval ResultBufferIsFull  The send operation failed because the buffer is temporarily full, or the processing capacity is maxed out.
@retval ResultNetworkConnectionIsLost  The network connection was lost.
*/
    //virtual nn::Result Write(const nn::pia::common::PacketOld&) {PIA_RETURN_RESULT(ResultNotImplemented);}


/*!
@brief  Gets the maximum number of recipients we can multicast to.
@details  The library controls packet generation so that the number of recipients specified in the recipient bitmap is never larger than this value.
@return  The maximum number of multicast recipients.
*/
    virtual u32 GetMulticastDestinationNum() const
    {
        return 1;
    }


/*!
@brief  Gets whether broadcasting is supported.
@details  If this value is <tt>true</tt>, you can specify <tt>STATION_INDEX_ALL</tt> to generate packets with more bits set than the value returned by the <tt>GetMulticastDestinationNum</tt> function.
@return  <tt>true</tt> if broadcasting is supported.
*/
    virtual bool IsBroadcastSupported() const
    {
        return false;
    }
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
