﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_ListBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>
#include <pia/common/common_ListNode.h>


namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
@brief  Represents the base class for lists.
*/
class ListBase : public nn::pia::common::RootObject
{
public:
/*!
@brief  Instantiates the object with default parameters (default constructor).
*/
    ListBase(void)
    {
        Init();
    }


/*!
@brief  Gets the number of elements in the list.
@return  Returns the number of elements in the list.
*/
    u32 Size() const
    {
        return m_Size;
    }


/*!
@brief  Determines whether the list is empty.
@return  Returns <tt>true</tt> if the list is empty.
*/
    bool IsEmpty() const
    {
        return (Size() == 0);
    }


protected:
    void Init();

    void InitListNode(ListNode* pNode)
    {
        pNode->InitListNode();
    }

    void ClearNode();

    void InsertBeforeNode(ListNode* pBasis, ListNode* pNode);

    void InsertAfterNode(ListNode* pBasis, ListNode* pNode);

    void EraseNode(ListNode* pNode);

    bool IsIncludeNode(const ListNode* pNode) const;


    void PushFrontNode(ListNode* pNode)
    {
        InsertAfterNode(&m_Terminator, pNode);
    }

    void PushBackNode(ListNode* pNode)
    {
        InsertBeforeNode(&m_Terminator, pNode);
    }

    ListNode* PopFrontNode();

    ListNode* PopBackNode();


    ListNode* FrontNode() const
    {
        return (m_Size > 0) ? m_Terminator.m_pNext : NULL;
    }

    ListNode* BackNode() const
    {
        return (m_Size > 0) ? m_Terminator.m_pPrev : NULL;
    }

    void RotateNode(ListNode* pNode);


    const ListNode* Terminator() const
    {
        return &m_Terminator;
    }

private:
    // Copying is prohibited.
    ListBase(const ListBase& rhs);
    ListBase& operator=(const ListBase& rhs);

private:
    ListNode m_Terminator;
    u32 m_Size;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
