﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_ListNode.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

class ListBase;

/*!
@cond PRIVATE
@brief  Represents the class for list nodes.
*/
class ListNode : public nn::pia::common::RootObject
{
    friend class ListBase;

public:
/*!
@brief  Instantiates the object with default parameters (default constructor).
*/
    ListNode()
    {
        InitListNode();
    }


/*!
Private:
@brief  Gets the next element.
@return  Returns the next element.
*/
    ListNode* Prev() const
    {
        return m_pPrev;
    }


/*!
Private:
@brief  Gets the previous element.
@return  Returns the previous element.
*/
    ListNode* Next() const
    {
        return m_pNext;
    }


    bool IsFreeListNode() const
    {
        return (m_pPrev == NULL) || (m_pNext == NULL);
    }

private:
    void InitListNode()
    {
        m_pPrev = NULL;
        m_pNext = NULL;
    }

    ListNode* m_pPrev;
    ListNode* m_pNext;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
