﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_RootObject.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/definitions.h>

#define NN_PIA_USE_ROOTOBJECT_NEW 1

#if NN_PIA_USE_ROOTOBJECT_NEW
#include <new>
#endif  // end of NN_PIA_USE_ROOTOBJECT_NEW


namespace nn
{
namespace pia
{
namespace common
{


/*!
@brief  This is the common base class used inside the Pia library.

@date 2012-04-23  Removed an unneeded destructor.
@date 2012-04-23  Initial version.
*/
class RootObject
{
public:
/*!
@brief  Instantiates the object.
*/
    RootObject(void)
    {
    }


#if NN_PIA_USE_ROOTOBJECT_NEW
/*!
@cond PRIVATE
@brief  Plain <tt>new</tt>.
*/
    static void* operator new(std::size_t size) throw();
    //! @endcond


/*!
@cond PRIVATE
@brief  Non-throwing <tt>new</tt>.
*/
    static void* operator new(std::size_t size, std::nothrow_t) throw();
    //! @endcond


/*!
@cond PRIVATE
@brief  Placement <tt>new</tt>.
*/
    static void* operator new(std::size_t, void* p) throw()
    {
        return p;
    }
    //! @endcond


/*!
@cond PRIVATE
@brief  Plain <tt>new</tt>.
*/
    static void* operator new[](std::size_t size) throw();
    //! @endcond


/*!
@cond PRIVATE
@brief  Non-throwing <tt>new</tt> (array version).
*/
    static void* operator new[](std::size_t size, std::nothrow_t) throw();
    //! @endcond


/*!
@cond PRIVATE
@brief  Placement <tt>new</tt> (array version).
*/
    static void* operator new[](std::size_t, void* p) throw()
    {
        return p;
    }
    //! @endcond


/*!
@cond PRIVATE
@brief  Delete operator.
*/
    static void operator delete(void* p);
    //! @endcond


/*!
@cond PRIVATE
@brief  Delete operator (array version).
*/
    static void operator delete[](void* p);
//! @endcond


#if NN_PIA_WIN || NN_PIA_NIN_WIN

    // On Win platform, a <tt>delete</tt> that corresponds to <tt>new</tt> is required.

/*!
@cond PRIVATE
@brief  Delete operator for non-throwing <tt>new</tt>.
*/
    static void operator delete(void* p, std::nothrow_t) throw();
    //! @endcond

/*!
@cond PRIVATE
@brief  Delete operator for placement <tt>new</tt>.
*/
    static void operator delete(void* p, void*) throw();
    //! @endcond

/*!
@cond PRIVATE
@brief  Delete operator for non-throwing <tt>new</tt> (array version).
*/
    static void operator delete[](void* p, std::nothrow_t) throw();
    //! @endcond

/*!
@cond PRIVATE
@brief  Delete operator for placement <tt>new</tt> (array version).
*/
    static void operator delete[](void* p, void*) throw();
//! @endcond

#endif

#endif  // end of NN_PIA_USE_ROOTOBJECT_NEW
};
}
}
}  // end of namespace nn::pia::common
