﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_ScopedLock.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
@brief  Locks while this instance is in existence.
*/
template <typename T>
class ScopedLock : public common::RootObject
{
public:
    explicit ScopedLock(T* pTarget)
      : m_pTarget(pTarget)
    {
        m_pTarget->Lock();
    }

    ~ScopedLock()
    {
        m_pTarget->Unlock();
    }

private:
    T* m_pTarget;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
