﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_StatisticValue.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>

namespace nn
{
namespace pia
{
namespace common
{


/*!
@cond PRIVATE
@brief  Statistics class.
*/
class StatisticValue : public nn::pia::common::RootObject
{
public:
    u32 m_Value;
    u32 m_ValueMax;
    u32 m_ValueMaxResult;
    u32 m_ValueMin;
    u32 m_ValueMinResult;
    u32 m_ValueSum;
    u32 m_Count;
    u32 m_Average;

/*!
@brief  Instantiates the object.
*/
    StatisticValue(void)
    {
        ResetValue();
    }

    void ResetValue()
    {
        m_Value = 0;
        m_ValueMax = 0;
        m_ValueMaxResult = 0;
        m_ValueMin = 0xFFFFFFFF;
        m_ValueMinResult = 0;
        m_ValueSum = 0;
        m_Count = 0;
        m_Average = 0;
    }

    void AddValue(u32 value)
    {
        m_Value = value;
        m_ValueSum += m_Value;
        m_Count++;
        if (m_Value > m_ValueMax)
        {
            m_ValueMax = m_Value;
        }
        if (m_Value < m_ValueMin)
        {
            m_ValueMin = m_Value;
        }
    }

    void CalcAverage()
    {
        m_Average = m_ValueSum / m_Count;
        m_ValueMaxResult = m_ValueMax;
        m_ValueMinResult = m_ValueMin;
    }
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
