﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_TreeMapBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>
#include <pia/common/common_TreeMapNodeBase.h>

namespace nn
{
namespace pia
{
namespace common
{

/*!
@cond PRIVATE
@brief  This is the default class for treemaps.
*/
class TreeMapBase : public common::RootObject
{
    typedef TreeMapNodeBase Node;

protected:
    TreeMapBase()
      : m_pRoot(NULL)
    {
    }


public:
/*!
@brief  Determines whether the object is empty.
@return  Returns <tt>true</tt> if the object is empty.
*/
    bool IsEmpty() const
    {
        return m_pRoot == NULL;
    }


/*!
@brief  Counts the number of elements.
@return  Returns the number of elements.
*/
    u32 Count() const;


protected:
    bool IsIncludeNodeBase(const Node* cpNode) const;

    void ClearNodeBase();

    Node* FrontNodeBase() const;
    Node* BackNodeBase() const;

    void InsertNodeRoot(Node* pNode);
    void InsertNodeLeft(Node* pNode, Node* pUp);
    void InsertNodeRight(Node* pNode, Node* pUp);
    void ReplaceNode(Node* pNode, Node* pOld);

    void EraseNodeBase(Node* pNode);

    Node* RotateRight(Node* pNode);
    Node* RotateLeft(Node* pNode);

protected:
    Node* GetRootBase() const
    {
        return m_pRoot;
    }

private:
    // Copying is prohibited.
    TreeMapBase(const TreeMapBase& rhs);
    TreeMapBase& operator=(const TreeMapBase& rhs);

    u32 CountImpl(const Node* cpNode) const;

    void ClearNodeBaseImpl(Node* pNode);

    void UpdateBalanceInsert(Node* pNode);

private:
    Node* m_pRoot;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
