﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_TreeMapNode.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>
#include <pia/common/common_TreeMapNodeBase.h>


namespace nn
{
namespace pia
{
namespace common
{

//! @cond PRIVATE
template <typename Key>
class TreeMap;
//! @endcond

/*!
@cond PRIVATE
@brief  Represents the class for treemap nodes.
*/
template <typename Key_>
class TreeMapNode : public common::TreeMapNodeBase
{
    friend class TreeMap<Key_>;

public:
    typedef Key_ Key;


/*!
@brief  Instantiates the object with default parameters (default constructor).
*/
    TreeMapNode()
      : TreeMapNodeBase(), m_Key()
    {
    }


/*!
@brief  Gets the key value.
@return  Returns the key value.
*/
    const Key& GetKey() const
    {
        return m_Key;
    }


private:
    void InitNode()
    {
        TreeMapNodeBase::InitNodeBase();
        m_Key = Key();
    }

public:
    TreeMapNode* Prev() const
    {
        return static_cast<TreeMapNode*>(PrevBase());
    }
    TreeMapNode* Next() const
    {
        return static_cast<TreeMapNode*>(NextBase());
    }

private:
    TreeMapNode* GetLeft() const
    {
        return static_cast<TreeMapNode*>(GetLeftBase());
    }
    TreeMapNode* GetRight() const
    {
        return static_cast<TreeMapNode*>(GetRightBase());
    }

private:
    Key m_Key;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
