﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     common_TreeMapNodeBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common/common_definitions.h>


namespace nn
{
namespace pia
{
namespace common
{

class TreeMapBase;

/*!
@cond PRIVATE
@brief  Represents the class for treemap nodes.
*/
class TreeMapNodeBase : public common::RootObject
{
    friend class TreeMapBase;

protected:
    TreeMapNodeBase()
    {
        InitNodeBase();
    }

protected:
    TreeMapNodeBase* PrevBase() const;

    TreeMapNodeBase* NextBase() const;

protected:
    void InitNodeBase();

    TreeMapNodeBase* GetLeftBase() const
    {
        return m_pLeft;
    }
    TreeMapNodeBase* GetRightBase() const
    {
        return m_pRight;
    }

private:
    // Copying is prohibited.
    TreeMapNodeBase(const TreeMapNodeBase& rhs);
    TreeMapNodeBase& operator=(const TreeMapNodeBase& rhs);

private:
    TreeMapNodeBase* m_pUp;
    TreeMapNodeBase* m_pLeft;
    TreeMapNodeBase* m_pRight;
    int m_Balance;
};
//! @endcond
}
}
}  // end of namespace nn::pia::common
