﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: inet_LanSessionInfo.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/inet/inet_LanSessionTypes.h>
#include <pia/transport/transport_StationConnectionInfo.h>
#include <pia/session/session_ISessionInfo.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
    @cond  PRIVATE
    @brief  Data class for LAN matchmaking sessions.
*/
class LanSessionInfo : public session::ISessionInfo
{
public:
    LanSessionInfo();
    virtual ~LanSessionInfo();

    virtual u32 GetGameMode() const;
    virtual u32 GetSessionId() const;
    virtual u32 GetCurrentParticipants() const;
    virtual u32 GetMinParticipants() const;
    virtual u32 GetMaxParticipants() const;
    virtual bool IsOpened() const;
    virtual nn::Result GetAttribute(u32* pAttribute, u8 idx) const;
    virtual LanSessionType GetSessionType() const;
    virtual nn::Result GetDescription(wchar_t* pBuffer, size_t size) const;
    virtual nn::Result GetDescriptionSize(size_t* pSize) const;
    virtual nn::Result GetApplicationDataBuf(void* pBuffer, size_t size);
    virtual nn::Result GetApplicationDataSize(size_t* pSize);

    virtual transport::StationConnectionInfo& GetStationConnectionInfo();

    nn::Result Serialize(bit8* pBuffer, size_t* pDataSize, size_t bufferSize) const;
    nn::Result Deserialize(const bit8* pData, size_t dataSize);
    size_t GetSerializedSize() const;

    virtual void SetGameMode(u32 gameMode);
    virtual void SetSessionId(u32 sessionId);
    virtual void SetCurrentParticipants(u32 participants);
    virtual void SetMinParticipants(u32 participants);
    virtual void SetMaxParticipants(u32 participants);
    virtual void SetOpen(bool isOpened);
    virtual void SetSessionType(LanSessionType type);
    virtual void SetAttribute(u32 attribute, u8 idx);
    virtual void SetDescription(const void* pDesc, size_t size);
    virtual void SetApplicationData(const void* pData, size_t size);
    virtual void SetStationConnectionInfo(const transport::StationConnectionInfo& info);

    virtual void Trace(u64 flag);
    virtual void Clear();
    virtual void Copy(const LanSessionInfo& lanSessionInfo);

private:
    u32 m_GameMode;
    u32 m_SessionId;
    u32 m_CurrentParticipants;
    u32 m_MinParticipants;
    u32 m_MaxParticipants;
    bool m_bIsOpened;

    u32 m_Attribute[LAN_ATTRIBUTE_MAX_SIZE];
    LanSessionType m_SessionType;
    wchar_t m_Description[LAN_DESCRIPTION_DATA_BUFFER_MAX_SIZE];
    size_t m_DescriptionSize;
    u8 m_ApplicationData[LAN_APPLICATION_DATA_BUFFER_MAX_SIZE];
    size_t m_ApplicationDataSize;

    transport::StationConnectionInfo m_StationConnectionInfo;

    LanSessionInfo(const LanSessionInfo& r);
    LanSessionInfo& operator=(const LanSessionInfo& r);

};

/*!
    @endcond
*/

}
}
}  // End of namespace nn::pia::inet.
