﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: inet_NexSessionSearchCriteriaOwner.h

  Copyright Nintendo. All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/inet/inet_definitions.h>

#include <pia/inet/inet_NexSessionTypes.h>
#include <pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  The condition class used when searching for a session.

  @see nn::pia::session::SessionSearchCriteria
*/
class NexSessionSearchCriteriaOwner : public nn::pia::session::SessionSearchCriteria
{
public:
/*!
    @brief      The default constructor.
*/
    NexSessionSearchCriteriaOwner() :
        session::SessionSearchCriteria(),
        m_OwnerPrincipalId(0)
    {
        m_KeyType = session::SessionSearchCriteria::SearchCriteriaKey_OwnerPrincipalId;
    }

/*!
    @brief      The constructor.
    @param[in]  ownerPrincipalId    Specify the principal ID of the owner as a search condition.
    @param[in]  resultOffset        Starting position of the entries to get as search results from the list of entries that match the search criteria.
    @param[in]  resultSize          Number of entries to get as search results from the list of entries that match the search criteria.
*/
    NexSessionSearchCriteriaOwner(u32 ownerPrincipalId, u32 resultOffset, u32 resultSize) :
        m_OwnerPrincipalId(ownerPrincipalId)
    {
        m_KeyType = session::SessionSearchCriteria::SearchCriteriaKey_OwnerPrincipalId;
        m_ResultOffset = resultOffset;
        m_ResultSize = resultSize;
    }

/*!
    @brief      The destructor.
*/
    virtual ~NexSessionSearchCriteriaOwner()
    {
    }

/*!
    @brief      Specifies the principal ID of the owner as a search condition.
    @param[in]  ownerPrincipalId    The principal ID of the owner to use as a search condition.
*/
    void SetOwnerPrincipalId(u32 ownerPrincipalId)
    {
        m_OwnerPrincipalId = ownerPrincipalId;
    }

/*!
    @brief      Assignment operator.
    @param[in]  r The object to assign.
    @return     See this object.
*/
    NexSessionSearchCriteriaOwner& operator=(const NexSessionSearchCriteriaOwner& r)
    {
        session::SessionSearchCriteria::operator=(r);
        m_OwnerPrincipalId = r.m_OwnerPrincipalId;
        return *this;
    }

/*!
    @cond PRIVATE
*/
    u32 GetOwnerPrincipalId() const
    {
        return m_OwnerPrincipalId;
    }
    //! @endcond

protected:

    // The copy constructor is sealed.
    NexSessionSearchCriteriaOwner(const NexSessionSearchCriteriaOwner&);

    u32 m_OwnerPrincipalId;

};
}
}
}  // End of namespace nn::pia::inet.

