﻿/*--------------------------------------------------------------------------------*
  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
 *--------------------------------------------------------------------------------*/


#pragma once

#include <pia/inet/inet_Definitions.h>

#include <pia/inet/inet_NexSessionTypes.h>
#include <pia/session/session_SessionSearchCriteria.h>

namespace nn
{
namespace pia
{
namespace inet
{

/*!
  @brief  The search condition class used when searching for a community.

  @see  nn::pia::session::SessionSearchCriteria
 */
class NexSessionSearchCriteriaParticipant : public session::SessionSearchCriteria
{
public:
/*!
    @brief  The default constructor.
*/
    NexSessionSearchCriteriaParticipant();

/*!
    @brief  The destructor.
*/
    virtual ~NexSessionSearchCriteriaParticipant();

/*!
    @brief  Sets the principal ID of the destination you are specifying as a search condition.
    @param[in] idList  The principal ID list for the destination you are specifying as a search condition.
    @param[in] idListSize  The principal ID list size for the destination you are specifying as a search condition.
    @return  Returns a successful result when search conditions are successfully specified.
    @retval nn::pia::ResultInvalidArgument  Indicates that the list is <tt>NULL</tt>, or the number of values is out of bounds. Programming error. Fix your program so that this error is not returned.
*/
    Result SetParticipantPrincipalId(const u32* idList, size_t idListSize);

/*!
    @cond PRIVATE
*/
    const u32* GetParticipantPrincipalId() const
    {
        return m_ParticipantPrincipalId;
    }
    size_t GetParticipantPrincipalIdSize() const
    {
        return m_ParticipantPrincipalIdNum;
    }
    //! @endcond

/*!
    @brief  Assignment operator.
    @param[in] rhs  The object to assign.
    @return  A reference to this object.
*/
    NexSessionSearchCriteriaParticipant& operator=(const NexSessionSearchCriteriaParticipant& rhs);

/*!
    @brief  Copies all settings.
    @param[in] rhs  The object to copy from.
*/
    void Copy(const NexSessionSearchCriteriaParticipant& rhs);

/*!
    @brief  Copy constructor.
    @param[in] rhs  The object to copy from.
*/
    NexSessionSearchCriteriaParticipant(const NexSessionSearchCriteriaParticipant& rhs);

/*!
    @brief  Resets the search conditions.
*/
    virtual void Reset();

protected:
    u32 m_ParticipantPrincipalId[PARTICIPANT_ARRAY_MAX_SIZE];
    size_t m_ParticipantPrincipalIdNum;

private:
    // SetResultRange is private.
    virtual void SetResultRange(u32 offset, u32 size);
};
}
}
}  // End of namespace nn::pia::inet.

