﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: inet_NexSessionTypes.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/inet/inet_definitions.h>

#if NN_PIA_NEX_NEW_FEATURE_FOR_MATCHMAKE
#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif
#include <nex.h>
#if NN_PIA_CTR
#pragma diag_warning 1300
#endif
#endif


namespace nn
{
namespace pia
{
namespace inet
{

const size_t GATHERINGHOLDER_MAX_SIZE = 100;

const size_t ATTRIBUTE_MAX_SIZE = 6; //!<  The number of attributes that can be specified in a session.
const size_t ATTRIBUTE_ARRAY_MAX_SIZE = 100;
const size_t PARTICIPANT_ARRAY_MAX_SIZE = 16;        //!<  The maximum length of the Principal ID array for users that can be specified in the search conditions.
const size_t APPLICATION_DATA_BUFFER_MAX_SIZE = 384; //!<  The maximum size that can be set as application-defined data.
const size_t APPLICATION_DATA_BUFFER_MAX_SIZE_INTERNAL = 512;
const size_t APPLICATION_DATA_BUFFER_DEFAULT_OFFSET = APPLICATION_DATA_BUFFER_MAX_SIZE_INTERNAL - APPLICATION_DATA_BUFFER_MAX_SIZE;

const size_t DESCRIPTION_DATA_BUFFER_MAX_SIZE = 256;

const size_t SEARCH_CRITERIA_MAX_NUM = 2;  //!<  The maximum number of search conditions that can be specified.

#if NN_PIA_NEX_NEW_FEATURE_FOR_MATCHMAKE
const u32 SESSION_SYSTEM_PASSWORD_LENGTH = nn::nex::MATCHMAKE_SESSION_SYSTEM_PASSWORD_LENGTH;  //!<  The system password's string length.
const u32 SESSION_USER_PASSWORD_LENGTH = nn::nex::MAX_MATCHMAKE_SESSION_USER_PASSWORD_LENGTH;  //!<  The user password's string length.

const u32 SESSION_UPDATE_SELECTION_PRIORITY_MINIMUM_INTERVAL_TIME = nn::nex::UPDATE_PROGRESS_SCORE_MINIMUM_INTERVAL_TIME;  //!<  Minimum update interval for the game progress score.
const u8 SESSION_MAX_SELECTION_PRIORITY = nn::nex::MAX_PROGRESS_SCORE;  //!<  Maximum configurable game progress score.

const u32 SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX = nn::nex::MATCHMAKE_SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX; //!<  Index of the base attribute value when selecting sessions with range broadening.
#else
const u32 SESSION_SYSTEM_PASSWORD_LENGTH = 16;
const u32 SESSION_USER_PASSWORD_LENGTH = 32;

const u32 SESSION_UPDATE_SELECTION_PRIORITY_MINIMUM_INTERVAL_TIME = 30;
const u8 SESSION_MAX_SELECTION_PRIORITY = 100;

const u32 SESSION_BROADEN_RANGE_ATTRIBUTE_INDEX = 1;
#endif

const u32 MAX_IP_ADDRESS_LENGTH = 15;

#if NN_PIA_ENABLE_MATCHMAKE_REFEREE
const size_t SESSION_PERSONAL_REPORT_BUFFER_MAX_SIZE = nn::nex::MatchmakeRefereeConstants::STATS_BUFFER_MAX_SIZE;
#endif


/*!
        @brief  Enumerates the session types.
*/
enum SessionType
{
    SESSION_TYPE_ANYBODY = 0,  //!<  Anybody can participate.
    SESSION_TYPE_FRIEND        //!<  Participation is only possible when there is a friend relationship with the host.
};

/*!
        @brief  Enumerates the selection methods for joining sessions when random matchmaking is in force.
*/
enum SelectionMethod
{
    SELECTION_METHOD_RANDOM = 0,        //!<  Random selection.
    //SELECTION_METHOD_NEAREST_NEIGHBOR,  //!<  Selects the session with the closest attribute value.
    //SELECTION_METHOD_BROADEN_RANGE,      //!<  Selects using range broadening.
    SELECTION_METHOD_BROADEN_RANGE_WITH_SELECTION_PRIORITY, //!<  Selected by range broadening and matchmaking priority.
    SELECTION_METHOD_SCORE_BASED //!<  Selected by score-conversion format.
};

#if NN_PIA_ENABLE_MATCHMAKE_REFEREE
enum PersonalGameResultFlag
{
    RESULT_FLAG_NONE = nn::nex::MatchmakeRefereeConstants::PERSONAL_ROUND_RESULT_FLAG_NONE,
    RESULT_FLAG_DISCONNECT = nn::nex::MatchmakeRefereeConstants::PERSONAL_ROUND_RESULT_FLAG_DISCONNECT,
    RESULT_FLAG_VIOLATION = nn::nex::MatchmakeRefereeConstants::PERSONAL_ROUND_RESULT_FLAG_VIOLATION
};

enum GameResult
{
    GAME_RESULT_LOSS = nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_LOSS,
    GAME_RESULT_WIN = nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_WIN,
    GAME_RESULT_DRAW = nn::nex::MatchmakeRefereeConstants::ROUND_RESULT_DRAW
};
#endif

}
}
}  // End of namespace nn::pia::inet.
