﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     inet_definitions.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common.h>

//n1872: NEX version information.
#if NN_PIA_NEX_3
#include <OnlineCore/src/version/nex_version.h>
#else
#include <version/nex_version.h>
#endif

#define NEX_VERSION_NUMBER(major, minor, micro) (((major) << 16) | ((minor) << 8) | ((micro) << 0))
#define NEX_CURRENT_VERSION_NUMBER NEX_VERSION_NUMBER(NEX_VERSION_MAJOR, NEX_VERSION_MINOR, NEX_VERSION_MICRO)

//n1589: Performs NAT property specification on the NEX side.
//n1589: If disabled, the NAT check is performed when the first matchmaking is started rather than when logging in, so that no time is wasted while waiting when just logging in.
#if NN_PIA_NEX_3
#define NN_PIA_ENABLE_NEX_NAT_PROPERTY_DETECTION 0
#else
//n1589: The NEX 2 series had no feature to send NAT property statistical information, so NAT properties had to be specified when logging in with NEX. This is not a problem as long as this information can be sent with the client monitoring feature.
#define NN_PIA_ENABLE_NEX_NAT_PROPERTY_DETECTION 0
#endif

//n1589: Incrementation of the EDM port is fixed to 1. (A value other than 1 is never encountered.)
#define NN_PIA_ENABLE_EDM_PORT_INCREMENT_FIX 1

#define NN_PIA_ENABLE_NEX_NAT_PATCH 0

//n1872: Internally use the Pia list class.
#define NN_PIA_ENABLE_PIA_OBJLIST 1

//n1872: If the session host uses an EIM router and clients use an EDM, a process is executed to find the success rate of NAT traversal (starting from NEX 3.4.4).
#if NN_PIA_WIN
#define NN_PIA_ENABLE_EDM_EIM_MASTER 0
#else  // CTR, CAFE
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3, 4, 4))
#define NN_PIA_ENABLE_EDM_EIM_MASTER 1
#else
#define NN_PIA_ENABLE_EDM_EIM_MASTER 0
#endif  // (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3,4,4))
#endif  // NN_PIA_WIN

//n1872: Makes responses using NEX RMC unnecessary (starting from NEX 3.0.0).
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3, 0, 0))
#define NN_PIA_ENABLE_NEX_RMC_ONEWAY 1
#else
#define NN_PIA_ENABLE_NEX_RMC_ONEWAY 0
#endif

#define NN_PIA_ENABLE_FEEDBACK_WITHOUT_GATEWAY 0

//n1872: Ignores packets from the NAT server in the stream after P2P communication starts.
#define NN_PIA_ENABLE_IGNORE_NAT_SERVER_ADDRESS 1

//n1872: NAT Resolves the DNS for the check server in the background.
#define NN_PIA_ENABLE_ASYNCRONOUS_DNS_RESOLUTION 1

//n1872: NAT Resolves the DNS for the check server without using NEX.
#define NN_PIA_ENABLE_RESOLVE_NAT_CHECK_SERVER 1

#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3, 6, 0))
#define NN_PIA_ENABLE_LOOSE_COUPLING_NEX 1
#else
#define NN_PIA_ENABLE_LOOSE_COUPLING_NEX 0
#endif

#define NN_PIA_EXPERIMENT_GET_LOCAL_URL 1

#define NN_PIA_EXPERIMENT_FORCED_HAIR_PIN 0

//n1872: Handles the situation when a different port is mapped with EIM.
#define NN_PIA_EXPERIMENT_EIM_DIFFERENT_PORT_MAPPING 1

#define NN_PIA_EXPERIMENT_EDM_CHECK_TARGET_LOCATION_EXPIRED 0

#define NN_PIA_EXPERIMENT_EDM_PORT_SPRAY 1

#define NN_PIA_EXPERIMENT_RETRY_NAT_TRAVERSAL 1

#define NN_PIA_EXPERIMENT_CHECK_PROBE_RECEIVED 1

#define NN_PIA_EXPERIMENT_CONFIRM_PROBE_ADDRESS 0

#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3, 8, 0))
//n1589: Adds support for the <tt>nex::NgsFacade</tt> interface.
#define NN_PIA_ENABLE_NGS_BRIDGE 1
#define NN_PIA_ENABLE_MATCHMAKE_SESSION_PASSWORD_ENABLED 1
#define NN_PIA_EXPERIMENT_LAST_GID_OPTION 1
#else
#define NN_PIA_ENABLE_NGS_BRIDGE 0
#define NN_PIA_ENABLE_MATCHMAKE_SESSION_PASSWORD_ENABLED 0
#define NN_PIA_EXPERIMENT_LAST_GID_OPTION 0
#endif

#define NN_PIA_EXPERIMENT_CHECK_JOINED_UNKNOWN_SESSION 0

//n1589: The normal definitions are from here down.
namespace nn
{
namespace pia
{
namespace inet
{

#if NN_PIA_ENABLE_EACH_NAT_TRAVERSAL_TIMEOUT
const s32 NAT_TRAVERSAL_TIMEOUT_DEFAULT = (5 * 1000);
const s32 NAT_TRAVERSAL_TIMEOUT_EDM_EXTENSION = 1500;
#else
const s32 NAT_TRAVERSAL_TIMEOUT_DEFAULT = (8 * 1000);
const s32 NAT_TRAVERSAL_TIMEOUT_EDM_EXTENSION = 1500;
#endif

const u32 NEX_INFINITE = 0xFFFFFFFF;
const u32 NEX_CONNECTION_ID_INVALID = 0;

//n1589: Used in place of <tt>nn::nex::ConnectionID</tt>.
typedef u32 ServerConnectionId;

const u16 FEEDBACK_DESTINATION_PORT = 50034;
}
}
}  // end of namespace nn::pia::inet

//n1589: The following header files are included for all classes in this module.
#include <pia/common/common_InetAddress.h>
#include <pia/common/common_StationAddress.h>
#include <pia/transport/transport_Protocol.h>
#include <pia/transport/transport_StationLocation.h>
#include <pia/transport/transport_StationLocationContainer.h>
#include <pia/transport/transport_StationConnectionInfo.h>
#include <pia/inet/inet_Api.h>
