﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     local_LocalCreateSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_CreateSessionSetting.h>
#include <pia/local/local_LocalSessionTypes.h>

#include <pia/common/common_SignatureSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

// Forward declarations.
class LocalCreateNetworkSetting;

/*!
@brief  Base class that manages the settings for the <tt>CreateSession</tt> function with a <tt>LocalNetwork</tt> object.

@details  This class is never instantiated by an application.
Use a class that inherits this class.

@see  nn::pia::local::UdsCreateSessionSetting

@date 2014-03-03  Initial version.
*/
class LocalCreateSessionSetting : public nn::pia::session::CreateSessionSetting
{
public:
/*!
@cond PRIVATE
@brief  Destroys the object.
*/
    virtual ~LocalCreateSessionSetting();
    //! @endcond

    const virtual LocalCreateNetworkSetting* GetLocalCreateNetworkSetting() const = 0;
    virtual void SetLocalCreateNetworkSetting(const LocalCreateNetworkSetting& createNetworkSetting) = 0;

/*!
@cond PRIVATE
*/
    const char* GetSignatureKeyData() const
    {
        return m_SignatureKeyData;
    }
    //! @endcond

/*!
@cond PRIVATE
*/
    size_t GetSignatureKeyDataSize() const
    {
        return m_SignatureKeyDataSize;
    }
    //! @endcond

/*!
@brief  Sets the session key and its size.

@details  Set the session key size to no more than <tt>@ref common::SignatureSetting::SIGNATURE_KEY_DATA_SIZE_MAX</tt>.

@param[in] pSignatureKeyData  The session key.
@param[in] signatureKeyDataSize  The size of the session key.

@return  Returns a <tt>Result</tt> value for which the <tt>IsSuccess</tt> function returns <tt>true</tt> if execution succeeds.

@retval ResultInvalidArgument  Indicates that an argument is invalid. Programming error. Fix your program so that this error is not returned.
*/
    nn::Result SetSignatureKey(const char* pSignatureKeyData, size_t signatureKeyDataSize);

    // Prints information that is useful for debugging.
    virtual void Trace(u64 flag) const;

protected:
    char m_SignatureKeyData[common::SignatureSetting::SIGNATURE_KEY_DATA_SIZE_MAX];
    size_t m_SignatureKeyDataSize;

/*!
@cond PRIVATE
@brief  Instantiates an object.
*/
    LocalCreateSessionSetting();
    //! @endcond

private:
    // The copy constructor is sealed.
    LocalCreateSessionSetting(const LocalCreateSessionSetting&);

    // The assignment operator is sealed.
    LocalCreateSessionSetting& operator=(const LocalCreateSessionSetting&);
};
}
}
}  // end of namespace nn::pia::local
