﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     local_LocalJoinSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_JoinSessionSetting.h>
#include <pia/local/local_LocalSessionTypes.h>

#include <pia/common/common_SignatureSetting.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
@brief  Base class that manages the settings for the <tt>JoinSession</tt> function with a <tt>LocalNetwork</tt> object.

@details  This class is never instantiated by an application.
Use a class that inherits this class.

@see  nn::pia::local::UdsJoinSessionSetting

@date 2014-05-28  Renamed the <tt>GetSessionId</tt> function to <tt>GetSessionId</tt>.
@date 2014-04-30  Added the <tt>GetSessionId</tt> function.
@date 2014-03-03  Initial version.
*/
class LocalJoinSessionSetting : public nn::pia::session::JoinSessionSetting
{
public:
/*!
@cond PRIVATE
@brief  Destroys the object.
*/
    virtual ~LocalJoinSessionSetting();
    //! @endcond

/*!
@cond PRIVATE
*/
    virtual const char* GetPassphrase() const = 0;
    //! @endcond

/*!
@cond PRIVATE
*/
    size_t GetPassphraseSize() const
    {
        return m_PassphraseSize;
    }
    //! @endcond

    virtual nn::Result SetPassphrase(const char* pPassphrase, size_t passphraseSize) = 0;

/*!
@cond PRIVATE
*/
    const char* GetSignatureKeyData() const
    {
        return m_SignatureKeyData;
    }
    //! @endcond

/*!
@cond PRIVATE
*/
    size_t GetSignatureKeyDataSize() const
    {
        return m_SignatureKeyDataSize;
    }
    //! @endcond

/*!
@cond PRIVATE
*/
    virtual u32 GetSessionId() const;
    //! @endcond

/*!
@brief  Sets the session key and its size.

@details  Set the session key size to no more than <tt>@ref common::SignatureSetting::SIGNATURE_KEY_DATA_SIZE_MAX</tt>.

@param[in] pSignatureKeyData  The session key.
@param[in] signatureKeyDataSize  The size of the session key.

@return  Returns a <tt>Result</tt> value for which the <tt>IsSuccess</tt> function returns <tt>true</tt> if execution succeeds.

@retval ResultInvalidArgument  Indicates that an argument is invalid. Programming error. Fix your program so that this error is not returned.
*/
    nn::Result SetSignatureKey(const char* pSignatureKeyData, size_t signatureKeyDataSize);

protected:
    size_t m_PassphraseSize;
    char m_SignatureKeyData[common::SignatureSetting::SIGNATURE_KEY_DATA_SIZE_MAX];
    size_t m_SignatureKeyDataSize;

/*!
@cond PRIVATE
@brief  Instantiates an object.
*/
    LocalJoinSessionSetting();
    //! @endcond

private:
    // The copy constructor is sealed.
    LocalJoinSessionSetting(const LocalJoinSessionSetting&);

    // The assignment operator is sealed.
    LocalJoinSessionSetting& operator=(const LocalJoinSessionSetting&);
};
}
}
}  // end of namespace nn::pia::local
