﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     local_LocalSessionStationInfo.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/local/local_definitions.h>
#include <pia/local/local_LocalNetworkSetting.h>
#include <pia/session/session_SessionStationInfo.h>

namespace nn
{
namespace pia
{
namespace local
{

/*!
@brief  Represents information about a station that is participating in a session.

@date 2014-05-27  Initial version.
*/
class LocalSessionStationInfo : public session::SessionStationInfo
{
public:
    LocalSessionStationInfo()
      : m_pStationInfoList(NULL), m_ListLength(0)
    {
    }
    virtual ~LocalSessionStationInfo()
    {
    }

/*!
@brief  Sets a buffer to store station information.

@details  Make the array length of lists provided as buffers equal to the maximum number of stations that can participate in the session.

@param[in] pStationInfoList  The buffer for storing station information.
@param[in] listLength  The length of lists that can be provided as buffers.
*/
    void SetStationInfoListBuf(LocalStationInfo* pStationInfoList, u8 listLength)
    {
        m_pStationInfoList = pStationInfoList;
        m_ListLength = listLength;
    }

/*!
@cond PRIVATE
*/
    LocalStationInfo* GetStationInfoListBuf()
    {
        return m_pStationInfoList;
    }
    //! @endcond

/*!
@cond PRIVATE
*/
    u8 GetStationInfoListLength() const
    {
        return m_ListLength;
    }
    //! @endcond

private:
    LocalStationInfo* m_pStationInfoList;
    u8 m_ListLength;
};
}
}
}  // end of namespace nn::pia::local
