﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     local_UdsCreateSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/local/local_LocalCreateSessionSetting.h>
#include <pia/local/local_UdsNetworkSetting.h>

namespace nn
{
namespace pia
{
namespace local
{
/*!
@brief  Class that manages the <tt>CreateSession</tt> settings for networks using the UDS library (local communication).

@date 2014-03-03  Initial version.
*/
class UdsCreateSessionSetting : public LocalCreateSessionSetting
{
public:
/*!
@brief  Instantiates an object.
*/
    UdsCreateSessionSetting();

/*!
@brief  Destroys the object.
*/
    virtual ~UdsCreateSessionSetting();

/*!
@cond PRIVATE
*/
    virtual const LocalCreateNetworkSetting* GetLocalCreateNetworkSetting() const
    {
        return &m_UdsCreateNetworkSetting;
    }
    //! @endcond

/*!
@brief  Specifies the settings used when generating the network.

@param[in] createNetworkSetting  Settings used when the network is generated.
*/
    virtual void SetLocalCreateNetworkSetting(const LocalCreateNetworkSetting& createNetworkSetting);

    // Prints information that is useful for debugging.
    virtual void Trace(u64 flag) const;

protected:
    UdsCreateNetworkSetting m_UdsCreateNetworkSetting;

private:
    // The copy constructor is sealed.
    UdsCreateSessionSetting(const UdsCreateSessionSetting&);

    // The assignment operator is sealed.
    UdsCreateSessionSetting& operator=(const UdsCreateSessionSetting&);
};
}
}
}  // end of namespace nn::pia::local
