﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     local_UdsJoinSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/local/local_LocalJoinSessionSetting.h>

namespace nn
{
namespace pia
{
namespace local
{
/*!
@brief  Class that manages <tt>JoinSession</tt> settings in <tt>LocalNetwork</tt> sessions.

@date 2014-03-03  Initial version.
*/

class UdsJoinSessionSetting : public LocalJoinSessionSetting
{
public:
    //! Specifies the minimum size of the passphrase used to encrypt the wireless layer.
    static const size_t PASSPHRASE_LENGTH_MIN = nn::uds::UDS_PASSPHRASE_LENGTH_MIN;

    //! Specifies the maximum size of the passphrase used to encrypt the wireless layer.
    static const size_t PASSPHRASE_LENGTH_MAX = nn::uds::UDS_PASSPHRASE_LENGTH_MAX;

/*!
@brief  Instantiates an object.
*/
    UdsJoinSessionSetting()
    {
    }

/*!
@brief  Destroys the object.
*/
    virtual ~UdsJoinSessionSetting() {};

/*!
@cond PRIVATE
*/
    virtual const char* GetPassphrase() const
    {
        return m_Passphrase;
    }
    //! @endcond

/*!
@brief  Sets the passphrase used for encrypting the wireless layer and its size.

@details  Set the size of the passphrase to a value of at least <tt>@ref PASSPHRASE_LENGTH_MIN</tt> but no more than <tt>@ref PASSPHRASE_LENGTH_MAX</tt>.

@param[in] pPassphrase  Specifies the passphrase to use for wireless layer encryption.
@param[in] passphraseSize  Specifies the size of the passphrase to use for wireless layer encryption.

@return  Returns a <tt>Result</tt> value for which the <tt>IsSuccess</tt> function returns <tt>true</tt> if execution succeeds.

@retval ResultInvalidArgument  Indicates that an argument is invalid. Programming error. Fix your program so that this error is not returned.
*/
    virtual nn::Result SetPassphrase(const char* pPassphrase, size_t passphraseSize);

protected:
    char m_Passphrase[PASSPHRASE_LENGTH_MAX];

private:
    // The copy constructor is sealed.
    UdsJoinSessionSetting(const UdsJoinSessionSetting&);

    // The assignment operator is sealed.
    UdsJoinSessionSetting& operator=(const UdsJoinSessionSetting&);
};
}
}
}  // end of namespace nn::pia::local
