﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: local_UdsNetworkFactory.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/local/local_definitions.h>
#include <pia/transport/transport_NetworkFactory.h>
#include <pia/local/local_LocalNetworkFactory.h>


namespace nn
{
namespace pia
{
namespace local
{


/*!
  @class  nn::pia::local::UdsNetworkFactory
  @brief  This class creates the necessary classes for networks using <tt>LocalNetwork</tt> objects. It implements the factory pattern.

  @details  Setting the <tt>UdsNetworkFactory</tt> class to <tt>@ref nn::pia::transport::Transport</tt> or <tt>@ref nn::pia::session::Session</tt>
           enables network-related processes (sending and receiving of data, session management, and so on) that are suitable for <tt>LocalNetwork</tt>.

  @see  nn::pia::transport::Transport::Setting, nn::pia::session::Session::Setting

  @date 2014-03-31  Added support for <tt>LocalNetwork</tt>.
  @date 2013-04-01  Noted the restriction on using send and receive functions on the <tt>UdsNode</tt> class when using <tt>PiaSession</tt>.
  @date 2013-03-28  Added the <tt>CreatePacketHandler()</tt> function.
  @date 2013-03-21  Added the <tt>CreateSignatureSettingStorage()</tt> function.
  @date 2013-03-14  Added the <tt>CreateCreateMeshJob()</tt> function.
  @date 2012-11-27  Added the <tt>IsSignatureRequired()</tt> function.
  @date 2012-10-29  Categorized member functions to improve readability of the API reference.
  @date 2012-04-06  Initial version.
*/
class UdsNetworkFactory : public nn::pia::local::LocalNetworkFactory
{
public:
/*!
     @name  Finish Initialization
     @{
*/

/*!
    @brief  Instantiates an object with default parameters (default constructor).
*/
    UdsNetworkFactory(void);


/*!
    @brief  The destructor.
*/
    virtual ~UdsNetworkFactory(void);

/*!
     end of Finish Initialization section.
     @}
*/

/*!
    @cond  PRIVATE
*/

/*!
    @brief  Prints information that is useful for debugging.

    @param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> API reference.
*/
    virtual void Trace(u64 flag) const;

/*!
    @brief  Instantiates an <tt>IMatchmakeSession</tt> object for the session.
*/
    virtual session::IMatchmakeSession* CreateMatchmakeSession();

    virtual u8 GetBrowseSessionInfoListMax() const;

    virtual nn::pia::session::ISessionInfoList* CreateSessionInfoList(u8 browseSessionInfoListNum);

/* For the auto-matchmaking feature. */


/*!
    @endcond
*/

protected:
private:
};
}
}
}  // End of namespace nn::pia::local.
