﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     platformCtr.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <nn/types.h>
#include <nn/Result.h>
#include <pia/platform/basicMacrosCtr.h>
#include <nn/uds.h>

#define NN_PIA_ENDIAN_BIG 0

namespace nn
{
namespace pia
{

/*!
@cond CTR_DOC
@brief  Enumerates <tt>StationIndex</tt> values.

@details  The Pia modules use <tt>StationIndex</tt> as a station identifier within the session regardless of the network type.
IDs were created with the intention of having the application handle information from each station as an array.

The maximum value changes whenever the maximum number of simultaneous connections that the Pia library can support changes.

@date 2013-02-01  Changed <tt>STATION_INDEX_MAX</tt> and <tt>MAX_STATION_NUM</tt> to 12 to match the maximum number of simultaneous connection supported by the Pia library.
@date 2012-04-05  Moved from the <tt>nn::pia::common</tt> namespace to the <tt>nn::pia</tt> namespace.



*/
enum StationIndex
{
    STATION_INDEX_1 = 0,                   //!< ID indicating Player 1.
    STATION_INDEX_2,                       //!< ID indicating Player 2.
    STATION_INDEX_3,                       //!< ID indicating Player 3.
    STATION_INDEX_4,                       //!< ID indicating Player 4.
    STATION_INDEX_5,                       //!< ID indicating Player 5.
    STATION_INDEX_6,                       //!< ID indicating Player 6.
    STATION_INDEX_7,                       //!< ID indicating Player 7.
    STATION_INDEX_8,                       //!< ID indicating Player 8.
    STATION_INDEX_9,                       //!< ID indicating Player 9.
    STATION_INDEX_10,                      //!< ID indicating Player 10.
    STATION_INDEX_11,                      //!< ID indicating Player 11.
    STATION_INDEX_12,                      //!< ID indicating Player 12.
    STATION_INDEX_MAX = STATION_INDEX_12,  //!< Maximum value of <tt>StationIndex</tt>.
    STATION_INDEX_INVALID = 0xfd,          //!< ID indicating a station that is not present in the session.
    STATION_INDEX_HOST = 0xfe,             //!< ID indicating the station that holds the role of host.
    STATION_INDEX_ALL = 0xff               //!< ID indicating all stations in the session.
};

const size_t MAX_STATION_NUM = 12;  //!< The maximum value for the number of stations that can participate in a Pia session.
}
}  // end of namespace nn::pia

//! @endcond
