﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     session_CreateSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_definitions.h>

#include <pia/session/session_types.h>

namespace nn
{
namespace pia
{
namespace session
{
/*!
@brief  Base class that manages settings specific to session creation.

@details  This class is never instantiated by an application. Use a class that inherits this class.

@date 2014-02-27  Initial version.
@if CTR_DOC
@see  nn::pia::local::UdsCreateSessionSetting
@endif
@see  nn::pia::inet::NexCreateSessionSetting
*/
class CreateSessionSetting : public nn::pia::common::RootObject
{
public:

/*!
@cond PRIVATE
@brief  Destroys the object.
*/
    virtual ~CreateSessionSetting() {};
    //! @endcond

/*!
@brief  Sets the minimum number of participants for the session to create.
@param[in] min  The minimum number of participants specified for the session to create.
*/
    void SetMinParticipants(u16 min)
    {
        m_MinParticipants = min;
    }
/*!
@brief  Gets the specified minimum number of participants.
@return  Returns the specified minimum number of participants.
*/
    u16 GetMinParticipants() const
    {
        return m_MinParticipants;
    }

/*!
@brief  Sets the maximum number of participants for the session to create.
@param[in] max  The maximum number of participants specified for the session to create.
*/
    void SetMaxParticipants(u16 max)
    {
        m_MaxParticipants = max;
    }
/*!
@brief  Gets the specified maximum number of participants.
@return  Returns the specified maximum number of participants.
*/
    u16 GetMaxParticipants() const
    {
        return m_MaxParticipants;
    }

    // Prints information that is useful for debugging.
    void Trace(u64 flag) const;

protected:
    u16 m_MinParticipants;
    u16 m_MaxParticipants;

    // Instantiates an object.
    CreateSessionSetting() :
        m_MinParticipants(0),
        m_MaxParticipants(0)
    {
    }

private:
    // The copy constructor is sealed.
    CreateSessionSetting(const CreateSessionSetting&);

    // The assignment operator is sealed.
    CreateSessionSetting& operator=(const CreateSessionSetting&);
};
}
}
}  // end of namespace nn::pia::session
