/*---------------------------------------------------------------------------*
  Project: Pia
  File: session_ISessionInfo.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_definitions.h>

namespace nn
{
namespace pia
{
namespace session
{

class SessionStationInfo;


/*!
    @brief  An interface to the session's information.
*/
class ISessionInfo : public common::RootObject
{
public:
    ISessionInfo()
    {
    }
    virtual ~ISessionInfo()
    {
    }

/*!
    @brief  Gets the session's game mode.
    @details  Returns a game mode value when Internet communication is ongoing.<br />
                During local communications, a communication mode ID is cast as a <tt>u32</tt> argument and returned.
    @return  Returns the session's game mode.
*/
    virtual u32 GetGameMode() const = 0;

/*!
    @brief  Gets the session's ID.
    @return  Returns the session ID.
*/
    virtual u32 GetSessionId() const = 0;

/*!
    @brief  Gets the number of people participating in a session.
    @return  Returns the number of people participating in a session.
*/
    virtual u32 GetCurrentParticipants() const = 0;

/*!
    @brief  Gets the minimum number of people who can participate in a session.
    @return  Returns the minimum number of people who can participate in a session.
*/
    virtual u32 GetMinParticipants() const = 0;

/*!
    @brief  Gets the maximum number of people who can participate in a session.
    @return  Returns the maximum number of people who can participate in a session.
*/
    virtual u32 GetMaxParticipants() const = 0;

/*!
    @brief  Gets whether a session is open for recruitment.
    @return  Returns <tt>true</tt> if the session is open for recruitment, and returns <tt>false</tt> if it is closed.
*/
    virtual bool IsOpened() const = 0;

/*!
    @brief  Clears session information.
*/
    virtual void Clear() = 0;

/*!
    @brief  Prints information that is useful for debugging.
    @param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> API reference.
*/
    virtual void Trace(u64 flag) = 0;

};
}
}
}  // end of namespace nn::pia::session
