/*---------------------------------------------------------------------------*
  Project: Pia
  File: session_ISessionInfoList.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_definitions.h>
#include <pia/session/session_ISessionInfo.h>

namespace nn
{
namespace pia
{
namespace session
{

/*!
    @brief  The interface for the session information list.

    @date 2014-02-27  Initial version.
*/
class ISessionInfoList : public common::RootObject
{
public:
    ISessionInfoList()
    {
    }
    virtual ~ISessionInfoList()
    {
    }

    typedef ISessionInfo** Iterator;
    typedef const ISessionInfo** ConstIterator;

/*!
    @brief  Gets the iterator for those that start with <tt>ISessionInfo*</tt> (<tt>const</tt> version).
    @return  Returns an iterator for the element at the start of the container.
*/
    virtual ConstIterator Begin() const = 0;
/*!
    @brief  Gets the iterator for those that start with <tt>ISessionInfo*</tt> (non-<tt>const</tt> version).
    @return  Returns an iterator for the element at the start of the container.
*/
    virtual Iterator Begin() = 0;
/*!
    @brief  Gets the iterator for the <tt>ISessionInfo*</tt> that corresponds to what follows the end of the container (<tt>const</tt> version).
    @return  Returns an iterator for the element that corresponds to what follows the end of the container.
*/
    virtual ConstIterator End() const = 0;
/*!
    @brief  Gets the iterator for the <tt>ISessionInfo*</tt> that corresponds to what follows the end of the container (non-<tt>const</tt> version).
    @return  Returns an iterator for the element that corresponds to what follows the end of the container.
*/
    virtual Iterator End() = 0;
/*!
    @brief  Gets the number of elements in the list.
    @return  Returns the number of elements in the list.
*/
    virtual size_t GetSize() const = 0;
/*!
    @brief  Gets the maximum number for the list.
    @return  Returns the maximum number for the list.
*/
    virtual size_t GetMaxSize() const = 0;
};
}
}
}  // end of namespace nn::pia::session
