﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: session_JoinSessionSetting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/session/session_definitions.h>

#include <pia/session/session_types.h>

namespace nn
{
namespace pia
{
namespace session
{

class ISessionInfo;
/*!
    @brief  Base class that manages settings specific to session joining.
    @details  This class is never instantiated by an application. Use a class that inherits this class.
    @date 2014-02-27  Initial version.

@if CTR_DOC
  @see  nn::pia::local::UdsJoinSessionSetting
@endif
  @see  nn::pia::inet::NexJoinSessionSetting
*/
class JoinSessionSetting : public nn::pia::common::RootObject
{
public:
    // The destructor.
    virtual ~JoinSessionSetting();

/*!
    @cond  PRIVATE
    @brief  Gets information for the target session to join.
    @return  Returns the target session information.
*/
    virtual const ISessionInfo* GetSessionInfoPtr() const;
    //! @endcond

/*!
    @brief  Sets target session information.
    @details  Sets target session information.
                Set the session information that was found using <tt>@ref Session::BrowseSessionAsync</tt>.
    @param[in] pSessionInfo  The target session's information.

    @see  Session::JoinSessionAsync, Session::BrowseSessionAsync
*/
    virtual void SetSessionInfoPtr(ISessionInfo* pSessionInfo);

/*!
    @cond  PRIVATE
    @brief  Prints information that is useful for debugging.
*/
    virtual void Trace(u64 flag) const;
    //! @endcond

protected:
    ISessionInfo* m_pSessionInfo;

    // The constructor.
    JoinSessionSetting();

private:
    // The copy constructor is sealed.
    JoinSessionSetting(const JoinSessionSetting&);

    // The assignment operator is sealed.
    JoinSessionSetting& operator=(const JoinSessionSetting&);
};
}
}
}  // end of namespace nn::pia::session
