﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     session_types.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once


namespace nn
{
namespace pia
{
namespace session
{

static const u32 NN_PIA_SESSION_JOIN_TIMEOUT_DEFAULT = 15000;    //!< The default value of the timeout time (in milliseconds) for the <tt>JoinMesh</tt> process.
static const u32 NN_PIA_SESSION_DESTROY_TIMEOUT_DEFAULT = 5000;  //!< The default value of the timeout time (in milliseconds) for the <tt>DestroyMesh</tt> process.
static const u32 NN_PIA_SESSION_LEAVE_TIMEOUT_DEFAULT = 5000;    //!< The default value of the timeout time (in milliseconds) for the <tt>LeaveMesh</tt> process.

static const s32 NN_PIA_SESSION_KEEP_ALIVE_INTERVAL_DEFAULT = 1000;  //!< The default value for the sending interval (in milliseconds) for keep-alive.
static const s32 NN_PIA_SESSION_MAX_SILENCE_TIME_DEFAULT = 10000;    //!< The default value for the amount of time without communication (in milliseconds) after which it is determined that the communication with the station has been disconnected.
static const s32 NN_PIA_SESSION_MAX_SILENCE_TIME_MAX = 30000;   //!< The maximum value that can be set for the amount of time without communication (in milliseconds) after which it is determined that the communication with the station has been disconnected.
static const s32 NN_PIA_SESSION_MAX_SILENCE_TIME_MIN = 1000;    //!< The minimum value that can be set for the amount of time without communication (in milliseconds) after which it is determined that the communication with the station has been disconnected.

static const u16 NN_PIA_SESSION_MAX_RTT_DEFAULT = 500;  //!< The default maximum total RTT (in milliseconds) for a relay route.

static const s32 NN_PIA_SESSION_INVALID_UPLINK_BIT_RATE_LOWER_LIMIT = -1;  //!< This constant represents an invalid bitrate in the bitrate detection feature.

/*!
@brief  Enumerated type for configuring the network topology of a mesh created with the <tt>Session</tt> class or the <tt>Mesh</tt> class.

@details  This feature will be added to the <tt>Session</tt> class and the <tt>Mesh</tt> class in a future release.
*/
enum NetworkTopology
{
    NetworkTopology_FullMesh = 0,  //!< Full-Mesh Network Topology
/*! @cond PRIVATE */
    NetworkTopology_Star,  //!< Star network topology (not available to third party developers).
/*! @endcond */
    NetworkTopology_RelayMesh  //!< Mesh Network Topology With Relay Connections Enabled
};


/*!
@brief  Enumerated type for setting the bitrate detection feature.
*/
enum BitRateCheckMode
{
    BitRateCheck_Disable = 0, //!< Do not measure the bitrate.
    BitRateCheck_Enable       //!< Measure the bitrate.
};

/*!
@cond PRIVATE
@brief  Enumerated type for configuring the host migration of a mesh created with the <tt>Session</tt> class or the <tt>Mesh</tt> class.

@details  This feature will be added to the <tt>Session</tt> class and the <tt>Mesh</tt> class in a future release.

*/
enum HostMigrationMode
{
    HostMigrationMode_None = 0,       //!< Do not use host migration.
    HostMigrationMode_SoleCandidate,  //!< Use host migration. Only one candidate for the next session host.
    HostMigrationMode_MultiCandidate  //!< Use host migration. Multiple candidates for the next session host.
};
//! @endcond

/*!
@brief  Constant that indicates an invalid <tt>SessionId</tt>.
*/
static const u32 INVALID_SESSION_ID = 0;

}
}
}
