﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     settings.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once


#define NN_PIA_PLATFORM_WIN (1)
#define NN_PIA_PLATFORM_CTR (2)
#define NN_PIA_PLATFORM_CAFE (3)
#define NN_PIA_PLATFORM_NIN_WIN (4)

#define NN_PIA_BUILD_DEBUG (1)
#define NN_PIA_BUILD_DEVELOP (2)
#define NN_PIA_BUILD_RELEASE (3)

#if defined(WIN32)
#if PIA_PLATFORM_NIN_WIN
#define NN_PIA_PLATFORM NN_PIA_PLATFORM_NIN_WIN
#else
#define NN_PIA_PLATFORM NN_PIA_PLATFORM_WIN
#endif

#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error  "PIA_BUILD_TARGET_XXX is not defined."
#endif

#elif defined(NN_PLATFORM_CTR)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_CTR

#if defined(NN_BUILD_NOOPT)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(NN_BUILD_VERBOSE)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#else
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#endif

#elif defined(PLATFORM) && (PLATFORM == CAFE)

#define NN_PIA_PLATFORM NN_PIA_PLATFORM_CAFE

//n1769 <tt>PIA_BUILD_TARGET_*</tt> macros are defined according to the <tt>commondefs.mk</tt> file of the build system.
//
#if defined(PIA_BUILD_TARGET_DEBUG)
#define NN_PIA_BUILD NN_PIA_BUILD_DEBUG
#elif defined(PIA_BUILD_TARGET_DEVELOP)
#define NN_PIA_BUILD NN_PIA_BUILD_DEVELOP
#elif defined(PIA_BUILD_TARGET_RELEASE)
#define NN_PIA_BUILD NN_PIA_BUILD_RELEASE
#else
#error  "PIA_BUILD_TARGET_XXX is not defined."
#endif

#else
#error  "This platform is not supported."
#endif


#define NN_PIA_CTR (NN_PIA_PLATFORM == NN_PIA_PLATFORM_CTR)
#define NN_PIA_WIN (NN_PIA_PLATFORM == NN_PIA_PLATFORM_WIN)
#define NN_PIA_CAFE (NN_PIA_PLATFORM == NN_PIA_PLATFORM_CAFE)
#define NN_PIA_NIN_WIN (NN_PIA_PLATFORM == NN_PIA_PLATFORM_NIN_WIN)

#define NN_PIA_DEBUG (NN_PIA_BUILD == NN_PIA_BUILD_DEBUG)
#define NN_PIA_DEVELOP (NN_PIA_BUILD == NN_PIA_BUILD_DEVELOP)
#define NN_PIA_RELEASE (NN_PIA_BUILD == NN_PIA_BUILD_RELEASE)

#define NN_PIA_UNDER_DEBUG (NN_PIA_BUILD <= NN_PIA_BUILD_DEBUG)
#define NN_PIA_UNDER_DEVELOP (NN_PIA_BUILD <= NN_PIA_BUILD_DEVELOP)

#define NN_PIA_ENABLE_TRACE (NN_PIA_UNDER_DEVELOP)
#define NN_PIA_ENABLE_ASSERT (NN_PIA_UNDER_DEVELOP)

#if defined(NN_PLATFORM_CTR)
//n2788: Macro to switch NEX libraries for library development.
#ifndef NN_PIA_NEX_3
#define NN_PIA_NEX_3 1
#endif
//n1872: Macro to enable or disable <tt>MultiUnitHeap</tt> in NEX. (It is enabled only when <tt>NN_PIA_NEX_3=0</tt>.)
#define NN_PIA_EXPERIMENT_UNUSE_NEX_MULTIUNITHEAP 0
#else
//n1872: Use NEX3 on platforms other than CTR.
#define NN_PIA_NEX_3 1
#define NN_PIA_EXPERIMENT_UNUSE_NEX_MULTIUNITHEAP 1
#endif

#define NN_PIA_EXPERIMENT_SHARE_CONNECTION_INFO_EARLY 1

#if NN_PIA_UNDER_DEVELOP
#define NN_PIA_ENABLE_FEEDBACK 0
#define NN_PIA_ENABLE_GATEWAY 0
#else
#define NN_PIA_ENABLE_FEEDBACK 0
#define NN_PIA_ENABLE_GATEWAY 0
#endif

#define NN_PIA_EXPERIMENT_RELAY_CONNECTION_TOPOLOGY 1

#define NN_PIA_EXPERIMENT_HM_MULTI_CANDIDATE 1

#define NN_PIA_ENABLE_NAT_TRAVERSAL_TIME_OPTIMIZATION 1
#define NN_PIA_ENABLE_EACH_NAT_TRAVERSAL_TIMEOUT 1

#define NN_PIA_ENABLE_TRANSPORT_CONNECTION_DIRECTION 1

#define NN_PIA_NEX_NEW_FEATURE_FOR_MATCHMAKE 1

#define NN_PIA_ENABLE_MATCHMAKE_REFEREE 0

#define NN_PIA_EXPERIMENT_USE_PRINT_HOOK 1

#define NN_PIA_EXPERIMENT_HANDSHAKE_CHECK 1 // n1769 Set to <tt>1</tt> when configuring to abandon any received data while the handshake between stations cannot yet be confirmed.

#define NN_PIA_EXPERIMENT_BAND_WIDTH_CHECK 1

#define NN_PIA_EXPERIMENT_OC_PACKET_V2 1
