﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: transport_ConnectionAnalysisData.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport/transport_definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
  @brief  The <tt>ConnectionAnalysisData</tt> structure contains parameters related to
   the connection quality (such as the round-trip time (RTT) and the packet loss rate).

  @date 2013-11-22  Changed the type for the variables that count packets and packet losses from <tt>u16</tt> to <tt>u32</tt>.
  @date 2013-11-14  Added a member variable that maintains a counter from the point at which measurement starts.
  @date 2013-10-29  Initial version.
 */
struct ConnectionAnalysisData
{
    static const size_t MAX_ENTRIES = MAX_STATION_NUM;

/*!
    @brief  Clears the member variables.
*/
    void Clear();


/*!
    @brief  Contains information required for tabulating analysis results for each <tt>Station</tt>.
*/
    struct Entry
    {
        s32 rtt;  //!<  RTT.

        u32 prevTotalPacketNum;  //!<  Previous number of packets.
        u32 currTotalPacketNum;  //!<  Current number of packets.

        u32 prevPacketLossNum;  //!<  Previous number of packets lost.
        u32 currPacketLossNum;  //!<  Current number of packets lost.

        bool valid;  //!<  Specifies <tt>true</tt> if this entry is valid.
    };

    struct Entry entry[MAX_ENTRIES];  //!<  Entry.
    s32 passedMilliSec;               //!<  Time elapsed (in milliseconds).


/*!
    @brief  Outputs the data to the console (except in the Release build).

    @param[in] bDetail  Set to <tt>true</tt> to also output data related to the internal implementation of Pia.
*/
    void Print(bool bDetail) const;


/*!
    @brief  Prints information that is useful for debugging.

    @param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> API reference.
*/
    void Trace(u64 flag) const;
};
}
}
}  // End of namespace nn::pia::transport.
