﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     transport_ProtocolEvent.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport/transport_definitions.h>

namespace nn
{
namespace pia
{
namespace transport
{


/*!
@cond PRIVATE
@brief  Protocol event class.
*/
class ProtocolEvent : public nn::pia::common::RootObject
{
public:
/*!
@brief  The event type.
*/
    enum Type
    {
        CONNECT,
        DISCONNECT
    };


/*!
@brief  Constructor.
*/
    ProtocolEvent(Type type, StationIndex stationId)
      : m_Type(type), m_StationIndex(stationId)
    {
    }


/*!
@brief  Gets the event type.
*/
    Type GetType() const
    {
        return m_Type;
    }


/*!
@brief  Gets the ID of the station.
*/
    StationIndex GetStationIndex() const
    {
        return m_StationIndex;
    }


/*!
@brief  Prints information useful for debugging.

@param[in] flag  Bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> type.
*/
    virtual void Trace(u64 flag) const;

private:
    Type m_Type;
    StationIndex m_StationIndex;
};
//! @endcond
}
}
}  // end of namespace nn::pia::transport
