﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     transport_ProtocolId.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport/transport_definitions.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
@cond PRIVATE
@brief  A struct representing a protocol ID.
*/
struct ProtocolId
{
public:
    struct Item
    {
#if NN_PIA_ENDIAN_BIG
        u16 m_Type;
        u16 m_Port;
#else
        u16 m_Port;
        u16 m_Type;
#endif
    };

    union
    {
        u32 m_Value;
        Item m_Item;
    };


/*!
@brief  Default constructor.
*/
    ProtocolId()
      : m_Value(INVALID.m_Value)
    {
    }


/*!
@brief  Converts from a <tt>u32</tt> value.
*/
    ProtocolId(u32 value)
      : m_Value(value)
    {
    }

/*!
@brief  Gets the type.
*/
    u16 GetType()
    {
        return m_Item.m_Type;
    }

/*!
@brief  Generates from the type and port.
*/
    ProtocolId(u16 type, u16 port)
    {
        m_Item.m_Type = type;
        m_Item.m_Port = port;
    }


/*!
@brief  Converts to a <tt>u32</tt> value.
*/
    operator u32() const
    {
        return m_Value;
    }


/*!
@brief  Invalid protocol ID.
*/
    static const ProtocolId INVALID;
};
//! @endcond
}
}
}  // end of namespace nn::pia::transport
