﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     transport_Singleton.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport/transport_definitions.h>

#include <pia/transport/transport_Api.h>
#include <pia/common/common_Log.h>
#include <pia/assert.h>


// Add this macro near the start of the CPP file that corresponds to the singleton.
#define NN_PIA_TRANSPORT_SINGLETON_DEFINE(CLASSNAME) CLASSNAME* CLASSNAME::s_pInstance = NULL


// For this macro, fill in the CPP file that corresponds to the singleton.
#define NN_PIA_TRANSPORT_SINGLETON_CREATE_INSTANCE_IMPL(CLASSNAME)                                                     \
    \
nn::Result CLASSNAME::CreateInstance(void)                                                                             \
    \
{                                                                                                                      \
        if (!IsInitialized())                                                                                          \
        {                                                                                                              \
        PIA_RETURN_RESULT_WITH_LOG(ResultNotInitialized, "PiaTransport is not initialized.");                          \
        }                                                                                                              \
        if (!IsInSetupMode())                                                                                          \
        {                                                                                                              \
        PIA_RETURN_RESULT_WITH_LOG(ResultInvalidState, "Singleton must be created between nn::pia::transport::BeginSetup() and nn::pia::transport::EndSetup().")    \
        }                                                                                                              \
        if (s_pInstance)                                                                                               \
        {                                                                                                              \
        PIA_RETURN_RESULT_WITH_LOG(ResultAlreadyExists, "Singleton is already created.");                              \
        }                                                                                                              \
                                                                                                                       \
/* <tt>RootObject::new</tt> is used if this class inherits <tt>RootObject</tt>.*/                              \
/* Do not use global <tt>new</tt>.                           */                                                \
        s_pInstance = new CLASSNAME();                                                                                 \
        PIA_ASSERT(PIA_IS_VALID_POINTER(s_pInstance));                                                                 \
                                                                                                                       \
        return ResultSuccess();                                                                                        \
}


// For this macro, fill in the CPP file that corresponds to the singleton.
#define NN_PIA_TRANSPORT_SINGLETON_DESTROY_INSTANCE_IMPL(CLASSNAME)                                               \
    \
void CLASSNAME::DestroyInstance(void)                                                                             \
    \
{                                                                                                          \
        if (s_pInstance)                                                                                          \
        {                                                                                                         \
/* <tt>RootObject::delete</tt> is used because this class inherits <tt>RootObject</tt>. */ \
/* Do not use global <tt>delete</tt>.                                        */            \
            delete s_pInstance;                                                                                   \
            s_pInstance = NULL;                                                                                   \
        }                                                                                                         \
    \
}


namespace nn
{
namespace pia
{
namespace transport
{
}
}
}  // end of namespace nn::pia::transport
