﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     transport_TransportAnalysisData.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport/transport_definitions.h>

#include <pia/transport/transport_PacketAnalysisData.h>
#include <pia/transport/transport_ConnectionAnalysisData.h>


namespace nn
{
namespace pia
{
namespace transport
{


/*!
@brief  Contains the results of analyzing send and receive packets, and parameters indicating the quality of the connection (RTT and packet loss) to each station.

@date 2013-10-29  Initial version.

*/
struct TransportAnalysisData
{
    struct PacketAnalysisData sendPacketAnalysisData;      //!< Sender packet analysis data.
    struct PacketAnalysisData recvPacketAnalysisData;      //!< Receiver packet analysis data.
    struct ConnectionAnalysisData connectionAnalysisData;  //!< Connection quality analysis data.

/*!
@brief  Outputs the data to the console.

@param[in] bDetail  Set to <tt>true</tt> to also output the protocol used internally by Pia.
*/
    void Print(bool bDetail) const;


/*!
@brief  Prints information that is useful for debugging.

@param[in] flag  Specifies the bitwise OR of trace flags. For more information, see the <tt>@ref TraceFlag</tt> type.
*/
    void Trace(u64 flag) const;
};
}
}
}  // end of namespace nn::pia::transport
