﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     transport_definitions.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once


#include <pia/result.h>
#include <pia/common/common_RootObject.h>
#include <pia/common/common_ErrorHandler.h>

#define NN_PIA_TRANSPORT_V3 1  // Transport for Pia 3.

#define NN_PIA_EXPERIMENT_PROTOCOL_FILTERING 1  // n1769 Filter <tt>ProtocolMessages</tt> received from suspect stations.

// For Pia library developers.
#define NN_PIA_TRANSPORT_ENABLE_DETAILED_TRACE 0
#define NN_PIA_EXPERIMENT_RELAY 0                 // Set to <tt>1</tt> to enable the code for the relay feature.
#define NN_PIA_EXPERIMENT_RELIABLE_DETAIL_INFO 0  // n1769 For testing. For measuring the performance of reliable communication.
#define NN_PIA_FEEDBACK_USE_COMPRESSION_CRYPTO 1  // Apply both compression and encryption.
#if NN_PIA_EXPERIMENT_MONITORING
#define NN_PIA_EXPERIMENT_BANDWIDTH_ESTIMATION 0  // n1769 Turns on the bandwidth estimation feature.
#else
#define NN_PIA_EXPERIMENT_BANDWIDTH_ESTIMATION 0  // n1769 Turns off the bandwidth estimation feature.
#endif
#define NN_PIA_EXPERIMENT_SEND_DOUBLE_DUMMY 0  // n1769

#define NN_PIA_EXPERIMENT_LATENCY_EMULATION_ERROR_HANDLING 1  // n1769 <tt>1</tt> if you are not going to ignore the error result from the transmission functions when using latency emulation.

#define NN_PIA_EXPERIMENT_PACKET_CHECK_SOURCE_INDEX 1

//n1589:test:off
#define NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME 0

// The macro related to optimization for use by Pia library developers. For each, promise that 0 is the traditional technique.
#define NN_PIA_OPTIMIZE_NO_PACKET_CONTAINER 0                     //n1769
#define NN_PIA_OPTIMIZE_USE_RESERVABLE_RINGBUF 1                  //n1769
#define NN_PIA_OPTIMIZE_MOVE_FROM_RELIABLE_IMPROVEMENT 1          //n1769
#define NN_PIA_OPTIMIZE_USE_RESERVATION_IN_UNRELIABLE_PROTOCOL 1  //n1769
#define NN_PIA_OPTIMIZE_USE_RESERVATION_IN_RELIABLE_PROTOCOL 1    //n1769

//n1589: The normal definitions are from here down.
namespace nn
{
namespace pia
{
namespace transport
{

// Protocol processing executes in the order of this value.
// Assign a small value to protocols to execute with higher priority.
static const u16 PROTOCOL_TYPE_RELAY = 0x0080;
static const u16 PROTOCOL_TYPE_KEEP_ALIVE = 0x00c0;
static const u16 PROTOCOL_TYPE_STATION = 0x0100;
static const u16 PROTOCOL_TYPE_MESH = 0x0200;
static const u16 PROTOCOL_TYPE_SYNC_CLOCK = 0x0210;
static const u16 PROTOCOL_TYPE_NAT = 0x0400;
static const u16 PROTOCOL_TYPE_GATEWAY = 0x0410;
static const u16 PROTOCOL_TYPE_LANMATCHMAKE = 0x0420;
static const u16 PROTOCOL_TYPE_BANDWIDTH_CHECKER = 0x0500;
static const u16 PROTOCOL_TYPE_RTT = 0x0600;
static const u16 PROTOCOL_TYPE_SYNC_OLD = 0x1800;
static const u16 PROTOCOL_TYPE_SYNC = 0x1810;
static const u16 PROTOCOL_TYPE_UNRELIABLE = 0x2000;
static const u16 PROTOCOL_TYPE_ROUNDROBIN_UNRELIABLE = 0x2100;
static const u16 PROTOCOL_TYPE_CLONE = 0x2400;
static const u16 PROTOCOL_TYPE_VOICE = 0x2800;
static const u16 PROTOCOL_TYPE_RELIABLE = 0x3000;
static const u16 PROTOCOL_TYPE_RELIABLE_BROADCAST = 0x7000;
static const u16 PROTOCOL_TYPE_SESSION = 0x7200;
static const u16 PROTOCOL_TYPE_FEEDBACK = 0x8000;
static const u16 PROTOCOL_TYPE_RELAY_SERVICE = 0x8200;
}
}
}  // end of namespace nn::pia::transport

//n1589: The following header files are included for all classes in this module.
