﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleFrameworkImplBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleFramework_Color.h"
#include "SampleFramework_ButtonDefines.h"

#include <stddef.h>  // For the <tt>NULL</tt> pointer.

#if defined(NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST)
#include "PiaTestLib/PiaTest.h"
#endif  // End of NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST


namespace nn
{
namespace pia
{
namespace sampleframework
{

class SampleFrameworkImplBase
{
public:
    SampleFrameworkImplBase(void)
    {
    }
    virtual ~SampleFrameworkImplBase(void)
    {
    }

    virtual bool SoundInitialize(const char streamFileName[], const char* voiceFileName[] = NULL, const u32 voiceFileNum = 0) = 0;
    virtual void SoundFinalize(void) = 0;
    virtual bool PlayBGM(void) = 0;
    virtual bool PauseBGM(void) = 0;
    virtual bool StopBGM(void) = 0;
    virtual bool PlayShortSound(const u32 voiceFileIndex) = 0;
    virtual bool PauseShortSoundAll(void) = 0;
    virtual bool PlayShortSoundAll(void) = 0;
    virtual bool StopShortSoundAll(void) = 0;
    virtual void ClearScreen(void) = 0;
    virtual void SetClearColor(const struct Color& clr) = 0;
    virtual void SetFontColor(const struct Color& clr) = 0;
    virtual void SetFontSize(int pixel) = 0;
    virtual int GetFontWidth(void) const = 0;
    virtual int GetFontHeight(void) const = 0;
    virtual void DrawText(s32 x, s32 y, const char* pText) = 0;
    virtual void DrawRect(s32 x, s32 y, s32 w, s32 h, const struct Color& clr) = 0;
    virtual void SetPointSize(f32 pointSize) = 0;
    virtual void DrawPoint(s32 x, s32 y, const struct Color& clr) = 0;
    virtual void SetLineWidth(f32 lineWidth) = 0;
    virtual void DrawLine(s32 x0, s32 y0, s32 x1, s32 y1, const struct Color& clr) = 0;
    virtual void ChangeRenderTarget(bool target) = 0;
    virtual void SwapBuffers(void) = 0;
    virtual void SkipFrame(void) = 0;
    virtual void BeginFrame(void) = 0;
    virtual void BeginFrameNoRender(void) = 0;
    virtual void EndFrame(void) = 0;
    virtual void EndFrameNoRender(void) = 0;
    virtual u32 GetFrameCount(void) const = 0;
    virtual bool SetFramePerSecond(s32 fps) = 0;
    virtual s32 GetFramePerSecond(void) const = 0;
    virtual bool IsHold(bit32 button) const = 0;
    virtual bool IsTrigger(bit32 button) const = 0;
    virtual bit32 GetTrigger() const = 0;
    virtual bool IsRelease(bit32 button) const = 0;
    virtual bool IsPenTriggered(void) const = 0;
    virtual bool IsPenTouched(void) const = 0;
    virtual bool IsPenReleased(void) const = 0;
    virtual bool GetLatestPenPosition(u16* x, u16* y) const = 0;
    virtual f32 GetAnalogX(void) = 0;
    virtual f32 GetAnalogY(void) = 0;
    virtual void GetAnalog(f32* x, f32* y) = 0;
    virtual bool GetAccelerationData(s16* x, s16* y, s16* z) const = 0;
    virtual u32 WaitForButton(u32 button) = 0;
    virtual void PrintCurrentThreadInfo(void) const = 0;
    virtual void SleepCurrentThreadMilliseconds(s32 msec) = 0;
    virtual void* AllocateDeviceMemory(size_t size) = 0;
    virtual void FreeDeviceMemory(void* ptr) = 0;
    virtual bool IsInMainThread(void) const = 0;
    virtual void Checker(void) = 0;
    virtual void GetPassedTimeString(char str[]) = 0;
    virtual void SetCachedPrintFlushSpan(u32 span) = 0;
    virtual bool IsSystemTerminationRequested(void) const = 0;
    virtual void SetProgramStatus(u32 sceneNo, bool isConnected, bool isHost, u32 stationNum, u32 linkLevel) = 0;
    virtual void SetProgramParamArray(bit32 paramArray[], size_t paramNum) = 0;
    virtual void SetProgramParam(size_t paramIndex, bit32 paramVar) = 0;
#if defined(NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST)
    virtual PiaTest::PiaTestCommand GetLastCommandFromHost() = 0;
#endif  // End of NN_PIA_INTERNAL_FRAMEWORK_FOR_PIATEST
private:
};
}
}
}  // end of namespace nn::pia::sampleframework
