﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleFramework_Color.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>


namespace nn
{
namespace pia
{
namespace sampleframework
{


/*!
  @brief  Contains color data (RGBA) as values of type <tt>f32</tt>.
 */
struct Color
{
    /*!
    @brief  Instantiates the object.

    @param[in] red  Specifies the red component.
    @param[in] green  Specifies the green component.
    @param[in] blue  Specifies the blue component.
    @param[in] alpha  Specifies the alpha component.
     */
    Color(f32 red, f32 green, f32 blue, f32 alpha)
      : r(red),
        g(green),
        b(blue),
        a(alpha)
    {
    }

    f32 r;  // Red.
    f32 g;  // Green.
    f32 b;  // Blue.
    f32 a;  // Alpha.
};

const Color COLOR_BLACK(0.0f, 0.0f, 0.0f, 1.0f);
const Color COLOR_GRAY(0.5f, 0.5f, 0.5f, 1.0f);
const Color COLOR_SILVER(0.75f, 0.75f, 0.75f, 1.0f);
const Color COLOR_WHITE(1.0f, 1.0f, 1.0f, 1.0f);
const Color COLOR_MAROON(0.5f, 0.0f, 0.0f, 1.0f);
const Color COLOR_RED(1.0f, 0.0f, 0.0f, 1.0f);
const Color COLOR_OLIVE(0.5f, 0.5f, 0.0f, 1.0f);
const Color COLOR_YELLOW(1.0f, 1.0f, 0.0f, 1.0f);
const Color COLOR_GREEN(0.0f, 0.5f, 0.0f, 1.0f);
const Color COLOR_LIME(0.0f, 1.0f, 0.0f, 1.0f);
const Color COLOR_TEAL(0.0f, 0.5f, 0.5f, 1.0f);
const Color COLOR_AQUA(0.0f, 1.0f, 1.0f, 1.0f);
const Color COLOR_NAVY(0.0f, 0.0f, 0.5f, 1.0f);
const Color COLOR_BLUE(0.0f, 0.0f, 1.0f, 1.0f);
const Color COLOR_PURPLE(0.5f, 0.0f, 0.5f, 1.0f);
const Color COLOR_FUCHSIA(1.0f, 0.0f, 1.0f, 1.0f);
const Color COLOR_DARKBLUE(0.0f, 0.0f, 0.5f, 1.0f);
}
}
}  // end of namespace nn::pia::sampleframework
