﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleFramework_VersionInfo.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/settings.h>
#include <pia/version.h>
#include "../../SampleLib/Print.h"

/*
 SampleFramework_VersionInfo.h
 Defined with the same namespaces in <tt>Shared</tt>.
 - To output version information to the log even if this function is not called in every sample.
 - To get the version information for the SDK at the time the sample was built.
 To satisfy both of these conditions above, the definitions in <tt>SampleFramework</tt> are made using <tt>weak</tt> symbols. When <tt>Shared_VersionInfo.h</tt> is referenced in a sample, what gets output to the log is the version information at the time the sample was built.
 When it is not referenced, nothing gets output to the log, as implemented in <tt>SampleFramework_VersionInfo.cpp</tt>.
 

*/

#if NN_PIA_CTR
#include <nn.h>
#include <nn/version.h>

extern void TraceBuildEnvironment() __attribute__((weak));
#endif

#if NN_PIA_WIN
#include <pia/platform/platformWin.h>
#endif

#if NN_PIA_CAFE
#include <cafe.h>
#include <sdk_ver.h>

extern void TraceBuildEnvironment() __attribute__((weak));
#endif

#if defined(NN_PIA_INCLUDE_NEX)

#if NN_PIA_NEX_3  // NEX 3.x.
#include <OnlineCore/src/version/nex_version.h>
#else  // NEX 2.x.
#include <version/nex_version.h>
#endif  // end of NN_PIA_NEX_3

#endif  // end of NN_PIA_INCLUDE_NEX


//extern void TraceBuildEnvironment() __attribute__ ((weak));

namespace nn
{
namespace pia
{
namespace sampleframework
{
class VersionInfo
{

public:
    static void TraceBuildEnvironment();
};
}
}
}  // end of namespace nn::pia::sampleframework
