﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Assert.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common.h>

#if NN_PIA_UNDER_DEVELOP
/*!
@brief  Defines an assertion macro for the samples.
 */
#define PIA_SAMPLE_ASSERT(exp)                                              \
    if (!(exp))                                                             \
    {                                                                       \
        SampleAssert(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, #exp); \
    }

/*!
@brief  Defines an assertion macro with a message for the samples.
 */
#define PIA_SAMPLE_ASSERTMSG(exp, ...)                                             \
    if (!(exp))                                                                    \
    {                                                                              \
        SampleAssert(PIA_CODE_POSITION_FILE, PIA_CODE_POSITION_LINE, __VA_ARGS__); \
    }

#else
#define PIA_SAMPLE_ASSERT(exp) ((void)0)
#define PIA_SAMPLE_ASSERTMSG(exp, ...) ((void)0)
#endif

/*!
@brief  Halts on an assertion if the <tt>result.IsFailure</tt> function returns <tt>true</tt>.
 */
#define PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result) \
    if (result.IsFailure())                         \
    {                                               \
        PIA_SAMPLE_LOG_RESULT(result);              \
        PIA_SAMPLE_ASSERT(false);                   \
    }

void SampleAssert(const char* cpPos, u32 line, const char* cpMsg, ...);


/*!
@brief  Forcibly terminates the program.
 */
#define PIA_SAMPLE_PANIC(...) PIA_PANIC(__VA_ARGS__)
