﻿/*---------------------------------------------------------------------------*
Project:  Pia
File:     Defines.h

Copyright Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.
*---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>

// Trace-related definitions.
#define SAMPLE_TRACE_ALWAYS (1 << 0)

#define SAMPLE_TRACE_COMMON_ALWAYS (1 << 1)
#define SAMPLE_TRACE_COMMON_DETAIL (1 << 2)
#define SAMPLE_TRACE_LOCAL_ALWAYS (1 << 3)
#define SAMPLE_TRACE_LOCAL_DETAIL (1 << 4)
#define SAMPLE_TRACE_TRANSPORT_ALWAYS (1 << 5)
#define SAMPLE_TRACE_TRANSPORT_DETAIL (1 << 6)
#define SAMPLE_TRACE_SYNC_ALWAYS (1 << 7)
#define SAMPLE_TRACE_SYNC_DETAIL (1 << 8)
#define SAMPLE_TRACE_INET_ALWAYS (1 << 9)
#define SAMPLE_TRACE_INET_DETAIL (1 << 10)
#define SAMPLE_TRACE_SESSION_ALWAYS (1 << 11)
#define SAMPLE_TRACE_SESSION_DETAIL (1 << 12)

#define SAMPLE_TRACE_APP1 (1 << 24)
#define SAMPLE_TRACE_APP2 (1 << 25)
#define SAMPLE_TRACE_APP3 (1 << 26)
#define SAMPLE_TRACE_APP4 (1 << 27)
#define SAMPLE_TRACE_APP5 (1 << 28)
#define SAMPLE_TRACE_APP6 (1 << 29)
#define SAMPLE_TRACE_APP7 (1 << 30)
#define SAMPLE_TRACE_APP8 (1 << 31)

#if 1
#define SAMPLE_TRACE_ERROR_ (1 << 13)
#define SAMPLE_TRACE_BUTTON_ (1 << 14)
#define SAMPLE_TRACE_UDS_ (1 << 15)
#define SAMPLE_TRACE_TRANSPORT_ (1 << 16)
#define SAMPLE_TRACE_UPDATE_ (1 << 17)
#define SAMPLE_TRACE_ROUTE_ (1 << 18)

#define SAMPLE_TRACE_SCENE_ SAMPLE_TRACE_APP1
#define SAMPLE_TRACE_AVATAR_ SAMPLE_TRACE_APP2
#define SAMPLE_TRACE_POSITION_ SAMPLE_TRACE_APP3
#define SAMPLE_TRACE_WORLD_ SAMPLE_TRACE_APP4
#define SAMPLE_TRACE_LATENCY_ SAMPLE_TRACE_APP5
#endif

#if NN_PIA_CTR
#include <nn/applet.h>

// On CTR systems, you call the main function of the sample program after calling the <tt>nn::applet::Enable</tt> function.
#define PIA_SAMPLE_MAIN_FUNC() \
    \
/* Declare the sample program main function.       */ \
void SampleMainFunc(void);     \
    \
void nnMain()                  \
    \
{                       \
        nn::applet::Enable();  \
        SampleMainFunc();      \
    \
}                       \
    \
void SampleMainFunc()

#elif NN_PIA_WIN
// Convert to functions for each console/GUI.
#ifdef _CONSOLE
#define PIA_SAMPLE_MAIN_FUNC() int main()
#else
#define PIA_SAMPLE_MAIN_FUNC() int WINAPI WinMain(HINSTANCE, HINSTANCE, LPSTR, int)
#endif  // _CONSOLE

#elif NN_PIA_NIN_WIN
#define PIA_SAMPLE_MAIN_FUNC() extern "C" void nnMain()

#elif NN_PIA_CAFE

// Defines the core that the main function runs on.
#define PIA_SAMPLE_MAIN_CORE 2

// In Cafe, check the operating core in the main function of the sample program, so that you can specify the core that the main function runs on. Then set up a thread so that you can run the sample program's main function on the target core.
// 
// 
#define PIA_SAMPLE_MAIN_FUNC()                                                                                                                                          \
    \
/* Declare the sample program main function.       */ \
void SampleMainFunc(void);                                                                                                                                              \
    \
/* Declare the entry function for the sample program thread. */ \
int SampleEntryFunc(int argc, void* argv)                                                                                                                               \
    {                                                                                                                                                                   \
        /* Implement code so that the <tt>SampleMainFunc</tt> function is called separately to avoid an unused argument warning. */                                          \
        NN_PIA_DUMMY_PARAM(argc);                                                                                                                                       \
        NN_PIA_DUMMY_PARAM(argv);                                                                                                                                       \
        SampleMainFunc();                                                                                                                                               \
        return 0;                                                                                                                                                       \
    \
}                                                                                                                                                                \
    \
const u32 SAMPLE_THREAD_STACK_SIZE = 2 * 1024 * 1024;                                                                                                                   \
    \
OSThread g_SampleThread;                                                                                                                                                \
    \
u8 g_SampleThreadStack[SAMPLE_THREAD_STACK_SIZE];                                                                                                                       \
    \
void RunSampleThread(void)                                                                                                                                              \
    {                                                                                                                                                                   \
        u16 sampleThreadAttribute = OS_THREAD_ATTR_DETACH;                                                                                                              \
                                                                                                                                                                        \
        if (PIA_SAMPLE_MAIN_CORE == 0)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE0;                                                                                                     \
        if (PIA_SAMPLE_MAIN_CORE == 1)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE1;                                                                                                     \
        if (PIA_SAMPLE_MAIN_CORE == 2)                                                                                                                                  \
            sampleThreadAttribute |= OS_THREAD_ATTR_AFFINITY_CORE2;                                                                                                     \
                                                                                                                                                                        \
        OSCreateThread(&g_SampleThread, SampleEntryFunc, 0, NULL, g_SampleThreadStack + SAMPLE_THREAD_STACK_SIZE, SAMPLE_THREAD_STACK_SIZE, 16, sampleThreadAttribute); \
        OSResumeThread(&g_SampleThread);                                                                                                                                \
    \
}                                                                                                                                                                \
    \
/* Main function */ \
int main()                                                                                                                                                              \
    {                                                                                                                                                                   \
        if (OSGetCoreId() == PIA_SAMPLE_MAIN_CORE)                                                                                                                      \
        {                                                                                                                                                               \
            SampleMainFunc();                                                                                                                                           \
        }                                                                                                                                                               \
        else                                                                                                                                                            \
        {                                                                                                                                                               \
            RunSampleThread();                                                                                                                                          \
                                                                                                                                                                        \
            while (OSIsThreadTerminated(&g_SampleThread) == FALSE)                                                                                                      \
            {                                                                                                                                                           \
                OSSleepSeconds(1);                                                                                                                                      \
            }                                                                                                                                                           \
        }                                                                                                                                                               \
        return 0;                                                                                                                                                       \
    \
}                                                                                                                                                                \
    \
void SampleMainFunc()
#else
#error  "Invalid platform";
#endif

