﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ErrorHandling.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>

namespace nn
{
class Result;
}


///< Contains member functions for error handling.
class ErrorHandling
{
public:
    // Initializes the module.
    static void Initialize(void);

    // End processing.
    static void Finalize(void);

    // Gets whether an error was detected.
    static bool IsErrorDetected(void);

    // The following calls are presumed if an error was detected.
    // Get information related to the error that occurred.
    static nn::Result GetResultCode(void);
    static const char* GetFilename(void);
    static u32 GetLineNo(void);

    static void PrintResult(nn::Result result);

    // Output the result code string expression maintained by this class to the debug output.
    static void Print(void);

    // Clears the <tt>nn::Result</tt> type value maintained by this class.
    static void Clear(void);

    // {secret}
    // n1769
    // Forcibly sets the result code. This function is provided so that you can intentionally cause an error to verify that your error handling sequence is correct.
    // 
    static void SetResultCode(nn::Result result);

    static const size_t FILENAME_MAX_LEN = 256;

    // <tt>nn::Result</tt> type error code-related information.
    static nn::Result s_result;
    static char s_filename[FILENAME_MAX_LEN];
    static u32 s_lineNo;

private:
    ///< The copy constructor and the assignment operator are private.
    ErrorHandling(void)
    {
    }
    ~ErrorHandling(void)
    {
    }
    ErrorHandling(const ErrorHandling&);
    ErrorHandling& operator=(const ErrorHandling&);

    static bool s_bInitialized;
};
