﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Thread.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Thread.h"
#include "Defines.h"
#include "Assert.h"
#include <pia/common.h>


#if NN_PIA_CTR
#include <nn/os.h>
#include <nn/fnd.h>
#endif


namespace
{
}  // end of anonymous namespace


///< Instantiates the object.
Thread::Thread(void)
{
}


///< Destroys the object.
Thread::~Thread(void)
{
}


#if NN_PIA_CTR
void Thread::Start(ThreadFunc pFunc, void* pArg, size_t stackSize, s32 priority)
{
    m_Thread.StartUsingAutoStack(pFunc, pArg, stackSize, priority);
}


bool Thread::IsAlive()
{
    return m_Thread.IsAlive();
}


void Thread::Join(void)
{
    m_Thread.Join();
}


void Thread::Finalize(void)
{
    m_Thread.Finalize();
}


void Thread::Detach(void)
{
    m_Thread.Detach();
}


void Thread::Sleep(s32 msec)
{
    nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(msec));
}


#elif NN_PIA_WIN


#error  "Not implemented."


#elif NN_PIA_CAFE


void Thread::Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, u16 attr, OSPriority priority)
{
    u16 attribute = attr;

    // Start a thread on the main core of the sample if affinity is not set.
    if ((attribute & (OS_THREAD_ATTR_AFFINITY_CORE0 | OS_THREAD_ATTR_AFFINITY_CORE1 | OS_THREAD_ATTR_AFFINITY_CORE2)) == 0)
    {
        switch (PIA_SAMPLE_MAIN_CORE)
        {
        case 0:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE0;
        }
        break;
        case 1:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE1;
        }
        break;
        case 2:
        {
            attribute |= OS_THREAD_ATTR_AFFINITY_CORE2;
        }
        break;
        }
    }

    OSCreateThread(&m_Thread,
                   pFunc,
                   1,
                   pArg,
                   static_cast<u8*>(pStackAddr) + stackSize,
                   stackSize,
                   priority,
                   attribute);

    OSResumeThread(&m_Thread);
}


bool Thread::IsAlive()
{
    return (!OSIsThreadTerminated(&m_Thread));
}

void Thread::Join(void)
{
    BOOL ret = OSJoinThread(&m_Thread, NULL);
    PIA_SAMPLE_ASSERTMSG(ret, "OSJoinThread() failed.");
}


void Thread::Detach(void)
{
    OSDetachThread(&m_Thread);
}


void Thread::Finalize(void)
{
    //n1769 Is anything special required for Cafe?
}


void Thread::Sleep(s32 msec)
{
    OSSleepMilliseconds(msec);
}


#endif  // end of NN_PIA_(platform)
