﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Thread.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>

#if NN_PIA_CAFE
#include <cafe.h>
#endif

#if NN_PIA_CTR
typedef void (*ThreadFunc)(void* pArg);
#elif NN_PIA_WIN
#error  "Not implemented."
#elif NN_PIA_NIN_WIN
// #error  "Not implemented."
#elif NN_PIA_CAFE
typedef int (*ThreadFunc)(int args, void* pArg);
#else
#error  "Invalid platform"
#endif


///< Represents a thread.
class Thread
{
public:
    Thread(void);
    ~Thread(void);

#if NN_PIA_CTR
    void Start(ThreadFunc pFunc, void* pArg, size_t stackSize, s32 priority = nn::os::DEFAULT_THREAD_PRIORITY);
#elif NN_PIA_WIN

#elif NN_PIA_NIN_WIN
    
#elif NN_PIA_CAFE
    void Start(ThreadFunc pFunc, void* pArg, void* pStackAddr, size_t stackSize, u16 attr = 0, OSPriority priority = OS_PRIORITY_APP_DEFAULT);
#else
#error  "Invalid platform"
#endif

    bool IsAlive();

    void Join(void);

    void Detach(void);

    void Finalize(void);

    ///< Makes the current thread sleep for the specified length of time. Specify the time in milliseconds.
    static void Sleep(s32 msec);


private:
    ///< The copy constructor and the assignment operator are private.
    Thread(const Thread&);
    Thread& operator=(const Thread&);

#if NN_PIA_CTR
    nn::os::Thread m_Thread;
#elif NN_PIA_WIN
#error  "Not implemented."
#elif NN_PIA_NIN_WIN
// #error  "Not implemented."
#elif NN_PIA_CAFE
    PIA_ATTRIBUTE_ALIGN(8) OSThread m_Thread;
#else
#error  "Invalid platform"
#endif
};
