﻿//
//------------------------------------------------------------
// Copyright(c) 2009-2010 by Digital Media Professionals Inc.
// All rights reserved.
//------------------------------------------------------------
// This source code is the confidential and proprietary
// of Digital Media Professionals Inc.
//------------------------------------------------------------
//

#pragma once

#include <nn/config.h>
#include <nn/gx.h>
#include <gles2/gl2ext.h>

namespace demo
{

/*!
        @brief  Converts OpenGL format texture data to PICA native format.
        
        The input texture data and output texture data must have the same data type.
        The following texture data types are supported: <tt>GL_RGBA_NATIVE_DMP</tt>: R8G8B8A8 (8 bits per component); <tt>GL_RGB_NATIVE_DMP</tt>: R8G8B8 (8 bits per component); and <tt>GL_LUMINANCE_ALPHA_NATIVE_DMP</tt>: L8A8 (8 bits per component).
        
        
        @param[in] format  Specifies the texture data format (<tt>GL_RGBA_NATIVE_DMP</tt>, <tt>GL_RGB_NATIVE_DMP</tt>, or <tt>GL_LUMINANCE_ALPHA_NATIVE_DMP</tt>).
        @param[in] width  Specifies the texture width in pixels.
        @param[in] height  Specifies the texture height in pixels.        
        @param[in] glData  Specifies a pointer to OpenGL format texture data.
        @param[in] dmpData  Specifies a pointer to PICA native format texture data. (It requires a region in the same size as <tt>glData</tt>.)
        @return  Returns <tt>true</tt> if conversion succeeds; returns <tt>false</tt> otherwise.
    */
bool ConvertGLTextureToNative(const GLenum format, const u32 width, const u32 height, void* glData, void* dmpData);
}

