﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     HomeButtonProcess.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "HomeButtonProcess.h"

#if NN_PIA_CAFE
#include <cafe/procui.h>
#endif

#include "Sndmic.h"
#include "Misc.h"

#include "Shared_Chat_Setup.h"


u32 HomeButtonProcess::s_HomeMenuStatus = 0;


void HomeButtonProcess::Initialize()
{
    //Initialize the HOME Menu status variable.
    s_HomeMenuStatus = 0;

    //Initialize ProcUI.
    ProcUIInitEx(&saveCallback, NULL);

    //Configure a region for storing and restoring MEM1.
    void* mem1Storage;
    u32 mem1Size;
    OSGetMemBound(OSMem_MEM1, (u32*)&mem1Storage, &mem1Size);
    mem1Storage = MEMAllocFromDefaultHeap(mem1Size);
    ProcUISetMEM1Storage(mem1Storage, mem1Size);

    //Set where to process system messages.
    ProcUIRegisterCallback(PROCUI_MESSAGE_ACQUIRE, callbackAcquiredForeground, NULL, 200);
    ProcUIRegisterCallback(PROCUI_MESSAGE_RELEASE, callbackReleaseForeground, NULL, 200);
}


void HomeButtonProcess::Finalize()
{
}


bool HomeButtonProcess::Process(SndMicDevice* pSndMicDevice)
{
    if (OSGetCoreId() != OSGetMainCoreId())
    {
        ProcUISubProcessMessages(TRUE);
    }

    //Process for system messages.
    ProcUIStatus status = ProcUIProcessMessages(TRUE);
    if (status == PROCUI_STATUS_EXIT)
    {
        //The process needs to be ended, but do not do anything here.
    }
    else if (status == PROCUI_STATUS_RELEASING)
    {
        //The HOME Button has been pressed. Release this process from the foreground and move it to the background.
        pSndMicDevice->Pause(NULL);  //Pause the sound and microphone.
                                     //        p_SndMicDevice = NULL;
        ProcUIDrawDoneRelease();
    }
    if (s_HomeMenuStatus)  // During the HOME Menu
    {
        if (s_HomeMenuStatus == 2)  // If 2, move from the background to the foreground.
        {
            SND_PARAM sndConfig;
            // The sound set is 8 kHz.
            sndConfig.freq = SND_FREQ_08KHZ;
            sndConfig.frameSample = Chat_GetVoiceProtocol()->GetAudioFrameLength();
            pSndMicDevice->Pause(&sndConfig);  //Resume the paused sound and microphone.
                                               //            p_SndMicDevice = sndmic;
            SAMPLE_PRINT("restart\n\n");
            Chat_GetVoiceProtocol()->ClearStreaming();
            s_HomeMenuStatus = 0;  //Foregound.
        }
        return true;
    }

    return false;
}


//Release callback.
u32 HomeButtonProcess::saveCallback(void* unused)
{
    OSSavesDone_ReadyToRelease();
    return 0;
}


//Request to move from the background to the foreground.
u32 HomeButtonProcess::callbackAcquiredForeground(void* unused)
{
    s_HomeMenuStatus = 2;
    return 0;
}


//Request to move from the foreground to the background.
u32 HomeButtonProcess::callbackReleaseForeground(void* unused)
{
    s_HomeMenuStatus = 1;
    return 0;
}
