﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ItemBase.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common.h>


class ItemBase
{
public:
    // Constructor.
    ItemBase()
      : m_pUserData(NULL)
    {
    }

    // Destructor.
    virtual ~ItemBase()
    {
    }

    // Sets the value to "Next."
    virtual void Next() = 0;

    // Sets the value to "Previous."
    virtual void Prev() = 0;

    // Rendering
    virtual void Draw(s32 x, s32 y) const = 0;

    // Dispatch.
    virtual void Dispatch() {};

    // Commit. Called to actually "apply" the changed value to the system.
    virtual void Commit() = 0;

    // Describes the operation corresponding to depression of the A Button.
    virtual void Exec()
    {
    }

    // Gets user data
    void* GetUserData() const
    {
        return m_pUserData;
    }

    // Sets user data.
    void SetUserData(void* pUserData)
    {
        m_pUserData = pUserData;
    }

private:
    void* m_pUserData;
};
