﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     MenuConnected.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "MenuConnected.h"

#include "Types.h"
#include "Config.h"
#include "Misc.h"
#include "Global.h"

#include "SampleLib/SampleLib.h"


namespace
{
class ItemConferenceModeMenu : public ItemBase
{
public:
    ItemConferenceModeMenu()
    {
    }
    virtual ~ItemConferenceModeMenu()
    {
    }

    virtual void Next()
    {
    }
    virtual void Prev()
    {
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Conference Mode Menu...");
        }
    }
    virtual void Commit()
    {
    }
    virtual void Exec()
    {
        *static_cast<bool*>(GetUserData()) = true;
    }
};


class ItemStopNetwork : public ItemBase
{
public:
    ItemStopNetwork()
    {
    }
    virtual ~ItemStopNetwork()
    {
    }

    virtual void Next()
    {
    }
    virtual void Prev()
    {
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Stop Network...");
        }
    }
    virtual void Commit()
    {
    }
    virtual void Exec()
    {
        *static_cast<bool*>(GetUserData()) = true;
    }
};

}  // end of anonymous namespace


MenuConnected::MenuConnected()
  : m_bReadyToGoConferenceMode(false),
    m_bReadyToStopNetwork(false)
{
    // Fills the pointer array with NULL.
    for (size_t i = 0; i < MAX_ITEMS; ++i)
    {
        m_paItem[i] = NULL;
    }

    // Registers the item object in the menu.
    m_paItem[0] = new ItemConferenceModeMenu;
    m_paItem[0]->SetUserData(&m_bReadyToGoConferenceMode);
    m_paItem[1] = new ItemStopNetwork;
    m_paItem[1]->SetUserData(&m_bReadyToStopNetwork);

    for (size_t i = 0; i < MAX_ITEMS; ++i)
    {
        if (m_paItem[i] != NULL)
        {
            m_MenuImpl.Register(m_paItem[i]);
        }
        else
        {
            break;
        }
    }
}


MenuConnected::~MenuConnected()
{
    // Clean up the constructed item objects.
    for (int i = MAX_ITEMS - 1; i >= 0; --i)
    {
        if (m_paItem[i] != NULL)
        {
            delete m_paItem[i];
            m_paItem[i] = NULL;
        }
    }
}


void MenuConnected::Dispatch()
{
    m_MenuImpl.Dispatch();
}


void MenuConnected::Draw() const
{
    m_MenuImpl.Draw(16, 80);
}
