﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     MenuImpl.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "MenuImpl.h"

#include <pia/common.h>

#include "Misc.h"
#include "SampleFramework/include/SampleFramework.h"


namespace
{
}  // end of anonymous namespace


// Constructor.
MenuImpl::MenuImpl()
  : m_paItem(),
    m_Items(0),
    m_CursorIdx(0),
    m_DispatchCalls(0)
{
    for (size_t i = 0; i < MAX_ITEMS; ++i)
    {
        m_paItem[i] = NULL;
    }
}


// Destructor.
MenuImpl::~MenuImpl()
{
}


bool MenuImpl::Register(ItemBase* pItem)
{
    if (!PIA_IS_VALID_POINTER(pItem))
    {
        PIA_BASIC_REPORT("MenuImpl::Register() got invalid argument.\n");
        return false;
    }

    if (m_Items >= MAX_ITEMS)
    {
        PIA_BASIC_REPORT("Item is full.\n");
        return false;
    }

    m_paItem[m_Items] = pItem;
    ++m_Items;
    return true;
}


void MenuImpl::Dispatch()
{
    ++m_DispatchCalls;

    nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();

    if (m_Items > 0)
    {
        // Executes when the A Button is pressed.
        if (f->IsTrigger(nn::pia::sampleframework::BUTTON_A))
        {
            m_paItem[m_CursorIdx]->Exec();
        }

        // Adjust parameters according to cursor's movement vertically and horizontally.
        if (f->IsTrigger(nn::pia::sampleframework::BUTTON_DOWN))
        {
            PIA_SAMPLE_ASSERT(m_Items > 0);
            m_CursorIdx = (m_CursorIdx + 1) % m_Items;
        }
        else if (f->IsTrigger(nn::pia::sampleframework::BUTTON_UP))
        {
            PIA_SAMPLE_ASSERT(m_Items > 0);
            if (m_CursorIdx == 0)
            {
                m_CursorIdx = m_Items - 1;
            }
            else
            {
                --m_CursorIdx;
            }
        }
        else if (f->IsTrigger(nn::pia::sampleframework::BUTTON_RIGHT))
        {
            m_paItem[m_CursorIdx]->Next();
        }
        else if (f->IsTrigger(nn::pia::sampleframework::BUTTON_LEFT))
        {
            m_paItem[m_CursorIdx]->Prev();
        }

        // Dispatch the registered items.
        for (size_t i = 0; i < m_Items; ++i)
        {
            m_paItem[i]->Dispatch();
        }
    }
}


void MenuImpl::Draw(s32 x, s32 y) const
{
    // Draw the registered items.
    const s32 H = 10;  // Height of one line
    for (size_t i = 0; i < m_Items; ++i)
    {
        m_paItem[i]->Draw(x, y + H * i);
    }

    // Render cursor
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (m_DispatchCalls % 40 < 20)
        {
            f->DrawText(x - 10, y + H * m_CursorIdx, ">");
        }
        else
        {
            f->DrawText(x - 7, y + H * m_CursorIdx, ">");
        }
    }
}


void MenuImpl::Commit()
{
    // Commit the registered items.
    for (size_t i = 0; i < m_Items; ++i)
    {
        m_paItem[i]->Commit();
    }
}


// Print information useful for debugging.
void MenuImpl::Trace(u64 flag) const
{
#if NN_PIA_ENABLE_TRACE
    PIA_TRACE(flag, "MenuImpl::Trace() called.");
#else   // else of NN_PIA_ENABLE_TRACE
    NN_PIA_DUMMY_PARAM(flag);
#endif  // end of NN_PIA_ENABLE_TRACE
}
