﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     MenuImpl.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/common.h>

#include "ItemBase.h"


class MenuImpl
{
public:
    // Constructor.
    MenuImpl();

    // Destructor.
    ~MenuImpl();

    // Register items.
    bool Register(ItemBase* pItem);

    // Dispatch.
    void Dispatch();

    // Rendering
    void Draw(s32 x, s32 y) const;

    // Apply the changes to the system.
    void Commit();

    // Get the cursor position.
    size_t GetCursorIndex() const
    {
        return m_CursorIdx;
    }

    // Prints information that is useful for debugging.
    void Trace(u64 flag) const;


private:
    // The copy constructor is sealed.
    MenuImpl(const MenuImpl&);

    // The assignment operator is sealed.
    MenuImpl& operator=(const MenuImpl&);

    static const size_t MAX_ITEMS = 64;

    ItemBase* m_paItem[MAX_ITEMS];
    size_t m_Items;
    size_t m_CursorIdx;
    size_t m_DispatchCalls;
};
