﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     MenuTop.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "MenuTop.h"

#include "Types.h"
#include "Config.h"
#include "Misc.h"
#include "Global.h"

#include "SampleLib/SampleLib.h"


namespace
{
class ItemNetworkMatchMode : public ItemBase
{
public:
    ItemNetworkMatchMode()
    {
    }
    virtual ~ItemNetworkMatchMode()
    {
    }

    virtual void Next()
    {
        g_MatchMode = SAMPLE_CHAT_NET_MATCH_FRIENDS;
    }
    virtual void Prev()
    {
        g_MatchMode = SAMPLE_CHAT_NET_MATCH_ANYBODY;
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Network Match Mode : %s", matchModeToStr(g_MatchMode));
        }
    }
    virtual void Commit()
    {
    }
};


class ItemGameMode : public ItemBase
{
public:
    ItemGameMode()
    {
    }
    virtual ~ItemGameMode()
    {
    }

    virtual void Next()
    {
        ++g_GameMode;
    }
    virtual void Prev()
    {
        --g_GameMode;
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Game Mode          : %d", g_GameMode);
        }
    }
    virtual void Commit()
    {
    }
};


class ItemMaxStationNum : public ItemBase
{
public:
    ItemMaxStationNum()
    {
    }
    virtual ~ItemMaxStationNum()
    {
    }

    virtual void Next()
    {
        g_MaxStationNum = clamp<size_t>(g_MaxStationNum + 1, 2, MAX_CLIENTS);
    }
    virtual void Prev()
    {
        g_MaxStationNum = clamp<size_t>(g_MaxStationNum - 1, 2, MAX_CLIENTS);
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Max Station Num    : %d", g_MaxStationNum);
        }
    }
    virtual void Commit()
    {
    }
};


class ItemStartNetwork : public ItemBase
{
public:
    ItemStartNetwork()
    {
    }
    virtual ~ItemStartNetwork()
    {
    }

    virtual void Next()
    {
    }
    virtual void Prev()
    {
    }
    virtual void Draw(s32 x, s32 y) const
    {
        nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
        if (PIA_IS_VALID_POINTER(f))
        {
            f->DrawText(x, y, "Start Network...");
        }
    }
    virtual void Commit()
    {
    }
    virtual void Exec()
    {
        *static_cast<bool*>(GetUserData()) = true;
    }
};

}  // end of anonymous namespace


MenuTop::MenuTop()
  : m_bReadyToGoNextScene(false)
{
    // Fills the pointer array with NULL.
    for (size_t i = 0; i < MAX_ITEMS; ++i)
    {
        m_paItem[i] = NULL;
    }

    // Registers the item object in the menu.
    m_paItem[0] = new ItemNetworkMatchMode;
    m_paItem[1] = new ItemGameMode;
    m_paItem[2] = new ItemMaxStationNum;
    m_paItem[3] = new ItemStartNetwork;
    m_paItem[3]->SetUserData(&m_bReadyToGoNextScene);

    for (size_t i = 0; i < MAX_ITEMS; ++i)
    {
        if (m_paItem[i] != NULL)
        {
            m_MenuImpl.Register(m_paItem[i]);
        }
        else
        {
            break;
        }
    }
}


MenuTop::~MenuTop()
{
    // Clean up the constructed item objects.
    for (int i = MAX_ITEMS - 1; i >= 0; --i)
    {
        if (m_paItem[i] != NULL)
        {
            delete m_paItem[i];
            m_paItem[i] = NULL;
        }
    }
}


void MenuTop::Dispatch()
{
    m_MenuImpl.Dispatch();
}


void MenuTop::Draw() const
{
    m_MenuImpl.Draw(16, 100);
}


bool MenuTop::IsReadyToGoNextScene() const
{
    return m_bReadyToGoNextScene;
}
