﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Misc.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once


#include <pia/common.h>

#include "SampleLib/SampleLib.h"

#include "Config.h"


#if NN_PIA_CTR
#define SAMPLE_PRINT NN_LOG
#define SAMPLE_ASSERT NN_ASSERT
#define SAMPLE_PANIC NN_PANIC
#elif NN_PIA_CAFE
#define SAMPLE_PRINT OSReport
#define SAMPLE_ASSERT ASSERT
#define SAMPLE_PANIC OSFatal
#endif


// Group of wrapper functions that simplify the calling of framework functions.
nn::pia::sampleframework::SampleFramework* getSampleFrameworkInstance();
void beginFrame(bool bNexDispatch = true);
void endFrame();
void changeRenderTarget(bool bUpper);
void drawText(s32 x, s32 y, const char* pFmt, ...);
void setClearColor(f32 r, f32 g, f32 b);
void swapBuffers();

// Clamp. <tt>orgValue</tt> is a value between <tt>minValue</tt> and <tt>maxValue</tt>.
template <typename T>
T clamp(T orgValue, T minValue, T maxValue)
{
    if (orgValue < minValue)
    {
        return minValue;
    }
    if (maxValue < orgValue)
    {
        return maxValue;
    }
    return orgValue;
}


// Useful functions.
bool isConnectedSession();


#if USE_VALIDATION_MODE

class Validation
{
public:
    static void initialize();

    static void enableTriangleWaveGeneration(bool enable);
    static void renderValidationMode();
    static void gotoNextValidation();
    static bool isPlayTriangleWave()
    {
        return s_bPlayTriangleWave;
    }

    static const size_t TRIANGLE_WAVE_DATA_SAMPLING_NUM = 144 * 8;
    static s16* getTriangleWaveData()
    {
        return s_TriangleWaveData;
    }

private:
    static const int ITEMS = 4;
    static void packetLossEmulation(int percentage);
    static void createTriangleWave(s16* out, size_t sampleNum);  // Creates a triangle wave.
    static int s_Mode;
    static bool s_bPlayTriangleWave;
    static s16 s_TriangleWaveData[];
};
#endif  // end of USE_VALIDATION_MODE
