﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SceneConnected.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SceneConnected.h"

#include <cstdlib>  // for rand(), srand()

#include <pia/session.h>
#include <pia/transport.h>

#include "Misc.h"
#include "Config.h"
#include "Types.h"
#include "MenuConnected.h"

#include "Shared_Chat_Setup.h"


namespace
{
void drawNetworkInformation()
{
    nn::pia::sampleframework::SampleFramework* f = nn::pia::sampleframework::SampleFramework::GetInstance();
    NetInformationStructure netInfo;
    setNetInfo(&netInfo);
    nn::pia::common::String string;
    nn::pia::common::InetAddress inetAddress;

    s32 y = 0;

    f->DrawText(0, 16 * y++, "----------------------------------------------------");
    f->DrawText(0, 16 * y++, "<<<<           Network Information...           >>>>");
    f->DrawText(0, 16 * y++, "----------------------------------------------------");
    netInfo.hostAddr.GetAddressString(&string);
    f->DrawText(0, 16 * y++, "  HOST            : %s", string.GetCharArray());
    netInfo.netMask.GetAddressString(&string);
    f->DrawText(0, 16 * y++, "  MASK            : %s", string.GetCharArray());
    netInfo.dns1.GetAddressString(&string);
    f->DrawText(0, 16 * y++, "  DNS1            : %s", string.GetCharArray());
    netInfo.dns2.GetAddressString(&string);
    f->DrawText(0, 16 * y++, "  DNS2            : %s", string.GetCharArray());
    netInfo.gateway.GetAddressString(&string);
    f->DrawText(0, 16 * y++, "  GATEWAY         : %s", string.GetCharArray());
#if NN_PIA_CTR
    f->DrawText(0, 16 * y++, "  MAC             : %02X:%02X:%02X:%02X:%02X:%02X", netInfo.mac[0], netInfo.mac[1], netInfo.mac[2], netInfo.mac[3], netInfo.mac[4], netInfo.mac[5]);
#endif
    f->DrawText(0, 16 * y++, "  MY StationId    : 0x%016llX", netInfo.myStationId.GetValue());
    f->DrawText(0, 16 * y++, "  MY PID          : %ld", netInfo.myPid);
    f->DrawText(0, 16 * y++, "  HOST PID        : %ld", netInfo.hostPid);
    f->DrawText(0, 16 * y++, "----------------------------------------------------");
}


void drawConnectedMenuCaption()
{
    nn::pia::sampleframework::SampleFramework* pFw = nn::pia::sampleframework::SampleFramework::GetInstance();

    pFw->DrawText(0, 8 * 4, "----------------------------------------");
    pFw->DrawText(0, 8 * 6, "<<<<       Network Connected        >>>>");
    pFw->DrawText(0, 8 * 8, "----------------------------------------");
}
}


s32 sceneConnected()
{
    setClearColor(0.0f, 0.0f, 0.7f);

    std::srand(1);

#if !ENABLE_VAD
    Chat_GetVoiceProtocol()->EnableVAD(false);
#endif

    {
        // You can change the audio frame length after PiaChat has been initialized, but the frame length must be the same for all stations.
        // 
        Chat_GetVoiceProtocol()->SetAudioFrameLength(nn::pia::chat::VoiceProtocol::AUDIO_FRAME_LENGTH_DEFAULT);
    }

    PIA_SAMPLE_PRINT("GetAudioDataSize() = %d\n", Chat_GetVoiceProtocol()->GetAudioDataSize());

    Chat_GetVoiceProtocol()->Trace(nn::pia::common::TRACE_FLAG_ALWAYS);

    MenuConnected menuConnected;

    s32 ret = SCENE_CLOSE_NETWORK;

    while (1)
    {
        beginFrame();

        menuConnected.Dispatch();

        changeRenderTarget(true);  // Upper screen.
        drawNetworkInformation();
        swapBuffers();

        changeRenderTarget(false);  // Lower screen.
        drawConnectedMenuCaption();
        menuConnected.Draw();
        swapBuffers();

        endFrame();

        // Escape the loop if the connection has been dropped.
        nn::Result r = nn::pia::transport::Transport::GetInstance()->CheckConnectionError();
        if (r.IsSuccess())
        {
            // No problem.
        }
        else if (r == nn::pia::ResultNetworkConnectionIsLost())
        {
            ret = SCENE_CLOSE_NETWORK;
            break;  // Network connection cut. Escape the loop.
        }
        else
        {
            PIA_SAMPLE_LOG_RESULT(r);
            PIA_SAMPLE_ASSERTMSG(false, "Transport::CheckConnectionError() returned unexpected Result.");
        }

        // Escape the loop even though the session connection status is questionable.
        if (!isConnectedSession())
        {
            ret = SCENE_CLOSE_NETWORK;
            break;
        }

        if (menuConnected.IsReadyToGoConferenceMode())
        {
            ret = SCENE_CONFERENCE;
            break;
        }

        if (menuConnected.IsReadyToStopNetwork())
        {
            ret = SCENE_CLOSE_NETWORK;
            break;
        }
    }

    return ret;
}
