﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Chat_Setup.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Shared_Chat_Setup.h"

#include "SampleLib/SampleLib.h"

#include <pia/transport.h>


namespace
{
u32 s_VoiceProtocolHandle = 0;
}  // end of anonymous namespace


nn::Result Chat_InitializePiaChat(size_t maxStationNum)
{
    PIA_BASIC_REPORT("Chat_InitializePiaChat() called.\n");

    nn::Result result;

    result = nn::pia::chat::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::chat::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    // Creates an instance of <tt>VoiceProtocol</tt>.
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(pTransport);
    const u16 PORT = 1;
    PIA_SAMPLE_ASSERT(s_VoiceProtocolHandle == 0);
    s_VoiceProtocolHandle = pTransport->CreateProtocol<nn::pia::chat::VoiceProtocol>(PORT);
    PIA_SAMPLE_ASSERT(s_VoiceProtocolHandle != 0);

    // Initializes the <tt>VoiceProtocol</tt> instance.
    nn::pia::chat::VoiceProtocol::Setting setting;
    setting.maxStationNum = maxStationNum;
    PIA_SAMPLE_ASSERT(Chat_GetVoiceProtocol());
    result = Chat_GetVoiceProtocol()->Initialize(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    result = nn::pia::chat::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

    return nn::ResultSuccess();
}


void Chat_FinalizePiaChat()
{
    nn::pia::chat::VoiceProtocol* pVoice = Chat_GetVoiceProtocol();
    if (pVoice)
    {
        pVoice->Finalize();
        nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
        if (pTransport)
        {
            pTransport->DestroyProtocol(s_VoiceProtocolHandle);
            s_VoiceProtocolHandle = 0;
        }
    }

    nn::pia::chat::Finalize();
}


nn::pia::chat::VoiceProtocol* Chat_GetVoiceProtocol()
{
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    if (pTransport == NULL)
    {
        PIA_BASIC_REPORT("Transport is not ready.\n");
        return NULL;
    }
    if (s_VoiceProtocolHandle == 0)
    {
        PIA_BASIC_REPORT("Handle of VoiceProtocol is zero.\n");
        return NULL;
    }

    return pTransport->GetProtocol<nn::pia::chat::VoiceProtocol>(s_VoiceProtocolHandle);
}
