﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SndmicCtr.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "Config.h"

/*---------------------------------------------------------------------------*
 * Includes
 *---------------------------------------------------------------------------*/
#include <nn.h>
#include <nn/mic.h>
#include <nn/snd.h>

#include <pia/chat.h>

#include "SampleFramework/include/SampleFramework.h"
#include "Wave.h"

#include "SndmicTypes.h"


// Forward declarations.
namespace nn
{
namespace pia
{
namespace chat
{
class VoiceProtocol;
}
}
}


/*---------------------------------------------------------------------------*
 *  Define
 *---------------------------------------------------------------------------*/
const int SOUND_BUFFER_NUM = 3;  // Number of the sound buffer.


/*---------------------------------------------------------------------------*
    Class
 *---------------------------------------------------------------------------*/
class SndMicDevice
{

public:
    // Functions.
    SNDMIC_ERROR_STATUS Initialize(SndMicConfigStructure* config, nn::pia::chat::VoiceProtocol* pVoiceProtocol);
    SNDMIC_ERROR_STATUS Finalize(void);
    SNDMIC_ERROR_STATUS Start(void);
    SNDMIC_ERROR_STATUS Stop(void);
    SNDMIC_ERROR_STATUS MicGainSet(u8 gain);
    SNDMIC_ERROR_STATUS MicGainGet(u8* gain) const;
    SNDMIC_ERROR_STATUS StartBGM(void);
    SNDMIC_ERROR_STATUS StopBGM(void);
    bool IsPlayingBGM(void) const;
    SNDMIC_ERROR_STATUS SetVolume(f32 volume);
    SNDMIC_ERROR_STATUS SetBGMVolume(f32 volume);
    void GetMicData(void);
    void MicDataThreadImpl(void);
    void SoundThreadFuncImpl(void);
    void FrameProcessThreadImpl(void);
    bool GetSndMicErrorFlag(void);
    void RomfileWavePlay(u32 number);
    bool GetReceiveAudioData(u8* audioBuf, u16 length);
    u8 Pause(void);
    void Dispatch(void);

private:
    void chatSendReceive(void);
    void sendWrapper(void* pData, size_t samples, u32 offset);
    void receiveWrapper(void* pData, u32 frameDataSize, nn::pia::StationIndex stationIndexArray[]);

    // Variables
    nn::os::Thread m_SoundThread;
    nn::os::Thread m_FrameProcessThread;
    nn::os::Event m_MicSyncEvent;
    nn::os::Event m_MicStopEvent;

    nn::snd::WaveBuffer m_WaveBuffer[SOUND_BUFFER_NUM];
    nn::snd::WaveBuffer m_MusicWaveBuffer;
    nn::snd::Voice* m_pVoice;
    nn::snd::Voice* m_pMusic;
    u8* m_SoundBuffer[SOUND_BUFFER_NUM];
    s16 m_MixedData[NN_SND_SAMPLES_PER_FRAME * 2];
    s8 m_SndBuf[4][nn::pia::chat::VoiceProtocol::AUDIO_DATA_SIZE_MAX];
    u16 m_SndBufWriteIdx;
    u16 m_SndBufReadIdx;
    SndMicConfigStructure m_Config;
    u32 m_SoundBufferSize;
    u32 m_AudioFrameSize;
    u16 m_CurrBufferIdx;
    u16 m_SoundBufferReqIdx;
    SND_PROCESS_STATUS m_SoundStatus;

    char m_MicBuffer[8192] NN_ATTRIBUTE_ALIGN(nn::mic::BUFFER_ALIGNMENT_ADDRESS);
    uptr m_MicBuffHalfPosition;
    u16 m_MicBufferSize;
    u16 m_MicPrevPosition;
    MIC_PROCESS_STATUS m_MicStatus;
    u8 m_MicGain;
    u8 m_Pause;
    u8* m_pWaveMemory[3];
    u8* m_pPlayWave;
    u32 m_FrameCount[3];
    u32 m_FramePlayNum;
    u32 m_WaveNumbber;
    WaveData m_WaveData[3];
    bool m_FramePlayReq;
    bool m_MicTimingError;
    char m_zero_buf[192 * 8 * 2];
    bool m_bChatSendReceiveReady;
};


/* EOF. */
