﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SndmicTypes.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "Config.h"


/*---------------------------------------------------------------------------*
 *  Enum
 *---------------------------------------------------------------------------*/
/* mic & snd error status. */
typedef enum
{
    SNDMIC_ERROR_NONE = 0,  // None error
    SNDMIC_ERROR_PARAM,     // Parameter error
    MIC_ERROR_PARAM,        // Parameter error
    MIC_ERROR_SEQ,          // Sequence error
    MIC_ERROR_SYS,          // System error
    MIC_ERROR_LIB,          // Library error
    MIC_WARN_FEW_DATA,      // Few data warning
    SND_ERROR_PARAM,        // Parameter error
    SND_ERROR_SEQ,          // Sequence error
    SND_ERROR_SYS,          // System error
    SND_ERROR_LIB,          // SND library error
    SND_ERROR_DSP           // DSP error
} SNDMIC_ERROR_STATUS;

/* mic process status. */
typedef enum
{
    SNDMIC_SEQ_NONE = 0,  // None process
    MIC_SEQ_INITED,       // Initialize process
    MIC_SEQ_START,        // Start process
    MIC_SEQ_STOP,         // Stop process
    SND_SEQ_INITED,       // Initialize process.
    SND_SEQ_START,        // Start process.
    SND_SEQ_STOPPING,     // Stopping process.
    SND_SEQ_STOPPED       // Stopped process.
} SNDMIC_SEQ_STATUS;

/* sampling frequency. */
typedef enum
{
    SND_FREQ_08KHZ = 0,  //  8kHz sampling.
    SND_FREQ_16KHZ,      // 16kHz sampling.
    SND_FREQ_32KHZ,      // 32kHz sampling.
    SND_FREQ_MAX         // Terminator.
} SND_FREQ;

/* snd process status. */
typedef enum
{
    SND_PROCESS_NONE = 0,  // None process
    SND_PROCESS_INIT,      // Initialize process
    SND_PROCESS_PLAY,      // Play process
    SND_PROCESS_STOP       // Stop process
} SND_PROCESS_STATUS;

/* mic process status. */
typedef enum
{
    MIC_PROCESS_NONE = 0,  // None process
    MIC_PROCESS_INIT,      // Initialize process
    MIC_PROCESS_START,     // Start process
    MIC_PROCESS_STOP       // Stop process
} MIC_PROCESS_STATUS;

/*---------------------------------------------------------------------------*
 *  Struct
 *---------------------------------------------------------------------------*/
typedef struct
{
    unsigned int sampleRate;
    unsigned int micGainLevel;
    unsigned int micGainMin;
    unsigned int micGainMax;
    unsigned int micGainStep;
} MIC_STATE_INFO;


typedef struct
{
    SND_FREQ freq;    // Sampling frequency.
    u32 frameSample;  // Sample of frame.
} SND_PARAM;


/* sndmic config parameter. */
typedef struct
{
    u32 frame_msec;  // Audio frame Length(mSec)
    u8 mic_gain;     // mic initialize gain
                     /*
                                  -> 31 : MIC Gain(26.0dB, ×20)
                                     43 : MIC Gain(32.0dB, ×40)
                                     55 : MIC Gain(38.0dB, ×80)
                                     67 : MIC Gain(44.0dB, ×160)
                            */
} SndMicConfigStructure;


/* EOF */
