﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Types.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Types.h"

#include <cstring>  // For <tt>memset</tt>.

#include <nn/ac.h>
#include <pia/session.h>
#include <pia/transport.h>

#include "SampleLib/SampleLib.h"


const char* getSceneName(s32 scene)
{
    switch (scene)
    {
    case SCENE_ERROR:
        return "SCENE_ERROR";
    case SCENE_NULL:
        return "SCENE_NULL";
    case SCENE_TOP:
        return "SCENE_TOP";
    case SCENE_OPEN_NETWORK:
        return "SCENE_OPEN_NETWORK";
    case SCENE_CONNECTED:
        return "SCENE_CONNECTED";
    case SCENE_CLOSE_NETWORK:
        return "SCENE_CLOSE_NETWORK";
    case SCENE_CONFERENCE:
        return "SCENE_CONFERENCE";
    case SCENE_PROGRAM_END:
        return "SCENE_PROGRAM_END";
    default:
        PIA_SAMPLE_ASSERTMSG(false, "Invalid scene number!");
        return NULL;
    }
}


const char* matchModeToStr(enum MatchMode mode)
{
    static const char* s_Str[] = { "ANYBODY", "FRIEND" };

    return s_Str[mode];
}


void setNetInfo(NetInformationStructure* pInfo)
{
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pInfo));

    nn::Result result;

#if NN_PIA_CTR
    nn::socket::InAddr addr1, addr2;
#else
    nn::ac::IpAddress addr;
#endif

    pInfo->connectionNum = 0;
    pInfo->hostPid = 0;
    pInfo->myPid = 0;
    pInfo->myStationId = 0xffffffffffffffff;

#if NN_PIA_CTR
    s32 sret;
    NN_PIA_DUMMY_PARAM_IN_RELEASE(sret);
    sret = nn::socket::GetPrimaryAddress(reinterpret_cast<u8*>(&addr1), reinterpret_cast<u8*>(&addr2));
    PIA_SAMPLE_ASSERT(sret == 0);
    pInfo->hostAddr = nn::pia::common::InetAddress(nn::pia::common::ByteOrder::Swap32(addr1.addr), 0);
    pInfo->netMask = nn::pia::common::InetAddress(nn::pia::common::ByteOrder::Swap32(addr2.addr), 0);

    sret = nn::socket::GetResolver(reinterpret_cast<u8*>(&addr1), reinterpret_cast<u8*>(&addr2));
    PIA_SAMPLE_ASSERT(sret == 0);
    pInfo->dns1 = nn::pia::common::InetAddress(nn::pia::common::ByteOrder::Swap32(addr1.addr), 0);
    pInfo->dns2 = nn::pia::common::InetAddress(nn::pia::common::ByteOrder::Swap32(addr2.addr), 0);

    sret = nn::socket::GetDefaultGateway(reinterpret_cast<u8*>(&addr1));
    PIA_SAMPLE_ASSERT(sret == 0);
    pInfo->gateway = nn::pia::common::InetAddress(nn::pia::common::ByteOrder::Swap32(addr1.addr), 0);

    sret = nn::socket::GetMacAddress(reinterpret_cast<u8*>(pInfo->mac));
    PIA_SAMPLE_ASSERT(sret == 0);
#else
    result = nn::ac::GetAssignedAddress(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    pInfo->hostAddr = nn::pia::common::InetAddress(addr, 0);

    result = nn::ac::GetAssignedSubnet(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    pInfo->netMask = nn::pia::common::InetAddress(addr, 0);

    result = nn::ac::GetAssignedPreferedDns(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    pInfo->dns1 = nn::pia::common::InetAddress(addr, 0);

    result = nn::ac::GetAssignedAlternativeDns(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    pInfo->dns2 = nn::pia::common::InetAddress(addr, 0);

    result = nn::ac::GetAssignedGateway(&addr);
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    pInfo->gateway = nn::pia::common::InetAddress(addr, 0);
//Cafe does not allow acquisition of the MAC address.
#endif

    // Fill with <tt>ff</tt> just in case if the subsequent process to get the ID fails.
    pInfo->myStationId = 0xffffffffffffffff;
    pInfo->myPid = 0xffffffff;
    pInfo->hostPid = 0xffffffff;

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    if (pSession != NULL)
    {
        pInfo->myStationId = pSession->GetLocalStationId();

        nn::pia::transport::StationManager* pManager = nn::pia::transport::StationManager::GetInstance();
        PIA_SAMPLE_ASSERT(pManager != NULL);

        nn::pia::transport::Station* pLocal = pManager->GetLocalStation();
        if (pLocal != NULL)
        {
            pLocal->GetPrincipalId(&(pInfo->myPid));
        }

        nn::pia::StationId hostId = pSession->GetHostStationId();
        nn::pia::transport::Station* pHost = pManager->GetStation(hostId);
        if (pHost != NULL)
        {
            pHost->GetPrincipalId(&(pInfo->hostPid));
        }
    }
}
