﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Types.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once


#include <pia/common.h>


const s32 SCENE_ERROR = -1;
const s32 SCENE_NULL = 0;
const s32 SCENE_TOP = 1;
const s32 SCENE_OPEN_NETWORK = 2;
const s32 SCENE_CONNECTED = 3;
const s32 SCENE_CLOSE_NETWORK = 4;
const s32 SCENE_CONFERENCE = 5;
const s32 SCENE_PROGRAM_END = 99;

const char* getSceneName(s32 scene);


enum MatchMode
{
    SAMPLE_CHAT_NET_MATCH_ANYBODY,
    SAMPLE_CHAT_NET_MATCH_FRIENDS
};


const char* matchModeToStr(enum MatchMode mode);


#if NN_PIA_CTR
const int MAX_CONNECTION_NUM = 8;
#else
const int MAX_CONNECTION_NUM = 16;
#endif

typedef struct
{
    nn::pia::common::InetAddress hostAddr; /* Host address information.        */
    nn::pia::common::InetAddress netMask;  /* Net mask information.            */
    nn::pia::common::InetAddress dns1;     /* First DNS address information.     */
    nn::pia::common::InetAddress dns2;     /* Second DNS address information.     */
    nn::pia::common::InetAddress gateway;  /* Gateway address information.     */
    u32 connectionNum;                     /* Connection number.              */
    u32 hostPid;                           /* host PID info.            */
    u32 myPid;                             /* My PID information.              */
    nn::pia::StationId myStationId;        /* my StationId info.        */
#if NN_PIA_CTR
    u8 mac[nn::socket::ETH_ALEN]; /* MAC address information.         */
#else
//Cafe does not allow acquisition of the MAC address.
#endif
} NetInformationStructure;


void setNetInfo(NetInformationStructure* pInfo);
