﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Wave.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "Config.h"

/*---------------------------------------------------------------------------*/
/* Includes */
#if NN_PIA_CTR
#include <nn/types.h>
#include <nn/fs.h>
#ifndef NN_BUILD_RELEASE  // HostIO cannot be used in the Release build.
#include <nn/hio.h>
#endif  // NN_BUILD_RELEASE
#else
#include <cafe.h>
#endif

/*---------------------------------------------------------------------------*/
/* Typedef */
typedef struct
{
    u16 formatId;
    u16 channel;
    u32 sampleRate;
    u32 transRate;
    u16 blockSize;
    u16 quantumBits;
} WaveFmt;

typedef struct
{
    u32 size;
    u8* buf;
} WaveData;

/*---------------------------------------------------------------------------*/
/* Prototypes */

#if NN_PIA_CTR
void LoadWaveCore(nn::fs::FileReader*, WaveFmt*, WaveData*, bool);
static inline void LoadWave(nn::fs::FileReader* reader, WaveFmt* fmt, WaveData* data)
{
    LoadWaveCore(reader, fmt, data, false);
}
static inline int GetWaveLength(nn::fs::FileReader* reader)
{
    WaveFmt fmt;
    WaveData data;
    LoadWaveCore(reader, &fmt, &data, true);
    return data.size;
}
#endif


/*---------------------------------------------------------------------------*/
/* End of file. */
/*---------------------------------------------------------------------------*/
