﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     main.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include <pia/common.h>
#include <pia/inet.h>
#include <pia/transport.h>
#include <pia/session.h>
#include <pia/chat.h>

#include "SampleLib/SampleLib.h"

#include "Shared_Basic_Setup.h"

#include "SceneTop.h"
#include "SceneNetwork.h"
#include "SceneConnected.h"
#include "SceneConference.h"

#include "Config.h"
#include "Types.h"
#include "Misc.h"
#include "Global.h"
#include "HomeButtonProcess.h"


namespace
{
// Initialize the render framework for the Pia sample program.
void initSampleFramework()
{
    nn::pia::sampleframework::SampleFramework::CreateInstance();
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(FRAME_PER_SECOND);
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetClearColor(nn::pia::sampleframework::Color(0.0f, 0.2f, 0.0f, 1.0f));
}

// Finalizes the rendering framework for the Pia sample program.
void finiSampleFramework()
{
    nn::pia::sampleframework::SampleFramework::DestroyInstance();
}


#if NN_PIA_CTR
// Initializes the CTR module.
void initCtrFeature()
{
    nn::applet::Enable();

    nn::fs::Initialize();
    const size_t ROMFS_BUFFER_SIZE = 1024 * 64;
    static char buffer[ROMFS_BUFFER_SIZE];
    NN_UTIL_PANIC_IF_FAILED(nn::fs::MountRom("rom:", 16, 16, buffer, ROMFS_BUFFER_SIZE, true));

    nn::os::InitializeAlarmSystem();
}
#endif


}  // end of anonymous namespace


#if NN_PIA_CTR
void nnMain()
#elif NN_PIA_CAFE
int main()
#else
#error
#endif
{
    PIA_BASIC_REPORT(PROGRAM_TITLE " program start.\n");
    Basic_SetProgramTitle(PROGRAM_TITLE);

#if NN_PIA_CTR
    initCtrFeature();
#elif NN_PIA_CAFE
    HomeButtonProcess::Initialize();
#endif

#if USE_VALIDATION_MODE
    Validation::initialize();
#endif

    // Initialize.
    initSampleFramework();

    // The central part of scene switching.
    s32 nextScene = SCENE_TOP;
    while (1)
    {
        PIA_BASIC_REPORT("nextScene = %s\n", getSceneName(nextScene));
        switch (nextScene)
        {
        case SCENE_TOP:
            nextScene = sceneTop();
            break;
        case SCENE_OPEN_NETWORK:
            nextScene = sceneOpenNetwork();
            break;
        case SCENE_CLOSE_NETWORK:
            nextScene = sceneCloseNetwork();
            break;
        case SCENE_CONNECTED:
            nextScene = sceneConnected();
            break;
        case SCENE_CONFERENCE:
            nextScene = sceneConference();
            break;
#if 0
        case SCENE_ERROR:
            nextScene = sceneError();
            break;
#endif
        default:
            PIA_SAMPLE_ASSERTMSG(false, "Unknown Scene number?");
            break;
        }

        if (nextScene == SCENE_PROGRAM_END)
        {
            break;
        }
    }

    // Finalize
    finiSampleFramework();

    PIA_BASIC_REPORT("AutomaticVerification program end.\n");

#if NN_PIA_CAFE
    return 0;
#endif
}
