﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Object.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Object.h"
#if NN_PIA_CAFE
#include <cafe/zipfile.h>
#endif

ReckoningStrategy Object::s_ReckoningStrategy;
SampleLargeData s_SampleLargeDataBuf;

void Object::Operate()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_RIGHT))
    {
        if (IsCloneRegistered())
        {
            UnregisterClone();
        }
        else
        {
            RegisterClone();
        }
    }
#if PIA_TEST_CODE
    OperateForPiaTest();
#else
    OperateElement();
#endif
}

bool Object::CleanupCore(nn::pia::clone::CloneBase* pClone)
{
    if (pClone->IsRegisteredWithProtocol())
    {
        UnregisterClone();
    }
    else
    {
        nn::Result r;
        // Only call UnregisterElement if already unregistered from Clone.
        if (m_ElementU1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementU1.GetId());
            r = pClone->UnregisterElement(&m_ElementU1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementU2.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementU2.GetId());
            r = pClone->UnregisterElement(&m_ElementU2);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementU3.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementU3.GetId());
            r = pClone->UnregisterElement(&m_ElementU3);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementR1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementR1.GetId());
            r = pClone->UnregisterElement(&m_ElementR1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementE1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementE1.GetId());
            r = pClone->UnregisterElement(&m_ElementE1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
        if (m_ElementC1.IsRegisteredWithCloneBase())
        {
            PIA_SAMPLE_PRINT("Clone Element Id:%x - Unregister\n", m_ElementC1.GetId());
            r = pClone->UnregisterElement(&m_ElementC1);
            PIA_SAMPLE_ASSERT(r.IsSuccess());
        }
    }

    bool ret = !m_ElementU1.IsRegisteredWithCloneBase() &&
               !m_ElementU2.IsRegisteredWithCloneBase() &&
               !m_ElementU3.IsRegisteredWithCloneBase() &&
               !m_ElementR1.IsRegisteredWithCloneBase() &&
               !m_ElementE1.IsRegisteredWithCloneBase() &&
               !m_ElementC1.IsRegisteredWithCloneBase();
    if (ret)
    {
        PIA_SAMPLE_PRINT("Clone Type:%s - Cleanup Process is Succeeded.\n", GetCloneTypeText());
    }
    return ret;
}
void Object::PrintValue(int y)
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
#if PIA_TEST_CODE
    if (m_ElementU1.IsValidValue())
    {
        pSampleFramework->DrawText(50, y, "0x%08x", static_cast<u32>(m_ElementU1.GetValue()));
    }
    if (m_ElementE1.GetValue() != NULL)
    {
        pSampleFramework->DrawText(170, y, "0x%08x",  *m_ElementE1.GetValue());
    }
    if (m_ElementR1.IsValidValue())
    {
        pSampleFramework->DrawText(280, y, "0x%08x",  m_ElementR1.GetValue());
    }
    if (m_ElementC1.IsValidValue())
    {
        pSampleFramework->DrawText(330, y, "%6.2f", m_ElementC1.GetValue());
    }
#else
    if (m_ElementU1.IsValidValue())
    {
        pSampleFramework->DrawText(50, y, "%3lld", m_ElementU1.GetValue());
    }
    if (m_ElementU2.IsValidValue())
    {
        pSampleFramework->DrawText(100, y, "%6.2f", m_ElementU2.GetValue());
    }
    if (m_ElementU3.IsValidValue())
    {
        pSampleFramework->DrawText(170, y, "%3d %6.2f", m_ElementU3.GetValue().m_V1, m_ElementU3.GetValue().m_V2);
    }
    if (m_ElementR1.IsValidValue())
    {
        pSampleFramework->DrawText(280, y, "%3d", m_ElementR1.GetValue());
    }
    if (m_ElementC1.IsValidValue())
    {
        pSampleFramework->DrawText(330, y, "%6.2f", m_ElementC1.GetValue());
    }
#endif
}


void Object::OperateElement()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = Clone_GetCloneProtocol();

    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_A))
    {
        if (m_ElementR1.IsReadyToSetValue())
        {
            u32 v = m_ElementR1.GetValue();
            m_ElementR1.SetValue(v < 100 ? v + 1 : 0);
        }
    }

    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_B))
    {
        if (m_ElementE1.IsReadyToSetValue())
        {
            u32 v = m_ElementR1.GetValue();
            nn::Result r = m_ElementE1.SetValue(v);
            if (r.IsSuccess())
            {
                PIA_SAMPLE_PRINT("SendEvent CID:%s-%d Arg:%d Clock:%d UsingBufferNum:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 v,
                                 pCloneProtocol->GetClock(),
                                 m_ElementE1.GetUsingBufferNum());
            }
        }
    }
    
    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_X))
    {
        if (m_ElementL1.IsReadyToSetValue())
        {
            for (int i = 0; i < sizeof(s_SampleLargeDataBuf.dataArray) / sizeof(s_SampleLargeDataBuf.dataArray[0]); ++i)
            {
                s_SampleLargeDataBuf.dataArray[i] = ::std::rand();
            }
#if NN_PIA_CTR
            s_SampleLargeDataBuf.hash = nn::util::Crc32::Calculate(s_SampleLargeDataBuf.dataArray, sizeof(s_SampleLargeDataBuf.dataArray));
#elif NN_PIA_CAFE
            s_SampleLargeDataBuf.hash = ZIPFILE_CalcCRC(reinterpret_cast<const u8*>(s_SampleLargeDataBuf.dataArray), sizeof(s_SampleLargeDataBuf.dataArray));
#else
            s_SampleLargeDataBuf.hash = 0;
#endif

            nn::Result r = m_ElementL1.SetValue(s_SampleLargeDataBuf);
            if (r.IsSuccess())
            {
                PIA_SAMPLE_PRINT("SendReliableLarge CID:%s-%d Hash:0x%08x Clock:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 s_SampleLargeDataBuf.hash,
                                 pCloneProtocol->GetClock());
            }
        }
    }
}

void Object::UpdateElement()
{
    if (m_ElementU1.IsReadyToSetValue())
    {
        u32 v = m_ElementU1.GetValue();
        m_ElementU1.SetValue(v < 100 ? v + 1 : 0);
    }
    if (m_ElementU2.IsReadyToSetValue())
    {
        f32 v = m_ElementU2.GetValue();
        m_ElementU2.SetValue(v > 1.0f ? v * 0.9f : 100.0f);
    }
    if (m_ElementU3.IsReadyToSetValue())
    {
        const StructValue& v = m_ElementU3.GetValue();
        StructValue vv;
        vv.m_V1 = v.m_V1 < 100 ? v.m_V1 + 1 : 0;
        vv.m_V2 = v.m_V2 + v.m_V1;
        m_ElementU3.SetValue(vv);
    }
    while (true)
    {
        const u32* cpArg = m_ElementE1.HandleNext();
        if (cpArg == NULL)
        {
            break;
        }
        PIA_SAMPLE_PRINT("RecvEvent CID:%s-%d Arg:%d Clock:%d SetterStationIndex:%d EventIndex:%d UsingBufferNum:%d\n",
                         GetCloneTypeText(),
                         m_CloneId,
                         *cpArg,
                         *m_ElementE1.GetClock(),
                         m_ElementE1.GetSetterStationIndex(),
                         m_ElementE1.GetEventIndex(),
                         m_ElementE1.GetUsingBufferNum());

        // Checks <tt>GetValue</tt>.
        const u32* cpArg2 = m_ElementE1.GetValue();
        if (cpArg2 == NULL)
        {
            break;
        }

        PIA_SAMPLE_ASSERT(*cpArg == *cpArg2);
    }
    if (m_ElementC1.IsReadyToSetValue())
    {
        f32 clock = static_cast<f32>(Clone_GetCloneProtocol()->GetClock());
        f32 sin = sinf(clock * 0.001f) * 50.0f;
        m_ElementC1.SetValue(sin);
    }
    if (IsEnableReceive())
    {
        if (m_ElementL1.IsValidValue())
        {
            const SampleLargeData& sampleLargeData = m_ElementL1.GetValue();
            if (sampleLargeData.hash != m_SampleLargeDataPreHash)
            {
                m_SampleLargeDataPreHash = sampleLargeData.hash;
#if NN_PIA_CTR
                u32 hash = nn::util::Crc32::Calculate(sampleLargeData.dataArray, sizeof(sampleLargeData.dataArray));
#elif NN_PIA_CAFE
                u32 hash = ZIPFILE_CalcCRC(reinterpret_cast<const u8*>(sampleLargeData.dataArray), sizeof(sampleLargeData.dataArray));
#else
                u32 hash = sampleLargeData.hash;
#endif
                PIA_SAMPLE_ASSERT(hash == sampleLargeData.hash);
                PIA_SAMPLE_PRINT("RecvReliableLarge CID:%s-%d Hash:0x%08x Clock:%d\n",
                                 GetCloneTypeText(),
                                 m_CloneId,
                                 hash,
                                 m_ElementL1.GetClock());
            }
        }
    }
}

void Object::TraceProfiling() const
{
    PIA_SAMPLE_PRINT("CID:%s-%d\n", GetCloneTypeText(), m_CloneId);
    for (u32 i = 0; i < MAX_STATIONS; ++i)
    {
        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
        if (GetClone().GetSendProfiler())
        {
            PIA_SAMPLE_PRINT("  Send (%d) %7d %4d  %7d %4d\n",
                             i,
                             GetClone().GetProfiledLatestSendSize(stationIndex),
                             GetClone().GetProfiledLatestSendCount(stationIndex),
                             GetClone().GetProfiledTotalSendSize(stationIndex),
                             GetClone().GetProfiledTotalSendCount(stationIndex));
        }
        if (GetClone().GetReceiveProfiler())
        {
            PIA_SAMPLE_PRINT("  Recv (%d) %7d %4d  %7d %4d\n",
                             i,
                             GetClone().GetProfiledLatestReceiveSize(stationIndex),
                             GetClone().GetProfiledLatestReceiveCount(stationIndex),
                             GetClone().GetProfiledTotalReceiveSize(stationIndex),
                             GetClone().GetProfiledTotalReceiveCount(stationIndex));
        }
    }
    PIA_SAMPLE_PRINT("ElementType:Reliable\n");
    for (u32 i = 0; i < MAX_STATIONS; ++i)
    {
        nn::pia::StationIndex stationIndex = static_cast<nn::pia::StationIndex>(i);
        if (m_ElementR1.GetSendProfiler())
        {
            PIA_SAMPLE_PRINT("  Send (%d) %7d %4d  %7d %4d\n",
                             i,
                             m_ElementR1.GetProfiledLatestSendSize(stationIndex),
                             m_ElementR1.GetProfiledLatestSendCount(stationIndex),
                             m_ElementR1.GetProfiledTotalSendSize(stationIndex),
                             m_ElementR1.GetProfiledTotalSendCount(stationIndex));
        }
        if (m_ElementR1.GetReceiveProfiler())
        {
            PIA_SAMPLE_PRINT("  Recv (%d) %7d %4d  %7d %4d\n",
                             i,
                             m_ElementR1.GetProfiledLatestReceiveSize(stationIndex),
                             m_ElementR1.GetProfiledLatestReceiveCount(stationIndex),
                             m_ElementR1.GetProfiledTotalReceiveSize(stationIndex),
                             m_ElementR1.GetProfiledTotalReceiveCount(stationIndex));
        }
    }
}

#if PIA_TEST_CODE

void Object::UpdateForPiaTest()
{
}

void Object::OperateForPiaTest()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    nn::pia::clone::CloneProtocol* pCloneProtocol = Clone_GetCloneProtocol();
    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_R))
    {
        if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_A))
        {
            if (m_ElementR1.IsRegisteredWithCloneBase())
            {
                if (m_ElementR1.IsReadyToSetValue())
                {
                    // Sends a random value.
                    u32 v = nn::nex::Platform::GetRandomNumber(0xffffffff) + 1;
                    nn::Result r = m_ElementR1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendReliable(v);
                        PIA_SAMPLE_PRINT("SendReliable CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendReliable(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_B))
        {
            if (m_ElementE1.IsRegisteredWithCloneBase())
            {
                if (m_ElementE1.IsReadyToSetValue())
                {
                    // Sends a random value.
                    u32 v = nn::nex::Platform::GetRandomNumber(0xffffffff) + 1;
                    nn::Result r = m_ElementE1.SetValue(v, g_IsEnableEventSendDelay);
                    if (r.IsSuccess())
                    {
                        SetParamSendEvent(v);
                        PIA_SAMPLE_PRINT("SendEvent CID:%s-%d Arg:0x%08x Clock:%d SendDelay:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock(), g_IsEnableEventSendDelay);
                    }
                    else
                    {
                        SetParamSendEvent(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_X))
        {
            if (m_ElementU1.IsRegisteredWithCloneBase())
            {
                if (m_ElementU1.IsReadyToSetValue())
                {
                    // Sends a random value.
                    u32 v = nn::nex::Platform::GetRandomNumber(0xffffffff) + 1;
                    nn::Result r = m_ElementU1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendUnreliable(v);
                        PIA_SAMPLE_PRINT("SendUnreliable CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendUnreliable(0);
                    }
                }
            }
        }
        if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
        {
            if (m_ElementC1.IsRegisteredWithCloneBase())
            {
                if (m_ElementC1.IsReadyToSetValue())
                {
                    // Sends a random value.
                    u32 v = nn::nex::Platform::GetRandomNumber(0xffffffff) + 1;
                    nn::Result r = m_ElementC1.SetValue(v);
                    if (r.IsSuccess())
                    {
                        SetParamSendReckoning(v);
                        PIA_SAMPLE_PRINT("SendReckoning CID:%s-%d Arg:0x%08x Clock:%d\n", GetCloneTypeText(), m_CloneId, v, pCloneProtocol->GetClock());
                    }
                    else
                    {
                        SetParamSendReckoning(0);
                    }
                }
            }
        }
    }
    else if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_L))
    {
        if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_B))
        {
            if (m_ElementE1.IsRegisteredWithProtocol())
            {
                const u32* cpArg = m_ElementE1.HandleNext();
                if (cpArg != NULL)
                {
                    PIA_SAMPLE_PRINT("RecvEvent CID:%s-%d Arg:%d Clock:%d SetterStationIndex:%d EventIndex:%d UsingBufferNum:%d\n",
                                     GetCloneTypeText(),
                                     m_CloneId,
                                     *cpArg,
                                     *m_ElementE1.GetClock(),
                                     m_ElementE1.GetSetterStationIndex(),
                                     m_ElementE1.GetEventIndex(),
                                     m_ElementE1.GetUsingBufferNum());

                }
            }
        }
    }
}

const char* Object::GetConnectionStatusText()
{
    static char connectionStatus[MAX_STATIONS + 1];
    for (int i = 0; i < MAX_STATIONS; ++i)
    {
        connectionStatus[i] = IsConnected(g_StationIdList[i]) ? '1' : '0';
    }
    connectionStatus[MAX_STATIONS] = '\0';
    return connectionStatus;
}
#endif


SendObject::SendObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void SendObject::Operate()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    NN_PIA_DUMMY_PARAM(pSampleFramework);

    Object::Operate();

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_PRINT("SendClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_PRINT("SendClone: Connected with not all\n");
        }
    }
#endif
}

void SendObject::Update()
{
    UpdateElement();
}

bool SendObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void SendObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterSendClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("RegisterSendClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("RegisterSendClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void SendObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterSendClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("UnregisterSendClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("UnregisterSendClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int SendObject::Print(int y, const nn::pia::sampleframework::Color& cColor)
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    if (m_Clone.IsRegisteredWithProtocol())
    {
#if PIA_TEST_CODE
        pSampleFramework->DrawText(10, y, "S %1x   R %s", m_Clone.GetId(), GetConnectionStatusText());
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MAX_STATIONS)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawText(10, y, "S %1x   R %c%c%c%c%c%c%c%c", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7]);
#endif
    }
    else
    {
        pSampleFramework->DrawText(10, y, "S %1x", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void SendObject::UpdatePiaTestData()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(SEND_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    bit32 isConnectedBitmap = 0;
    for (int i = 0; i < MAX_STATIONS; ++i)
    {
        nn::pia::StationId stationId;
        nn::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(SEND_CONNECTED, isConnectedBitmap);
}

void SendObject::SetParamSendUnreliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_UNRELIABLE_SEND, v);
}

void SendObject::SetParamSendReliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_RELIABLE_SEND, v);
}

void SendObject::SetParamSendEvent(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_EVENT_SEND, v);
}

void SendObject::SetParamSendReckoning(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEND_RECKONING_SEND, v);
}
#endif


ReceiveObject::ReceiveObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void ReceiveObject::Operate()
{
    Object::Operate();
}

void ReceiveObject::Update()
{
    UpdateElement();
}

bool ReceiveObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void ReceiveObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterReceiveClone(&m_Clone, m_StationIndex, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("RegisterReceiveClone Success (StationIndex:%d, cloneId:%d)\n", m_StationIndex, m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("RegisterReceiveClone Failure (StationIndex:%d, cloneId:%d)\n", m_StationIndex, m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void ReceiveObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterReceiveClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("UnregisterReceiveClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("UnregisterReceiveClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int ReceiveObject::Print(int y, const nn::pia::sampleframework::Color& cColor)
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    pSampleFramework->DrawText(10, y, "");
    if (m_Clone.IsRegisteredWithProtocol())
    {
        pSampleFramework->DrawText(10, y, "R %1d:%1d R %c", m_Clone.GetId(), m_Clone.GetSendStationIndex(), m_Clone.IsConnected() ? '1' : '0');

        // Show RTT.
        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
        nn::pia::StationId stationId;
        nn::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, m_Clone.GetSendStationIndex());
        if (result.IsSuccess())
        {
            nn::pia::StationIndex localStationIndex;
            result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&localStationIndex, pSession->GetLocalStationId());
            nn::pia::transport::Station* pStation = nn::pia::transport::StationManager::GetInstance()->GetStation(stationId);
            if (result.IsSuccess() && pStation && localStationIndex != m_Clone.GetSendStationIndex())
            {
                s32 rtt = pStation->GetRtt();
                pSampleFramework->DrawText(180, y, "RTT: %d", rtt);
            }
        }
    }
    else
    {
        pSampleFramework->DrawText(10, y, "R %1d:%1d", m_CloneId, m_StationIndex);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void ReceiveObject::UpdatePiaTestData()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(RECEIVE_IS_REGISTERED + m_StationIndex, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    pSampleFramework->SetProgramParam(RECEIVE_IS_CONNECTED + m_StationIndex, m_Clone.IsConnected() ? 1 : 0);

    pSampleFramework->SetProgramParam(RECEIVE_UNRELIABLE + m_StationIndex, m_ElementU1.IsValidValue() ? m_ElementU1.GetValue() : 0);
    pSampleFramework->SetProgramParam(RECEIVE_RELIABLE + m_StationIndex, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
    pSampleFramework->SetProgramParam(RECEIVE_EVENT + m_StationIndex, m_ElementE1.GetValue() != NULL ? *m_ElementE1.GetValue() : 0);
    pSampleFramework->SetProgramParam(RECEIVE_RECKONING + m_StationIndex, m_ElementC1.IsValidValue() ? m_ElementC1.GetValue() : 0);
}
#endif

AtomicObject::AtomicObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);
    m_Clone.RegisterElement(&m_ElementE1, 0x21);
    m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    m_Clone.RegisterElement(&m_ElementC1, 0x31);
    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void AtomicObject::Operate()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    Object::Operate();

    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_LEFT))
    {
        if (m_Clone.IsRegisteredWithProtocol())
        {
            if (m_Clone.GetLockStatus() == nn::pia::clone::AtomicSharingClone::LOCK_STATUS_UNLOCKED)
            {
                m_Clone.TryLock();
            }
            else
            {
                m_Clone.Unlock();
            }
        }
    }

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_PRINT("AtomicSharingClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_PRINT("AtomicSharingClone: Connected with not all\n");
        }
    }
#endif
}

void AtomicObject::Update()
{
    UpdateElement();
    if (m_ElementE1.IsDropEvent())
    {
        PIA_SAMPLE_PRINT("AtomicSharingClone: DropEvent\n");
    }
}

bool AtomicObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void AtomicObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterAtomicSharingClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("RegisterAtomicSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("RegisterAtomicSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void AtomicObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterAtomicSharingClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("UnregisterAtomicSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("UnregisterAtomicSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int AtomicObject::Print(int y, const nn::pia::sampleframework::Color& cColor)
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    pSampleFramework->DrawText(10, y, "");
    if (m_Clone.IsRegisteredWithProtocol())
    {
        const char* cpLockStatusText = "";
        switch (m_Clone.GetLockStatus())
        {
        case nn::pia::clone::AtomicSharingClone::LOCK_STATUS_UNLOCKED:
        {
            cpLockStatusText = "Unlocked";
        }
        break;
        case nn::pia::clone::AtomicSharingClone::LOCK_STATUS_TRY_LOCK:
        {
            cpLockStatusText = "TryLock";
        }
        break;
        case nn::pia::clone::AtomicSharingClone::LOCK_STATUS_LOCKED:
        {
            cpLockStatusText = "Locked";
        }
        break;
        }
#if PIA_TEST_CODE
        pSampleFramework->DrawText(10, y, "A %1d   R %s %s", m_Clone.GetId(), GetConnectionStatusText(), cpLockStatusText);
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MAX_STATIONS)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawText(10, y, "A %1d   R %c%c%c%c%c%c%c%c %s", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7], cpLockStatusText);
#endif
    }
    else
    {
        pSampleFramework->DrawText(10, y, "A %1d", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void AtomicObject::UpdatePiaTestData()
{
    if (m_CloneId != 0)
    {
        return;
    }

    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(ATOMIC_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    bit32 isConnectedBitmap = 0;
    for (int i = 0; i < MAX_STATIONS; ++i)
    {
        nn::pia::StationId stationId;
        nn::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(ATOMIC_CONNECTED, isConnectedBitmap);

    pSampleFramework->SetProgramParam(ATOMIC_LOCK_STATUS, m_Clone.GetLockStatus());

    pSampleFramework->SetProgramParam(ATOMIC_UNRELIABLE, m_ElementU1.IsValidValue() ? m_ElementU1.GetValue() : 0);
    pSampleFramework->SetProgramParam(ATOMIC_RELIABLE, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
    pSampleFramework->SetProgramParam(ATOMIC_EVENT, m_ElementE1.GetValue() != NULL ? *m_ElementE1.GetValue() : 0);
    pSampleFramework->SetProgramParam(ATOMIC_RECKONING, m_ElementC1.IsValidValue() ? m_ElementC1.GetValue() : 0);
}

void AtomicObject::SetParamSendUnreliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_UNRELIABLE_SEND, v);
}

void AtomicObject::SetParamSendReliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_RELIABLE_SEND, v);
}

void AtomicObject::SetParamSendEvent(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_EVENT_SEND, v);
}

void AtomicObject::SetParamSendReckoning(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(ATOMIC_RECKONING_SEND, v);
}
#endif


SequentialObject::SequentialObject()
{
    m_Clone.RegisterElement(&m_ElementU1, 0x01);
    m_Clone.RegisterElement(&m_ElementU2, 0x02);
    m_Clone.RegisterElement(&m_ElementU3, 0x03);
    m_Clone.RegisterElement(&m_ElementR1, 0x11);

    // EventCloneElement and ReckoningCloneElement cannot be used with SequentialSharingClone.
    //m_Clone.RegisterElement(&m_ElementE1, 0x21);
    //m_ElementC1.SetReckoningStrategy(&s_ReckoningStrategy);
    //m_Clone.RegisterElement(&m_ElementC1, 0x31);

    m_Clone.RegisterElement(&m_ElementL1, 0x41);

    m_Clone.SetProfiler(&m_SendProfiler, &m_ReceiveProfiler);
    m_ElementR1.SetProfiler(&m_SendProfilerForElement, &m_ReceiveProfilerForElement);
}

void SequentialObject::Operate()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    NN_PIA_DUMMY_PARAM(pSampleFramework);

    Object::Operate();

#if PIA_TEST_CODE
#else
    if (pSampleFramework->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
    {
        if (m_Clone.IsConnectedWithAll())
        {
            PIA_SAMPLE_PRINT("SequentialSharingClone: Connected with all\n");
        }
        else
        {
            PIA_SAMPLE_PRINT("SequentialSharingClone: Connected with not all\n");
        }
    }
#endif
}

void SequentialObject::Update()
{
    UpdateElement();
}

bool SequentialObject::Cleanup()
{
    return CleanupCore(&m_Clone);
}

void SequentialObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterSequentialSharingClone(&m_Clone, m_CloneId);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("RegisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("RegisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

void SequentialObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterSequentialSharingClone(&m_Clone);
    if (result.IsSuccess())
    {
        PIA_SAMPLE_PRINT("UnregisterSequentialSharingClone Success (cloneId:%d)\n", m_CloneId);
    }
    else
    {
        PIA_SAMPLE_PRINT("UnregisterSequentialSharingClone Failure (cloneId:%d)\n", m_CloneId);
        PIA_SAMPLE_ASSERT(false);
    }
}

int SequentialObject::Print(int y, const nn::pia::sampleframework::Color& cColor)
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    pSampleFramework->SetFontColor(cColor);
    pSampleFramework->DrawText(10, y, "");
    if (m_Clone.IsRegisteredWithProtocol())
    {
#if PIA_TEST_CODE
        pSampleFramework->DrawText(10, y, "Q %1d   R %s", m_Clone.GetId(), GetConnectionStatusText());
#else
        char isConnected[8];
        for (int i = 0; i < 8; ++i)
        {
            if (i < MAX_STATIONS)
            {
                isConnected[i] = m_Clone.IsConnected(g_StationIdList[i]) ? '1' : '0';
            }
            else
            {
                isConnected[i] = '0';
            }
        }
        pSampleFramework->DrawText(10, y, "Q %1d   R %c%c%c%c%c%c%c%c", m_Clone.GetId(), isConnected[0], isConnected[1], isConnected[2], isConnected[3], isConnected[4], isConnected[5], isConnected[6], isConnected[7]);
#endif
    }
    else
    {
        pSampleFramework->DrawText(10, y, "Q %1d", m_CloneId);
    }
    y += pSampleFramework->GetFontHeight() - 1;
    PrintValue(y);
    y += pSampleFramework->GetFontHeight();
    return y;
}

#if PIA_TEST_CODE
void SequentialObject::UpdatePiaTestData()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    pSampleFramework->SetProgramParam(SEQUENTIAL_IS_REGISTERED, m_Clone.IsRegisteredWithProtocol() ? 1 : 0);

    bit32 isConnectedBitmap = 0;
    for (int i = 0; i < MAX_STATIONS; ++i)
    {
        nn::pia::StationId stationId;
        nn::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationId(&stationId, static_cast<nn::pia::StationIndex>(i));
        if (result.IsSuccess() && m_Clone.IsConnected(stationId))
        {
            isConnectedBitmap |= (0x1 << i);
        }
    }
    pSampleFramework->SetProgramParam(SEQUENTIAL_CONNECTED, isConnectedBitmap);

    pSampleFramework->SetProgramParam(SEQUENTIAL_UNRELIABLE, m_ElementU1.IsValidValue() ? m_ElementU1.GetValue() : 0);
    pSampleFramework->SetProgramParam(SEQUENTIAL_RELIABLE, m_ElementR1.IsValidValue() ? m_ElementR1.GetValue() : 0);
}

void SequentialObject::SetParamSendUnreliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEQUENTIAL_UNRELIABLE_SEND, v);
}

void SequentialObject::SetParamSendReliable(u32 v)
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramParam(SEQUENTIAL_RELIABLE_SEND, v);
}
#endif
