﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ReckoningStrategy.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "ReckoningStrategy.h"

bool ReckoningStrategy::Estimate(Value* pValue, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    if (accessor.GetNum() <= 0)
    {
        return false;
    }
    if (clock > accessor.GetClock(0))
    {
        *pValue = accessor.GetSample(0);
        return true;
    }
    if (clock < accessor.GetClock(accessor.GetNum() - 1))
    {
        *pValue = accessor.GetSample(accessor.GetNum() - 1);
        return true;
    }

    f32 tt = static_cast<f32>(static_cast<s32>(clock - cpWork->m_BaseClock));
    *pValue = (cpWork->m_ParamA * tt + cpWork->m_ParamB) * tt + cpWork->m_ParamC;
    return true;
}

bool ReckoningStrategy::CheckSample(bool* pIsReliable, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    NN_PIA_DUMMY_PARAM(cpSetValueArg);

    if (accessor.GetNum() <= 0)
    {
        *pIsReliable = true;
        return true;
    }

    f32 estimatedValue;
    bool r = Estimate(&estimatedValue, accessor, clock, cpWork);
    if (r == false)
    {
        return true;
    }

    f32 diff = value - estimatedValue;
    return (diff <= -5.0f || diff >= 5.0f);
}

void ReckoningStrategy::MakeSample(Sample* pSample, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const
{
    NN_PIA_DUMMY_PARAM(cpSetValueArg);
    NN_PIA_DUMMY_PARAM(accessor);
    NN_PIA_DUMMY_PARAM(clock);
    NN_PIA_DUMMY_PARAM(cpWork);

    *pSample = value;
}

void ReckoningStrategy::OnUpdateSample(const Accessor& accessor, int index, Work* pWork) const
{
    NN_PIA_DUMMY_PARAM(index);

    if (accessor.GetNum() <= 0)
    {
        pWork->m_ParamA = 0.0f;
        pWork->m_ParamB = 0.0f;
        pWork->m_ParamC = 0.0f;
        pWork->m_BaseClock = 0;
        return;
    }

    f32 v0 = accessor.GetSample(0);
    nn::pia::clone::ClockValue c0 = accessor.GetClock(0);

    pWork->m_ParamC = v0;
    pWork->m_BaseClock = c0;

    if (accessor.GetNum() >= 3)
    {
        nn::pia::clone::ClockValue c1 = accessor.GetClock(1);
        nn::pia::clone::ClockValue c2 = accessor.GetClock(2);

        PIA_SAMPLE_ASSERT(c0 != c1 && c1 != c2);
        {
            f32 t1 = static_cast<f32>(static_cast<s32>(c1 - pWork->m_BaseClock));
            f32 t2 = static_cast<f32>(static_cast<s32>(c2 - pWork->m_BaseClock));
            f32 v1 = accessor.GetSample(1) - pWork->m_ParamC;
            f32 v2 = accessor.GetSample(2) - pWork->m_ParamC;

            pWork->m_ParamA = (t2 * v1 - t1 * v2) / (t1 * t2 * (t1 - t2));
            pWork->m_ParamB = v1 / t1 - pWork->m_ParamA * t1;
            return;
        }
    }

    pWork->m_ParamA = 0.0f;

    if (accessor.GetNum() >= 2)
    {
        nn::pia::clone::ClockValue c1 = accessor.GetClock(1);

        PIA_SAMPLE_ASSERT(c0 != c1);
        {
            f32 t1 = static_cast<f32>(static_cast<s32>(c1 - pWork->m_BaseClock));
            f32 v1 = accessor.GetSample(1) - pWork->m_ParamC;

            pWork->m_ParamB = v1 / t1;
            return;
        }
    }

    pWork->m_ParamB = 0.0f;
}
