﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ReckoningStrategy.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

// Define an algorithm for estimating an f32 value here.
// This indicates how to use ReckoningCloneElement, not how to indicate an appropriate estimation method.
class ReckoningStrategy
{
public:
    typedef f32 Value;
    typedef f32 Sample;
    typedef void SetValueArg;

    struct Work
    {
        f32 m_ParamA;
        f32 m_ParamB;
        f32 m_ParamC;
        nn::pia::clone::ClockValue m_BaseClock;
    };

    static const size_t BUFFER_SIZE = 3;

    typedef nn::pia::reckoning::ReckoningCloneElementBase::SampleAccessor<Sample> Accessor;

    bool Estimate(Value* pValue, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;
    bool CheckSample(bool* pIsReliable, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;
    void MakeSample(Sample* pSample, const Value& value, const SetValueArg* cpSetValueArg, const Accessor& accessor, nn::pia::clone::ClockValue clock, const Work* cpWork) const;
    void OnUpdateSample(const Accessor& accessor, int index, Work* pWork) const;
};
