﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleConfig.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleLib/SampleLib.h"
#include <cstdio>

#if NN_PIA_CTR
#define USE_PIA_LOCAL 1
#else
#define USE_PIA_LOCAL 0
#endif

#ifdef PIA_TEST_BUILD
// Code for automatic verification. Ignore this code as a sample.
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#if USE_PIA_LOCAL
#include <pia/local.h>
#include "Shared_Local_Setup.h"
#endif

#include "Shared_Basic_Setup.h"
#include "Shared_Common_Setup.h"
#include "Shared_Inet_Setup.h"
#include "Shared_Nex_Setup.h"
#include "Shared_Reckoning_Setup.h"
#include "Shared_Session_Setup.h"
#include "Shared_Transport_Setup.h"
#include "Shared_Clone_Setup.h"

#define PROGRAM_TITLE "CloneProtocol"

#if NN_PIA_WIN
#define snprintf(...) _snprintf(__VA_ARGS__)
#endif

#if PIA_TEST_CODE
#if NN_PIA_CTR
const size_t MAX_STATIONS = 12;
#else
const size_t MAX_STATIONS = 16;
#endif
#else
const size_t MAX_STATIONS = 8;  // Defines the maximum number of stations.
#endif

const bool ENABLE_HOST_MIGRATION = true;  //!< Enables/disables the host migration feature.

extern nn::pia::StationId g_StationIdList[MAX_STATIONS];
nn::pia::StationIndex ConvertStationIdToStationIndex(nn::pia::StationId stationId);

#if PIA_TEST_CODE
const u32 MAX_SILENT_TIME = 45000;

// In the order in which stored in the array used by PiaTest.
enum ParamKey
{
    MAX_STATION_NUM = 0x00,
    STATION_INDEX,
    SESSION_PROCESSING,

    CLONE_PROTOCOL_STATE = 0x10,
    CLOCK_TYPE,
    ENABLE_EVENT_SEND_DELAY,
    CLOCK_VALUE,

    CURSOR_POS = 0x20,

    SEND_UNRELIABLE_SEND = 0x30,
    SEND_RELIABLE_SEND,
    SEND_EVENT_SEND,
    SEND_RECKONING_SEND,
    SEND_IS_REGISTERED,
    SEND_CONNECTED,

    ATOMIC_UNRELIABLE = 0x40,
    ATOMIC_RELIABLE,
    ATOMIC_EVENT,
    ATOMIC_RECKONING,
    ATOMIC_UNRELIABLE_SEND,
    ATOMIC_RELIABLE_SEND,
    ATOMIC_EVENT_SEND,
    ATOMIC_RECKONING_SEND,
    ATOMIC_IS_REGISTERED,
    ATOMIC_CONNECTED,
    ATOMIC_LOCK_STATUS,

    SEQUENTIAL_UNRELIABLE = 0x50,
    SEQUENTIAL_RELIABLE,
    SEQUENTIAL_UNRELIABLE_SEND,
    SEQUENTIAL_RELIABLE_SEND,
    SEQUENTIAL_IS_REGISTERED,
    SEQUENTIAL_CONNECTED,

    RECEIVE_UNRELIABLE = 0x60,
    RECEIVE_RELIABLE = 0x70,
    RECEIVE_EVENT = 0x80,
    RECEIVE_RECKONING = 0x90,
    RECEIVE_IS_REGISTERED = 0xa0,
    RECEIVE_IS_CONNECTED = 0xb0
};

extern bool g_IsEnableEventSendDelay;

#endif
