﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Transport_Setup.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>
#include <pia/transport.h>

typedef nn::Result (*InitializeAdditionalProtocolFunction)();
typedef void (*FinalizeAdditionalProtocolFunction)();

nn::Result Transport_InitializePiaTransport(const nn::pia::transport::Transport::Setting& setting);

nn::Result Transport_InitializePiaTransport(
    const nn::pia::transport::Transport::Setting& setting,
    u16 unreliableProtocolPort,
    u32* pUnreliableProtocolHandle,  //n1769: When <tt>NULL</tt> is passed, the <tt>UnreliableProtocol</tt> initialization process is not finished.
    u16 reliableProtocolPort,
    u32* pReliableProtocolHandle,  //n1769: When <tt>NULL</tt> is passed, the <tt>ReliableProtocol</tt> initialization process is not finished.
    size_t reliableSendBufNum = 64,
    size_t reliableRecvBufNum = 64,
    InitializeAdditionalProtocolFunction pAdditionalFunction = NULL,
    size_t unreliableRecvBufNum = 32);

nn::Result Transport_InitializePiaTransport(
    nn::pia::transport::NetworkFactory* pFactory,
    u16 unreliableProtocolPort,
    u32* pUnreliableProtocolHandle,  //n1769: When <tt>NULL</tt> is passed, the <tt>UnreliableProtocol</tt> initialization process is not finished.
    u16 reliableProtocolPort,
    u32* pReliableProtocolHandle,  //n1769: When <tt>NULL</tt> is passed, the <tt>ReliableProtocol</tt> initialization process is not finished.
    size_t reliableSendBufNum = 64,
    size_t reliableRecvBufNum = 64,
    InitializeAdditionalProtocolFunction pAdditionalFunction = NULL,
    size_t unreliableRecvBufNum = 32);

void Transport_FinalizePiaTransport();

void Transport_FinalizePiaTransport(
    u32 unreliableProtocolHandle,  //n1769: When <tt>0</tt> is passed, the <tt>UnreliableProtocol</tt> finalization process is not finished.
    u32 reliableProtocolHandle,    //n1769: When <tt>0</tt> is passed, the <tt>ReliableProtocol</tt> finalization process is not finished.
    FinalizeAdditionalProtocolFunction pAdditionalFunction = NULL);
