﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Object.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Object.h"
#include "Shared_Clone_Setup.h"

// Reverse the StructValue bytes so it can be serialized.
template <>
void nn::pia::clone::ReverseSerializePolicy<RectPosition>::Reverse(RectPosition* pTo, const RectPosition* cpFrom)
{
    ReverseValue(&pTo->x, &cpFrom->x);
    ReverseValue(&pTo->y, &cpFrom->y);
}

SendObject::SendObject()
{
    m_Clone.RegisterElement(&m_ElementIncrementNumber, 0x01);
    m_Clone.RegisterElement(&m_ElementRectPosition, 0x02);
    m_Clone.RegisterElement(&m_ElementStationNumber, 0x03);
    m_Clone.RegisterElement(&m_ElementIsStartJointSession, 0x04);
    m_Clone.RegisterElement(&m_ElementOperationType, 0x05);
}

void SendObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterSendClone(&m_Clone, m_CloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}

void SendObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterSendClone(&m_Clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}

ReceiveObject::ReceiveObject()
{
    m_Clone.RegisterElement(&m_ElementIncrementNumber, 0x01);
    m_Clone.RegisterElement(&m_ElementRectPosition, 0x02);
    m_Clone.RegisterElement(&m_ElementStationNumber, 0x03);
    m_Clone.RegisterElement(&m_ElementIsStartJointSession, 0x04);
    m_Clone.RegisterElement(&m_ElementOperationType, 0x05);
}

void ReceiveObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterReceiveClone(&m_Clone, m_StationIndex, m_CloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
}

void ReceiveObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterReceiveClone(&m_Clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
}


AtomicObject::AtomicObject()
{
    m_Clone.RegisterElement(&m_ElementIncrementNumber, 0x01);
    m_Clone.RegisterElement(&m_ElementRectPosition, 0x02);
}

void AtomicObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterAtomicSharingClone(&m_Clone, m_CloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}

void AtomicObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterAtomicSharingClone(&m_Clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}


SequentialObject::SequentialObject()
{
    m_Clone.RegisterElement(&m_ElementIncrementNumber, 0x01);
    m_Clone.RegisterElement(&m_ElementRectPosition, 0x02);
}

void SequentialObject::RegisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->RegisterSequentialSharingClone(&m_Clone, m_CloneId);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}

void SequentialObject::UnregisterClone()
{
    nn::Result result = Clone_GetCloneProtocol()->UnregisterSequentialSharingClone(&m_Clone);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
    }
    PIA_SAMPLE_ASSERT(result.IsSuccess());
    NN_PIA_DUMMY_PARAM_IN_RELEASE(result);
}
