﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Object.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

namespace
{
}

// The base class for the game object that corresponds to Clone.
class Object
{
public:
    virtual void RegisterClone()
    {
    }
    virtual void UnregisterClone()
    {
    }
    virtual nn::Result TryLock()
    {
        PIA_SAMPLE_ASSERT(false);
        PIA_RETURN_RESULT(nn::pia::ResultInvalidState);
    }
    virtual nn::Result Unlock()
    {
        PIA_SAMPLE_ASSERT(false);
        PIA_RETURN_RESULT(nn::pia::ResultInvalidState);
    }
    virtual nn::pia::clone::AtomicSharingClone::LockStatus GetLockStatus() const
    {
        PIA_SAMPLE_ASSERT(false);
        return nn::pia::clone::AtomicSharingClone::LOCK_STATUS_UNLOCKED;
    }
    nn::pia::clone::CloneBase::Id m_CloneId;
    nn::pia::clone::UnreliableCloneElement<u32> m_ElementIncrementNumber;
    nn::pia::clone::UnreliableCloneElement<u32> m_ElementStationNumber;
    nn::pia::clone::UnreliableCloneElement<RectPosition, nn::pia::clone::NetworkByteOrderSerializePolicy<RectPosition> > m_ElementRectPosition;
    nn::pia::clone::ReliableCloneElement<bool> m_ElementIsStartJointSession;
    nn::pia::clone::ReliableCloneElement<u32> m_ElementOperationType;
protected:
};


// A game object that uses SendClone.
class SendObject : public Object
{
public:
    SendObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::SendClone m_Clone;

protected:
};

// A game object that uses ReceiveClone.
class ReceiveObject : public Object
{
public:
    ReceiveObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::ReceiveClone m_Clone;
    nn::pia::StationIndex m_StationIndex;

protected:
};

// A game object that uses AtomicSharingClone.
class AtomicObject : public Object
{
public:
    AtomicObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::AtomicSharingClone m_Clone;
    virtual nn::Result TryLock()
    {
        return m_Clone.TryLock();
    }
    virtual nn::Result Unlock()
    {
        return m_Clone.Unlock();
    }
    virtual nn::pia::clone::AtomicSharingClone::LockStatus GetLockStatus() const
    {
        return m_Clone.GetLockStatus();
    }

protected:
};

// A game object that uses SequentialSharingClone.
class SequentialObject : public Object
{
public:
    SequentialObject();
    virtual void RegisterClone();
    virtual void UnregisterClone();
    nn::pia::clone::SequentialSharingClone m_Clone;

protected:
};
