﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ObjectManager.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "ObjectManager.h"
#include "Shared_Clone_Setup.h"

namespace
{
}

ObjectManager g_ObjectManager;

ObjectManager::ObjectManager()
{
    m_SendObject.m_CloneId = CLONE_ID_OFFSET_STATION + 0x0;

    for (int i = 0; i < SAMPLE_GAME_PLAYER_MAX; ++i)
    {
        m_ReceiveObject[i].m_CloneId = CLONE_ID_OFFSET_STATION + 0x0;
        m_ReceiveObject[i].m_StationIndex = static_cast<nn::pia::StationIndex>(i);
    }

    for (int i = 0; i < SAMPLE_TEAM_NUM_IN_GAME + 1; ++i)
    {
        m_AtomicObject[i].m_CloneId = CLONE_ID_OFFSET_COMMON_OBJECT + 0x10 * i + 0x0;
        m_SequentialObject[i].m_CloneId = CLONE_ID_OFFSET_COMMON_OBJECT + 0x10 * i + 0x1;
    }
}

void ObjectManager::RegisterAll()
{
    m_SendObject.RegisterClone();
    for (int i = 0; i < SAMPLE_GAME_PLAYER_MAX; ++i)
    {
        m_ReceiveObject[i].RegisterClone();
    }
    for (int i = 0; i < SAMPLE_TEAM_NUM_IN_GAME + 1; ++i)
    {
        m_AtomicObject[i].RegisterClone();
        m_SequentialObject[i].RegisterClone();
    }
}

void ObjectManager::UnregisterAll()
{
    m_SendObject.UnregisterClone();
    for (int i = 0; i < SAMPLE_GAME_PLAYER_MAX; ++i)
    {
        m_ReceiveObject[i].UnregisterClone();
    }
    for (int i = 0; i < SAMPLE_TEAM_NUM_IN_GAME + 1; ++i)
    {
        m_AtomicObject[i].UnregisterClone();
        m_SequentialObject[i].UnregisterClone();
    }
}

SendObject* ObjectManager::GetSendObject()
{
    return &m_SendObject;
}


ReceiveObject* ObjectManager::GetReceiveObject(nn::pia::StationId stationId)
{
    if (stationId == nn::pia::STATION_ID_INVALID)
    {
        return NULL;
    }

    nn::pia::StationIndex stationIndex;
    nn::Result result = nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&stationIndex, stationId);
    if (result.IsFailure())
    {
        return NULL;
    }
    PIA_SAMPLE_ASSERT(stationIndex < SAMPLE_GAME_PLAYER_MAX);
    return &m_ReceiveObject[stationIndex];
}

AtomicObject* ObjectManager::GetTeamAtomicObject(int teamIndex)
{
    if (teamIndex < 0 || teamIndex >= SAMPLE_TEAM_NUM_IN_GAME)
    {
        return NULL;
    }
    return &m_AtomicObject[teamIndex];
}

SequentialObject* ObjectManager::GetTeamSequentialObject(int teamIndex)
{
    if (teamIndex < 0 || teamIndex >= SAMPLE_TEAM_NUM_IN_GAME)
    {
        return NULL;
    }
    return &m_SequentialObject[teamIndex];
}

AtomicObject* ObjectManager::GetCommonAtomicObject()
{
    return &m_AtomicObject[SAMPLE_TEAM_NUM_IN_GAME];
}

SequentialObject* ObjectManager::GetCommonSequentialObject()
{
    return &m_SequentialObject[SAMPLE_TEAM_NUM_IN_GAME];
}
