﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleObjectStatus.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"
#include "Shared_Clone_Setup.h"
#include "Object.h"

enum SampleObjectStatusType
{
    OBJECT_STATUS_TYPE_NONE = 0,
    OBJECT_STATUS_TYPE_STATION,
    OBJECT_STATUS_TYPE_TEAM_ATOMIC,
    OBJECT_STATUS_TYPE_TEAM_SEQUENTIAL,
    OBJECT_STATUS_TYPE_COMMON_ATOMIC,
    OBJECT_STATUS_TYPE_COMMON_SEQUENTIAL
};

struct SampleObjectStatus
{
    nn::pia::StationId stationId;
    RectPosition pos;
    u32 latestIncrementNumber;
    u32 stationNumber;
    SampleObjectStatusType type;
    Object* pObject;
    int teamIndex;

    bool isStartJointSession;
    bool isUpdataStartJointSession;
    u32 operationType;

    void Clear()
    {
        stationId = nn::pia::STATION_ID_INVALID;
        pos.x = 0;
        pos.y = 0;
        latestIncrementNumber = 0;
        stationNumber = 0;
        type = OBJECT_STATUS_TYPE_NONE;
        pObject = NULL;
        teamIndex = -1;

        isStartJointSession = false;
        isUpdataStartJointSession = false;
        operationType = 0;
    }

    void Update()
    {
        if (Clone_GetCloneProtocol()->GetState() != nn::pia::clone::CloneProtocol::STATE_ACTIVE)
        {
            return;
        }

        if (type == OBJECT_STATUS_TYPE_STATION && stationId == nn::pia::STATION_ID_INVALID)
        {
            return;
        }
        if (type != OBJECT_STATUS_TYPE_STATION && teamIndex == -1)
        {
            return;
        }

        PIA_SAMPLE_ASSERT(pObject != NULL);
        if (pObject->m_ElementIncrementNumber.IsValidValue())
        {
            latestIncrementNumber = pObject->m_ElementIncrementNumber.GetValue();
        }
        if (pObject->m_ElementRectPosition.IsValidValue())
        {
            pos = pObject->m_ElementRectPosition.GetValue();
        }
        if (pObject->m_ElementStationNumber.IsValidValue())
        {
            stationNumber = pObject->m_ElementStationNumber.GetValue();
        }

        if (pObject->m_ElementIsStartJointSession.IsValidValue())
        {
            if(!isUpdataStartJointSession)
            {
                isStartJointSession = pObject->m_ElementIsStartJointSession.GetValue();
                isUpdataStartJointSession = false;
            }
        }
        if (pObject->m_ElementOperationType.IsValidValue())
        {
            operationType = pObject->m_ElementOperationType.GetValue();
        }    }


};
