﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     TeamConstitutionManager.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include "SampleConfig.h"

class TeamConstitutionManager
{
public:
    TeamConstitutionManager();

    struct TeamConstitution
    {
        TeamConstitution()
        {
            Clear();
        }

        void Clear()
        {
            for (int i = 0; i < SAMPLE_TEAM_NUM_IN_GAME; ++i)
            {
                sessionIdTable[i] = nn::pia::session::INVALID_SESSION_ID;
                for (int j = 0; j < SAMPLE_TEAM_MEMBER_MAX; ++j)
                {
                    stationIdTable[i][j] = nn::pia::STATION_ID_INVALID.GetValue();
                }
            }
        }

        void Trace() const
        {
            PIA_SAMPLE_LOG("TeamConstitution Trace\n");
            for (int i = 0; i < SAMPLE_TEAM_NUM_IN_GAME; ++i)
            {
                PIA_SAMPLE_LOG("stationIdTable[%d] : ", i);
                for (int j = 0; j < SAMPLE_TEAM_MEMBER_MAX; ++j)
                {
                    PIA_SAMPLE_LOG("[%d]%lld ", j, stationIdTable[i][j]);
                }
                PIA_SAMPLE_LOG("\n");
            }
        }

        u32 sessionIdTable[SAMPLE_TEAM_NUM_IN_GAME];
        u64 stationIdTable[SAMPLE_TEAM_NUM_IN_GAME][SAMPLE_TEAM_MEMBER_MAX];
    };

    void Setup();
    void Reset();
    void Update();

    bool JoinEvent(nn::pia::StationId stationId);
    void LeaveEvent(nn::pia::StationId stationId);
    bool SessionHostChangedEvent();

    nn::pia::StationId GetStationId(int teamIndex, int memberIndex) const;
    int GetTeamIndex(nn::pia::StationId stationId) const;
    int GetMemberIndex(nn::pia::StationId stationId) const;
    u32 GetTeamSessionId(int teamIndex) const;
    u32 GetTeamMemberNum(int teamIndex) const;
    u32 GetTeamNum() const;

private:
    bool AssignTeamMember(nn::pia::StationId stationId);
    bool CancelTeamMember(nn::pia::StationId stationId);

    nn::pia::clone::AtomicSharingClone m_Clone;
    nn::pia::clone::ReliableCloneElement<TeamConstitution, nn::pia::clone::NetworkByteOrderSerializePolicy<TeamConstitution> > m_ElementR;
    TeamConstitution m_TeamConstitution;
    bool m_IsNeedSend;
    nn::pia::clone::ClockValue m_LatestInfoClock;
};

extern TeamConstitutionManager g_TeamConstitutionManager;
