﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Clone_Setup.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "Shared_Clone_Setup.h"

namespace
{
u32 s_hClone = 0;
}

void Clone_InitializePiaClone()
{
    nn::pia::clone::CloneProtocol::Setting setting;
    setting.clockType = nn::pia::clone::CloneProtocol::Setting::CLOCK_TYPE_RTC;
    setting.clockPerSec = 1000.0f;
    setting.clockPriorityJudgeMethod = nn::pia::clone::CloneProtocol::Setting::CLOCK_PRIORITY_JUDGE_METHOD_SIMPLE;
    Clone_InitializePiaClone(setting);
}

void Clone_InitializePiaClone(const nn::pia::clone::CloneProtocol::Setting& setting)
{
    nn::Result r;

    // Be sure to finish the initialization of the transport module beforehand.
    PIA_SAMPLE_ASSERT(nn::pia::transport::IsInitialized());

    r = nn::pia::clone::Initialize();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    r = nn::pia::clone::BeginSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    s_hClone = nn::pia::transport::Transport::GetInstance()->CreateProtocol<nn::pia::clone::CloneProtocol>();
    PIA_SAMPLE_ASSERT(s_hClone != 0);

    r = Clone_GetCloneProtocol()->Initialize(setting);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);

    r = nn::pia::clone::EndSetup();
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(r);
}

void Clone_FinalizePiaClone()
{
    Clone_GetCloneProtocol()->Finalize();

    nn::pia::transport::Transport::GetInstance()->DestroyProtocol(s_hClone);

    nn::pia::clone::Finalize();
}

nn::pia::clone::CloneProtocol* Clone_GetCloneProtocol()
{
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::clone::CloneProtocol>(s_hClone);
}
