﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Nex_Setup.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/settings.h>
#include <pia/platform.h>

#if NN_PIA_CTR
#pragma diag_suppress 1300
#endif

#include <nex.h>

#include <OnlineCore/src/version/nex_version.h>
#include <OnlineCore/src/Core/CallContext.h>
#include <OnlineCore/src/Platform/Core/PlatformDecl.h>
#include <OnlineCore/src/Platform/Core/GlobalVariables.h>
#include <OnlineCore/src/Platform/Core/SystemErrorResult.h>
#include <OnlineCore/src/Platform/Core/qList.h>
#include <OnlineCore/src/Platform/Core/Type.h>
#include <OnlineCore/src/Platform/Core/SystemClock.h>
#include <OnlineCore/src/Platform/Core/UpdatableTimeProvider.h>
#include <OnlineCore/src/Platform/Core/ConsoleIO.h>
#include <OnlineCore/src/Platform/Stack/Core/InetAddress.h>
#include <OnlineCore/src/Transport/UDP/Packet.h>
#include <OnlineCore/src/Transport/PRUDP/PRUDPEndPoint.h>
#include <OnlineCore/src/Transport/Interface/RootTransport.h>
#include <OnlineCore/src/Transport/Interface/SocketTransport.h>
#include <OnlineCore/src/Transport/Interface/TransportError.h>
#include <OnlineCore/src/Transport/Interface/TransportAdapter.h>
#include <OnlineCore/src/Transport/Interface/StationURL.h>
#include <NetZ/src/Core/DOCore/Station.h>
#include <NetZ/src/Core/DOCore/Session.h>
#include <NetZ/src/Core/DOCore/SessionDescription.h>
#include <NetZ/src/Core/DOCore/DOHandle.h>
#include <NetZ/src/Core/DOCore/DirectStream.h>
#include <NetZ/src/Core/DOCore/DirectTransport.h>
#include <NetZ/src/Extensions/SessionSelection/ConsoleSessionEnumerator.h>
#include <NetZ/HostMigration.h>
#include <RendezVous/Services/NATTraversal/src/Client/NATTraversalClient.h>

#if NN_PIA_CTR
#pragma diag_warning 1300
#endif

#define SAMPLE_SESSION_ROLE_HOST nn::nex::SESSION_ROLE_MASTER
#define SAMPLE_SESSION_ROLE_DUPLICA nn::nex::SESSION_ROLE_DUPLICA
#define SAMPLE_SESSION_ROLE_UNDEFINED nn::nex::SESSION_ROLE_UNDEFINED

// NEX version macro.
#define NEX_VERSION_NUMBER(major, minor, micro) (((major) << 16) | ((minor) << 8) | ((micro) << 0))
#define NEX_CURRENT_VERSION_NUMBER NEX_VERSION_NUMBER(NEX_VERSION_MAJOR, NEX_VERSION_MINOR, NEX_VERSION_MICRO)

#include "SampleLib/SampleLib.h"

// <tt>nn::Result</tt> definitions for broadly categorizing errors that are not expressed by <tt>nn::Result</tt> in the sample.
enum SAMPLE_RESULT_NEX_DESCRIPTION
{
    NEX_INVALID_STATE = 10,           // The NEX API was called at an inappropriate time.
    FAILED_AUTO_MATCHMAKE_SESSION,    // Random matchmaking failed.
    FAILED_CREATE_MATCHMAKE_SESSION,  // The process for creating a matchmaking session failed.
    FAILED_JOIN_MATCHMAKE_SESSION,    // The process for joining a matchmaking session failed.
    FAILED_OPEN_PARTICIPATION,        // OpenParticipation failed.
    FAILED_GET_HOST_CONNECTION_INFO,  // GetHostStationConnectionInfo failed.
    NEX_INTERNAL_ERROR                // NEX error.
};

PIA_DEFINE_RESULT(
    SampleNexInvalidState,
    nn::Result::LEVEL_USAGE,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    NEX_INVALID_STATE);

PIA_DEFINE_RESULT(
    SampleNexInternalError,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    NEX_INTERNAL_ERROR);

PIA_DEFINE_RESULT(
    SampleFailedAutoMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_AUTO_MATCHMAKE_SESSION);

PIA_DEFINE_RESULT(
    SampleFailedCreateMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_CREATE_MATCHMAKE_SESSION);

PIA_DEFINE_RESULT(
    SampleFailedJoinMatchmakeSession,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_JOIN_MATCHMAKE_SESSION);

PIA_DEFINE_RESULT(
    SampleFailedOpenParticipation,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_OPEN_PARTICIPATION);

PIA_DEFINE_RESULT(
    SampleFailedGetHostStationConnectionInfo,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_GET_HOST_CONNECTION_INFO);

//-----------------------------------------------------------------------------------------
// Declarations of External Functions
//-----------------------------------------------------------------------------------------

nn::Result Nex_InitializeNex(
    size_t memorySize,
    size_t emergencyMemorySize,
    nn::nex::Core::ThreadMode threadMode);
void Nex_FinalizeNex();

// Displays the protocol call context error.
void Nex_ReportProtocolCallContext(nn::nex::ProtocolCallContext& context, const char* pHeaderText = NULL);

// Gets an <tt>NgsFacade</tt> instance.
nn::nex::NgsFacade* Nex_GetNgsFacadeInstance();

// Gets whether connected to a game server.
bool Nex_IsNgsConnected();

#if !NN_PIA_WIN
nn::nex::MatchmakeExtensionClient* Nex_GetMatchmakeExtensionClientInstance();
#endif

void Nex_InitializeErrorHandling();
void Nex_FinalizeErrorHandling();
bool Nex_CheckInternalError();
bool Nex_CheckLastAPIError();

u32 Nex_GetInternalErrorCode();
u32 Nex_GetLastAPIErrorCode();

size_t Nex_GetPeakMemoryUsage();
