﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SceneFriendsMatchmake.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/SampleLib.h"
#include "Shared_Inet_Setup.h"
#include "Shared_Session_Setup.h"
#include <pia/common/common_FixedObjList.h>

namespace
{
typedef nn::pia::common::FixedObjList<FriendId, FRIENDS_LIST_MAX> OnlineFriendsList;
OnlineFriendsList s_OnlineFriendsList;
}

bool IsJoinable(FriendPresence* pPresence, u64 mask)
{
#if NN_PIA_CTR
    return pPresence->IsJoinable(mask);
#elif NN_PIA_CAFE
    return nn::fp::IsJoinable(pPresence, mask);
#endif
}

FriendPresence GetFriendPresence(FriendId id)
{
    FriendPresence friendPresence;
#if NN_PIA_CTR
    nn::friends::GetFriendPresence(&friendPresence, &id);
#elif NN_PIA_CAFE
    nn::fp::GetFriendPresence(&friendPresence, &id, 1);
#endif
    return friendPresence;
}

u32 GetFriendPrincipalId(FriendId id)
{
#if NN_PIA_CTR
    return id.principalId;
#elif NN_PIA_CAFE
    return id;
#endif
}

void UpdateFriendPresence()
{
    size_t friendsNum = 0;
    FriendId friendIdList[FRIENDS_LIST_MAX];
    FriendPresence friendPresenceList[FRIENDS_LIST_MAX];

#if NN_PIA_CTR
    nn::friends::GetFriendKeyList(friendIdList, &friendsNum, 0, FRIENDS_LIST_MAX);
    nn::friends::GetFriendPresence(friendPresenceList, friendIdList, friendsNum);
#elif NN_PIA_CAFE
    nn::fp::GetFriendList(friendIdList, &friendsNum, 0, FRIENDS_LIST_MAX);
    nn::fp::GetFriendPresence(friendPresenceList, friendIdList, friendsNum);
#endif
    s_OnlineFriendsList.Clear();
    for (size_t i = 0; i < friendsNum; i++)
    {
        PIA_SAMPLE_LOG("Presence[%d]\n", i);
        PIA_SAMPLE_LOG("  IsOnline              : %d\n", friendPresenceList[i].isOnline);
        PIA_SAMPLE_LOG("  GameId                : %08x\n", friendPresenceList[i].gameMode.joinGameId);
        PIA_SAMPLE_LOG("  GameMode              : %u\n", friendPresenceList[i].gameMode.joinGameMode);
        PIA_SAMPLE_LOG("  GroupId(GatheringId)  : %u\n", friendPresenceList[i].gameMode.joinGroupId);
        PIA_SAMPLE_LOG("  MatchmakeSystemType   : %u\n", friendPresenceList[i].gameMode.matchmakeSystemType);
        PIA_SAMPLE_LOG("  Owner                 : %u\n", friendPresenceList[i].gameMode.ownerPrincipalId);
        PIA_SAMPLE_LOG("  JoinAvailabilityFlag  : %08x\n", friendPresenceList[i].gameMode.joinAvailabilityFlag);
#if NN_PIA_CTR
        PIA_SAMPLE_LOG("  IsInviteMe            : %d\n", friendPresenceList[i].isInvitingMe);
#endif
        PIA_SAMPLE_LOG("  IsJoinable            : %d\n", IsJoinable(&friendPresenceList[i], FRIENDS_MATCHMAKE_GAME_MODE_MASK));

        if (friendPresenceList[i].isOnline && IsJoinable(&friendPresenceList[i], FRIENDS_MATCHMAKE_GAME_MODE_MASK))
        {
            FriendId* pNode = s_OnlineFriendsList.TryBirthBack();
            if (pNode)
            {
                *pNode = friendIdList[i];
            }
        }
    }
}

SampleSceneNo SceneFriendsMatchmake()
{
    nn::Result result;
    u32 target = 0;
    MatchmakeCommand command = MC_NONE;
    SampleSceneNo nextSceneNo = SCENE_COMMUNICATE;

    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SCENE_FRIENDS_MATCHMAKE,
        false,
        false,
        0);

    nn::pia::inet::NexCreateSessionSetting createSessionSetting;
    nn::pia::inet::NexJoinSessionSetting joinSessionSetting;

    // Sets the game mode in the range of 0-63 in order to link with Friend Presence.
    createSessionSetting.SetGameMode(FRIENDS_MATCHMAKE_GAME_MODE + GetIncrementalGameModeValueByBitRateCheckMode());
    createSessionSetting.SetSessionType(nn::pia::inet::SESSION_TYPE_FRIEND);
    createSessionSetting.SetMaxParticipants(SAMPLE_MAX_ENTRY);
    createSessionSetting.SetSessionUserPassword(SAMPLE_USER_PASSWORD, SAMPLE_USER_PASSWORD_LENGTH);
    createSessionSetting.SetAttribute(MATCHMAKE_ATTR_INDEX, g_IsEnableHostMigration);
    createSessionSetting.SetOpenParticipation(true);
    createSessionSetting.SetDescription(SAMPLE_DESCRIPTION_FRIENDS);

    // Starts from Friend Presence.
    if (g_IsFromFriendList)
    {
        g_IsFromFriendList = false;

        // Gets the matchmaking session information from Friend Presence.
        FriendPresence friendPresence = GetFriendPresence(g_FriendIdForJoinIn);
        if (IsJoinable(&friendPresence, FRIENDS_MATCHMAKE_GAME_MODE_MASK))
        {
            joinSessionSetting.SetSessionId(friendPresence.gameMode.joinGroupId);
            command = MC_JOIN;
        }
    }

    joinSessionSetting.SetSessionUserPassword(SAMPLE_USER_PASSWORD, SAMPLE_USER_PASSWORD_LENGTH);

    s_OnlineFriendsList.Clear();

    while (command == MC_NONE)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        // Up/Down: Select a session.
        if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_UP))
        {
            target = (FRIENDS_LIST_MAX + target - 1) % FRIENDS_LIST_MAX;
        }
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_DOWN))
        {
            target = (target + 1) % FRIENDS_LIST_MAX;
        }
        // B: Reset
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_B))
        {
            command = MC_BACK;
        }
        // Y: Updates Friend Presence.
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
        {
            UpdateFriendPresence();
        }
        // X: Create a session.
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_X))
        {
            command = MC_CREATE;
        }
        // A: Session joining.
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_A))
        {
            OnlineFriendsList::Iterator it = s_OnlineFriendsList.Begin();
            for (int i = 0; i < FRIENDS_LIST_MAX; i++)
            {
                if (it == s_OnlineFriendsList.End())
                {
                    break;
                }
                if (i == target)
                {
                    FriendPresence friendPresence = GetFriendPresence(*it);
                    joinSessionSetting.SetSessionId(friendPresence.gameMode.joinGroupId);
                    command = MC_JOIN;
                }
                ++it;
            }
        }

        // Lower screen.
        nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
        nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
        int line = 0;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[%s Demo] %d", PROGRAM_NAME, g_FrameNo);
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Scene: Select Friend's Session.");
        line += FONT_SIZE;
        line += FONT_SIZE;

        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[ Online Friend PrincipalID List ] Found:%01d ", s_OnlineFriendsList.Size());
        line += FONT_SIZE;
        OnlineFriendsList::Iterator it = s_OnlineFriendsList.Begin();
        for (int i = 0; i < FRIENDS_LIST_MAX; i++)
        {
            if (it != s_OnlineFriendsList.End())
            {
                if (i == target)
                {
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_RED);
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, " PID=%d", GetFriendPrincipalId(*it));
                    line += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);

                    FriendPresence friendPresence = GetFriendPresence(*it);
                    int y = LOWER_SCREEN_HEIGHT - (FONT_SIZE * 14);
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  GameId   : %08x\n", friendPresence.gameMode.joinGameId);
                    y += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  GameMode : %u\n", friendPresence.gameMode.joinGameMode);
                    y += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  GroupId  : %u\n", friendPresence.gameMode.joinGroupId);
                    y += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  Type     : %u\n", friendPresence.gameMode.matchmakeSystemType);
                    y += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  Owner    : %u\n", friendPresence.gameMode.ownerPrincipalId);
                    y += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(10, y, "  Flag     : %08x\n", friendPresence.gameMode.joinAvailabilityFlag);
                    y += FONT_SIZE;
                }
                else
                {
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, " PID=%d", GetFriendPrincipalId(*it));
                    line += FONT_SIZE;
                }
                ++it;
            }
            else
            {
                if (i == target)
                {
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_RED);
                }
                else
                {
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
                }
                nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  -----");
                nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
                line += FONT_SIZE;
            }
        }

        line = LOWER_SCREEN_HEIGHT - (FONT_SIZE * 5);
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[ Manual ]");
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  UP/DOWN: Select Friend's Session");
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  X: Create New Session for Friends");
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Y: Update Friend Presence");
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  A: Start Join-in Friend's Session");
        line += FONT_SIZE;
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  B: Back");
        line += FONT_SIZE;

        nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        // Upper screen.
        nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);
        nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

        ++g_FrameNo;
        Dispatch();

        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

        // Checks the connection status.
        result = nn::pia::session::Session::GetInstance()->CheckConnectionError();
        if (result.IsFailure())
        {
            s_OnlineFriendsList.Clear();
            SampleErrorType errorType = (result == nn::pia::ResultSessionConnectionIsLost()) ? SESSION_LOST_ERROR : CONNECTION_LOST_ERROR;
            return GetNextSceneByErrorResult(result, errorType);
        }
    }

    if (command == MC_CREATE)
    {
        SampleSessionAsyncStatus sampleSessionAsyncStatus = Session_CreateSession(&createSessionSetting, &result);
        if (sampleSessionAsyncStatus != SAMPLE_SESSION_ASYNC_SUCCESS)
        {
            PIA_SAMPLE_LOG_RESULT(result);
            nextSceneNo = GetNextSceneByErrorResult(result, SESSION_CREATE_ERROR);
        }
        else
        {
            // Updates the Friend Presence game mode information.
            nextSceneNo = SetGameMode(FRIENDS_MATCHMAKE_GAME_MODE + GetIncrementalGameModeValueByBitRateCheckMode(), nn::nex::MATCHMAKE_SYSTEM_TYPE_FRIEND, true, SAMPLE_DESCRIPTION_FRIENDS);
        }
    }
    else if (command == MC_JOIN)
    {
        SampleSessionAsyncStatus sampleSessionAsyncStatus = Session_JoinSession(&joinSessionSetting, &result);
        if (sampleSessionAsyncStatus != SAMPLE_SESSION_ASYNC_SUCCESS)
        {
            PIA_SAMPLE_LOG_RESULT(result);
            nextSceneNo = GetNextSceneByErrorResult(result, SESSION_JOIN_ERROR);
        }
        else
        {
            // Updates the Friend Presence game mode information.
            nextSceneNo = SetGameMode(FRIENDS_MATCHMAKE_GAME_MODE + GetIncrementalGameModeValueByBitRateCheckMode(), nn::nex::MATCHMAKE_SYSTEM_TYPE_FRIEND, true, SAMPLE_DESCRIPTION_FRIENDS);
        }
    }
    else if (command == MC_BACK)
    {
        nextSceneNo = SCENE_SELECT_MATCHMAKE_TYPE;
        Session_CleanupSession();
    }

    s_OnlineFriendsList.Clear();

    return nextSceneNo;
}
