﻿/*---------------------------------------------------------------------------*
  Project: Pia
  File: SceneMatchmake.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law. They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/SampleLib.h"

#include "Shared_Inet_Setup.h"
#include "Shared_Nex_Setup.h"
#include "Shared_Session_Setup.h"

namespace
{
    bit8 s_TokenBrowse[nn::pia::transport::Station::IDENTIFICATION_TOKEN_MAX_DATA_SIZE] = "Browse";
    bit8 s_TokenRandom[nn::pia::transport::Station::IDENTIFICATION_TOKEN_MAX_DATA_SIZE] = "Random";
    bit8 s_TokenFriend[nn::pia::transport::Station::IDENTIFICATION_TOKEN_MAX_DATA_SIZE] = "Friend";
};

SampleSceneNo SceneSelectMatchmakeType()
{
    nn::Result result;
    SampleSceneNo nextSceneNo = SCENE_SELECT_MATCHMAKE_TYPE;

    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SCENE_SELECT_MATCHMAKE_TYPE,
        false,
        false,
        0);

    // Automatically transition to the friend-joining Friends_Matchmake scene if starting from Friend Presence + HOME Menu.
    if (g_IsFromFriendList)
    {
        nextSceneNo = SCENE_FRIENDS_MATCHMAKE;
    }
    // If information for rejoining sessions is being saved, join the session via the Browse_Matchmake scene.
    else if (GetSessionId() != nn::pia::session::INVALID_SESSION_ID)
    {
        nextSceneNo = SCENE_BROWSE_MATCHMAKE;
    }
    // The only scene with local communication is Browse_Matchmake.
    else if (g_NetworkType == TYPE_LOCAL)
    {
        nextSceneNo = SCENE_BROWSE_MATCHMAKE;
        g_IsEnableHostMigration = true;
    }
    else
    {
        // Loop for selecting the matchmake type.
        while (nextSceneNo == SCENE_SELECT_MATCHMAKE_TYPE)
        {
            nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

            // Y Button:
            if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
            {
                nextSceneNo = SCENE_BROWSE_MATCHMAKE;
                g_IsEnableHostMigration = true;
            }
            // B Button.
            if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_B))
            {
                nextSceneNo = SCENE_RESET;
            }
            // A Button:
            if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_A))
            {
                nextSceneNo = SCENE_RANDOM_MATCHMAKE;
                g_IsEnableHostMigration = true;
            }
            // X Button:
            if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_X))
            {
#if ENABLE_SERVERLESS
// Friend matchmaking without a server is not supported.
#else
                nextSceneNo = SCENE_FRIENDS_MATCHMAKE;
                // In this sample, host migration is turned off for friend matchmaking.
                // With friend matchmaking, matchmaking is possible between the host and users that have friendships
                // with the host, so if the host changes, the users that can be matched will also change.
                g_IsEnableHostMigration = false;
#endif
            }

            // Lower screen.
            nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
            nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
            int line = 0;
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[%s Demo] %d", PROGRAM_NAME, g_FrameNo);
            line += FONT_SIZE;
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Scene: SelectMatchmake.");
            line += FONT_SIZE;
            line += FONT_SIZE;

            line = LOWER_SCREEN_HEIGHT - (FONT_SIZE * 6);
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[ Manual ]");
            line += FONT_SIZE;
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Y: Start BrowseMatchmake");
            line += FONT_SIZE;
#if ENABLE_SERVERLESS
#else
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  X: Start FriendsMatchmake");
            line += FONT_SIZE;
#endif
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  A: Start RandomMatchmake");
            line += FONT_SIZE;
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  B: Back");
            line += FONT_SIZE;

            nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

            // Upper screen.
            nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);
            nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

            nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
            ++g_FrameNo;

            Dispatch();

            // Checks the connection status.
            if (g_NetworkType == TYPE_INTERNET)
            {
                // AC
                if (!Inet_IsConnected())
                {
                    PIA_SAMPLE_LOG("Inet_IsConnected Disconnected \n");
                    SetCommunicationErrorType(CONNECTION_LOST_ERROR);
                    return SCENE_ERROR;
                }
            }
        }
    }
    if (nextSceneNo != SCENE_RESET)
    {
        if (g_NetworkType == TYPE_INTERNET)
        {
            PIA_SAMPLE_LOG("nn::pia::inet::NexFacade::UpdateNotificationDataAsync is called.\n");
            nn::nex::NotificationEvents::NotificationEvents eventType;
            if (nextSceneNo == SCENE_RANDOM_MATCHMAKE)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType1;
            }
            else if (nextSceneNo == SCENE_BROWSE_MATCHMAKE)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType2;
            }
            else if (nextSceneNo == SCENE_FRIENDS_MATCHMAKE)
            {
                eventType = nn::nex::NotificationEvents::GameNotificationType3;
            }
            u32 param1 = Nex_GetNgsFacadeInstance()->GetCredentials()->GetPrincipalID();
            u32 param2 = 0;
            const wchar_t strParam[] = L"MatchmakeType is decided";

            result = nn::pia::inet::NexFacade::GetInstance()->UpdateNotificationDataAsync(eventType, param1, param2, strParam, sizeof(strParam)/sizeof(wchar_t));
            if (result.IsFailure())
            {
                PIA_SAMPLE_LOG("Calling nn::pia::inet::NexFacade::UpdateNotificationDataAsync is failed.\n");
            }
            else
            {
                do {
                    nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

                    // Lower screen.
                    nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
                    int line = 0;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[%s Demo] %d", PROGRAM_NAME, g_FrameNo);
                    line += FONT_SIZE;
                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Scene: SelectMatchmake.");
                    line += FONT_SIZE;
                    line += FONT_SIZE;

                    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "Updating NotificationData...");
                    line += FONT_SIZE;

                    nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

                    // Upper screen.
                    nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);
                    nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

                    nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
                    ++g_FrameNo;

                    Dispatch();

                    if (!Inet_IsConnected())
                    {
                        PIA_SAMPLE_LOG("Inet_IsConnected Disconnected \n");
                        SetCommunicationErrorType(CONNECTION_LOST_ERROR);
                        return SCENE_ERROR;
                    }
                } while (!nn::pia::inet::NexFacade::GetInstance()->IsCompletedUpdateNotificationData());

                if (nn::pia::inet::NexFacade::GetInstance()->GetUpdateNotificationDataResult().IsSuccess())
                {
                    PIA_SAMPLE_LOG("nn::pia::inet::NexFacade::UpdateNotificationDataAsync process is success.\n");
                }
                else
                {
                    PIA_SAMPLE_LOG("nn::pia::inet::NexFacade::UpdateNotificationDataAsync process is failed.\n");
                }
            }
        }

        nn::pia::transport::Station::IdentificationToken token;
        bit8* pTokenData;
        switch (nextSceneNo)
        {
        case SCENE_BROWSE_MATCHMAKE:
        {
            pTokenData = s_TokenBrowse;
        }
        break;
        case SCENE_RANDOM_MATCHMAKE:
        {
            pTokenData = s_TokenRandom;
        }
        break;
        case SCENE_FRIENDS_MATCHMAKE:
        {
            pTokenData = s_TokenFriend;
        }
        break;
        }
        ::std::memcpy(token.data, pTokenData, nn::pia::transport::Station::IDENTIFICATION_TOKEN_MAX_DATA_SIZE);

        nn::pia::session::Session::StartupSetting startupSetting;
        startupSetting.bUsingHostMigration = g_IsEnableHostMigration;
        startupSetting.cryptoMode = nn::pia::common::CryptoSetting::MODE_AES_128;
        startupSetting.maxSilenceTime = 10000;
        startupSetting.keepAliveSendingInterval = 500;
        if(g_IsEnableBitRateCheck)
        {
            startupSetting.uplinkBitRateLowerLimit = 200 * 1000;
            startupSetting.bitRateCheckPacketSize = 1024;
        }
        startupSetting.pToken = &token;

        result = Session_StartupSession(startupSetting);
        if (result.IsFailure())
        {
            return GetNextSceneByErrorResult(result, SESSION_STARTUP_ERROR);
        }

        // Set the pulse interval for the synchronization clock.
        // The default value is 2000 ms. This generally does not need to be changed, but we set it to 1000 ms here.
        nn::pia::session::Session::GetInstance()->SetSyncClockPulseInterval(1000);
        PIA_SAMPLE_ASSERT(nn::pia::session::Session::GetInstance()->GetSyncClockPulseInterval() == 1000);
    }
    return nextSceneNo;
}
