﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ScenePrepareLocal.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleConfig.h"


#include <pia/common.h>
#include <pia/common/common_Event.h>
#include <pia/transport/transport_StationConnectionInfo.h>

#include "SampleLib/Defines.h"
#include "SampleLib/ErrorHandling.h"
#include "SampleLib/SampleLib.h"
#include "SampleLib/Thread.h"

#include "Shared_Transport_Setup.h"
#include "Shared_Session_Setup.h"

#if NN_PIA_CTR
#include <pia/local.h>
#include "Shared_Local_Setup.h"
#endif

/*!

    @brief  CPP file with processing specific to local communication.

*/

namespace
{
#if NN_PIA_CTR
const char* stepStrings[] = {
    "Initialize local module",
    "Completed"
};  // 1 step.
#endif

enum InitializeStep
{
    STEP_INITIALIZE_PIA_MODULES = 0,
    STEP_MAX
};
}

// Displays the progress of <tt>InitializePiaLocal</tt> on the screen.
SampleSceneNo ScenePrepareLocal()
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SCENE_PREPARE_LOCAL,
        false,
        false,
        0,
        0);

#if NN_PIA_CTR
    u32 step = 0;
    nn::Result result;

    while (step < STEP_MAX)
    {
        switch (step)
        {
        case STEP_INITIALIZE_PIA_MODULES:
            // Initializes PiaLocal.
            result = Local_InitializePiaLocal(SAMPLE_TITLE_ID, SAMPLE_SUB_ID, ENABLE_SAMPLE_HOST_MIGRATION);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            // Initializes PiaTransport.
            {
                u32 unreliableProtocolHandle;
                nn::pia::transport::Transport::Setting transportSetting;
                nn::pia::local::UdsNetworkFactory localUdsFactory;
                transportSetting.pFactory = &localUdsFactory;
                transportSetting.maxStationNum = SAMPLE_MAX_ENTRY;
                transportSetting.maxSendThreadBufferNum = 16;
                transportSetting.maxReceiveThreadBufferNum = 32;
                result = Transport_InitializePiaTransport(
                    transportSetting,
                    0,
                    &unreliableProtocolHandle,
                    0,
                    NULL);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                // Sets the UnreliableProtocolHandle to use for communication.
                SetUnreliableProtocolHandle(unreliableProtocolHandle);
            }
            // Initializes PiaSession.
            {
                nn::pia::session::Session::Setting sessionSetting;
                nn::pia::local::UdsNetworkFactory localUdsFactory;
                sessionSetting.pNetworkFactory = &localUdsFactory;
                sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
                sessionSetting.browsedSessionInfoListNum = SESSION_LIST_MAX;
                result = Session_InitializePiaSession(sessionSetting);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
                // Sets the event callback.
                result = nn::pia::session::Session::GetInstance()->RegisterSessionEventCallback(SampleSessionCallback);
                PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
            }
            ++step;
            break;
        }
    }


    nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

    // Lower screen.
    nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(false);
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
    int line = 0;
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[%s Demo]", PROGRAM_NAME);
    line += FONT_SIZE;
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "  Scene: Local Initializing process.");
    line += FONT_SIZE;
    line += FONT_SIZE;

    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "   START");
    line += FONT_SIZE;
    for (u32 i = 0; i < STEP_MAX; i++)
    {
        if (i == step)
        {
            nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "     |");
            line += FONT_SIZE;
            nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_RED);
        }
        else
        {
            line += FONT_SIZE;
        }
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "   %s", stepStrings[step]);
        line += FONT_SIZE;
    }
    line += FONT_SIZE;
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "   END");
    line += FONT_SIZE;
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFontColor(nn::pia::sampleframework::COLOR_WHITE);
    nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

    // Upper screen.
    nn::pia::sampleframework::SampleFramework::GetInstance()->ChangeRenderTarget(true);

    line = UPPER_SCREEN_HEIGHT - FONT_SIZE;
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(0, line, "[ Version ] %s", GetPiaVersionString());
    line += FONT_SIZE;

    nn::pia::sampleframework::SampleFramework::GetInstance()->SwapBuffers();

    nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

    return SCENE_SELECT_MATCHMAKE_TYPE;
#else
    return SCENE_START;
#endif
}
