﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SceneRandomMatchmake.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "SampleLib/SampleLib.h"
#include "Shared_Inet_Setup.h"
#include "Shared_Session_Setup.h"

namespace
{
#if PIA_TEST_CODE
void PiaTestCallbackForJoinRandomSession()
{
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    PIA_ASSERT(PIA_IS_VALID_POINTER(pSampleFramework));

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pSession));

    pSampleFramework->SetProgramParam( SAMPLE_PROGRAMPARAM_GETJOINSEQUENCE, pSession->GetJoinAutoMatchmakeSequence() );
}
#endif
}

SampleSceneNo SceneRandomMatchmake()
{
    nn::Result result;
    SampleSceneNo nextSceneNo = SCENE_COMMUNICATE;

    nn::pia::sampleframework::SampleFramework::GetInstance()->SetProgramStatus(
        SCENE_RANDOM_MATCHMAKE,
        false,
        false,
        0);

// Settings for the matchmaking session to create.
#if ENABLE_SERVERLESS
    nn::pia::inet::LanCreateSessionSetting sessionSetting;
    sessionSetting.SetSessionType(nn::pia::inet::LAN_SESSION_TYPE_ANYBODY);
#else
    nn::pia::inet::NexCreateSessionSetting sessionSetting;
    sessionSetting.SetSessionType(nn::pia::inet::SESSION_TYPE_ANYBODY);
#endif
    sessionSetting.SetGameMode(MATCHMAKE_GAME_MODE + MATCHMAKE_RANDOM + GetIncrementalGameModeValueByBitRateCheckMode());
    sessionSetting.SetMaxParticipants(SAMPLE_MAX_ENTRY);
    sessionSetting.SetAttribute(MATCHMAKE_ATTR_INDEX, g_IsEnableHostMigration);
    sessionSetting.SetOpenParticipation(true);  // Allows participation as soon as session is created.

    // Search condition settings
    const size_t cSearchCriteriaNum = 2;
#if ENABLE_SERVERLESS
    nn::pia::inet::LanSessionSearchCriteria searchCriteriaList[cSearchCriteriaNum];
#else
    nn::pia::inet::NexSessionSearchCriteria searchCriteriaList[cSearchCriteriaNum];
#endif

    // Specifies a single attribute.
    searchCriteriaList[0].SetGameMode(MATCHMAKE_GAME_MODE + MATCHMAKE_RANDOM + GetIncrementalGameModeValueByBitRateCheckMode());
    searchCriteriaList[0].SetMaxParticipants(SAMPLE_MAX_ENTRY);
    searchCriteriaList[0].SetAttribute(MATCHMAKE_ATTR_INDEX, g_IsEnableHostMigration);

    // Specifies multiple attributes.
    // (The code example is for specifying multiple attributes, but the search is made on the above-described specification for a single attribute, so this is not used for actual searching.)
    searchCriteriaList[1].SetGameMode(MATCHMAKE_GAME_MODE + MATCHMAKE_RANDOM + GetIncrementalGameModeValueByBitRateCheckMode());
    searchCriteriaList[1].SetMaxParticipantsRange(SAMPLE_MAX_ENTRY, SAMPLE_MAX_ENTRY);
    const size_t arraySize = 5;
    u32 attributeArray[arraySize] = { 2, 4, 6, 11, 72 };
    searchCriteriaList[1].SetAttribute(MATCHMAKE_ATTR_INDEX, attributeArray, arraySize);

#if PIA_TEST_CODE
    RegisterPiaTestCallback(PiaTestCallbackForJoinRandomSession);
#endif
    // Loop that does matchmaking.
    SampleSessionAsyncStatus sampleSessionAsyncStatus = Session_JoinRandomSession(
        &sessionSetting,
        searchCriteriaList,
        cSearchCriteriaNum,
        &result);
    if (sampleSessionAsyncStatus != SAMPLE_SESSION_ASYNC_SUCCESS)
    {
        PIA_SAMPLE_LOG_RESULT(result);
        if (sampleSessionAsyncStatus == SAMPLE_SESSION_ASYNC_CANCELED)
        {
            nextSceneNo = SCENE_SELECT_MATCHMAKE_TYPE;
            Session_CleanupSession();
        }
        else
        {
            nextSceneNo = GetNextSceneByErrorResult(result, SESSION_JOIN_ERROR);
        }
    }
    else
    {
        nextSceneNo = SetGameMode(MATCHMAKE_GAME_MODE + MATCHMAKE_RANDOM + GetIncrementalGameModeValueByBitRateCheckMode(), nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY, false, SAMPLE_DESCRIPTION_RANDOM);
    }
#if PIA_TEST_CODE
    RegisterPiaTestCallback(NULL);
#endif
    return nextSceneNo;
}
