﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Inet_Setup.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef SHARED_INET_SETUP_H_
#define SHARED_INET_SETUP_H_


#include "SampleLib/SampleLib.h"

#include <pia/common.h>
#include <pia/transport.h>
#include <pia/session.h>
#include <pia/inet.h>

#if NN_PIA_CTR
#include <nn/friends.h>
#elif NN_PIA_CAFE
#include <nn/fp.h>
#endif

#if NN_PIA_CTR
#define ENABLE_SERVERLESS 0   // Do not use the server.
#define ENABLE_LAN_NETWORK 0  // Use an existing server on the WAN.
#elif NN_PIA_CAFE
#define ENABLE_SERVERLESS 0   // Do not use the server.
#define ENABLE_LAN_NETWORK 0  // Use an existing server on the WAN.
#elif NN_PIA_WIN
#define ENABLE_SERVERLESS 0   // Do not use the server.
#define ENABLE_LAN_NETWORK 0  // Use an existing server on the WAN.
#endif

#if ENABLE_SERVERLESS
#include <pia/inet/inet_LanFacade.h>
#include <pia/inet/inet_LanNetworkFactory.h>
#include <pia/inet/inet_LanSessionTypes.h>
#include <pia/inet/inet_LanCreateSessionSetting.h>
#include <pia/inet/inet_LanJoinSessionSetting.h>
#include <pia/inet/inet_LanSessionSearchCriteria.h>
#endif


// <tt>nn::Result</tt> definitions for broadly categorizing errors that are not handled by <tt>nn::Result</tt> in the sample.
enum SAMPLE_RESULT_INET_DESCRIPTION
{
    FAILED_NETWORK_IFUP = 1,          // The network interface is down.
    FAILED_FRIENDS_LOGIN,             // A process related to logging in to the friend server failed.
    FAILED_NEX_LOGIN                  // A process related to logging in to the game server failed.
};

PIA_DEFINE_RESULT(
    SampleFailedNetworkInterfaceUp,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_NETWORK_IFUP);

PIA_DEFINE_RESULT(
    SampleFailedFriendsLogin,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_FRIENDS_LOGIN);

PIA_DEFINE_RESULT(
    SampleFailedNexLogin,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    FAILED_NEX_LOGIN);


// Parameter to use for logging in.
#if NN_PIA_CTR

// Use an existing server on the WAN.
#define ENABLE_LAN_NETWORK 0

#define MATCHMAKE_GAME_ID    0x0FE00305
#define MATCHMAKE_ACCESS_KEY _T("12345678")
#define MATCHMAKE_KEY_HASH   "deed2a88"
#endif


// Parameter to use for logging in.
#if NN_PIA_CAFE

// Use an existing server on the WAN.
#define ENABLE_LAN_NETWORK 0

#define MATCHMAKE_GAME_ID    0x1F940003
#define MATCHMAKE_ACCESS_KEY _T("12345678")
#define MATCHMAKE_KEY_HASH   "deed2a88"
#endif


namespace
{
const size_t SAMPLE_MTU_SIZE = nn::pia::inet::DEFAULT_MTU_SIZE;
}

// Initialize PiaInet.
nn::Result Inet_InitializePiaInet(size_t mtuSize = SAMPLE_MTU_SIZE);

// Finalize PiaInet.
void Inet_FinalizePiaInet();

// Get the NetworkFactory required for Transport initialization.
nn::pia::transport::NetworkFactory* Inet_GetNetworkFactory();

// Initializes AC and Socket.
nn::Result Inet_InitializeNetwork();

// Finalizes AC and Socket.
void Inet_FinalizeNetwork();

// Checks the status of AC.
bool Inet_IsConnected();

// Gets local system's address.
u32 Inet_GetLocalAddress();

// Gets the network error code.
u32 Inet_GetNetworkErrorCode();

// Log in to the game server.
nn::Result Inet_Login(u32 timeout = 30*1000);

// Log out from the game server.
nn::Result Inet_Logout();

// Gets the login error code.
s32 Inet_GetLoginNetworkErrorCode();

// Friend Presence-related.
#if NN_PIA_CTR
nn::Result Inet_UpdateGameMode(nn::friends::GameMode gameMode, const wchar_t* pDescription);
#elif NN_PIA_CAFE
nn::Result Inet_UpdateGameMode(nn::fp::GameMode gameMode, const wchar_t* pDescription);
#endif


#endif // SHARED_INET_SETUP_H_
