﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ControllerData.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleConfig.h"
#include "ControllerData.h"

#include "SampleLib/SampleLib.h"
#include <string.h>  // For <tt>memset</tt>.

// Clear controller data.
void ControllerData::Clear()
{
    memset(&m_KeyData, 0, sizeof(m_KeyData));
    memset(&m_TouchData, 0, sizeof(m_TouchData));
    memset(&m_StickData, 0, sizeof(m_StickData));

#if PIA_TEST_CODE
    memset(&m_SyncCheckData, 0, sizeof(m_SyncCheckData));
#endif
}


// Convert the framework button ID into a key.
bit32 ControllerData::KeyToButton(Key key)
{
    switch (key)
    {
    case KEY_UP:
        return nn::pia::sampleframework::BUTTON_UP;
    case KEY_DOWN:
        return nn::pia::sampleframework::BUTTON_DOWN;
    case KEY_LEFT:
        return nn::pia::sampleframework::BUTTON_LEFT;
    case KEY_RIGHT:
        return nn::pia::sampleframework::BUTTON_RIGHT;
    case KEY_A:
        return nn::pia::sampleframework::BUTTON_A;
    case KEY_B:
        return nn::pia::sampleframework::BUTTON_B;
    case KEY_X:
        return nn::pia::sampleframework::BUTTON_X;
    case KEY_Y:
        return nn::pia::sampleframework::BUTTON_Y;
    default:
        return 0;
    }
}


// Access the controller data to send.
const void* ControllerReader::GetData(DataId dataId) const
{
    switch (dataId)
    {
    case DATA_ID_KEY:
        return &m_KeyData;
    case DATA_ID_TOUCH:
        return &m_TouchData;
    case DATA_ID_STICK:
        return &m_StickData;

#if PIA_TEST_CODE
    case DATA_ID_SYNC_CHECK:
        return &m_SyncCheckData;
#endif

    default:
        return NULL;
    }
}


// Load the controller data from the framework and create the send data.
void ControllerReader::Read()
{
    Clear();

    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();

    for (u32 i = 0; i < KEY_NUM; ++i)
    {
        bit32 button = KeyToButton(static_cast<Key>(i));
        bit32 flag = 0x1 << i;
        if (pSampleFramework->IsHold(button))
        {
            m_KeyData.m_HoldFlag |= flag;
        }
        if (pSampleFramework->IsTrigger(button))
        {
            m_KeyData.m_TriggerFlag |= flag;
        }
        if (pSampleFramework->IsRelease(button))
        {
            m_KeyData.m_ReleaseFlag |= flag;
        }
    }

    if (pSampleFramework->IsPenTouched())
    {
        m_TouchData.m_Touch = 1;
        pSampleFramework->GetLatestPenPosition(&m_TouchData.m_X, &m_TouchData.m_Y);
    }

    pSampleFramework->GetAnalog(&m_StickData.m_X, &m_StickData.m_Y);
}

#if PIA_TEST_CODE
// Set the data for detecting desync.
void ControllerReader::SetSyncCheckKeyData(u32 n, u32 holdFlag, u32 triggerFlag, u32 releaseFlag)
{
    m_SyncCheckData.m_KeyData[n].m_HoldFlag = holdFlag;
    m_SyncCheckData.m_KeyData[n].m_TriggerFlag = triggerFlag;
    m_SyncCheckData.m_KeyData[n].m_ReleaseFlag = releaseFlag;
}

void ControllerReader::SetSyncCheckTouchData(u32 n, f32 x, f32 y, u8 touch)
{
    m_SyncCheckData.m_TouchData[n].m_X = x;
    m_SyncCheckData.m_TouchData[n].m_Y = y;
    m_SyncCheckData.m_TouchData[n].m_Touch = touch;
}

void ControllerReader::SetSyncCheckStickData(u32 n, u16 x, u16 y)
{
    m_SyncCheckData.m_StickData[n].m_X = x;
    m_SyncCheckData.m_StickData[n].m_Y = y;
}
#endif

// Get the save destination for the controller data received.
void* ControllerAccessor::GetDataPtr(DataId dataId)
{
    switch (dataId)
    {
    case DATA_ID_KEY:
        return &m_KeyData;
    case DATA_ID_TOUCH:
        return &m_TouchData;
    case DATA_ID_STICK:
        return &m_StickData;

#if PIA_TEST_CODE
    case DATA_ID_SYNC_CHECK:
        return &m_SyncCheckData;
#endif

    default:
        return NULL;
    }
}
