﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleConfig.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>
#include "Shared_Nex_Setup.h"
#include <pia/sync.h>


#ifdef PIA_TEST_BUILD
// Code for automatic verification. Ignore this code as a sample.
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

#if NN_PIA_CTR
#define USE_PIA_LOCAL 1
#else
#define USE_PIA_LOCAL 0
#endif

#define ENABLE_HOST_MIGRATION 1

#define PROGRAM_TITLE "SyncProtocol"

const size_t MAX_STATIONS = 8;  // Defines the maximum number of stations.
const size_t FRAME_PER_SECONDS = 60;
const u32 NEX_DISPATCH_PERIOD = (FRAME_PER_SECONDS == 60 ? 6 : 3);
const u32 DELAY_MAX = nn::pia::sync::SyncProtocol::FRAME_DELAY_MAX;
const bool ENABLE_CHANGE_DELAY = true;

const size_t MEMSIZE = 10 * 1024 * 1024;  // Memory size, in bytes, provided with the nn::pia::common::Initialize function.

// When not using NetZ, a small size is sufficient.
// Adjust it according to the application specifications.
const u32 SAMPLE_NEX_MEMORY_SIZE = 1 * 1024 * 1024;
const u32 SAMPLE_NEX_EMERGENCY_MEMORY_SIZE = 50 * 1024;

const nn::nex::Core::ThreadMode SAMPLE_NEX_THREAD_MODE = nn::nex::Core::ThreadModeUnsafeTransportBuffer;

const u32 MATCHMAKE_GAME_MODE_OFFSET = 0;
const u32 MATCHMAKE_GAME_MODE_BASE = 40000;  // Game mode.
const u32 MATCHMAKE_GAME_MODE = MATCHMAKE_GAME_MODE_BASE + 100000 * NN_PIA_PLATFORM + 1000000 * PIA_TEST_CODE + MATCHMAKE_GAME_MODE_OFFSET;

#if USE_PIA_LOCAL
const u32 SAMPLE_TITLE_ID = 0xFF320;
const u8 SAMPLE_SUB_ID = 0x00;
const char SAMPLE_PASSPHRASE[] = "aaabbccc0123456789!";
const char SAMPLE_SIGNATURE_KEY_DATA[] = "syncprotocol key";
#endif
