﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Demo.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>


#if NN_PIA_CTR
#define ENABLE_PIA_LOCAL 1
#else
#define ENABLE_PIA_LOCAL 0
#endif

// Indicates the mode that is executing.
// With inet, starts with automatch, and then determines whether to be the host or a client.
enum GameMode
{
#if ENABLE_PIA_LOCAL
    GAME_MODE_LOCAL_MASTER,
    GAME_MODE_LOCAL_CLIENT,
#endif
    GAME_MODE_INET_AUTOMATCH,
    GAME_MODE_INET_MASTER,
    GAME_MODE_INET_CLIENT,
#if ENABLE_PIA_LOCAL
    GAME_MODE_LOCAL_ERROR,
#endif
    GAME_MODE_INET_ERROR,
    GAME_MODE_EXIT
};


struct OptionalParameter
{
    s32  offset;  // This value must be set in advance by the caller of the DecideGameMode function.

    // The following values are set in the DecideGameMode function.
    bool bRelayEmulation;
    bool bPacketLossEmulation;
    s32  sendPacketLossPercentage;
    s32  recvPacketLossPercentage;
    s32  relayPreset;
    bool bLatencyEmulation;
    s32  latencyPreset;
    s32  demandingBitRate;

    OptionalParameter()
    {
        Initialize(0);
    }

    void Initialize(s32 ofs)
    {
        offset = ofs;

        bRelayEmulation = false;
        bPacketLossEmulation = false;
        sendPacketLossPercentage = 0;
        recvPacketLossPercentage = 0;
        relayPreset = 0;
        bLatencyEmulation = false;
        latencyPreset = 0;
        demandingBitRate = 0;
    }
};


GameMode DecideGameMode(const char* pTitle, OptionalParameter* pOpt = NULL);


void SetRelayEmulation(nn::pia::StationIndex c1, nn::pia::StationIndex c2);
