﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Common.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef PROFILING_TEST_COMMON_H_
#define PROFILING_TEST_COMMON_H_

#include "Shared_Inet_Setup.h"
#include "Shared_Nex_Setup.h"

#define ENABLE_LOCAL 0
#define ENABLE_NEX_NETZ 0

#define ENABLE_CRYPTO 0
#define ENABLE_CALL_CONTEXT 0

// Set to 1 to measure the CPU load
#define ENABLE_CPU_PROFILING 0
#define CPU_PROFILING_PERIOD 10
#define CPU_PROFILING_REPEAT 1
#if NN_PIA_CAFE
// Clear data cache before Dispatch processing (Cafe only)?
#define USE_CLEAR_DATACACHE 1
#endif

#define ENABLE_MEMORY_PROFILING 0

#define ENABLE_WATERMARK 0

#define ENABLE_LATENCY_PROFILING (!(ENABLE_CPU_PROFILING))
#define LATENCY_PROFILING_PERIOD 10

//If set to n1769 1, the sending and receiving of packets can be intentionally delayed in the transport send/receive thread.
#define ENABLE_LATENCY_EMULATION 0

// Does not affect other measurements if always ON
#define ENABLE_BLOCK_TIME_PROFILING 1
#define BLOCK_TIME_PROFILING_PERIOD 10
#define BLOCK_TIME_PROFILING_REPEAT 0

//n1769 Whether to execute communication using PiaChat
#define ENABLE_VOICE_COMMUNICATION 0
//n1769 Whether to execute Unreliable communication
#define ENABLE_UNRELIABLE_COMMUNICATION 1
// Whether to execute RoundRobin(R2)Unreliable communication instead of Unreliable communication
#define ENABLE_R2_UNRELIABLE 0
// Number of stations to send to each time when using RoundRobin
#define R2_UNRELIABLE_STATION_NUM 2

//n1769 Whether to execute Reliable communication
#define ENABLE_RELIABLE_COMMUNICATION 0

//n2908 Whether to enable HostMigration.
#define ENABLE_HOST_MIGRATION 0

//n2004 Whether to sign
#if ENABLE_CPU_PROFILING
#if ENABLE_LOCAL
#define ENABLE_SIGNATURE 0
#else
#define ENABLE_SIGNATURE 1
#endif  // ENABLE_LOCAL
#else
#define ENABLE_SIGNATURE 1
#endif  // ENABLE_CPU_PROFILING

#if ENABLE_NEX_NETZ
#else
// Whether to use relay connection feature
#define USE_RELAY_CONNECTION 0
#if USE_RELAY_CONNECTION
// Advanced settings for relay connection feature
#define SAMPLE_MAX_RELAY_NUM 10  // Maximum number of relay requests that can be assigned to one station
#define SAMPLE_MAX_RTT 400       // Maximum total RTT for relayed route
#endif                           // USE_RELAY_CONNECTION
#endif                           // ENABLE_NEX_NETZ

#if NN_PIA_NEX_3
//n1872 If NEX 3.x, set to 0 because the multi-unit heap cannot be used.
#define USE_MULTIUNITHEAP 0
//n1872 Whether DirectStream is used
#define USE_DIRECTSTREAM 1
//n1872 Whether to use send context when using "Reliable" transmission in DirectStream.
#define USE_CONTEXT 0
//n1872 If NEX 3.x, use the UnsafeTransportBuffer mode.
#define SAMPLE_NEX_THREAD_MODE nn::nex::Core::ThreadModeUnsafeTransportBuffer
#else
//n1872 Whether the multi-unit heap is used
#define USE_MULTIUNITHEAP 1
//n1872 If NEX 2.x, set to 0 because there is no DirectStream.
#define USE_DIRECTSTREAM 0
//n1872 If NEX 2.x, use the one of the thread modes shown below.
#define SAMPLE_NEX_THREAD_MODE nn::nex::Core::ThreadModeUnsafeTransportBuffer
//#define SAMPLE_NEX_THREAD_MODE nn::nex::Core::ThreadModeUnsafeUser
#endif

//n1872: The nex::MemoryMonitor feature was eliminated from NEX 3.6.0.
#if (NEX_CURRENT_VERSION_NUMBER >= NEX_VERSION_NUMBER(3, 6, 0))
#define USE_MEMORY_MONITOR 0
#else
#define USE_MEMORY_MONITOR 1
#endif

// Macro definition for automatic verification
#ifdef PIA_TEST_BUILD
#define PIA_TEST_CODE 1
#else
#define PIA_TEST_CODE 0
#endif

//n1872 When ON, repeats the connection/disconnection process automatically
#define ENABLE_AUTO_SESSION_REPEAT 0
#define AUTO_SESSION_REPEAT_COMMUNICATION_BASE_TIME_HOST 4000
#define AUTO_SESSION_REPEAT_COMMUNICATION_RANDOM_TIME_HOST 8000
#define AUTO_SESSION_REPEAT_COMMUNICATION_BASE_TIME_CLIENT 4000
#define AUTO_SESSION_REPEAT_COMMUNICATION_RANDOM_TIME_CLIENT 8000

// Condition group used when building or searching for matchmaking sessions on a NEX game server.
const u32 MATCHMAKE_GAME_MODE_OFFSET = 0;
const u32 MATCHMAKE_GAME_MODE_BASE = 35600;  // Game mode.
const u32 MATCHMAKE_GAME_MODE = MATCHMAKE_GAME_MODE_BASE + 100000 * NN_PIA_PLATFORM + MATCHMAKE_GAME_MODE_OFFSET;

/* Local-related. */
const bit32 SAMPLE_TITLE_ID = 0xFF312;      //!< Defines the unique ID of this program.
const u8 SAMPLE_SUB_ID = 0x00;              //!< Defines the sub-ID.
const char SAMPLE_PASSPHRASE[] = __FILE__;  //!< Defines the passphrase used to generate the encryption key for UDS communication. In an actual application, make sure that the passphrase is complex and sufficiently long.


// The number of retries (matchmaking is attempted this number of times when it fails)
#define MATCHMAKE_TRY_NUM 10

// When enabled, log output is faster, but the order relationship of logs of other libraries (such as NEX) is no longer the execution order.
#define ENABLE_CACHED_PRINT 1

#if ENABLE_CPU_PROFILING
#define ENABLE_TRACE_PIA 0
#define ENABLE_TRACE_PIA_DETAIL 0
#define ENABLE_TRACE_NEX 0
#define ENABLE_TRACE_NEX_DETAIL 0
#else
#define ENABLE_TRACE_PIA 1
#define ENABLE_TRACE_NEX 1
#define ENABLE_TRACE_PIA_DETAIL 0
#define ENABLE_TRACE_NEX_DETAIL 0
#endif

// Number of connected persons
#if NN_PIA_CTR
#define MATCHMAKE_PARTICIPANTS_MAX 12
#define GAME_PARTICIPANTS_NUM 12
#else
#define MATCHMAKE_PARTICIPANTS_MAX 24
#define GAME_PARTICIPANTS_NUM 24
#endif

// Whether to use automatic matchmaking.
#define USE_AUTO_MATCHMAKE 1

#define P2P_SESSION_FRAME_PER_SECOOND 30
#define P2P_COMMUNICATION_PERIOD 1

// NEX dispatch frequency.
#define NEX_DISPATCH_PERIOD 1

// NEX dispatch frequency during a P2P session.
// 3 game frames = 6 picture frames = around 100 ms.
#define NEX_DISPATCH_PERIOD_IN_P2P_SESSION 3

// Time to wait for a dispatch.
#define NEX_DISPATCH_TIMEOUT 0
#define PIA_DISPATCH_TIMEOUT 0

// PiaInet MTU size.
#define PIA_INET_MTU_SIZE nn::pia::inet::DEFAULT_MTU_SIZE
//#define PIA_INET_MTU_SIZE   nn::pia::inet::MIN_MTU_SIZE

#if ENABLE_NEX_NETZ


// The memory size is set to a larger value for the sample demo.
// Adjust it according to the application specifications.
#define SAMPLE_NEX_MEMORY_SIZE (4 * 1024 * 1024)
#define SAMPLE_NEX_EMERGENCY_MEMORY_SIZE (400 * 1024)

#else

// When not using NetZ, a small size is sufficient.
// Adjust it according to the application specifications.
#define SAMPLE_NEX_MEMORY_SIZE (1 * 1024 * 1024)
#define SAMPLE_NEX_EMERGENCY_MEMORY_SIZE (50 * 1024)

#endif

// Allocate extra memory for development applications.
// For the Release version, the value acquired with the nn::pia::common::GetMemoryUsage function is described here.
#if NN_PIA_CTR
#define SAMPLE_PIA_MEM_SIZE (4 * 1024 * 1024)
#else
#define SAMPLE_PIA_MEM_SIZE (8 * 1024 * 1024)
#endif

#define SAMPLE_TRACE_SEND SAMPLE_TRACE_APP1
#define SAMPLE_TRACE_RECEIVE SAMPLE_TRACE_APP2

enum
{
    STATE_INITIAL,
    STATE_SETUP_AUTOMATCH,
    STATE_CREATING_SESSION,
    STATE_JOINING_SESSION,
    STATE_WAIT_FOR_PARTICIPANT,
    STATE_GAME,
    STATE_LEAVING_SESSION,
    STATE_DESTROYING_SESSION,
    STATE_CLEANUP
};

void NexDispatch();
void PiaDispatch();
void PiaPostDispatch();
void DrawScreen();
void DrawError(nn::Result result, s32 networkErrorCode);

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
void ClearDataCache();
#endif
#endif

nn::Result CalcAsyncProcess();

#if ENABLE_NEX_NETZ
nn::Result WaitCallContext(const nn::nex::CallContext& callContext);
#else
#if ENABLE_CALL_CONTEXT
nn::Result WaitCallContext(const nn::pia::common::CallContext& callContext);
#endif
#endif

nn::Result WaitForButton(u32 button, u32* pTrigger);

#if PIA_TEST_CODE
void UpdatePiaTestData();
#endif

extern u8 g_State;
extern u32 g_MatchMakeLoopCnt;
extern u32 g_MainLoopCnt;

#if ENABLE_NEX_NETZ
extern nn::nex::HostMigrationCallback* s_pHostMigrationCallback;
#else
extern nn::pia::common::CallContext g_CallContext;
#endif

extern u32 g_NexDispatchPeriod;

#endif
