﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Internet.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "ProfilingTest_Internet.h"
#include "ProfilingTest_Main.h"
#include "ProfilingTest_P2P.h"

#include "Shared_Nex_Setup.h"
#include "Shared_Inet_Setup.h"
#include "Shared_Nex_Matchmake.h"

#include "Shared_Transport_Setup.h"
#include "Shared_Session_Setup.h"

#if ENABLE_NEX_NETZ
#if NN_PIA_CAFE
#include <nn/act.h>
#endif
#else
#include <pia/transport/transport_Transport.h>
#include <pia/session/session_Session.h>
#include <pia/util.h>
#endif


namespace
{

#if ENABLE_NEX_NETZ
// Gathering ID to use for matchmaking
nn::nex::GatheringID s_JoinedGatheringId = INVALID_GATHERINGID;
nn::nex::NetZ* s_pNetZ = NULL;
nn::nex::NATTraversalClient* s_pNatTraversalClient = NULL;
#endif
};

#if ENABLE_NEX_NETZ&& NN_PIA_NEX_3
namespace nn
{
namespace nex
{
extern void InitDOClasses();
}
}
#endif

nn::Result InitializeInternet()
{
    nn::Result result;

    result = Nex_InitializeNex(
        SAMPLE_NEX_MEMORY_SIZE,
        SAMPLE_NEX_EMERGENCY_MEMORY_SIZE,
        SAMPLE_NEX_THREAD_MODE);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#if ENABLE_NEX_NETZ
#else
    size_t mtuSize = PIA_INET_MTU_SIZE;
    PIA_SAMPLE_LOG("MTU: %d\n", mtuSize);
    result = Inet_InitializePiaInet(mtuSize);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#endif
    PIA_SAMPLE_LOG("InitializeInternet: succeeded\n");
    return nn::ResultSuccess();
}

void FinalizeInternet()
{
#if ENABLE_NEX_NETZ
#else
    Inet_FinalizePiaInet();
#endif

    Nex_FinalizeNex();
}

nn::Result Login()
{
    nn::Result result;

    // Connect to the Internet
    result = Inet_InitializeNetwork();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("Login:Inet_InitializeNetwork failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }

#if ENABLE_MEMORY_PROFILING  //n1769
    //Nex_EnableMultiUnitHeapPeakRecording();
#endif

#if ENABLE_NEX_NETZ&& NN_PIA_NEX_3
    nn::nex::NetZ::SetDOClassInitializationCallback(&nn::nex::InitDOClasses);
#endif

#if ENABLE_NEX_NETZ
    // Timeout settings for NAT traversal processing.
    nn::nex::RootTransport::SetNATTraversalTimeout(MATCHMAKE_PARTICIPANTS_MAX * 5000);

#if ENABLE_LAN_NETWORK
#else
    // Disable LAN sessions
    nn::nex::LANSessionDiscovery::Disable();
#endif
#endif

#if ENABLE_TRACE_NEX
    nn::nex::EventLog::GetInstance()->SetLogLevel(nn::nex::EventLog::Debug);

#if ENABLE_TRACE_NEX_DETAIL
    nn::nex::TraceLog::GetInstance()->GetOutputFormat()->ShowLocalTime();
#endif

    nn::nex::TraceLog::GetInstance()->SetFlag(
        TRACE_ALWAYS
#if ENABLE_TRACE_NEX_DETAIL
        |
        TRACE_NAT
//        |TRACE_CONNECTION
//        | D_TRACE_TRANSPORT_PACKET
#if 0
        | TRACE_INIT
        | TRACE_TERMINATE
        | TRACE_FAULT

//        | D_TRACE_NAT_TRAVERSAL
        | D_TRACE_TRANSPORT_EXPORT

        | TRACE_TIME
        | TRACE_CONNECTION
        | TRACE_TEARDOWN
        | TRACE_UPDATE
        | TRACE_TRANSPORT
        | TRACE_PHBDR
        | TRACE_OPERATION
        | TRACE_DUPSPACE
        | TRACE_LOADBALANCING
        | TRACE_SESSIONDISCOVERY
        | TRACE_SCHEDULER

        | D_TRACE_TEST
        | D_TRACE_DO_MIGRATION
        | D_TRACE_THREAD
        | D_TRACE_CONSTRUCT
        | D_TRACE_TRANSPORT
        | D_TRACE_TRANSPORT_PACKET
        | D_TRACE_TRANSPORT_END_POINT
        | D_TRACE_TRANSPORT_SEND_MULTI
        | D_TRACE_TRANSPORT_BUNDLING
        | D_TRACE_TRANSPORT_RELIABLE
        | D_TRACE_TRANSPORT_ROUTING
        | D_TRACE_TRANSPORT_HOST_MIGRATION
        | D_TRACE_DO_MESSAGE_BUNDLING
#endif
#endif  // ENABLE_TRACE_NEX_DETAIL
        );
#endif  // ENABLE_TRACE_NEX

    // Log in to the game server.
    result = Inet_Login();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("Login:Inet_Login: failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        // Clean up due to logout.
        return result;
    }

#if ENABLE_NEX_NETZ
    PIA_SAMPLE_LOG("Login: Bind to Matchmaking Service\n");
    Nex_InitializeMatchmake();
#endif

    PIA_SAMPLE_LOG("Login: succeeded\n");
    return nn::ResultSuccess();
}

void Logout()
{
#if ENABLE_NEX_NETZ
    Nex_FinalizeMatchmake();
#endif

    Inet_Logout();
    // Disconnect from the Internet.
    Inet_FinalizeNetwork();
}

nn::Result StartupInternet()
{
    nn::Result result;
    nn::pia::common::Time startTime = nn::pia::common::Time::GetTime();

#if ENABLE_NEX_NETZ
    s_pNatTraversalClient = qNew nn::nex::NATTraversalClient();

    PIA_SAMPLE_LOG("StartupInternet: Bind to nat traversal service\n");
    if (!s_pNatTraversalClient->Bind(Nex_GetNgsFacadeInstance()->GetCredentials()))
    {
        PIA_SAMPLE_LOG("StartupInternet: failed to bind to nat traversal service\n");
        return SampleNexInvalidState();
    }

    // Enables NAT traversal.
    PIA_SAMPLE_LOG("StartupInternet: StartNATSession\n");
    nn::nex::CallContext callContext;
    bool bResult = nn::nex::ConnectivityManager::GetInstance()->StartNATSession(&callContext);
    if (!bResult)
    {
        PIA_SAMPLE_LOG("StartupInternet: ConnectivityManager::GetInstance()->StartNATSession failed.\n");
        return SampleNexInvalidState();
    }

    result = WaitCallContext(callContext);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("StartupInternet: WaitCallContext failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }

    if (callContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        PIA_SAMPLE_LOG("StartupInternet: StartNATSession failed:%d\n", callContext.GetState());
        return SampleNexInvalidState();
    }


#else
#endif  // ENABLE_NEX_NETZ
    nn::pia::common::TimeSpan timeSpan = nn::pia::common::Time::GetTime() - startTime;
    PIA_SAMPLE_PRINT("StartupInternet: succeeded: time:%d [ms]\n", timeSpan.GetMSec());
    return nn::ResultSuccess();
}

void CleanupInternet()
{
    nn::Result result;
#if ENABLE_NEX_NETZ
    // Ends NAT traversal
    PIA_SAMPLE_LOG("CleanupInternet: StopNATSession\n");
    nn::nex::ConnectivityManager::GetInstance()->StopNATSession();

    if (s_pNatTraversalClient != NULL)
    {
        s_pNatTraversalClient->Unbind();
        qDelete s_pNatTraversalClient;
    }
#else
#endif
}

nn::Result StartupSession()
{
    nn::Result result;

#if ENABLE_NEX_NETZ
    s_pNetZ = qNew nn::nex::NetZ();
    PIA_SAMPLE_ASSERT(s_pNetZ != NULL);
    if (!s_pNetZ->IsValid())
    {
        PIA_SAMPLE_LOG("NetZ is invalid!!\n");
        qDelete s_pNetZ;
        return SampleNexInvalidState();
    }
#if NN_PIA_NEX_3
#else
    nn::nex::BundlingPolicy::GetInstance()->Enable();
    nn::nex::BundlingPolicy::GetInstance()->SetMaximumFlushDelay(75);
    nn::nex::BundlingPolicy::GetInstance()->SetFlushCheckPeriod(50);
    nn::nex::Stream::GetSettings(nn::nex::Stream::DO).GetBundling().Enable(30);
#endif

#if USE_DIRECTSTREAM
    // Enable sending and receiving of DirectStream.
    // Note: The library discards any packets received before calling the DirectStream::Enable function.
    nn::nex::DirectStream::GetInstance()->Enable();
#else
    // Set the receive callback
    nn::nex::DirectTransport::GetInstance()->RegisterUserReceiveCallback(P2PReceiveCallback);
#endif

    // HostMigration is necessary to continue processing when a station leaves a session.
    // A NetZ object is automatically unregistered when it is deleted.
    nn::nex::HostMigrationExtension::Register();
#if NN_PIA_NEX_3
    s_pHostMigrationCallback = qNew nn::nex::HostMigrationCallback(
        Nex_GetMatchmakeClient(), nn::nex::MatchmakeExtensionClient::StaticUpdateSessionHost);
#else
    s_pHostMigrationCallback = qNew nn::nex::HostMigrationCallback(Nex_GetMatchmakeClient());
#endif

#else

    nn::pia::session::Session::StartupSetting startupSetting;
    startupSetting.bUsingHostMigration = static_cast<bool>(ENABLE_HOST_MIGRATION);

#if ENABLE_CRYPTO
    // Do not use the host migration feature.
    startupSetting.cryptoMode = nn::pia::common::CryptoSetting::MODE_AES_128;
    startupSetting.maxSilenceTime = nn::pia::session::NN_PIA_SESSION_MAX_SILENCE_TIME_DEFAULT;
    startupSetting.keepAliveSendingInterval = nn::pia::session::NN_PIA_SESSION_KEEP_ALIVE_INTERVAL_DEFAULT;
#endif

    result = Session_StartupSession(startupSetting);
    if(result == nn::pia::ResultNetworkConnectionIsLost())
    {
        return result;
    }
    else
    {
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
    }
#endif
    return nn::ResultSuccess();
}

void CleanupSession()
{

#if ENABLE_NEX_NETZ
    if (s_pHostMigrationCallback != NULL)
    {
        qDelete s_pHostMigrationCallback;
    }

#if USE_DIRECTSTREAM
    // Disable sending/receiving of DirectStream.
    nn::nex::DirectStream::GetInstance()->Disable();
    CancelSendP2PDataReliableNetZ();
#endif

    if (s_pNetZ != NULL)
    {
        nn::Result result = DestroyP2PSession(s_pNetZ);
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("CleanupSession: DestroyP2PSession failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
        }

        qDelete s_pNetZ;
    }
#else
    Session_CleanupSession();
#endif
}

//-----------------------------------------------------------------------------
// Main loop.
//-----------------------------------------------------------------------------
void MainLoopInternet()
{
    nn::Result result;
    bool isExit = false;

    while (!isExit)
    {
        Nex_ResetMatchmakeNetworkErrorCode();

        // Initialize Internet communication, log into game server.
        result = Login();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("MainLoopInternet: Login failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            isExit = true;
            ShowErrorCodeLoop(result);
        }
        else
        {
            isExit = MainLoopInternetCore();
        }
        // Log out of game server and terminate Internet communication.
        Logout();
    }
}

bool MainLoopInternetCore()
{
    nn::Result result;
    bool isExitMainLoop = true;
    bool isExitMatchmakeLoop = false;

    PIA_SAMPLE_LOG("MainLoopInternetCore\n");
    for (g_MatchMakeLoopCnt = 0; g_MatchMakeLoopCnt < MATCHMAKE_TRY_NUM; ++g_MatchMakeLoopCnt)
    {
        g_State = STATE_INITIAL;

#if ENABLE_AUTO_SESSION_REPEAT
        //n1872
        // Calculates wait time until automatch processing starts.
        // Keep values apart by a suitable amount because stations cannot be put in a single session if they are too close together.
        //(should use a better random number... )
        // 
        s32 span = (nn::pia::common::Time::GetTime().GetTick() % 1000) * 3 + 2000;

        PIA_SAMPLE_PRINT("[*] Waiting for %d ms to start automatch.\n", span);
#if ENABLE_CACHED_PRINT
        // Outputs the above print text
        if (nn::pia::common::CachedPrint::GetInstance() != NULL)
        {
            nn::pia::common::CachedPrint::GetInstance()->Flush();
        }
#endif
        // Clears screen to differentiate from a freeze.
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

// Waits for the prescribed amount of time.
#if NN_PIA_CTR
        nn::os::Thread::Sleep(nn::fnd::TimeSpan::FromMilliSeconds(span));
#elif NN_PIA_WIN
        Sleep(span);
#elif NN_PIA_CAFE
        OSSleepMilliseconds(span);
#endif
#else  // ENABLE_AUTO_SESSION_REPEAT
        PIA_SAMPLE_PRINT("Press A : Start automatch\n");
        PIA_SAMPLE_PRINT("Press B : Logout\n");
#if ENABLE_MEMORY_PROFILING
        PIA_SAMPLE_PRINT("Press Y : Logout and NEX Finalize\n");
#endif  // ENABLE_MEMORY_PROFILING
        bit32 button;
        // Wait for button input.
        bit32 enableButton = nn::pia::sampleframework::BUTTON_A | nn::pia::sampleframework::BUTTON_B
#if ENABLE_MEMORY_PROFILING
                             | nn::pia::sampleframework::BUTTON_Y
#endif  // ENABLE_MEMORY_PROFILING
            ;
        result = WaitForButton(enableButton, &button);
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("MainLoopInternetCore: WaitForButton failed.\n");
            isExitMainLoop = true;
            break;
        }
        if (button == nn::pia::sampleframework::BUTTON_B)
        {
            isExitMainLoop = false;
            break;
        }
#if ENABLE_MEMORY_PROFILING
        if (button == nn::pia::sampleframework::BUTTON_Y)
        {
            isExitMainLoop = true;
            break;
        }
#endif  // ENABLE_MEMORY_PROFILING
#endif  // ENABLE_AUTO_SESSION_REPEAT

        g_State = STATE_SETUP_AUTOMATCH;
        result = ProcessMatchmakeInternet();
        if (result.IsFailure())
        {
            // Finalize and restart Pia and NEX if the network connection is lost or an NEX error occurs.
            // 
            if (result == SampleFailedNetworkInterfaceUp() ||
                result == SampleNexInternalError())
            {
                isExitMainLoop = true;
                isExitMatchmakeLoop = true;
            }
            // Display the error code of network errors that must be displayed if the matchmaking session fails.
            // Retry matchmaking if there is an error that does not need to be shown. (Such errors are not returned when using AutoMatchmake.)
            else if (result == SampleFailedJoinMatchmakeSession())
            {
                if (Nex_GetMatchmakeNetworkErrorCode() == 0)
                {
                    continue;
                }
            }
            // Retry matchmaking if the session closes or is closed.
            else if (result == SampleSessionStateDisconnected())
            {
                continue;
            }
            else
            {
                // A different Pia error was returned.
                nn::pia::util::HandlingType handlingType = nn::pia::util::GetHandlingType(result);

                switch(handlingType)
                {
                case nn::pia::util::HANDLING_LEAVE:
                case nn::pia::util::HANDLING_LEAVE_AND_CLEANUP:
                case nn::pia::util::HANDLING_RETRY:
                    // The JoinRandomSessionAsync asynchronous process never returns a Result handled as shown above.
                case nn::pia::util::HANDLING_LOGOUT:
                case nn::pia::util::HANDLING_CLEANUP:
                    isExitMainLoop = false;
                    isExitMatchmakeLoop = true;
                    break;
                case nn::pia::util::HANDLING_RECONNECT:
                case nn::pia::util::HANDLING_REINITIALIZE:
                    isExitMainLoop = true;
                    isExitMatchmakeLoop = true;
                    break;
                case nn::pia::util::HANDLING_IGNORABLE:
                    // The JoinRandomSessionAsync asynchronous process never returns a Result handled as shown above.
                    PIA_SAMPLE_LOG("ignorable result\n");
                    break;
                case nn::pia::util::HANDLING_PROGRAMMING_ERROR:
                case nn::pia::util::HANDLING_VOID:
                default:
                    PIA_TRACE_RESULT(result);
                    PIA_SAMPLE_LOG("programming error\n");
                    PIA_SAMPLE_ASSERT(result.IsSuccess());
                    break;
                }
            }
            ShowErrorCodeLoop(result);
        }

        if (isExitMatchmakeLoop)
        {
            break;
        }
    }

    g_State = STATE_CLEANUP;
    return isExitMainLoop;
}

nn::Result ProcessMatchmakeInternet()
{
    nn::Result result;

    result = StartupInternet();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet: StartupInternet failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        CleanupInternet();
        return result;
    }

    nn::nex::PrincipalID MyPrincipalID = INVALID_PRINCIPALID;

    bool bSuccess = false;
    bool bMatchmakeConnected = false;
    bool bP2PConnected = false;
    // Warning measures at Release build.
    (void)&bMatchmakeConnected;
    (void)&bP2PConnected;

// Start automatic matchmaking.
#if ENABLE_NEX_NETZ
#if USE_AUTO_MATCHMAKE
    // Try to create a session or join another open session.
    result = Nex_AutoMatchmake(
        &s_JoinedGatheringId,
        &MyPrincipalID,
        MATCHMAKE_GAME_MODE,
        MATCHMAKE_PARTICIPANTS_MAX,
        "PiaSession ProfilingTest");
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Nex_AutoMatchmake failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        CleanupInternet();
        return result;
    }
#else
    // Searches for a matchmaking session.
    // Blocked until an internal search completes.
    nn::nex::qList<nn::nex::GatheringHolder> listGatheringHolder;
    result = Nex_SearchMatchmakeSession(
        MATCHMAKE_GAME_MODE,
        MATCHMAKE_PARTICIPANTS_MAX,
        2,
        &listGatheringHolder);
    if (result.IsFailure())
    {
        // Search failed.
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet:Nex_SearchMatchmakeSession failed.\n");
    }
    else if (listGatheringHolder.size() < 1)
    {
        // If the matchmaking session is not found, create a new one.
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet:no matchmake session is found.\n");

        // Create a new session.
        result = Nex_CreateMatchmakeSession(
            &s_JoinedGatheringId,
            MATCHMAKE_GAME_MODE,
            MATCHMAKE_PARTICIPANTS_MAX,
            "PiaSession ProfilingTest");
        if (result.IsFailure())
        {
            // When the program has been properly implemented, the only causes are that wireless is turned off, the system is entering Sleep Mode, or server communication has timed out.
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet:Nex_CreateMatchmakeSession failed.\n");
        }
        else
        {
            MyPrincipalID = Nex_GetMatchmakeClient()->GetCredentials()->GetPrincipalID();
        }
    }
    else
    {
        // If the matchmaking session was found, join the first session in the search list.
        s_JoinedGatheringId = listGatheringHolder.front().Get()->GetID();
        result = Nex_JoinMatchmakeSession(s_JoinedGatheringId);
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet:Nex_JoinMatchmakeSession failed.\n");
        }
    }

    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG_RESULT(result);
        CleanupInternet();
        return result;
    }
#endif

#if ENABLE_NEX_NETZ
#else
    // Transmit the matchmaking session GatheringID to NexFacade.
    nn::pia::inet::NexFacade::GetInstance()->UpdateGatheringID(s_JoinedGatheringId);
#endif

    bMatchmakeConnected = true;

    result = StartupSession();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet: StartupSession failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        CleanupSession();
        CleanupInternet();
        return result;
    }
    if (MyPrincipalID == Nex_GetMatchmakeClient()->GetCredentials()->GetPrincipalID())
    {
        // Session host.
        g_State = STATE_CREATING_SESSION;
        result = CreateP2PSession();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: CreateP2PSession failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            // cleaned up below
        }
        else
        {
            result = Nex_OpenParticipation(s_JoinedGatheringId);
            if (result.IsFailure())
            {
                PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Nex_OpenParticipation failed.\n");
                PIA_SAMPLE_LOG_RESULT(result);
                // cleaned up below
            }
            else
            {
                bP2PConnected = true;
                bSuccess = true;
            }
        }
    }
    else
    {
        // Session client.
        nn::nex::qList<nn::nex::StationURL> lstURLs;
        result = Nex_GetHostStationURLs(
            s_JoinedGatheringId,
            &lstURLs);
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Nex_GetHostStationURLs failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            // cleaned up below
        }
        else
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Nex_GetHostStationURLs succeeded\n");
            nn::nex::qList<nn::nex::StationURL>::iterator it = lstURLs.begin();
            while (it != lstURLs.end())
            {
                PIA_SAMPLE_LOG("ProcessMatchmakeInternet: URL:%ls\n", it->CStr());
                ++it;
            }

            g_State = STATE_JOINING_SESSION;
            result = JoinP2PSession(lstURLs);
            if (result.IsFailure())
            {
                PIA_SAMPLE_LOG("ProcessMatchmakeInternet: JoinP2PSession failed.\n");
                PIA_SAMPLE_LOG_RESULT(result);
                // cleaned up below
            }
            else
            {
                bP2PConnected = true;
                bSuccess = true;
            }
        }
    }
#else
    StartupSession();

    nn::pia::inet::NexCreateSessionSetting createSessionSetting;
    createSessionSetting.SetGameMode(MATCHMAKE_GAME_MODE);
    createSessionSetting.SetMinParticipants(1);
    createSessionSetting.SetMaxParticipants(MATCHMAKE_PARTICIPANTS_MAX);
    createSessionSetting.SetDescription(_T("PiaSession ProfilingTest"));
    createSessionSetting.SetOpenParticipation(true);

    nn::pia::inet::NexSessionSearchCriteria sessionSearchCriteria;
    sessionSearchCriteria.SetGameMode(MATCHMAKE_GAME_MODE);
    sessionSearchCriteria.SetMinParticipants(1);
    sessionSearchCriteria.SetMaxParticipants(MATCHMAKE_PARTICIPANTS_MAX);
    sessionSearchCriteria.SetVacantOnly(true);
    sessionSearchCriteria.SetOpenedOnly(true);

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    PIA_SAMPLE_ASSERT(pSession != NULL);

    result = pSession->JoinRandomSessionAsync(&createSessionSetting, &sessionSearchCriteria, 1);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("ProcessMatchmakeInternet: JoinRandomSessionAsync failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }

    while (true)
    {
        if (pSession->IsCompletedJoinRandomSession())
        {
            result = pSession->GetJoinRandomSessionResult();
            if (result.IsFailure())
            {
                PIA_SAMPLE_LOG("ProcessMatchmakeInternet: JoinRandomSessionAsync failed.\n");
                PIA_SAMPLE_LOG_RESULT(result);
                CleanupInternet();
                bMatchmakeConnected = false;
                bP2PConnected = false;
                bSuccess = false;
                break;
                //return result;
            }
            else
            {
                bMatchmakeConnected = true;
                bP2PConnected = true;
                bSuccess = true;
                break;
            }
        }
        result = CalcAsyncProcess();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Connection is lost\n");
            PIA_SAMPLE_LOG_RESULT(result);
            CleanupInternet();
            return result;
        }
    }
#endif
    PIA_SAMPLE_LOG("ProcessMatchmakeInternet: MatchmakeConnected:%d, P2PConnected:%d\n", bMatchmakeConnected, bP2PConnected);

#if ENABLE_NEX_NETZ
#else
    // During a Pia P2P session, the CPU load can be lessened by lowering the frequency of calls to NEX Dispatch.
    // The call period can be extended to as much as a 100-millisecond interval.
    g_NexDispatchPeriod = NEX_DISPATCH_PERIOD_IN_P2P_SESSION;
#endif

    // Wait until enough people have gathered.
    if (bSuccess)
    {
        Nex_SetOwnerPID(MyPrincipalID);

#if ENABLE_NEX_NETZ
        // Enables host migration.
        s_pHostMigrationCallback->Register(s_JoinedGatheringId);
#endif

        result = WaitForParticipantLoop();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: WaitForParticipantLoop failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            bSuccess = false;
            // cleaned up below
        }

#if ENABLE_NEX_NETZ
        // Disables host migration.
        s_pHostMigrationCallback->Unregister(s_JoinedGatheringId);
#endif
    }

    // Start the main send/receive processing.
    if (bSuccess)
    {
#if ENABLE_NEX_NETZ
        // Enables host migration.
        s_pHostMigrationCallback->Register(s_JoinedGatheringId);
#endif

        result = GameLoop();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: GameLoop failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            bSuccess = false;
            // cleaned up below
        }

#if ENABLE_NEX_NETZ
        // Disables host migration.
        s_pHostMigrationCallback->Unregister(s_JoinedGatheringId);
#endif
    }

    g_NexDispatchPeriod = NEX_DISPATCH_PERIOD;

#if ENABLE_CACHED_PRINT
    // If caching print statements has been enabled, flush the cache now because NEX prints cannot be cached.
    if (nn::pia::common::CachedPrint::GetInstance() != NULL)
    {
        nn::pia::common::CachedPrint::GetInstance()->Flush();
    }
#endif

#if ENABLE_NEX_NETZ
    if (bMatchmakeConnected)
    {
        // Leave the session.
        // Delete the matchmaking session without migrating hosts if the local station is the owner.
        if (Nex_LeaveMatchmakeSession(s_JoinedGatheringId, Nex_GetOwnerPID(), false).IsFailure())
        {
            PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Nex_LeaveMatchmakeSession failed.\n");
        }
    }
#else
    if (bMatchmakeConnected)
    {
        result = pSession->LeaveSessionAsync();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("LeaveSessionAsync: failed.\n");
            PIA_SAMPLE_LOG_RESULT(result);
            return result;
        }
        while (true)
        {
            if (pSession->IsCompletedLeaveSession())
            {
                result = pSession->GetLeaveSessionResult();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_LOG("ProcessMatchmakeInternet: LeaveSession failed");
                    PIA_SAMPLE_LOG_RESULT(result);
                }
                break;
            }

            result = CalcAsyncProcess();
            if (result.IsFailure())
            {
                PIA_SAMPLE_LOG("ProcessMatchmakeInternet: Connection is lost\n");
                PIA_SAMPLE_LOG_RESULT(result);
            }
        }
    }
#endif

    CleanupSession();
    CleanupInternet();

    // Return the result if an error occurs at this time.
    if (!bSuccess)
    {
        return result;
    }

    PIA_SAMPLE_LOG("ProcessMatchmakeInternet: succeeded\n");
    return nn::ResultSuccess();
}

bool IsNgsConnected()
{
    if (Nex_GetNgsFacadeInstance())
    {
        return Nex_GetNgsFacadeInstance()->IsConnected();
    }
    return false;
}
