﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Main.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


//n1589: todo2: Manualization.
/*
Operation Method:

After starting the demo, a blue screen appears.
Press the A Button to advance to the log-in process, search for a matchmaking session, join one if found, or create one if none is found.
To test with multiple devices, press the A Button on one of the devices, and advance until WaitForParticipantLoop appears in the log. Then, press the A Button on the other devices.
*/

#include "ProfilingTest_Main.h"
#if NN_PIA_CTR
#include "ProfilingTest_Local.h"
#endif
#include "ProfilingTest_Internet.h"
#include "ProfilingTest_P2P.h"
#include "ProfilingTest_Profiling.h"
#include "ProfilingTest_Setting.h"
#include "ProfilingTest_MemoryMonitor.h"

#include "Shared_Common_Setup.h"
#include "Shared_Nex_Matchmake.h"
#include "Shared_Inet_Setup.h"
#include "Shared_Nex_Setup.h"
#include "Shared_Transport_Setup.h"
#include "Shared_Session_Setup.h"

#include <pia/version.h>
#include <pia/common/common_Trace.h>
#include <pia/common/common_Scheduler.h>
#include <pia/common/common_Time.h>
#include <pia/util.h>
#include <pia/chat.h>

#if NN_PIA_CTR
#include <nn/version.h>
#elif NN_PIA_CAFE
#include <sdk_ver.h>
#endif

#define ENABLE_STATION_ADDRESS_DUPLICATION_CHECK 1
#if ENABLE_STATION_ADDRESS_DUPLICATION_CHECK
#include <pia/transport/transport_StationManager.h>
#endif

#define EXPERIMENT_JOIN_DENY_AT_FIRST_TIME 0

// The time to wait alone after creating a session
#define P2P_SESSION_ORPHAN_WAIT_TIME (10 * 60 * 1000)

#define P2P_SESSION_LOG_PERIOD (10 * 1000)

#define SCREEN_WIDTH 400
#define SCREEN_HEIGHT 240
#define DRAW_FONT_WIDTH 8
#if NN_PIA_CTR
#define DRAW_FONT_HEIGHT 8
#elif NN_PIA_WIN
#define DRAW_FONT_HEIGHT 14
#elif NN_PIA_CAFE
#define DRAW_FONT_HEIGHT 14
#endif

#define DRAW_POSITION_TITLE_X 0
#define DRAW_POSITION_TITLE_Y (1 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_STATE_X 0
#define DRAW_POSITION_STATE_Y (2 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_GAME_FRAME_X 0
#define DRAW_POSITION_GAME_FRAME_Y (3 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_NUM_STATIONS_X 0
#define DRAW_POSITION_NUM_STATIONS_Y (4 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_IS_HOST_X 0
#define DRAW_POSITION_IS_HOST_Y (5 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_SEND_COUNT_X 0
#define DRAW_POSITION_SEND_COUNT_Y (6 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_RECEIVE_COUNT_X 0
#define DRAW_POSITION_RECEIVE_COUNT_Y (7 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_JOIN_TIME_X 0
#define DRAW_POSITION_JOIN_TIME_Y (8 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_NEX_DISPATCH_COUNT_X 0
#define DRAW_POSITION_NEX_DISPATCH_COUNT_Y (9 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_SEND_COUNT_RELIABLE_X 0
#define DRAW_POSITION_SEND_COUNT_RELIABLE_Y (10 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_SEND_FAILED_COUNT_RELIABLE_X 0
#define DRAW_POSITION_SEND_FAILED_COUNT_RELIABLE_Y (11 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_RECEIVE_COUNT_RELIABLE_X 0
#define DRAW_POSITION_RECEIVE_COUNT_RELIABLE_Y (12 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_MEMORY_PEAK_SIZE_X 0
#define DRAW_POSITION_MEMORY_PEAK_SIZE_Y (13 * DRAW_FONT_HEIGHT)
#define DRAW_POSITION_JOB_NUM_X 0
#define DRAW_POSITION_JOB_NUM_Y (15 * DRAW_FONT_HEIGHT)

#if ENABLE_BLOCK_TIME_PROFILING
#define BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM 5
#endif

#define DC_SIZE_MAIN 2 * 1024 * 1024
#define DC_SIZE_SUB 512 * 1024

// Global variables are defined here
u8 g_State = STATE_INITIAL;
u32 g_MatchMakeLoopCnt = 0;
u32 g_MainLoopCnt = 0;

#if ENABLE_NEX_NETZ
nn::nex::HostMigrationCallback* s_pHostMigrationCallback;
#else
#if ENABLE_CALL_CONTEXT
nn::pia::common::CallContext g_CallContext;
#endif
#endif

u32 g_NexDispatchPeriod = NEX_DISPATCH_PERIOD;

namespace
{
u32 s_NexDispatchCount = 0;

#if ENABLE_NEX_NETZ
#else
u32 s_UnreliableProtocolHandle = 0;
u32 s_ReliableProtocolHandle = 0;
#if ENABLE_R2_UNRELIABLE
u32 s_R2UnreliableProtocolHandle = 0;
#endif
#if ENABLE_VOICE_COMMUNICATION
u32 s_VoiceProtocolHandle = 0;
#endif
#endif

// Pia heap memory.
PIA_ATTRIBUTE_ALIGN(PIA_HEAP_ALIGN) u8 s_PiaHeapBuffer[SAMPLE_PIA_MEM_SIZE];

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
// Memory for clearing cache.
PIA_ATTRIBUTE_ALIGN(32) u8 s_DataCacheClearBuffer[DC_SIZE_MAIN];
#endif
#endif

bool IsInSession(u8 state)
{
    switch (state)
    {
    case STATE_CREATING_SESSION:
    case STATE_JOINING_SESSION:
    case STATE_WAIT_FOR_PARTICIPANT:
    case STATE_GAME:
    case STATE_LEAVING_SESSION:
    case STATE_DESTROYING_SESSION:
        return true;
    }
    return false;
}

#if ENABLE_BLOCK_TIME_PROFILING
Statistic<nn::pia::common::TimeSpan> s_NexSchedulerDispatchTimeStatistic = Statistic<nn::pia::common::TimeSpan>();
#if ENABLE_NEX_NETZ
#else
Statistic<nn::pia::common::TimeSpan> s_PiaSchedulerDispatchTimeStatistic = Statistic<nn::pia::common::TimeSpan>();
Statistic<nn::pia::common::TimeSpan> s_PiaSchedulerPostDispatchTimeStatistic = Statistic<nn::pia::common::TimeSpan>();
#endif
#endif


#if ENABLE_NEX_NETZ
#else
#if ENABLE_LATENCY_PROFILING
#if ENABLE_UNRELIABLE_COMMUNICATION
u32 s_LatencyProfilingProtocolUnreliableHandle = 0;
LatencyStatistic s_LatencyStatisticUnreliable[MATCHMAKE_PARTICIPANTS_MAX];
#endif
#if ENABLE_RELIABLE_COMMUNICATION
u32 s_LatencyProfilingProtocolReliableHandle = 0;
LatencyStatistic s_LatencyStatisticReliable[MATCHMAKE_PARTICIPANTS_MAX];
#endif
#endif
#endif

#if ENABLE_CPU_PROFILING
#if CPU_PROFILING_REPEAT

#if ENABLE_RELIABLE_COMMUNICATION
u32 s_NumSuccessSendReliable = 0;
u32 s_NumPacketRecvReliable = 0;
u32 s_NumFailSendReliable = 0;
#endif
#if ENABLE_UNRELIABLE_COMMUNICATION
u32 s_NumSuccessSendUnreliable = 0;
u32 s_NumPacketRecvUnreliable = 0;
u32 s_NumPacketDropUnreliable = 0;
#endif

#endif  // CPU_PROFILING_REPEAT
#else
MemoryMonitor s_MemMon;
#endif
};

//-----------------------------------------------------------------------------
// External function definitions
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Dispatch process.
//-----------------------------------------------------------------------------
nn::Result GetConnectionStatus()
{

#if ENABLE_NEX_NETZ
    // Status check for NEX system error.
    if (Nex_CheckInternalError())
    {
        PIA_SAMPLE_LOG("GetConnectionStatus: NEX internal error occurs\n");
        return SampleNexInternalError();
    }
#if ENABLE_LOCAL
#else
    // Status check for network connection to AC.
    if (!Inet_IsConnected())
    {
        PIA_SAMPLE_LOG("GetConnectionStatus: Unable to reach the internet.\n");
        return SampleFailedNetworkInterfaceUp();
    }
#endif
#else
    // Status check for network connection when sending and receiving.
    //If the network connection is abnormal:
    if (nn::pia::session::Session::GetInstance()->CheckConnectionError() == nn::pia::ResultNetworkConnectionIsLost())
    {
        //Returns when the network connection is interrupted because the wireless switch is turned off or communication fails.
        PIA_SAMPLE_LOG("CheckConnection: Connection is lost.\n");
        return nn::pia::ResultNetworkConnectionIsLost();
    }
#endif

    return nn::ResultSuccess();
}

void NexDispatch()
{

    {
// NEX dispatch
#if ENABLE_NEX_NETZ
#if ENABLE_BLOCK_TIME_PROFILING
        nn::pia::common::Time startTime, endTime;
        startTime.SetNow();
#endif
        nn::nex::Scheduler* pScheduler = nn::nex::Scheduler::GetInstance();
        if (pScheduler)
        {
            pScheduler->Dispatch(NEX_DISPATCH_TIMEOUT);
        }
#if ENABLE_BLOCK_TIME_PROFILING
        endTime.SetNow();
        s_NexSchedulerDispatchTimeStatistic.Record(endTime - startTime);
#endif
        s_NexDispatchCount++;
#else  // ENABLE_NEX_NETZ

#if ENABLE_LOCAL&& NN_PIA_CTR
        NN_PIA_DUMMY_PARAM(g_NexDispatchPeriod);
#else  // ENABLE_LOCAL && NN_PIA_CTR

        if ((nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount() % g_NexDispatchPeriod) == 0)
        {
#if ENABLE_BLOCK_TIME_PROFILING
            nn::pia::common::Time startTime, endTime;
            startTime.SetNow();
#endif
            nn::nex::Scheduler* pScheduler = nn::nex::Scheduler::GetInstance();
            if (pScheduler)
            {
                pScheduler->Dispatch(NEX_DISPATCH_TIMEOUT);
            }
#if ENABLE_BLOCK_TIME_PROFILING
            endTime.SetNow();
            s_NexSchedulerDispatchTimeStatistic.Record(endTime - startTime);
#endif
            s_NexDispatchCount++;
        }

#endif  // ENABLE_LOCAL && NN_PIA_CTR

#endif  // ENABLE_NEX_NETZ
    }
#if ENABLE_CPU_PROFILING
#else
#if USE_MULTIUNITHEAP
    s_MemMon.Update();
#endif
#endif
}

void PiaDispatch()
{
#if ENABLE_NEX_NETZ
#else
    {
        nn::pia::common::Scheduler* pScheduler = nn::pia::common::Scheduler::GetInstance();
        if (pScheduler)
        {
#if ENABLE_BLOCK_TIME_PROFILING
            nn::pia::common::Time startTime, endTime;
            startTime.SetNow();
#endif
            pScheduler->Dispatch(PIA_DISPATCH_TIMEOUT);
#if ENABLE_BLOCK_TIME_PROFILING
            endTime.SetNow();
            s_PiaSchedulerDispatchTimeStatistic.Record(endTime - startTime);
#endif
        }

#if ENABLE_STATION_ADDRESS_DUPLICATION_CHECK
        nn::pia::transport::StationManager* pStationManager = nn::pia::transport::StationManager::GetInstance();
        if (pStationManager)
        {
            // Check whether there is a station with the same <tt>StationAddress</tt> among the valid stations belonging to the <tt>StationManager</tt>.
            int i, j;
            i = 0;
            for (nn::pia::transport::StationManager::Iterator it = pStationManager->Begin(); it != pStationManager->End(); ++it)
            {
                nn::pia::transport::StationManager::Iterator itBuf = it;
                j = i + 1;
                for (++itBuf; itBuf != pStationManager->End(); ++itBuf)
                {
                    if (it->GetStationAddress() == itBuf->GetStationAddress())
                    {
                        PIA_SAMPLE_LOG("[TEST] found same StationAddress.\n");
                        PIA_SAMPLE_LOG("Station Idx:%d [0x%08x]:\n", i, &(*it));
                        it->Trace(nn::pia::common::TRACE_FLAG_ALWAYS);
                        PIA_SAMPLE_LOG("Station Idx:%d [0x%08x]:\n", j, &(*itBuf));
                        itBuf->Trace(nn::pia::common::TRACE_FLAG_ALWAYS);

                        nn::pia::session::Mesh::GetInstance()->TraceStationConnectionInfoList(nn::pia::common::TRACE_FLAG_ALWAYS);

                        PIA_SAMPLE_ASSERT(false);
                    }
                    ++j;
                }
                ++i;
            }
        }
#endif
    }
#endif
}

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
void ClearDataCache()
{
    u32 range = (OSGetCoreId() == 1) ? DC_SIZE_MAIN : DC_SIZE_SUB;
    DCZeroRange(static_cast<void*>(s_DataCacheClearBuffer), range);
}
#endif
#endif

void DrawScreen()
{
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_TITLE_X,
        DRAW_POSITION_TITLE_Y,
        "PiaInet Sample / ProfilingTest");

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_STATE_X,
        DRAW_POSITION_STATE_Y,
        "State:%d",
        g_State);

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_GAME_FRAME_X,
        DRAW_POSITION_GAME_FRAME_Y,
        "GameFrame:%d",
        nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount());

    u16 stationNum = 0;
    bool isHost = false;
    if (IsInSession(g_State))
    {
        stationNum = GetStationNum();
        isHost = IsSessionHost();
    }

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_NUM_STATIONS_X,
        DRAW_POSITION_NUM_STATIONS_Y,
        "NumStations:%d",
        stationNum);

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_IS_HOST_X,
        DRAW_POSITION_IS_HOST_Y,
        "IsHost:%d",
        isHost);

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_SEND_COUNT_X,
        DRAW_POSITION_SEND_COUNT_Y,
        "SendCount:%d",
        GetP2PSendCount());

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_RECEIVE_COUNT_X,
        DRAW_POSITION_RECEIVE_COUNT_Y,
        "ReceiveCount:%d",
        GetP2PReceiveCount());

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_JOIN_TIME_X,
        DRAW_POSITION_JOIN_TIME_Y,
        "JoinTime:%d [ms]",
        GetJoinTime());

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_NEX_DISPATCH_COUNT_X,
        DRAW_POSITION_NEX_DISPATCH_COUNT_Y,
        "NexDispatchCount:%d",
        s_NexDispatchCount);

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_SEND_COUNT_RELIABLE_X,
        DRAW_POSITION_SEND_COUNT_RELIABLE_Y,
        "SendCountReliable:%d",
        GetP2PSendCountReliable());

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_SEND_FAILED_COUNT_RELIABLE_X,
        DRAW_POSITION_SEND_FAILED_COUNT_RELIABLE_Y,
        "SendFailedCountReliable:%d",
        GetP2PSendFailedCountReliable());

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_RECEIVE_COUNT_RELIABLE_X,
        DRAW_POSITION_RECEIVE_COUNT_RELIABLE_Y,
        "ReceiveCountReliable:%d",
        GetP2PReceiveCountReliable());

#if ENABLE_NEX_NETZ
#if USE_MULTIUNITHEAP
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_MEMORY_PEAK_SIZE_X,
        DRAW_POSITION_MEMORY_PEAK_SIZE_Y,
        "MultiUnitHeap peak size:%d",
        Nex_GetMultiUnitHeapPeakMemorySize());
#else
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_MEMORY_PEAK_SIZE_X,
        DRAW_POSITION_MEMORY_PEAK_SIZE_Y,
        "NEX Heap peak size:%d",
        Nex_GetPeakMemoryUsage());
#endif
#endif

#if ENABLE_NEX_NETZ
#else
    u32 jobNum = 0;
    if (nn::pia::common::Scheduler::GetInstance() != NULL)
    {
        jobNum = nn::pia::common::Scheduler::GetInstance()->GetJobNum();
    }

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(
        DRAW_POSITION_JOB_NUM_X,
        DRAW_POSITION_JOB_NUM_Y,
        "Job:%d",
        jobNum);
#endif

#if ENABLE_CPU_PROFILING
#else
#if USE_MULTIUNITHEAP
    s_MemMon.Render();
#endif
#endif
}

//-----------------------------------------------------------------------------
// Render error screen.
//-----------------------------------------------------------------------------
void DrawError(nn::Result result, s32 networkErrorCode)
{
    int lineHeigth = nn::pia::sampleframework::SampleFramework::GetInstance()->GetFontHeight();
    int x = 0;
    int y = 0;

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "PiaInet Sample / AutoMatch");

    y += lineHeigth;

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "GameFrame:%d", nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount());

    y += lineHeigth;

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "Error occurs.");

#if ENABLE_LOCAL
    NN_PIA_DUMMY_PARAM(result);
#else
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "  Result: 0x%08X", result.GetPrintableBits());
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "   Level:       %4d", result.GetLevel());
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "   Summary:     %4d", result.GetSummary());
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "   Module:      %4d", result.GetModule());
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "   Description: %4d", result.GetDescription());
#endif

    y += lineHeigth;

    if (networkErrorCode != 0)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "  Network error code: %d", networkErrorCode);
    }
    else
    {
        y += lineHeigth;
    }

    y += lineHeigth;

    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "[ Manual ]");
    nn::pia::sampleframework::SampleFramework::GetInstance()->DrawText(x, y += lineHeigth, "  A: Exit and Restart.");
}

void ShowErrorCodeLoop(nn::Result result)
{
    // Display the error code for any errors.
    if (result.IsFailure())
    {
        bool trigger = false;
        s32 networkErrorCode = 0;

#if ENABLE_LOCAL
#else
        // Use the NgsFacade::GetLastLoginErrorCode function to get login errors.
        if (result == SampleFailedNexLogin())
        {
            networkErrorCode = Inet_GetLoginNetworkErrorCode();
        }
        // Use the util::NetworkErrorCodeConverter function to convert Pia errors into the corresponding network error code.
        else if (nn::pia::util::IsPiaResult(result))
        {
            networkErrorCode = nn::pia::util::NetworkErrorCodeConverter::Convert(result);
        }
        // Use the Nex_GetInternalErrorCode function to get NEX errors.
        else if (result == SampleNexInternalError())
        {
            networkErrorCode = static_cast<s32>(Nex_GetInternalErrorCode());
        }
#endif

        // Output to log.
        PIA_SAMPLE_PRINT("Result        : 0x%08X\n", result.GetPrintableBits());
        PIA_SAMPLE_PRINT("  Level       : %4d\n", result.GetLevel());
        PIA_SAMPLE_PRINT("  Summary     : %4d\n", result.GetSummary());
        PIA_SAMPLE_PRINT("  Module      : %4d\n", result.GetModule());
        PIA_SAMPLE_PRINT("  Description : %4d\n", result.GetDescription());
        if (networkErrorCode != 0)
        {
            PIA_SAMPLE_PRINT("ErrorCode     : %d\n", networkErrorCode);
        }

#if ENABLE_AUTO_SESSION_REPEAT
        nn::pia::common::Time startTime;
        nn::pia::common::Time endTime;
        startTime.SetNow();
#endif
        // Display the error code until the A Button is pressed.
        while (!trigger)
        {
            nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if ENABLE_AUTO_SESSION_REPEAT
            endTime.SetNow();
            if ((endTime - startTime).GetMSec() > 2000)
            {
                trigger = true;
            }
#elif PIA_TEST_CODE
            trigger = true;
#else
            if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_A))
            {
                trigger = true;
            }
#endif
            NexDispatch();
            PiaDispatch();

            DrawError(result, networkErrorCode);
#if PIA_TEST_CODE
            UpdatePiaTestData();
#endif
            nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
        }
    }
}

nn::Result CalcAsyncProcess()
{
    nn::Result result;

    nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

    // Confirm the network connection status.
    result = GetConnectionStatus();

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
    ClearDataCache();
#endif
#endif

    NexDispatch();
    PiaDispatch();

    DrawScreen();
#if PIA_TEST_CODE
    UpdatePiaTestData();
#endif
    nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();

    return result;
}

nn::Result WaitCallContext(
#if ENABLE_NEX_NETZ
    const nn::nex::CallContext& callContext
#else
    const nn::pia::common::CallContext& callContext
#endif
    )
{
    nn::Result result;

#if ENABLE_BLOCK_TIME_PROFILING
    s_NexSchedulerDispatchTimeStatistic.Reset();
#if ENABLE_NEX_NETZ
#else
    s_PiaSchedulerDispatchTimeStatistic.Reset();
#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
    nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->ResetProtocolStatistic();
#endif
#if NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
    nn::pia::transport::ThreadStreamManager::GetInstance()->ResetThreadStreamStatistic();
#endif
#endif
#endif

    while (true)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        result = GetConnectionStatus();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("WaitForCallContext: Connection is lost.\n");
            return result;
        }

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
        ClearDataCache();
#endif
#endif
        NexDispatch();
        // With Pia, common::Scheduler::Dispatch is not called automatically by the library while waiting on CallContext.
        // You must call pia::common::Scheduler::Dispatch while waiting for CallContext to complete.
        PiaDispatch();

#if ENABLE_NEX_NETZ
        if (callContext.GetState() != nn::nex::CallContext::CallInProgress)
        {
#else
        if (callContext.GetState() != nn::pia::common::CallContext::CallInProgress)
        {
#endif
            break;
        }

        DrawScreen();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

#if ENABLE_BLOCK_TIME_PROFILING
    s_NexSchedulerDispatchTimeStatistic.Print("WaitCallContext NexScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
#if ENABLE_NEX_NETZ
#else
    s_PiaSchedulerDispatchTimeStatistic.Print("WaitCallContext PiaScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
    nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->PrintProtocolStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
#endif
#if NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
    nn::pia::transport::ThreadStreamManager::GetInstance()->PrintThreadStreamStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
#endif
#endif
#endif

    return nn::ResultSuccess();
}

//-----------------------------------------------------------------------------
// Wait for button input.
//-----------------------------------------------------------------------------
nn::Result WaitForButton(u32 button, u32* pTrigger)
{
    nn::Result result = nn::ResultSuccess();

    PIA_SAMPLE_ASSERT(pTrigger != NULL);

    // Loops until button input is detected or the network connection status becomes abnormal.
    while (result.IsSuccess())
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(button))
        {
            break;
        }

        // Confirm the network connection status.
        result = GetConnectionStatus();
#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
        ClearDataCache();
#endif
#endif
        NexDispatch();
        PiaDispatch();

        DrawScreen();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    PIA_SAMPLE_LOG("WaitForButton: IsTrigger:%x\n", button);
    *pTrigger = nn::pia::sampleframework::SampleFramework::GetInstance()->GetTrigger();
    return result;
}

#if PIA_TEST_CODE
//-----------------------------------------------------------------------------
// Update the PiaTest data.
//-----------------------------------------------------------------------------
void UpdatePiaTestData()
{
    //PIA_SAMPLE_PRINT("UpdatePiaTestData Begin\n");
    nn::pia::sampleframework::SampleFramework* pSampleFramework = nn::pia::sampleframework::SampleFramework::GetInstance();
    if (!PIA_IS_VALID_POINTER(pSampleFramework))
    {
        return;
    }

#if ENABLE_CPU_PROFILING
    if (Profiler::GetInstance()->IsCounting())
    {
        return;
    }
#endif

    nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
    if (!PIA_IS_VALID_POINTER(pSession))
    {
        return;
    }

    pSampleFramework->SetProgramStatus(
        g_State,
        pSession->GetStatus() == nn::pia::session::Session::STATUS_CONNECTED_SESSION,
        pSession->GetLocalStationId() == pSession->GetHostStationId(),
        pSession->GetStationNum());

    // Session.
    nn::pia::StationIndex localStationIndex;
    nn::pia::transport::Transport::GetInstance()->ConvertToStationIndex(&localStationIndex, pSession->GetLocalStationId());
    pSampleFramework->SetProgramParam(0x00, localStationIndex);
    //PIA_SAMPLE_PRINT("UpdatePiaTestData End\n");
}
#endif


//-----------------------------------------------------------------------------
// Wait for enough players to join.
//-----------------------------------------------------------------------------
nn::Result WaitForParticipantLoop()
{
    nn::Result result;

    PIA_SAMPLE_LOG("WaitForParticipantLoop:\n");
    g_State = STATE_WAIT_FOR_PARTICIPANT;

    PIA_SAMPLE_PRINT("Press A : Move on to game loop\n");

    // Wait until the prescribed number of people have joined.
    nn::pia::common::Time tiReadyStart = nn::pia::common::Time::GetTime();
    nn::pia::common::Time tiWaitStart = tiReadyStart;
    u32 waitCnt = 0;
    while (true)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if ENABLE_LOCAL&& NN_PIA_CTR
#else
#if ENABLE_NEX_NETZ
        /* 
           If the session owner disconnects and only one station remains, host migration will not take place and the session will become ownerless, preventing anyone from connecting.
           To prevent this, the following function is used to determine whether all other stations have left a session.
           If we detect that there are no other stations in the session, we exit the loop here and retry matchmaking.
        
*/
        if (s_pHostMigrationCallback->SessionHostIsOrphan())
        {
            PIA_SAMPLE_LOG("WaitForParticipantLoop: detected as orphan\n");
            return SampleSessionStateDisconnected();
        }
#endif
#endif

        result = GetConnectionStatus();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("WaitForParticipantLoop: Connection is lost.\n");
            return result;
        }

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
        ClearDataCache();
#endif
#endif
        NexDispatch();
        PiaDispatch();

        u16 numStations = GetStationNum();
        nn::pia::common::Time nowTime = nn::pia::common::Time::GetTime();
        if (nowTime - tiReadyStart >= 3000)
        {
            // Display the number of people who are currently connected.
            PIA_SAMPLE_LOG("WaitForParticipantLoop: waiting... NumStations:%d/%d\n", numStations, MATCHMAKE_PARTICIPANTS_MAX);

            // Display connection information.
            TraceP2PSession();

            tiReadyStart = nowTime;
            waitCnt++;
        }

        // Check how many people have connected.
        if (numStations >= GAME_PARTICIPANTS_NUM)
        {
            // The prescribed number of people have connected and preparations for entering the main loop are complete.
            PIA_SAMPLE_PRINT("WaitForParticipantLoop: NumStations %d >= %d\n", numStations, GAME_PARTICIPANTS_NUM);
            break;
        }

        if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_A))
        {
            PIA_SAMPLE_PRINT("WaitForParticipantLoop: next step\n");
            break;
        }
#if ENABLE_AUTO_SESSION_REPEAT
        // If you have been waiting with less than half the prescribed number of people, it may be that the session has been split into several sessions, so leave that kind of session once and perform matchmaking again so participants start gathering to a single session.
        // Also, if waiting for awhile with more than half the prescribed number of people and still no one joins, leave once, and then perform matchmaking again.
        // 
        // 
        // 
        else if ((waitCnt > 5 && numStations <= GAME_PARTICIPANTS_NUM / 2) || waitCnt > 15)
        {
            PIA_SAMPLE_PRINT("WaitForParticipantLoop: retry Automatchmake [waitCnt %d, numStations %d]\n", waitCnt, numStations);
            break;
        }
#endif
#if ENABLE_BLOCK_TIME_PROFILING
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_L))
        {
            s_NexSchedulerDispatchTimeStatistic.Print("Statistic NexScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            s_NexSchedulerDispatchTimeStatistic.Reset();
#if ENABLE_NEX_NETZ
#else
            s_PiaSchedulerDispatchTimeStatistic.Print("Statistic PiaScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            s_PiaSchedulerDispatchTimeStatistic.Reset();
#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
            nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->PrintProtocolStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->ResetProtocolStatistic();
#endif
#if NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
            nn::pia::transport::ThreadStreamManager::GetInstance()->PrintThreadStreamStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            nn::pia::transport::ThreadStreamManager::GetInstance()->ResetThreadStreamStatistic();
#endif
#endif
        }
#endif

        if (numStations == 1 && nowTime - tiWaitStart >= P2P_SESSION_ORPHAN_WAIT_TIME)
        {
            // Retry matchmaking if no one joins the session for a fixed amount of time.
            PIA_SAMPLE_LOG("WaitForParticipantLoop: time passed as orphan\n");
            return SampleSessionStateDisconnected();
        }

#if ENABLE_NEX_NETZ
#else
        if (nn::pia::session::Session::GetInstance()->CheckConnectionError() == nn::pia::ResultSessionConnectionIsLost())
        {
            // Retry matchmaking if the session closes while waiting.
            PIA_SAMPLE_LOG("WaitForParticipantLoop: Session::CheckConnectionError() Session DISCONNECTED\n");
            return SampleSessionStateDisconnected();
        }
#endif

        DrawScreen();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    PIA_SAMPLE_LOG("WaitForParticipantLoop: succeeded\n");
    return nn::ResultSuccess();
}

//-----------------------------------------------------------------------------
// The game loop after matchmaking.
//-----------------------------------------------------------------------------
nn::Result GameLoop()
{
    PIA_SAMPLE_LOG("GameLoop:\n");

    nn::Result result;
    g_State = STATE_GAME;
    nn::pia::common::Time tiLogStart = nn::pia::common::Time::GetTime();

    PIA_SAMPLE_PRINT("Press B : Finish game loop\n");

    PrintProfilingSetting();
#if ENABLE_CPU_PROFILING
    PIA_SAMPLE_PRINT("Press A : Start profiling for game loop\n");
#endif
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
    bool bProfiling = false;
    ClearCounters();
#endif
#if ENABLE_WATERMARK
    PIA_SAMPLE_PRINT("Press X : Print watermark result\n");
#endif
#if ENABLE_MEMORY_PROFILING
    PIA_SAMPLE_PRINT("Press R : Print memory report\n");
#endif
#if ENABLE_RELIABLE_COMMUNICATION
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
    PIA_SAMPLE_PRINT("Press START : Start reliable communication\n");
    bool bSendReliableTrigger = false;  // A reliable send is not executed until this flag becomes true.
#else
    bool bSendReliableTrigger = true;
#endif
#endif
#if ENABLE_VOICE_COMMUNICATION
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
    PIA_SAMPLE_PRINT("Press START : Start voice communication\n");
    bool bSendVoiceTrigger = false;  // <tt>VoiceProtocol</tt> <tt>Send</tt> is not executed until this flag becomes true.
#else
    bool bSendVoiceTrigger = true;
#endif
#endif
#if ENABLE_UNRELIABLE_COMMUNICATION
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
    PIA_SAMPLE_PRINT("Press START : Start unreliable communication\n");
    bool bSendUnreliableTrigger = false;  // An unreliable send is not executed until this flag becomes true.
#else
    bool bSendUnreliableTrigger = true;
#endif
#endif
#if ENABLE_LATENCY_PROFILING
#if ENABLE_NEX_NETZ
#else
    PIA_SAMPLE_PRINT("Press Up : Print latency report\n");
#if ENABLE_UNRELIABLE_COMMUNICATION
    for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
    {
        s_LatencyStatisticUnreliable[i].Reset();
    }
#endif
#if ENABLE_RELIABLE_COMMUNICATION
    for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
    {
        s_LatencyStatisticReliable[i].Reset();
    }
#endif
#endif
#endif  // ENABLE_LATENCY_PROFILING
#if ENABLE_BLOCK_TIME_PROFILING
    bool bBlockTime = false;
#endif
#if ENABLE_AUTO_SESSION_REPEAT
    nn::pia::common::Time startTime = nn::pia::common::Time::GetTime();
    // The time until leaving a session and the time until discarding a session are made to be a somewhat close in value.
    // 
    s32 span = 0;
    if (IsSessionHost())
    {
        span = AUTO_SESSION_REPEAT_COMMUNICATION_BASE_TIME_HOST;
        span += startTime.GetTick() % AUTO_SESSION_REPEAT_COMMUNICATION_RANDOM_TIME_HOST;
        PIA_SAMPLE_PRINT("[*] Setup a disconnection timer to %d ms.(host)\n", span);
    }
    else
    {
        span = AUTO_SESSION_REPEAT_COMMUNICATION_BASE_TIME_CLIENT;
        span += startTime.GetTick() % AUTO_SESSION_REPEAT_COMMUNICATION_RANDOM_TIME_CLIENT;
        PIA_SAMPLE_PRINT("[*] Setup a disconnection timer to %d ms.(client)\n", span);
    }

#endif

    while (true)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

        if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_B))
        {
            break;
        }
#if ENABLE_CPU_PROFILING
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_A))
        {
#if ENABLE_NEX_NETZ
            PIA_SAMPLE_PRINT("GameLoop:StartProfilingTarget:StationNum: %d\n", GetStationNum());
#endif
            s_NexDispatchCount = 0;
            Profiler::GetInstance()->StartProfilingTarget(CPU_PROFILING_PERIOD);
            bProfiling = true;
        }
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_Y))
        {
            if (Profiler::GetInstance()->IsCounting())
            {
                Profiler::GetInstance()->FinishProfiling();
                PIA_SAMPLE_PRINT("GameLoop:FinishProfiling:NexDispatchCount:%d\n", s_NexDispatchCount);
            }
            else
            {
                s_NexDispatchCount = 0;
                Profiler::GetInstance()->StartProfilingTarget(0);
            }
            bProfiling = false;
        }
#endif
#if ENABLE_WATERMARK
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_X))
        {
// Based on the watermark value, stores the internal buffer peak value.
// This peak value is used during the next initialization of the Pia library, so the memory starts from a state of being tuned.
// 
#if ENABLE_MEMORY_PROFILING
            RecordPeakValues();
#endif
            PrintWatermark();
            PIA_SAMPLE_PRINT("Send: %d, Recv: %d\n", GetP2PSendCount(), GetP2PReceiveCount());
        }
#endif
#if ENABLE_MEMORY_PROFILING
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_R))
        {
#if ENABLE_NEX_NETZ
#if USE_MULTIUNITHEAP
            size_t memSize = Nex_GetMultiUnitHeapPeakMemorySize();
            PIA_SAMPLE_PRINT("MultiUnitHeap peak memory usage: %d bytes. (%d KB)\n", memSize, memSize / 1024);
#else
            size_t memSize = Nex_GetPeakMemoryUsage();
            PIA_SAMPLE_PRINT("NEX peak memory usage: %d bytes. (%d KB)\n", memSize, memSize / 1024);
#endif
#else
            size_t memSize = nn::pia::common::GetMemoryUsage();
            PIA_SAMPLE_PRINT("Pia memory usage: %d bytes. (%d KB)\n", memSize, memSize / 1024);
#endif
        }
#endif
#if ENABLE_BLOCK_TIME_PROFILING
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_L))
        {
            bBlockTime = true;
        }
#endif
#if ENABLE_LATENCY_PROFILING
#if ENABLE_NEX_NETZ
#else
        // Measures latency of game frames.
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_UP))
        {
            char buf[64];
#if ENABLE_UNRELIABLE_COMMUNICATION
            for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
            {
                if (s_LatencyStatisticUnreliable[i].GetCount() > 0)
                {
                    sprintf(buf, "Latency Unreliable to %d", i);
                    s_LatencyStatisticUnreliable[i].Print(buf, 0);
                }
                s_LatencyStatisticUnreliable[i].Reset();
            }
#endif
#if ENABLE_RELIABLE_COMMUNICATION
            for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
            {
                if (s_LatencyStatisticReliable[i].GetCount() > 0)
                {
                    sprintf(buf, "Latency Reliable to %d", i);
                    s_LatencyStatisticReliable[i].Print(buf, 0);
                }
                s_LatencyStatisticReliable[i].Reset();
            }
#endif
        }
        // Measures time for sent data to round trip, without performing VSync synchronization.
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_DOWN))
        {
            char buf[64];
            nn::pia::common::Time latencyProfileStartTime, latencyProfileEndTime;

            // Initialize measurement variable.
            latencyProfileStartTime.SetNow();
            for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
            {
#if ENABLE_UNRELIABLE_COMMUNICATION
                s_LatencyStatisticUnreliable[i].Reset();
#endif
#if ENABLE_RELIABLE_COMMUNICATION
                s_LatencyStatisticReliable[i].Reset();
#endif
            }

            nn::pia::common::Time t = nn::pia::common::Time::GetTime();
            nn::pia::common::TimeSpan timeSpan = nn::pia::common::TimeSpan(1);
            // To send back the received data as quickly as possible, this does not render or perform other actions.
            do
            {
                // Check the connection status.
                result = GetConnectionStatus();
                if (result.IsFailure())
                {
                    PIA_SAMPLE_LOG("GameLoop: Connection is lost.\n");
                    return result;
                }
#if ENABLE_NEX_NETZ
#if ENABLE_LOCAL
#else
                if (s_pHostMigrationCallback->SessionHostIsOrphan())
                {
                    // Also exit when there is no one else in the session
                    PIA_SAMPLE_LOG("GameLoop: detected as orphan\n");
                    return SampleSessionStateDisconnected();
                }
#endif
#else
                if (nn::pia::session::Session::GetInstance()->CheckConnectionError() == nn::pia::ResultSessionConnectionIsLost())
                {
                    PIA_SAMPLE_LOG("GameLoop: Session::CheckConnectionError() Session DISCONNECTED\n");
                    return SampleSessionStateDisconnected();
                }
#endif
                // Dispatch for receiving.
                NexDispatch();
                PiaDispatch();
#if ENABLE_UNRELIABLE_COMMUNICATION
                result = ProfileLatency(false, s_LatencyProfilingProtocolUnreliableHandle, s_LatencyStatisticUnreliable, MATCHMAKE_PARTICIPANTS_MAX);
#endif
#if ENABLE_RELIABLE_COMMUNICATION
                result = ProfileLatency(true, s_LatencyProfilingProtocolReliableHandle, s_LatencyStatisticReliable, MATCHMAKE_PARTICIPANTS_MAX);
#endif
                // Dispatch for sending.
                PiaDispatch();

                latencyProfileEndTime.SetNow();

                while (nn::pia::common::Time::GetTime() - t < timeSpan)
                {
                }
                t = t + timeSpan;
            } while ((latencyProfileEndTime - latencyProfileStartTime).GetSec() < LATENCY_PROFILING_PERIOD);

            // Output measurements.
            for (int i = 0; i < MATCHMAKE_PARTICIPANTS_MAX; ++i)
            {
#if ENABLE_UNRELIABLE_COMMUNICATION
                if (s_LatencyStatisticUnreliable[i].GetCount() > 0)
                {
                    sprintf(buf, "Latency Unreliable to %d", i);
                    s_LatencyStatisticUnreliable[i].Print(buf, 0);
                }
                s_LatencyStatisticUnreliable[i].Reset();
#endif
#if ENABLE_RELIABLE_COMMUNICATION
                if (s_LatencyStatisticReliable[i].GetCount() > 0)
                {
                    sprintf(buf, "Latency Reliable to %d", i);
                    s_LatencyStatisticReliable[i].Print(buf, 0);
                }
                s_LatencyStatisticReliable[i].Reset();
#endif
            }
        }
#endif  // ENABLE_NEX_NETZ
#endif  // ENABLE_LATENCY_PROFILING
        else if (nn::pia::sampleframework::SampleFramework::GetInstance()->IsTrigger(nn::pia::sampleframework::BUTTON_START))
        {
#if ENABLE_RELIABLE_COMMUNICATION
            bSendReliableTrigger = true;
#endif
#if ENABLE_UNRELIABLE_COMMUNICATION
            bSendUnreliableTrigger = true;
#endif
#if ENABLE_VOICE_COMMUNICATION
            bSendVoiceTrigger = true;
#endif
        }

#if ENABLE_CACHED_PRINT
        if (nn::pia::common::CachedPrint::GetInstance() != NULL)
        {
            nn::pia::common::CachedPrint::GetInstance()->Flush();
        }
#endif

#if ENABLE_CPU_PROFILING
#if CPU_PROFILING_REPEAT
        if (bProfiling)
        {
            // If the profiler is stopped, start profiling.
            if (!Profiler::GetInstance()->IsCounting())
            {
                s_NexDispatchCount = 0;
#if ENABLE_NEX_NETZ
                PIA_SAMPLE_PRINT("GameLoop:StartProfilingTarget:StationNum: %d\n", GetStationNum());
#endif
                Profiler::GetInstance()->StartProfilingTarget(CPU_PROFILING_PERIOD);
// Record the send/receive counters when profiling starts, and output to a log for the amount changed until next time.
#if ENABLE_UNRELIABLE_COMMUNICATION
                {
                    u32 sendCount = GetP2PSendCount() - s_NumSuccessSendUnreliable;
                    u32 recvCount = GetP2PReceiveCount() - s_NumPacketRecvUnreliable;
                    u32 dropCount = GetP2PDropCount() - s_NumPacketDropUnreliable;
                    // An unreliable transmission is sent to all stations with a single call, so we divide the receive count by the number of stations.
                    PIA_SAMPLE_PRINT("Unreliable (send, recv, ratio, drop): (%d, %d, %3.2f, %d)\n",
                                     sendCount,
                                     recvCount,
                                     static_cast<f32>(recvCount / (GetStationNum() - 1)) / static_cast<f32>(sendCount) * 100.0f,
                                     dropCount);
                    s_NumSuccessSendUnreliable = GetP2PSendCount();
                    s_NumPacketRecvUnreliable = GetP2PReceiveCount();
                    s_NumPacketDropUnreliable = GetP2PDropCount();
                }
#endif  // ENABLE_UNRELIABLE_COMMUNICATION
#if ENABLE_RELIABLE_COMMUNICATION
                {
                    u32 sendCount = GetP2PSendCountReliable() - s_NumSuccessSendReliable;
                    u32 recvCount = GetP2PReceiveCountReliable() - s_NumPacketRecvReliable;
                    u32 failCount = GetP2PSendFailedCountReliable() - s_NumFailSendReliable;
#if ENABLE_NEX_NETZ
                    // NetZ reliable transmissions are sent to each station per call, so we divide the receive count by the send/receive count.
                    PIA_SAMPLE_PRINT("Reliable (send, recv, ratio, send_fail): (%d, %d, %3.2f, %d)\n",
                                     sendCount,
                                     recvCount,
                                     static_cast<f32>(recvCount) / static_cast<f32>(sendCount) * 100.0f,
                                     failCount);
#else
                    PIA_SAMPLE_PRINT("Reliable (send, recv, ratio, send_fail): (%d, %d, %3.2f, %d)\n",
                                     sendCount,
                                     recvCount,
                                     static_cast<f32>(recvCount / (GetStationNum() - 1)) / static_cast<f32>(sendCount) * 100.0f,
                                     failCount);
#endif  // ENABLE_NEX_NETZ
                    s_NumSuccessSendReliable = GetP2PSendCountReliable();
                    s_NumPacketRecvReliable = GetP2PReceiveCountReliable();
                    s_NumFailSendReliable = GetP2PSendFailedCountReliable();
                }
#endif  // ENABLE_RELIABLE_COMMUNICATION
            }
        }
#endif  // CPU_PROFILING_REPEAT
#endif  // ENABLE_CPU_PROFILING

#if ENABLE_BLOCK_TIME_PROFILING
#if BLOCK_TIME_PROFILING_REPEAT
        if ((nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount() % (BLOCK_TIME_PROFILING_PERIOD * P2P_SESSION_FRAME_PER_SECOOND)) == 0)
        {
            bBlockTime = true;
        }
#endif
        if (bBlockTime)
        {
            s_NexSchedulerDispatchTimeStatistic.Print("Statistic NexScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            s_NexSchedulerDispatchTimeStatistic.Reset();
#if ENABLE_NEX_NETZ
#else
            s_PiaSchedulerDispatchTimeStatistic.Print("Statistic PiaScheduler::Dispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            s_PiaSchedulerDispatchTimeStatistic.Reset();
            s_PiaSchedulerPostDispatchTimeStatistic.Print("Statistic PiaScheduler::PostDispatch()", BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            s_PiaSchedulerPostDispatchTimeStatistic.Reset();
#if PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
            nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->PrintProtocolStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            nn::pia::transport::Transport::GetInstance()->GetProtocolManager()->ResetProtocolStatistic();
#endif  // PIA_STATISTIC_PROTOCOL_BLOCK_TIME_ENABLE
#if NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
            nn::pia::transport::ThreadStreamManager::GetInstance()->PrintThreadStreamStatistic(BLOCK_TIME_PROFILING_PRINT_MAX_DATA_NUM);
            nn::pia::transport::ThreadStreamManager::GetInstance()->ResetThreadStreamStatistic();
#endif  // NN_PIA_ENABLE_STATISTIC_THREAD_STREAM_BLOCK_TIME
#endif  // ENABLE_NEX_NETZ
            bBlockTime = false;
        }
#endif  // ENABLE_BLOCK_TIME_PROFILING

        result = GetConnectionStatus();
        if (result.IsFailure())
        {
            PIA_SAMPLE_LOG("GameLoop: Connection is lost.\n");
            return result;
        }

#if ENABLE_NEX_NETZ
#if ENABLE_LOCAL
#else
        if (s_pHostMigrationCallback->SessionHostIsOrphan())
        {
            // Also exit when there is no one else in the session
            PIA_SAMPLE_LOG("GameLoop: detected as orphan\n");
            return SampleSessionStateDisconnected();
        }
#endif
#else
        if (nn::pia::session::Session::GetInstance()->CheckConnectionError() == nn::pia::ResultSessionConnectionIsLost())
        {
            PIA_SAMPLE_LOG("GameLoop: Session::CheckConnectionError() Session DISCONNECTED\n");
            return SampleSessionStateDisconnected();
        }
#endif

// Dispatch (1st time)
#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
        ClearDataCache();
#endif
#endif
        NexDispatch();
        PiaDispatch();

// Receive.
#if ENABLE_VOICE_COMMUNICATION
        if (IsVoiceDataArrived())   // Starts measurement mode once at least one PiaChat data is delivered. 
        {
            bSendVoiceTrigger = true;
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
            if (!bProfiling)
            {
                bProfiling = true;
                if (!Profiler::GetInstance()->IsCounting())
                {
                    s_NexDispatchCount = 0;
                    Profiler::GetInstance()->StartProfilingTarget(CPU_PROFILING_PERIOD);
                }
            }
#endif
        }
#endif  // end of ENABLE_VOICE_COMMUNICATION
#if ENABLE_UNRELIABLE_COMMUNICATION
        if (GetP2PReceiveCount() > 0)
        {
            bSendUnreliableTrigger = true;
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
            if (!bProfiling)
            {
                bProfiling = true;
                if (!Profiler::GetInstance()->IsCounting())
                {
                    s_NexDispatchCount = 0;
                    Profiler::GetInstance()->StartProfilingTarget(CPU_PROFILING_PERIOD);
                }
            }
#endif
        }
#if ENABLE_NEX_NETZ
#if USE_DIRECTSTREAM
        ReceiveP2PData();
#endif
#else
#if ENABLE_R2_UNRELIABLE
        result = ReceiveP2PDataR2(s_R2UnreliableProtocolHandle);
#else
        result = ReceiveP2PData(s_UnreliableProtocolHandle);
#endif  // ENABLE_R2_UNRELIABLE
#endif  // ENABLE_NEX_NETZ
#endif

#if ENABLE_RELIABLE_COMMUNICATION  //n1769 Reliable receive processing
        // If you receive reliable data even once, that will trigger Reliable transmission on the local station, as well.
        // 
        if (GetP2PReceiveCountReliable() > 0)
        {
            bSendReliableTrigger = true;
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
            if (!bProfiling)
            {
                bProfiling = true;
                if (!Profiler::GetInstance()->IsCounting())
                {
                    s_NexDispatchCount = 0;
                    Profiler::GetInstance()->StartProfilingTarget(CPU_PROFILING_PERIOD);
                }
            }
#endif
        }
#if ENABLE_NEX_NETZ
#if USE_DIRECTSTREAM
        ReceiveP2PData();
#endif
#else
        result = ReceiveP2PDataReliable(s_ReliableProtocolHandle);
#endif
#endif

#if ENABLE_VOICE_COMMUNICATION
        if (bSendVoiceTrigger)
        {
            result = SendP2PDataVoice(s_VoiceProtocolHandle);
        }
#endif
#if ENABLE_UNRELIABLE_COMMUNICATION
        // Send.
        if (bSendUnreliableTrigger && (nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount() % P2P_COMMUNICATION_PERIOD) == 0)
        {
#if ENABLE_NEX_NETZ
            result = SendP2PDataNetZ();
#else
#if ENABLE_R2_UNRELIABLE
            result = SendP2PDataR2(s_R2UnreliableProtocolHandle, R2_UNRELIABLE_STATION_NUM);
#else
            result = SendP2PData(s_UnreliableProtocolHandle);
#endif
#endif
        }
#endif

#if ENABLE_RELIABLE_COMMUNICATION  //n1769 Reliable send processing
        if (bSendReliableTrigger && (nn::pia::sampleframework::SampleFramework::GetInstance()->GetFrameCount() % P2P_COMMUNICATION_PERIOD) == 0)
        {
#if ENABLE_NEX_NETZ
            result = SendP2PDataReliableNetZ();
#else
            result = SendP2PDataReliable(s_ReliableProtocolHandle);
#endif
        }
#endif

// Dispatch (2nd time)
#if ENABLE_NEX_NETZ
        NexDispatch();
#endif
        PiaDispatch();

#if ENABLE_LATENCY_PROFILING
#if ENABLE_NEX_NETZ
#else
#if ENABLE_UNRELIABLE_COMMUNICATION
        result = ProfileLatency(false, s_LatencyProfilingProtocolUnreliableHandle, s_LatencyStatisticUnreliable, MATCHMAKE_PARTICIPANTS_MAX);
#endif
#if ENABLE_RELIABLE_COMMUNICATION
        result = ProfileLatency(true, s_LatencyProfilingProtocolReliableHandle, s_LatencyStatisticReliable, MATCHMAKE_PARTICIPANTS_MAX);
#endif
#endif
#endif

        nn::pia::common::Time nowTime = nn::pia::common::Time::GetTime();
        if (nowTime - tiLogStart >= P2P_SESSION_LOG_PERIOD)
        {
            // Display connected stations.
            tiLogStart = nn::pia::common::Time::GetTime();
            TraceP2PSession();
        }

#if ENABLE_AUTO_SESSION_REPEAT
        nowTime = nn::pia::common::Time::GetTime();
        if ((nowTime - startTime).GetMSec() > span)
        {
            PIA_SAMPLE_PRINT("GameLoop: AutoSessionRepeatTimer expired!\n");
            DrawScreen();
#if PIA_TEST_CODE
            UpdatePiaTestData();
#endif
            nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
            break;
        }
#endif

        DrawScreen();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }

    PIA_SAMPLE_LOG("GameLoop: succeeded\n");
    return nn::ResultSuccess();
}

nn::Result InitializeAdditionalProtocols()
{
#if ENABLE_NEX_NETZ
#else
    nn::Result result;

#if ENABLE_R2_UNRELIABLE || ENABLE_VOICE_COMMUNICATION || (ENABLE_LATENCY_PROFILING&& ENABLE_UNRELIABLE_COMMUNICATION) || (ENABLE_LATENCY_PROFILING&& ENABLE_RELIABLE_COMMUNICATION)
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
#endif

#if ENABLE_R2_UNRELIABLE
    s_R2UnreliableProtocolHandle = pTransport->CreateProtocol<nn::pia::transport::RoundRobinUnreliableProtocol>();
    PIA_SAMPLE_ASSERT(s_R2UnreliableProtocolHandle != 0);
    result = pTransport->GetProtocol<nn::pia::transport::RoundRobinUnreliableProtocol>(s_R2UnreliableProtocolHandle)->Initialize();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("InitializeAdditionalProtocols: GetProtocol<nn::pia::transport::RoundRobinUnreliableProtocol>(s_R2UnreliableHandle)->Initialize failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }
#endif
#if ENABLE_VOICE_COMMUNICATION
    s_VoiceProtocolHandle = pTransport->CreateProtocol<nn::pia::chat::VoiceProtocol>();
    PIA_SAMPLE_ASSERT(s_VoiceProtocolHandle != 0);
    nn::pia::chat::VoiceProtocol::Setting voiceSetting;
    result = pTransport->GetProtocol<nn::pia::chat::VoiceProtocol>(s_VoiceProtocolHandle)->Initialize(voiceSetting);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("InitializeAdditionalProtocols: GetProtocol<nn::pia::chat::VoiceProtocol>(s_VoiceProtocolHandle)->Initialize failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }
#endif
#if ENABLE_LATENCY_PROFILING
    const u16 cLatencyProfilingPort = 0x10;

#if ENABLE_UNRELIABLE_COMMUNICATION
    s_LatencyProfilingProtocolUnreliableHandle = pTransport->CreateProtocol<nn::pia::transport::UnreliableProtocol>(cLatencyProfilingPort);
    PIA_SAMPLE_ASSERT(s_LatencyProfilingProtocolUnreliableHandle != 0);
    result = pTransport->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_LatencyProfilingProtocolUnreliableHandle)->Initialize();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("InitializeAdditionalProtocols: GetProtocol<nn::pia::transport::UnreliableProtocol>(s_LatencyProfilingProtocolUnreliableHandle)->Initialize failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }
#endif
#if ENABLE_RELIABLE_COMMUNICATION
    s_LatencyProfilingProtocolReliableHandle = pTransport->CreateProtocol<nn::pia::transport::ReliableProtocol>(cLatencyProfilingPort);
    PIA_SAMPLE_ASSERT(s_LatencyProfilingProtocolReliableHandle != 0);
    result = pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_LatencyProfilingProtocolReliableHandle)->Initialize(32, 32);
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("InitializeAdditionalProtocols: GetProtocol<nn::pia::transport::ReliableProtocol>(s_LatencyProfilingProtocolReliableHandle)->Initialize failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        return result;
    }
#endif
#endif  // ENABLE_LATENCY_PROFILING
#endif  // ENABLE_NEX_NETZ
    return nn::ResultSuccess();
}

void FinalizeAdditionalProtocols()
{
#if ENABLE_NEX_NETZ
#else
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();

    if (pTransport != NULL)
    {
#if ENABLE_VOICE_COMMUNICATION
        {
            nn::pia::chat::VoiceProtocol* pProtocol = pTransport->GetProtocol<nn::pia::chat::VoiceProtocol>(s_VoiceProtocolHandle);
            if (pProtocol != NULL)
            {
                pProtocol->Finalize();
                pTransport->DestroyProtocol(s_VoiceProtocolHandle);
            }
        }
#endif
#if ENABLE_R2_UNRELIABLE
        {
            nn::pia::transport::RoundRobinUnreliableProtocol* pProtocol = pTransport->GetProtocol<nn::pia::transport::RoundRobinUnreliableProtocol>(s_R2UnreliableProtocolHandle);
            if (pProtocol != NULL)
            {
                pProtocol->Finalize();
                pTransport->DestroyProtocol(s_R2UnreliableProtocolHandle);
            }
        }
#endif
#if ENABLE_LATENCY_PROFILING
#if ENABLE_UNRELIABLE_COMMUNICATION
        {
            nn::pia::transport::UnreliableProtocol* pProtocol = pTransport->GetProtocol<nn::pia::transport::UnreliableProtocol>(s_LatencyProfilingProtocolUnreliableHandle);
            if (pProtocol != NULL)
            {
                pProtocol->Finalize();
                pTransport->DestroyProtocol(s_LatencyProfilingProtocolUnreliableHandle);
            }
        }
#endif
#if ENABLE_RELIABLE_COMMUNICATION
        {
            nn::pia::transport::ReliableProtocol* pProtocol = pTransport->GetProtocol<nn::pia::transport::ReliableProtocol>(s_LatencyProfilingProtocolReliableHandle);
            if (pProtocol != NULL)
            {
                pProtocol->Finalize();
                pTransport->DestroyProtocol(s_LatencyProfilingProtocolReliableHandle);
            }
        }
#endif
#endif  // ENABLE_LATENCY_PROFILING
    }
#endif  // ENABLE_NEX_NETZ
}

//-----------------------------------------------------------------------------
// Internal function definitions
//-----------------------------------------------------------------------------
nn::Result InitializeNetwork()
{
#if ENABLE_LOCAL&& NN_PIA_CTR
    return InitializeLocal();
#else
    return InitializeInternet();
#endif
}

void FinalizeNetwork()
{
#if ENABLE_LOCAL&& NN_PIA_CTR
    FinalizeLocal();
#else
    FinalizeInternet();
#endif
}

void MainRoutine();

//-----------------------------------------------------------------------------
// Entry point.
//-----------------------------------------------------------------------------
PIA_SAMPLE_MAIN_FUNC()
{
    PIA_SAMPLE_PRINT("*******************************\n");
    PIA_SAMPLE_PRINT("PiaInet Sample / ProfilingTest\n");
    PIA_SAMPLE_PRINT("*******************************\n\n");

#if NN_PIA_DEBUG
    PIA_SAMPLE_PRINT("Build : Debug\n");
#elif NN_PIA_DEVELOP
    PIA_SAMPLE_PRINT("Build : Development\n");
#elif NN_PIA_RELEASE
    PIA_SAMPLE_PRINT("Build : Release\n");
#endif

    PIA_SAMPLE_PRINT("PIA REVISION:%s\n", PIA_VERSION_STRING " " PIA_VER_STR(PIA_VERSION_DATE) " r" PIA_VER_STR(PIA_VERSION_REVISION));
    PIA_SAMPLE_PRINT("Nex Version : %s\n", NEX_VERSION_STRING);
#if NN_PIA_CTR
    PIA_SAMPLE_PRINT("SDK Version : %s\n", NN_CURRENT_SDK_VERSION);
    PIA_SAMPLE_PRINT("Firmware Version : %s\n", NN_CURRENT_FIRMWARE_VERSION);
#elif NN_PIA_CAFE
    PIA_SAMPLE_PRINT("SDK Version : %s\n", CAFE_OS_SDK_VERSION_STRING);
    PIA_SAMPLE_PRINT("RunningCore : %d\n", OSGetCoreId());
#endif

#if NN_PIA_CTR
    nn::fnd::DateTime dateTime = nn::fnd::DateTime::GetNow();
    PIA_SAMPLE_PRINT("Date : %d/%02d/%02d %02d:%02d:%02d\n",
                     dateTime.GetYear(),
                     dateTime.GetMonth(),
                     dateTime.GetDay(),
                     dateTime.GetHour(),
                     dateTime.GetMinute(),
                     dateTime.GetSecond());
#elif NN_PIA_WIN
    SYSTEMTIME st;
    ::GetLocalTime(&st);

    PIA_SAMPLE_PRINT("Date : %d/%02d/%02d %02d:%02d:%02d\n",
                     st.wYear,
                     st.wMonth,
                     st.wDay,
                     st.wHour,
                     st.wMinute,
                     st.wSecond)
#elif NN_PIA_CAFE
    OSCalendarTime td;
    OSTicksToCalendarTime(OSGetTime(), &td);
    PIA_SAMPLE_PRINT("Date : %d/%02d/%02d %02d:%02d:%02d\n",
                     td.year,
                     td.mon + 1,
                     td.mday,
                     td.hour,
                     td.min,
                     td.sec)
#endif

    PIA_SAMPLE_PRINT("TRANSPORT_V3:%d\n", NN_PIA_TRANSPORT_V3);

    PIA_SAMPLE_PRINT("ENABLE_LOCAL:%d\n", ENABLE_LOCAL);
    PIA_SAMPLE_PRINT("ENABLE_SIGNATURE:%d\n", ENABLE_SIGNATURE);
    PIA_SAMPLE_PRINT("ENABLE_CACHED_PRINT:%d\n", ENABLE_CACHED_PRINT);
    PIA_SAMPLE_PRINT("ENABLE_NEX_NETZ:%d\n", ENABLE_NEX_NETZ);
    PIA_SAMPLE_PRINT("ENABLE_CRYPTO:%d\n", ENABLE_CRYPTO);
    PIA_SAMPLE_PRINT("USE_MULTIUNITHEAP:%d\n", USE_MULTIUNITHEAP);

    nn::pia::sampleframework::SampleFramework::CreateInstance();
    nn::pia::sampleframework::SampleFramework::GetInstance()->SetFramePerSecond(P2P_SESSION_FRAME_PER_SECOOND);
//    nn::pia::sampleframework::SampleFramework::GetInstance()->SetCacedPrintFlushSpan(30 * 5);

#if ENABLE_CPU_PROFILING
#if NN_PIA_CTR
    nn::os::InitializeAlarmSystem();
#endif
    Profiler::CreateInstance();
    PIA_SAMPLE_PRINT("start profiling idle\n");
    Profiler::GetInstance()->StartProfilingIdle(CPU_PROFILING_PERIOD);

    PIA_SAMPLE_PRINT("start profiling dummy loop\n");
    Profiler::GetInstance()->StartProfilingBase(CPU_PROFILING_PERIOD);
    ProfilingDummyLoop();
#elif ENABLE_MEMORY_PROFILING
#if NN_PIA_CTR
    nn::os::InitializeAlarmSystem();
#endif
    Profiler::CreateInstance();
#endif

    // With while(1), hangs on a compiler warning, so loop using a sufficiently large value.
    while (g_MainLoopCnt++ < 10000)
    {
        PIA_SAMPLE_PRINT("****** start MainRoutine() [%d] ******\n", g_MainLoopCnt);
        MainRoutine();
        PIA_SAMPLE_PRINT("****** end   MainRoutine() [%d] ******\n", g_MainLoopCnt);
    }

#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
    Profiler::DestroyInstance();
#endif

    nn::pia::sampleframework::SampleFramework::DestroyInstance();
}

#if EXPERIMENT_JOIN_DENY_AT_FIRST_TIME

bool joinApprovalCallback(const nn::pia::transport::Station::IdentificationToken* pRequesterToken)
{
    NN_PIA_DUMMY_PARAM(pRequesterToken);

    static s32 s_ApprovalCount = 0;
    PIA_SAMPLE_LOG("joinApprovalCallback:%d\n", s_ApprovalCount);
    if (s_ApprovalCount == 0)
    {
        s_ApprovalCount++;
        return false;
    }
    else
    {
        return true;
    }
}

#endif

void MainRoutine()
{
    nn::Result result;

    u64 traceFlags = 0
#if ENABLE_TRACE_PIA
                     | nn::pia::common::TRACE_FLAG_ALWAYS | nn::pia::common::TRACE_FLAG_COMMON | nn::pia::common::TRACE_FLAG_TRANSPORT | nn::pia::common::TRACE_FLAG_SESSION | nn::pia::common::TRACE_FLAG_INET | nn::pia::common::TRACE_FLAG_LOCAL
#if ENABLE_TRACE_PIA_DETAIL
                     | nn::pia::common::TRACE_FLAG_INET_NAT | nn::pia::common::TRACE_FLAG_INET_NAT_DETAIL | nn::pia::common::TRACE_FLAG_INET_SOCKET | nn::pia::common::TRACE_FLAG_INET_INIT | nn::pia::common::TRACE_FLAG_INET_NAT_PROBE | nn::pia::common::TRACE_FLAG_SESSION_CONNECT | nn::pia::common::TRACE_FLAG_SESSION_LEAVE | nn::pia::common::TRACE_FLAG_SESSION_UPDATE
#if 0
        | nn::pia::common::TRACE_FLAG_COMMON_JOB
        | nn::pia::common::TRACE_FLAG_LOCAL_TRANSPORT
        | nn::pia::common::TRACE_FLAG_TRANSPORT_PACKET_SEND
        | nn::pia::common::TRACE_FLAG_TRANSPORT_PACKET_RECEIVE
        | nn::pia::common::TRACE_FLAG_TRANSPORT_RELAY
        | nn::pia::common::TRACE_FLAG_SESSION_CONNECT
        | nn::pia::common::TRACE_FLAG_SESSION_LEAVE
        | nn::pia::common::TRACE_FLAG_SESSION_UPDATE
#endif
#endif  // ENABLE_TRACE_PIA_DETAIL
#endif  // ENABLE_TRACE_PIA
        ;

    // Always used with this sample to trace PiaCommon.
    result = Common_InitializePiaCommon(s_PiaHeapBuffer, SAMPLE_PIA_MEM_SIZE, traceFlags, ENABLE_CACHED_PRINT);
    PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

#if NN_PIA_ENABLE_FEEDBACK
    NN_PIA_FEEDBACK();
#endif

#if ENABLE_CPU_PROFILING
    nn::pia::common::Log::SetGlobalObject(NULL);
#else  // ENABLE_CPU_PROFILING
    AppTrace::GetInstance()->SetFlag(
        SAMPLE_TRACE_ALWAYS
#if 0
        | SAMPLE_TRACE_SEND
        | SAMPLE_TRACE_RECEIVE
#endif
        );
#endif  // ENABLE_CPU_PROFILING

#if ENABLE_WATERMARK
    StartWatermark();
#endif

    result = InitializeNetwork();
    if (result.IsFailure())
    {
        PIA_SAMPLE_LOG("MainRoutine: InitializeNetwork failed.\n");
        PIA_SAMPLE_LOG_RESULT(result);
        ShowErrorCodeLoop(result);
        goto CLEANUP;
    }

#if ENABLE_NEX_NETZ
#else
    {
        nn::pia::transport::NetworkFactory* pNetworkFactory = NULL;

#if ENABLE_LOCAL&& NN_PIA_CTR
        nn::pia::local::UdsNetworkFactory networkFactory;
#else  // ENABLE_LOCAL && NN_PIA_CTR
#if ENABLE_LAN_NETWORK
        nn::pia::inet::NexFacade::EnableLanNetwork(true);
        nn::pia::inet::NexLanNetworkFactory networkFactory;
#else
        nn::pia::inet::NexNetworkFactory networkFactory;
#endif
#endif  // ENABLE_LOCAL && NN_PIA_CTR
        pNetworkFactory = &networkFactory;

        nn::pia::transport::Transport::Setting setting;
        setting.pFactory = pNetworkFactory;
        setting.maxStationNum = GAME_PARTICIPANTS_NUM;  // Maximum number of stations
        size_t reliableSendBufNum = 0;
        size_t reliableRecvBufNum = 0;
        size_t unreliableRecvBufNum = 0;
        bool ret = UpdateSetting(&setting, &reliableSendBufNum, &reliableRecvBufNum, &unreliableRecvBufNum);
        PIA_SAMPLE_ASSERT(ret);
        NN_PIA_DUMMY_PARAM_IN_RELEASE(ret);

// Get ready to use latency emulation feature. Set the number of buffers.
#if ENABLE_LATENCY_EMULATION
        {
            nn::pia::transport::Transport::DebugSetting debugSetting;
            debugSetting.sendThreadLatencyEmulationPacketNum = 128;
            debugSetting.receiveThreadLatencyEmulationPacketNum = 256;
            debugSetting.bPacketLossEmulation = false;
            result = nn::pia::transport::Transport::SetDebugSetting(debugSetting);
            PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

            PIA_SAMPLE_PRINT("Latency emulation buf num: (send, receive) = (%d, %d)\n",
                             debugSetting.sendThreadLatencyEmulationPacketNum,
                             debugSetting.receiveThreadLatencyEmulationPacketNum);
        }
#else
        PIA_SAMPLE_PRINT("Latency emulation is disabled.\n");
#endif  // end of ENABLE_LATENCY_EMULATION

        // Each of the Pia modules must be initialized in this order
        result = Transport_InitializePiaTransport(
            setting,
            0,
#if ENABLE_UNRELIABLE_COMMUNICATION
            &s_UnreliableProtocolHandle,
#else
            NULL,
#endif
            1,
#if ENABLE_RELIABLE_COMMUNICATION
            &s_ReliableProtocolHandle,
#else
            NULL,
#endif
            reliableSendBufNum,
            reliableRecvBufNum,
            InitializeAdditionalProtocols,
            unreliableRecvBufNum);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        NN_PIA_DUMMY_PARAM(s_UnreliableProtocolHandle);
        NN_PIA_DUMMY_PARAM(s_ReliableProtocolHandle);

// Set the time used for latency emulation.
#if ENABLE_LATENCY_EMULATION
        {
            nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMgr));
            const s32 SEND_MIN = 0;
            const s32 SEND_MAX = 500;
            const s32 RECV_MIN = 0;
            const s32 RECV_MAX = 500;
            pMgr->SetSendThreadLatencyEmulation(SEND_MIN, SEND_MAX);
            pMgr->SetReceiveThreadLatencyEmulation(RECV_MIN, RECV_MIN);
            PIA_SAMPLE_PRINT("Latency emulation time (send) (min, max) = (%d, %d)\n", SEND_MIN, SEND_MAX);
            PIA_SAMPLE_PRINT("Latency emulation time (recv) (min, max) = (%d, %d)\n", RECV_MIN, RECV_MAX);
        }
#endif  // end of ENABLE_LATENCY_EMULATION
        /*
        nn::pia::session::Session::DebugSetting debugSessionSetting;
        debugSessionSetting.isEnableRelayEmulation = true;
        nn::pia::session::Session::SetDebugSetting(debugSessionSetting);
*/
        nn::pia::session::Session::Setting sessionSetting;
        sessionSetting.pNetworkFactory = pNetworkFactory;
// Configure the network topology.
#if USE_RELAY_CONNECTION
        sessionSetting.networkTopology = nn::pia::session::NetworkTopology_RelayMesh;
#else
        sessionSetting.networkTopology = nn::pia::session::NetworkTopology_FullMesh;
#endif  // end of USE_RELAY_CONNECTION

#if ENABLE_LOCAL
        // The setting for the number of <tt>SessionInfo</tt> to browse has an impact on memory.
        sessionSetting.browsedSessionInfoListNum = 1;
#endif
        result = Session_InitializePiaSession(sessionSetting);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);

#if USE_RELAY_CONNECTION
        // Advanced settings for relay connection feature after Session initialization
        nn::pia::session::Session* pSession = nn::pia::session::Session::GetInstance();
        result = pSession->SetMaxNumOfRelayRoutesAssignedToStation(SAMPLE_MAX_RELAY_NUM);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
        result = pSession->SetMaxRtt(SAMPLE_MAX_RTT);
        PIA_SAMPLE_ASSERT_RESULT_IS_SUCCESS(result);
#endif  // end of USE_RELAY_CONNECTION
    }
#endif

    ClearCounters();  // Clears to zero for the send/receive counters.

// Main loop.
#if ENABLE_LOCAL&& NN_PIA_CTR
    MainLoopLocal();
#else
    MainLoopInternet();
#endif

CLEANUP:
// Finalization must occur in the reverse order of initialization
// Even if finalization is called in an uninitialized state, it succeeds.
#if ENABLE_NEX_NETZ
#else
    Session_FinalizePiaSession();
    Transport_FinalizePiaTransport(
        s_UnreliableProtocolHandle,
        s_ReliableProtocolHandle,
        FinalizeAdditionalProtocols);
#endif
    FinalizeNetwork();
    Common_FinalizePiaCommon();

    PIA_SAMPLE_PRINT("MainRoutine() end.\n");
}
