﻿/*---------------------------------------------------------------------------*
Project:  Pia
File:     ProfilingTest_MemoryMonitor.h

Copyright Nintendo.  All rights reserved.

These coded instructions, statements, and computer programs contain
proprietary information of Nintendo of America Inc. and/or Nintendo
Company Ltd., and are protected by Federal copyright law.  They may
not be disclosed to third parties or copied or duplicated in any form,
in whole or in part, without the prior written consent of Nintendo.
*---------------------------------------------------------------------------*/


#ifndef PROFILINGTEST_MEMORYMONITOR_H_
#define PROFILINGTEST_MEMORYMONITOR_H_

#include <cstddef>

#include "ProfilingTest_StatisticalRecord.h"

#include <pia/common/common_Time.h>


///< This class monitors things like the amount of memory used.
///< This class is currently designed for use with MultiUnitHeap.
class MemoryMonitor
{
public:
    MemoryMonitor(void);
    ~MemoryMonitor(void);

    void Update(void);                  ///< Updates the memory use information.
    void Render(void) const;            ///< Writes information the LCD screen.
    void PrintMemoryUsage(void) const;  ///< Prints information to the log output. Does not print anything, however, if nothing has changed compared to the previous output.

    void StartAllocationMonitor(void);
    void StopAllocationMonitor(void);
    void PrintAllocationResult(void) const;

private:
    ///< Constants
    static const size_t SPAN = 32;  ///< Size of the history that records memory use.
    static const int WORKBUF_SIZE = 256;

    ///< The copy constructor and the assignment operator are private.
    MemoryMonitor(const MemoryMonitor&);
    MemoryMonitor& operator=(const MemoryMonitor&);

    int m_workBuf[WORKBUF_SIZE / sizeof(int)];
    StatisticalRecord m_memRec;

    std::size_t m_currMemUsage;
    std::size_t m_prevMemUsage;

    nn::pia::common::Time m_standardTime;

    bool m_AllocationMonitoring;
};


#endif  // end of PROFILINGTEST_MEMORYMONITOR_H_
