﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_P2P.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef PROFILING_TEST_P2P_H_
#define PROFILING_TEST_P2P_H_

#include "ProfilingTest_Common.h"

// Send data size.
#if ENABLE_CPU_PROFILING || ENABLE_MEMORY_PROFILING
#define SEND_DATA_SIZE 512
#define SEND_DATA_SIZE_RELIABLE 128
#else
#define SEND_DATA_SIZE 128  // nn::pia::inet::COMMUNICATION_DATA_SIZE_MAX
#define SEND_DATA_SIZE_RELIABLE 1024
#endif

nn::Result CreateP2PSession();
nn::Result JoinP2PSession(const nn::nex::qList<nn::nex::StationURL>& lstURL);
nn::Result JoinP2PSession(const nn::pia::transport::StationConnectionInfo& connectionInfo);

#if ENABLE_NEX_NETZ
nn::Result DestroyP2PSession(nn::nex::NetZ* pNetZ);
nn::Result SendP2PDataNetZ();
nn::Result SendP2PDataReliableNetZ();
#if USE_DIRECTSTREAM
void ReceiveP2PData();
void CancelSendP2PDataReliableNetZ();
#else
void P2PReceiveCallback(nn::nex::StationID sourceId, u8* buffer, u32 size);
#endif  // USE_DIRECTSTREAM
#else
nn::Result DestroyP2PSession();
nn::Result LeaveP2PSession();
nn::Result SendP2PData(u32 unreliableProtocolHandle);
nn::Result SendP2PDataR2(u32 r2UnreliableProtocolHandle, u32 stationNum);
nn::Result ReceiveP2PData(u32 unreliableProtocolHandle);
nn::Result ReceiveP2PDataR2(u32 r2UnreliableProtocolHandle);
nn::Result SendP2PDataReliable(u32 reliableProtocolHandle);
nn::Result ReceiveP2PDataReliable(u32 reliableProtocolHandle);
nn::Result SendP2PDataVoice(u32 voiceProtocolHandle);
nn::Result ReceiveP2PDataVoice(u32 voiceProtocolHandle);
bool IsVoiceDataArrived();
void P2PReceiveCallback(nn::pia::StationId sourceId, bit8* buffer, u32 size);
#if ENABLE_LATENCY_PROFILING
typedef Statistic<nn::pia::common::TimeSpan> LatencyStatistic;
nn::Result ProfileLatency(bool isReliable, u32 protocolHandle, LatencyStatistic* pStatistic, u32 statisticNum);
#endif
#endif

void TraceP2PSession();

u16 GetStationNum();
bool IsSessionHost();
u32 GetJoinTime();
u32 GetP2PSendCount();
u32 GetP2PReceiveCount();
u32 GetP2PDropCount();
#if ENABLE_NEX_NETZ
u32 GetP2PDropCount(nn::nex::ConnectionID id);
#else
u32 GetP2PDropCount(nn::pia::StationId id);
#endif
u32 GetP2PSendCountReliable();
u32 GetP2PSendFailedCountReliable();
u32 GetP2PReceiveCountReliable();
void ClearCounters();
#endif
