﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Profiling.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "ProfilingTest_Profiling.h"
#include "ProfilingTest_P2P.h"

#if NN_PIA_CTR
#include <nn/version.h>
#elif NN_PIA_CAFE
#include <sdk_ver.h>
#endif

#if ENABLE_CPU_PROFILING

#if NN_PIA_CTR
#include <nn/ac.h>
#include <nn/cfg.h>
#elif NN_PIA_CAFE
#include <nn/ac.h>
#else
#error  "Invalid platform";
#endif

#include <pia/transport/transport_ThreadStreamManager.h>

void ProfilingDummyLoop()
{
    while (true)
    {
        nn::pia::sampleframework::SampleFramework::GetInstance()->BeginFrame();

#if NN_PIA_CAFE
#if USE_CLEAR_DATACACHE
        // Puts into a state where no cache hit occurs for the operating core data cache.
        ClearDataCache();
#endif
#endif
        // Insert into a dummy loop because there is a possibility that GetTime cannot be ignored
        nn::pia::common::Time nowTime = nn::pia::common::Time::GetTime();
        NN_PIA_DUMMY_PARAM(nowTime);

        if (!Profiler::GetInstance()->IsCounting())
        {
            break;
        }

        //result = Dispatch();
        nn::Result result = nn::ResultSuccess();
        if (result.IsFailure())
        {
            return;
        }

        DrawScreen();
#if PIA_TEST_CODE
        UpdatePiaTestData();
#endif
        nn::pia::sampleframework::SampleFramework::GetInstance()->EndFrame();
    }
}

#endif  // ENABLE_CPU_PROFILING

void PrintProfilingSetting()
{
    PIA_SAMPLE_PRINT("******** Profiling Setting ********\n");

    PIA_SAMPLE_PRINT("Application : PiaInet Sample / ProfilingTest\n");

#if NN_PIA_DEBUG
    PIA_SAMPLE_PRINT("Build : Debug\n");
#elif NN_PIA_DEVELOP
    PIA_SAMPLE_PRINT("Build : Development\n");
#elif NN_PIA_RELEASE
    PIA_SAMPLE_PRINT("Build : Release\n");
#endif

#if NN_PIA_CTR
    PIA_SAMPLE_PRINT("SDK Version : %s\n", NN_CURRENT_SDK_VERSION);
    PIA_SAMPLE_PRINT("Firmware Version : %s\n", NN_CURRENT_FIRMWARE_VERSION);
#elif NN_PIA_CAFE
    PIA_SAMPLE_PRINT("SDK Version : %s\n", CAFE_OS_SDK_VERSION_STRING);
#endif
    PIA_SAMPLE_PRINT("Nex Version : %s\n", NEX_VERSION_STRING);

#if ENABLE_NEX_NETZ
    PIA_SAMPLE_PRINT("Library : NetZ\n");
#else
    PIA_SAMPLE_PRINT("Library : Pia\n");
#endif

#if NN_PIA_CTR
    nn::fnd::DateTime dateTime = nn::fnd::DateTime::GetNow();
    PIA_SAMPLE_PRINT("Date : %d/%02d/%02d %02d:%02d:%02d\n",
                     dateTime.GetYear(),
                     dateTime.GetMonth(),
                     dateTime.GetDay(),
                     dateTime.GetHour(),
                     dateTime.GetMinute(),
                     dateTime.GetSecond());
#elif NN_PIA_CAFE
    OSCalendarTime calenderTime;
    OSTicksToCalendarTime(OSGetTime(), &calenderTime);
    PIA_SAMPLE_PRINT("Date : %d/%02d/%02d %02d:%02d:%02d\n",
                     calenderTime.year,
                     calenderTime.mon + 1,
                     calenderTime.mday,
                     calenderTime.hour,
                     calenderTime.min,
                     calenderTime.sec);
#endif

#if NN_PIA_CTR
    nn::ac::ApType apType;
    nn::ac::Initialize();
    nn::Result r = nn::ac::GetConnectingApType(&apType);
    const char* apTypeText = "Unknown";
    if (r.IsSuccess())
    {
        switch (apType)
        {
        case nn::ac::CTR::AP_TYPE_NONE:
            apTypeText = "None";
            break;
        case nn::ac::CTR::AP_TYPE_USER_SETTING_1:
            apTypeText = "UserSetting1";
            break;
        case nn::ac::CTR::AP_TYPE_USER_SETTING_2:
            apTypeText = "UserSetting2";
            break;
        case nn::ac::CTR::AP_TYPE_USER_SETTING_3:
            apTypeText = "UserSetting3";
            break;
        case nn::ac::CTR::AP_TYPE_NINTENDO_WIFI_USB_CONNECTOR:
            apTypeText = "USBConnector";
            break;
        case nn::ac::CTR::AP_TYPE_NINTENDO_ZONE:
            apTypeText = "NintendoZone";
            break;
        case nn::ac::CTR::AP_TYPE_WIFI_STATION:
            apTypeText = "WiFiStation";
            break;
        case nn::ac::CTR::AP_TYPE_HOTSPOT:
            apTypeText = "HotSpot";
            break;
        };
    }
    PIA_SAMPLE_PRINT("AP type : %s\n", apTypeText);
#endif

#if NN_PIA_CTR
    nn::cfg::Initialize();
    PIA_SAMPLE_PRINT("TransferableId : %016llx\n", nn::cfg::CTR::GetTransferableId(0xFF000));
#endif

    PIA_SAMPLE_PRINT("Frame per second : %d\n", P2P_SESSION_FRAME_PER_SECOOND);
#if ENABLE_VOICE_COMMUNICATION
    PIA_SAMPLE_PRINT("Voice communication is enabled.");
#else
    PIA_SAMPLE_PRINT("Voice communication is disabled.");
#endif
#if ENABLE_UNRELIABLE_COMMUNICATION
    PIA_SAMPLE_PRINT("Unreliable communication period : %d\n", P2P_COMMUNICATION_PERIOD);
    PIA_SAMPLE_PRINT("Unreliable send data size : %d byte\n", SEND_DATA_SIZE);
#else
    PIA_SAMPLE_PRINT("Unreliable communication is disabled.\n");
#endif

#if ENABLE_RELIABLE_COMMUNICATION
    PIA_SAMPLE_PRINT("Reliable communication period : %d\n", P2P_COMMUNICATION_PERIOD);
    PIA_SAMPLE_PRINT("Reliable send data size : %d byte\n", SEND_DATA_SIZE_RELIABLE);
#else
    PIA_SAMPLE_PRINT("Reliable communication is disabled.\n");
#endif

#if ENABLE_NEX_NETZ
    PIA_SAMPLE_PRINT("Dispatch timeout : %d ms\n", NEX_DISPATCH_TIMEOUT);
#else
    PIA_SAMPLE_PRINT("Dispatch timeout : %d ms\n", PIA_DISPATCH_TIMEOUT);
#endif

    PIA_SAMPLE_PRINT("Matchmake participants max : %d\n", MATCHMAKE_PARTICIPANTS_MAX);
    PIA_SAMPLE_PRINT("Game participants num : %d\n", GAME_PARTICIPANTS_NUM);

    PIA_SAMPLE_PRINT("NEX thread mode : %d\n", nn::nex::Core::GetThreadMode());
    PIA_SAMPLE_PRINT("NEX TransportBufferThreadSleepSpan : %d ms\n", nn::nex::RootTransport::GetTransportBufferThreadSleepSpan());
    PIA_SAMPLE_PRINT("NEX TransportBufferThreadSendPriority : %d\n", nn::nex::RootTransport::GetTransportBufferThreadSendPriority());
    PIA_SAMPLE_PRINT("NEX TransportBufferThreadRecvPriority : %d\n", nn::nex::RootTransport::GetTransportBufferThreadRecvPriority());
    PIA_SAMPLE_PRINT("NEX dispatch period in p2p session: %d\n", NEX_DISPATCH_PERIOD_IN_P2P_SESSION);

#if ENABLE_NEX_NETZ
#else
    nn::pia::transport::ThreadStreamManager* pMgr = nn::pia::transport::ThreadStreamManager::GetInstance();
    if (pMgr)
    {
        PIA_SAMPLE_PRINT("Send thread priority : %d\n", pMgr->GetSendThreadPriority());
        PIA_SAMPLE_PRINT("Send thread sleep span : %d ms\n", pMgr->GetSendThreadSleepTimeSpan());
        PIA_SAMPLE_PRINT("Receive thread priority : %d\n", pMgr->GetReceiveThreadPriority());
        PIA_SAMPLE_PRINT("Receive thread sleep span : %d ms\n", pMgr->GetReceiveThreadSleepTimeSpan());
    }
#endif

#if ENABLE_NEX_NETZ
    PIA_SAMPLE_PRINT("Station : %s\n", nn::nex::Session::GetInstance()->IsADuplicationMaster() ? "Host" : "Client");
#else
    PIA_SAMPLE_PRINT("Station : %s\n",
                     nn::pia::session::Session::GetInstance()->GetHostStationId() == nn::pia::session::Session::GetInstance()->GetLocalStationId() ? "Host" : "Client");
#endif

    PIA_SAMPLE_PRINT("***********************************\n");
}


#if ENABLE_WATERMARK
// Start the monitoring feature of the numbers of various buffer elements within Pia using WatermarkManager.
void StartWatermark()
{
    if (PIA_IS_VALID_POINTER(nn::pia::common::WatermarkManager::GetInstance()))
    {
        nn::pia::common::WatermarkManager::GetInstance()->EnableAllWatermark();
    }
    else
    {
        PIA_SAMPLE_PRINT("It seems that WatermarkManager instance is not ready.\n");
    }
}

// Print out the measured result from the execution of the StartWatermark function until now.
void PrintWatermark()
{
    nn::pia::common::WatermarkManager* pMgr = nn::pia::common::WatermarkManager::GetInstance();
    if (PIA_IS_VALID_POINTER(pMgr))
    {
        for (int i = 0; i <= nn::pia::common::WatermarkManager::KEY_MAX; ++i)
        {
            nn::pia::common::Watermark* pMark = pMgr->GetWatermark(i);
            PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMark));
            if (pMark->GetUpdateCount() > 0)
            {
                PIA_SAMPLE_PRINT("Watermark: <%s>  max: %lld, min: %lld, latest: %lld, Updates: %lld\n",
                                 pMark->GetName(),
                                 pMark->GetMaxValue(),
                                 pMark->GetMinValue(),
                                 pMark->GetLatestValue(),
                                 pMark->GetUpdateCount());
            }
        }
    }
    else
    {
        PIA_SAMPLE_PRINT("It seems that WatermarkManager instance is not ready.\n");
    }

#if ENABLE_CACHED_PRINT
    if (nn::pia::common::CachedPrint::GetInstance() != NULL)
    {
        nn::pia::common::CachedPrint::GetInstance()->Flush();
    }
#endif

}
#endif  // end of ENABLE_WATERMARK
