﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Setting.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "ProfilingTest_Setting.h"

#include <pia/common.h>

#include "ProfilingTest_Common.h"


namespace
{
#if NN_PIA_CTR
nn::pia::transport::Transport::Setting s_Setting(NULL, 0, 32, 32, 0);
#elif NN_PIA_CAFE
nn::pia::transport::Transport::Setting s_Setting(NULL, 0, 32, 32, 0);
#elif NN_PIA_WIN
nn::pia::transport::Transport::Setting s_Setting(NULL, 0, 32, 32, 0);
#endif
size_t s_ReliableSendBufNum = 64;
size_t s_ReliableRecvBufNum = 64;
size_t s_UnreliableRecvBufNum = 32;
}  // end of anonymous namespace


bool UpdateSetting(nn::pia::transport::Transport::Setting* pSetting,
                   size_t* pReliableSendBufNum,
                   size_t* pReliableRecvBufNum,
                   size_t* pUnreliableRecvBufNum)
{
    // Check arguments.
    if (!PIA_IS_VALID_POINTER(pSetting) ||
        !PIA_IS_VALID_POINTER(pReliableSendBufNum) ||
        !PIA_IS_VALID_POINTER(pReliableRecvBufNum) ||
        !PIA_IS_VALID_POINTER(pUnreliableRecvBufNum))
    {
        PIA_SAMPLE_PRINT("UpdateSetting(): Invalid pointer.");
        return false;
    }

    pSetting->maxSendThreadBufferNum = s_Setting.maxSendThreadBufferNum;
    pSetting->maxReceiveThreadBufferNum = s_Setting.maxReceiveThreadBufferNum;

    *pReliableSendBufNum = s_ReliableSendBufNum;
    *pReliableRecvBufNum = s_ReliableRecvBufNum;
    *pUnreliableRecvBufNum = s_UnreliableRecvBufNum;

    return true;
}


bool RecordPeakValues(void)
{
    // Checks if the state is one where the Watermark feature can be used.
    nn::pia::common::WatermarkManager* pMgr = nn::pia::common::WatermarkManager::GetInstance();
    if (!PIA_IS_VALID_POINTER(pMgr))
    {
        PIA_SAMPLE_PRINT("RecordPeakValues(): WatermarkManager is not ready.");
        return false;
    }

    // If arrives here, this is the net process.
    // If in a state where measurement with Watermark has not been performed once, stops with an assert.
    // 
    struct Table
    {
        int key;              // Key corresponding to Watermark.
        size_t defaultValue;  // Default value.
        size_t* pValue;       // Pointer indicating the write destination.
    };

    Table table[] = {
        { nn::pia::common::WatermarkManager::KEY_SENDTHREADSTREAM_BUFFER_NUM, 32, &(s_Setting.maxSendThreadBufferNum) },
        { nn::pia::common::WatermarkManager::KEY_RECEIVETHREADSTREAM_BUFFER_NUM, 32, &(s_Setting.maxReceiveThreadBufferNum) },
        { nn::pia::common::WatermarkManager::KEY_RELIABLE_PROTOCOL_SEND_BUFFER_NUM, 64, &s_ReliableSendBufNum },
        { nn::pia::common::WatermarkManager::KEY_RELIABLE_PROTOCOL_RECEIVE_BUFFER_NUM, 64, &s_ReliableRecvBufNum },
        { nn::pia::common::WatermarkManager::KEY_UNRELIABLE_PROTOCOL_RECEIVE_BUFFER_NUM, 32, &s_UnreliableRecvBufNum },
    };

    for (int i = 0; i < sizeof(table) / sizeof(Table); ++i)
    {
        nn::pia::common::Watermark* pMark = pMgr->GetWatermark(table[i].key);
        PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pMark));
        if (pMark->GetUpdateCount() > 0)
        {
            *(table[i].pValue) = static_cast<size_t>(pMark->GetMaxValue());
        }
        else
        {
            PIA_SAMPLE_PRINT("Watermark::GetUpdateCount() is zero.  So, peak value is not updated.\n");
        }
    }

    return true;
}
