﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     ProfilingTest_Setting.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef PROFILING_TEST_SETTING_H_
#define PROFILING_TEST_SETTING_H_

#include <pia/transport.h>


// Updates member variables of the Setting structure for passing to the Transport_InitializePiaTransport function and registers the Watermark value of the ReliableProtocol sending and receiving buffers.
// 
// Only the 5 member variables related to the capacity of the 5 types of buffers in the transport module are updated.
// When a NULL pointer, etc. is passed, false is returned.
// If this function is called before the RecordPeakValues function has ever been called, then the structure passed in the pSetting argument is set to the default values.
// 
bool UpdateSetting(nn::pia::transport::Transport::Setting* pSetting,
                   size_t* pReliableSendBufNum,
                   size_t* pReliableRecvBufNum,
                   size_t* pUnreliableRecvBufNum);

// Call after performing typical communications for a while.
// Gets the Watermark values of the transport module buffers, and records the peak value of usage of each buffer.
// If you call the UpdateSetting function after calling this function, the peak values recorded here are written to the Setting structure
// 
// The function returns false if the Watermark feature cannot be used.
bool RecordPeakValues(void);


#endif
