﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     SampleStationStatus.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef PIA_SAMPLE_STATIONSTATUS_H_
#define PIA_SAMPLE_STATIONSTATUS_H_

#include <pia/platform.h>

struct RectPosition  //!< Stores rectangle position coordinates.
{
    u16 x;
    u16 y;
};

struct PacketLossCheckNum  //!< Group of variables for checking packet loss.
{
    u32 firstNum;
    u32 latestNum;
    u32 lossCount;

    f32 CalcPacketLoss() const
    {
        if (firstNum == 0 || latestNum == firstNum)
        {
            return 0;
        }
        return static_cast<f32>(lossCount) / static_cast<f32>(latestNum - firstNum);
    }
};

class SampleStationStatus
{
public:
    static const size_t SERIALIZED_SIZE = 16;

    nn::pia::StationId stationId;
    nn::pia::StationIndex stationIndex;
    RectPosition pos;

    nn::pia::StationId routeInfoId;

    s32 rtt;

    PacketLossCheckNum unreliable;
    PacketLossCheckNum roundRobin;

    u32 reliableSendCount;
    u32 reliableReceiveCount;

    void Startup(nn::pia::StationId id, nn::pia::StationIndex index);
    void Cleanup();

    void Serialize(void* pBuf, bool isRoundRobin) const;
    void Deserialize(const void* pData, bool isRoundRobin);

    void UpdateInfo(bool enableRelay);
    void ResetPacketLossCheck();
};

#endif  // PIA_SAMPLE_STATIONSTATUS_H_
