﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Nex_Matchmake.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "Shared_Inet_Setup.h"

#if ENABLE_SERVERLESS
nn::Result Nex_InitializeMatchmake()
{
    return nn::pia::ResultInvalidState();
}
nn::Result Nex_FinalizeMatchmake()
{
    return nn::pia::ResultInvalidState();
}
nn::nex::MatchmakeExtensionClient* Nex_GetMatchmakeClient()
{
    return NULL;
}
nn::Result Nex_AutoMatchmake(
    nn::nex::GatheringID* pJoinedGatheringId,
    nn::nex::PrincipalID* pOwnerPID,
    u32 matchmakeMode,
    u16 matchmakeMaxParticipants,
    const char* matchmakeDescription)
{
    NN_PIA_DUMMY_PARAM(pJoinedGatheringId);
    NN_PIA_DUMMY_PARAM(pOwnerPID);
    NN_PIA_DUMMY_PARAM(matchmakeMode);
    NN_PIA_DUMMY_PARAM(matchmakeMaxParticipants);
    NN_PIA_DUMMY_PARAM(matchmakeDescription);
    return nn::pia::ResultInvalidState();
}
nn::Result Nex_OpenParticipation(nn::nex::GatheringID gatheringID)
{
    return nn::pia::ResultInvalidState();
}
nn::Result Nex_LeaveMatchmakeSession(
    nn::nex::GatheringID gatheringID,
    nn::nex::PrincipalID ownerPID,
    bool hostMigration)
{
    NN_PIA_DUMMY_PARAM(gatheringID);
    NN_PIA_DUMMY_PARAM(ownerPID);
    NN_PIA_DUMMY_PARAM(hostMigration);
    return nn::pia::ResultInvalidState();
}

nn::Result Nex_GetHostStationURLs(
    nn::nex::GatheringID joinedGatheringId,
    nn::nex::qList<nn::nex::StationURL>* pStationURLs)
{
    NN_PIA_DUMMY_PARAM(joinedGatheringId);
    NN_PIA_DUMMY_PARAM(pStationURLs);
    return nn::pia::ResultInvalidState();
}

nn::Result Nex_SearchMatchmakeSession(
    u32 gameMode,
    u16 maxParticipants,
    u32 rangeSize,
    nn::nex::qList<nn::nex::GatheringHolder>* plistGatheringHolder)
{
    NN_PIA_DUMMY_PARAM(gameMode);
    NN_PIA_DUMMY_PARAM(maxParticipants);
    NN_PIA_DUMMY_PARAM(rangeSize);
    NN_PIA_DUMMY_PARAM(plistGatheringHolder);
    return nn::pia::ResultInvalidState();
}

nn::Result Nex_CreateMatchmakeSession(
    nn::nex::GatheringID* pJoinedGatheringID,
    u32 gameMode,
    u16 maxParticipants,
    const char* description)
{
    NN_PIA_DUMMY_PARAM(pJoinedGatheringID);
    NN_PIA_DUMMY_PARAM(gameMode);
    NN_PIA_DUMMY_PARAM(maxParticipants);
    NN_PIA_DUMMY_PARAM(description);
    return nn::pia::ResultInvalidState();
}
nn::Result Nex_JoinMatchmakeSession(nn::nex::GatheringID gatheringID)
{
    NN_PIA_DUMMY_PARAM(gatheringID);
    return nn::pia::ResultInvalidState();
}
u32 Nex_GetMatchmakeNetworkErrorCode()
{
    return 0;
}
void Nex_ResetMatchmakeNetworkErrorCode()
{
}
nn::nex::qVector<nn::nex::qByte>* Nex_GetMatchmakeSessionSignatureKey()
{
    return NULL;
}
nn::pia::common::SignatureSetting* Nex_GetMatchmakeSessionSignatureSetting()
{
    return NULL;
}
nn::nex::PrincipalID Nex_GetOwnerPID()
{
    return 0;
}
void Nex_SetOwnerPID(nn::nex::PrincipalID id)
{
}
#else
#include "SampleLib/SampleLib.h"
#include "Shared_Nex_Matchmake.h"
#include "Shared_Basic_Setup.h"
#include <pia/common.h>
#include <pia/inet/inet_NexFacade.h>

// Index of the search attribute to use
#define MATCHMAKE_ATTR_INDEX 0

// The maximum search attribute value to use
#define MATCHMAKE_ATTR_MAX 2

// Use AutoMatchmake, which lets you join the same matchmaking session unlimited times. (NEX 3 series only.)
#define ENABLE_JOIN_SAME_MATCHMAKE 0

namespace
{
// Instance required for matchmaking.
nn::nex::MatchmakeExtensionClient* s_pMatchMakingClient = NULL;
bool s_IsBind = false;

// Matchmaking network error code.
u32 s_MatchmakeNetworkErrorCode = 0;

// Pointer to the shared session key obtained from the server when the matchmaking session was established.
nn::nex::qVector<nn::nex::qByte>* s_pMatchmakeSessionKey;

// Size of the signature key configured in the SignatureSetting object.
const u32 cSignatureKeyDataSize = 32;

// Settings for the signature feature to use a shared session key in a PiaSession object.
nn::pia::common::SignatureSettingWithKeyBuffer<cSignatureKeyDataSize> s_SignatureSetting;

// Specify the principal ID of the host.
nn::nex::PrincipalID s_OwnerPID = INVALID_PRINCIPALID;
};

//-----------------------------------------------------------------------------
// Event handler to process notifications from the NGS
//-----------------------------------------------------------------------------
class SampleNotificationEventHandler : public nn::nex::NotificationEventHandler
{
public:
    virtual void ProcessNotificationEvent(const nn::nex::NotificationEvent& oEvent)
    {
        nn::nex::StringStream ss;
        ss << oEvent;
        PIA_SAMPLE_LOG("ProcessNotificationEvent:%ls\n", ss.CStr());

        // Per-message processing
        switch (oEvent.GetType())
        {
        case nn::nex::NotificationEvents::ParticipationEvent:
        {
            switch (oEvent.GetSubType())
            {
            case nn::nex::ParticipationEvents::Participate:
            {
                PIA_SAMPLE_LOG("ParticipationEvents::Participate: [PID %u] join [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            case nn::nex::ParticipationEvents::EndParticipation:
            {
                PIA_SAMPLE_LOG("ParticipationEvents::EndParticipation: [PID %u] end [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            case nn::nex::ParticipationEvents::Disconnect:
            {
                PIA_SAMPLE_LOG("ParticipationEvents::Disconnect: [PID %u] disconnect [session %u]\n", oEvent.GetParam2(), oEvent.GetParam1());
            }
            break;
            default:
            {
                PIA_SAMPLE_LOG("ParticipationEvents: unsupported participation event:%d\n", oEvent.GetSubType());
            }
            break;
            }
        }
        break;
        case nn::nex::NotificationEvents::OwnershipChangeEvent:
        {
            PIA_SAMPLE_LOG(
                "NotificationEvents::OwnershipChangeEvent: [session %u] changed owner to [PID %u]\n",
                oEvent.GetParam1(),
                oEvent.GetParam2());

            // Record the PID of the new owner
            s_OwnerPID = oEvent.GetParam2();
        }
        break;
        case nn::nex::NotificationEvents::GatheringUnregistered:
        {
            // Record the GID of the deleted gathering
            nn::nex::GatheringID unregisteredGatheringId = oEvent.GetParam1();
            PIA_SAMPLE_LOG("NotificationEvents::GatheringUnregistered: [session %u] unregistered\n", unregisteredGatheringId);
            NN_PIA_DUMMY_PARAM(unregisteredGatheringId);
        }
        break;
        case nn::nex::NotificationEvents::HostChangeEvent:
        {
            PIA_SAMPLE_LOG("NotificationEvents::HostChangeEvent: [session %u] changed host\n", oEvent.GetParam1());
        }
        break;
        default:
        {
            PIA_SAMPLE_LOG("NotificationEvents: unsupported notification event:%d\n", oEvent.GetType());
        }
        break;
        }
    }
};
// Event handler to process notifications
static SampleNotificationEventHandler s_NotificationEventHandler;

nn::Result Nex_InitializeMatchmake()
{
    // Creates the instance required for matchmaking.
    PIA_SAMPLE_LOG("Nex_InitializeMatchmake:qNew nn::nex::MatchmakeExtensionClient.\n");
    s_pMatchMakingClient = qNew nn::nex::MatchmakeExtensionClient();
    PIA_SAMPLE_ASSERT(s_pMatchMakingClient != NULL);

    s_pMatchmakeSessionKey = qNew nn::nex::qVector<nn::nex::qByte>();

    // Sets the certificate for using the matchmaking service.
    if (!s_pMatchMakingClient->Bind(Nex_GetNgsFacadeInstance()->GetCredentials()))
    {
        PIA_SAMPLE_LOG("Nex_InitializeMatchmake:Bind failed.\n");
        return SampleNexInvalidState();
    }
    // Register an event handler to process notifications from the NGS.
    Nex_GetNgsFacadeInstance()->RegisterNotificationEventHandler(&s_NotificationEventHandler);

    s_IsBind = true;
    return nn::ResultSuccess();
}

nn::Result Nex_FinalizeMatchmake()
{
    if (s_pMatchMakingClient)
    {
        if (s_IsBind)
        {
            s_pMatchMakingClient->Unbind();
            s_IsBind = false;
            PIA_SAMPLE_LOG("Nex_FinalizeMatchmake:Unbind.\n");
        }

        // Delete event handler
        if (Nex_GetNgsFacadeInstance())
        {
            Nex_GetNgsFacadeInstance()->UnregisterNotificationEventHandler(&s_NotificationEventHandler);
        }

        // Destroys the instance required for matchmaking.
        PIA_SAMPLE_LOG("Nex_FinalizeMatchmake:qDelete nn::nex::MatchmakeExtensionClient.\n");
        qDelete s_pMatchMakingClient;
        s_pMatchMakingClient = NULL;
    }
    if (s_pMatchmakeSessionKey)
    {
        s_pMatchmakeSessionKey->clear();
        qDelete s_pMatchmakeSessionKey;
        s_pMatchmakeSessionKey = NULL;
    }

    return nn::ResultSuccess();
}

nn::nex::MatchmakeExtensionClient* Nex_GetMatchmakeClient()
{
    return s_pMatchMakingClient;
}

//-----------------------------------------------------------------------------
// Run automatic matchmaking.
// Get the necessary gathering and then join or create a session.
//-----------------------------------------------------------------------------
nn::Result Nex_AutoMatchmake(
    nn::nex::GatheringID* pJoinedGatheringID,
    nn::nex::PrincipalID* pOwnerPID,
    u32 matchmakeMode,
    u16 matchmakeMaxParticipants,
    const char* matchmakeDescription)
{
    PIA_SAMPLE_LOG("Nex_AutoMatchmake:matchmakeMode:%d, matchmakeMaxParticipants:%d\n", matchmakeMode, matchmakeMaxParticipants);

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_AutoMatchmake:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    // A room is created when no session matches the criteria.
    nn::nex::MatchmakeSession* const pMatchmakeSession = qNew nn::nex::MatchmakeSession();
    pMatchmakeSession->SetOpenParticipation(false);

    // The description string setting of MatchmakeSession is optional.
    if (matchmakeDescription == NULL)
    {
        pMatchmakeSession->SetDescription(_T("PiaInet Automatch"));
    }
    else
    {
        pMatchmakeSession->SetDescription(matchmakeDescription);
    }

    // The minimum and maximum number of participants.
    pMatchmakeSession->SetMinParticipants(1);
    pMatchmakeSession->SetMaxParticipants(matchmakeMaxParticipants);

    // Game mode.
    pMatchmakeSession->SetGameMode(matchmakeMode);

    // Search attributes that the application can freely define.
    // Because this is a sample, attribute values for sessions are randomized.
    pMatchmakeSession->SetAttribute(MATCHMAKE_ATTR_INDEX, nn::nex::Platform::GetRandomNumber(MATCHMAKE_ATTR_MAX + 1));

    // Matchmaking system type.
    pMatchmakeSession->SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // Binary data that the application can freely set.
    nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
    applicationBuffer.push_back(1);
    applicationBuffer.push_back(2);
    applicationBuffer.push_back(3);
    pMatchmakeSession->SetApplicationBuffer(applicationBuffer);

    // Transfer ownership when the owner logs out?
    pMatchmakeSession->SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // The MatchmakeSession instance is destroyed by the InGatheringHolder destructor.
    nn::nex::GatheringHolder oInGatheringHolder = pMatchmakeSession;
    nn::nex::GatheringHolder oOutGatheringHolder;  // The gathering that was joined.

    // Search for or create a matchmaking session.
    nn::nex::ProtocolCallContext oContext;

#if !ENABLE_JOIN_SAME_MATCHMAKE
    // Execute AutoMatchmake by using nn::nex::MatchmakeExtensionClient::AutoMatchmake, which takes multiple search criteria.

    // Create search criteria.
    nn::nex::qList<nn::nex::MatchmakeSessionSearchCriteria> criteriaList;

    // The first criteria uses a perfect match.
    {
        nn::nex::MatchmakeSessionSearchCriteria criteria;

        // The minimum and maximum number of participants.
        criteria.SetMinParticipants(1);
        criteria.SetMaxParticipants(matchmakeMaxParticipants);

        // Game mode.
        criteria.SetGameMode(matchmakeMode);

        // Search attributes that the application can freely define. Originally fixed at 0.
        criteria.SetAttribute(MATCHMAKE_ATTR_INDEX, 0);

        // Matchmaking system type.
        criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // Register to the list.
        criteriaList.push_back(criteria);
    }

    // The second criteria is specified within a range.
    {
        nn::nex::MatchmakeSessionSearchCriteria criteria;

        // The minimum and maximum number of participants.
        criteria.SetMinParticipants(1);
        criteria.SetMaxParticipants(matchmakeMaxParticipants);

        // Game mode.
        criteria.SetGameMode(matchmakeMode);

        // Search attributes that the application can freely define. All ranges that the attributes can take.
        criteria.SetAttributeWithRange(MATCHMAKE_ATTR_INDEX, 0, MATCHMAKE_ATTR_MAX);
        criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

        // Register the criteria with the list. (Add it to the end of the list because it is the second search criteria.)
        criteriaList.push_back(criteria);
    }

    // Due to the specifications of AutoMatchmake, after you leave a matchmaking session, you cannot find the session.
    bool bSuccess = s_pMatchMakingClient->AutoMatchmake(
        &oContext,
        criteriaList,
        oInGatheringHolder,
        &oOutGatheringHolder,
        _T("AutoMatchmake message"));
#else
    // Execute AutoMatchmake by using nn::nex::MatchmakeExtensionClient::AutoMatchmake, which takes a gathering ID. (NEX 3 series only.)
    PIA_SAMPLE_LOG("Nex_AutoMatchmake: ENABLE_JOIN_SAME_MATCHMAKE on\n");
    // Searches for a matchmaking session.
    // Blocked until an internal search completes.
    nn::nex::qList<nn::nex::GatheringHolder> listGatheringHolder;
    nn::Result result = Nex_SearchMatchmakeSession(
        s_pMatchMakingClient,
        matchmakeMode,
        matchmakeMaxParticipants,
        2,
        &listGatheringHolder);
    if (result.IsFailure())
    {
        // Search failed.
        PIA_SAMPLE_LOG("Nex_AutoMatchmake:Nex_SearchMatchmakeSession failed.\n");
    }

    // Create a qList of nn::nex::GatheringID objects from the list of nn::nex::GatheringHolder objects obtained from the search results.
    nn::nex::qList<nn::nex::GatheringID> listGatheringId;
    listGatheringId.clear();
    if (listGatheringHolder.size() < 1)
    {
        PIA_SAMPLE_LOG("Nex_AutoMatchmake:no matchmake session is found.\n");
        listGatheringId.clear();
    }
    else
    {
        nn::nex::qList<nn::nex::GatheringHolder>::const_iterator it = listGatheringHolder.begin();
        while (it != listGatheringHolder.end())
        {
            nn::nex::GatheringID gid = it->Get()->GetID();
            listGatheringId.push_back(gid);
            ++it;
        }
    }

    // Unlike the standard AutoMatch function, this function does not remove a matchmaking session from the list of joinable sessions after three prior join-ins.
    // Consider using the AutoMatch function that takes multiple search criteria before using this AutoMatch function.
    bool bSuccess = s_pMatchMakingClient->AutoMatchmake(
        &oContext,
        listGatheringId,
        oInGatheringHolder,
        &oOutGatheringHolder,
        _T("AutoMatchmake message"));
#endif

    if (!bSuccess)
    {
        PIA_SAMPLE_LOG("Failed to AutoMatchmake\n");
        return SampleFailedAutoMatchmakeSession();
    }

    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        Nex_ReportProtocolCallContext(oContext, "Failed to Nex_AutoMatchmake\n");
        return SampleFailedAutoMatchmakeSession();
    }

    *pJoinedGatheringID = oOutGatheringHolder->GetID();
    *pOwnerPID = oOutGatheringHolder->GetOwnerPID();

    //Get the key sent by the game server as the signature key for the P2P session.
    s_pMatchmakeSessionKey->clear();
    *s_pMatchmakeSessionKey = reinterpret_cast<nn::nex::MatchmakeSession*>(oOutGatheringHolder.Get())->GetSessionKey();

    PIA_SAMPLE_LOG(
        "[PID Info >> JoinedGID:%d OwnerPID:%d HostPID:%d MyPID:%d]\n",
        oOutGatheringHolder->GetID(),
        oOutGatheringHolder->GetOwnerPID(),
        oOutGatheringHolder->GetHostPID(),
        s_pMatchMakingClient->GetCredentials()->GetPrincipalID());

    return nn::ResultSuccess();
}


//-----------------------------------------------------------------------------
// Open a matchmaking session.
//-----------------------------------------------------------------------------
nn::Result Nex_OpenParticipation(
    nn::nex::GatheringID gatheringID)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_OpenParticiaption:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    nn::nex::ProtocolCallContext oContext;
    if (s_pMatchMakingClient->OpenParticipation(&oContext, gatheringID))
    {
        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            Nex_ReportProtocolCallContext(oContext, "Nex_OpenParticipation:Failed to open participation\n");
            return SampleFailedOpenParticipation();
        }
    }
    else
    {
        PIA_SAMPLE_LOG("Nex_OpenParticipation:OpenParticipation failed (Startup)\n");
        return SampleFailedOpenParticipation();
    }
    return nn::ResultSuccess();
}

//-----------------------------------------------------------------------------
// Leave the matchmaking session.
//-----------------------------------------------------------------------------
nn::Result Nex_LeaveMatchmakeSession(
    nn::nex::GatheringID gatheringID,
    nn::nex::PrincipalID ownerPID,
    bool hostMigration)
{
    PIA_SAMPLE_LOG("Nex_LeaveMatchmakeSession\n");

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_AutoMatchmake:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    if (gatheringID == INVALID_GATHERINGID)
    {
        PIA_SAMPLE_LOG("Invalid gathering.\n");
        return SampleNexInvalidState();
    }

    // Check the connection status with the MCS server.
    if (!s_pMatchMakingClient->IsConnected())
    {
        PIA_SAMPLE_LOG("Already disconnected to server.\n");
        return SampleNexInvalidState();
    }

    nn::nex::ProtocolCallContext oContext;

    PIA_SAMPLE_LOG(
        "[PID Info >> OwnerPID:%d MyPID:%d]\n",
        ownerPID,
        s_pMatchMakingClient->GetCredentials()->GetPrincipalID());

    // If it is a matchmaking session and there is no host migration, deletes the matchmaking session.
    if (s_pMatchMakingClient->GetCredentials()->GetPrincipalID() == ownerPID && !hostMigration)
    {
        PIA_SAMPLE_LOG("Unregister gathering ... \n");
        if (!s_pMatchMakingClient->UnregisterGathering(&oContext, gatheringID))
        {
            PIA_SAMPLE_LOG("UnregisterGathering call failed ... \n");
            return SampleNexInvalidState();
        }

        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            Nex_ReportProtocolCallContext(oContext, "failed to Unregister gathering!!\n");
            return SampleNexInvalidState();
        }
        PIA_SAMPLE_LOG("Unregister gathering successful.\n");
    }
    // If host migration is due to the owner or if not the owner, exits the matchmaking session.
    else
    {
        PIA_SAMPLE_LOG("End participation ... \n");
        if (!s_pMatchMakingClient->EndParticipation(&oContext, gatheringID))
        {
            PIA_SAMPLE_LOG("Failed to end participation\n");
            return SampleNexInvalidState();
        }

        oContext.Wait();
        if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
        {
            // If it has already been deleted, treat this as a normal exit.
            if (oContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
            {
                PIA_SAMPLE_LOG("matchmake session is already unregistered. don't care\n");
                return nn::ResultSuccess();
            }

            Nex_ReportProtocolCallContext(oContext, "Failed to end participation\n");
            return SampleNexInvalidState();
        }
    }

    return nn::ResultSuccess();
}

nn::Result Nex_GetHostStationURLs(
    nn::nex::GatheringID joinedGatheringID,
    nn::nex::qList<nn::nex::StationURL>* pStationURLs)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_GetHostStationURLs:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    // List the session URLs and try to join them.
    nn::nex::ProtocolCallContext oContext;
    if (!s_pMatchMakingClient->GetSessionURLs(&oContext, joinedGatheringID, pStationURLs))
    {
        PIA_SAMPLE_LOG("Failed Request URLs...\n");
        return SampleNexInvalidState();
    }

    oContext.Wait();
    if (oContext.GetState() != nn::nex::CallContext::CallSuccess)
    {
        // We may have joined a matchmaking session that has been deleted.
        if (oContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
        {
            PIA_SAMPLE_LOG("matchmake session is already unregistered.\n");
        }
        Nex_ReportProtocolCallContext(oContext, "Failed to obtain a list of URLs to connect to the game host\n");
        return SampleNexInvalidState();
    }

    return nn::ResultSuccess();
}


//-----------------------------------------------------------------------------
// Searching for a matchmaking session.
//-----------------------------------------------------------------------------
nn::Result Nex_SearchMatchmakeSession(
    u32 matchmakeMode,
    u16 matchmakeMaxParticipants,
    u32 rangeSize,
    nn::nex::qList<nn::nex::GatheringHolder>* plistGatheringHolder)
{
    PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:start.\n");

    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:MatchmakeExtensionClient is not created.\n");
        PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:end.\n");
        return SampleNexInvalidState();
    }

    /* Start search condition settings. */
    // For more information about each setting, see the CTR-NEX API Reference or other documents.
    nn::nex::MatchmakeSessionSearchCriteria criteria;

    // Minimum and maximum number of participants.
    criteria.SetMinParticipants(1);
    criteria.SetMaxParticipants(matchmakeMaxParticipants);

    // Application-specific game mode.
    criteria.SetGameMode(matchmakeMode);

    // Search attributes that the application can freely define. (In this version, the range is specified.)
    criteria.SetAttributeWithRange(MATCHMAKE_ATTR_INDEX, 0, MATCHMAKE_ATTR_MAX);

    // Matchmaking system type.
    criteria.SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // Search only for sessions that can be joined.
    criteria.SetExcludeLocked(true);

    // Exclude full sessions.
    criteria.SetVacantOnly(true);
    /* Terminates search criteria settings. */

    // Search range.
    nn::nex::ResultRange range;
    range.SetSize(rangeSize);  // Maximum number obtained.
    range.SetOffset(0);        // Offset value.

    // Empties the save list.
    plistGatheringHolder->clear();

    nn::nex::ProtocolCallContext searchContext;
    nn::Result result;

    // Start the asynchronous search process.
    if (!s_pMatchMakingClient->BrowseMatchmakeSession(&searchContext, criteria, range, plistGatheringHolder))
    {
        // The asynchronous search process failed to start.
        PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:BrowseMatchmakeSession failed.(Startup)\n");
        result = SampleNexInvalidState();
    }
    else
    {
        // Wait for the asynchronous search process to end.
        searchContext.Wait();
        if (!searchContext.GetOutcome())
        {
            // The asynchronous search process failed.
            PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:BrowseMatchmakeSession failed.(Search)\n");

            // Empties the save list.
            plistGatheringHolder->clear();
            result = SampleNexInvalidState();
        }
        else
        {
            result = nn::ResultSuccess();
        }
    }

    PIA_SAMPLE_LOG("Nex_SearchMatchmakeSession:end.\n");
    return result;
}

//-----------------------------------------------------------------------------
// Open a matchmaking session.
//-----------------------------------------------------------------------------
nn::Result Nex_CreateMatchmakeSession(
    nn::nex::GatheringID* pJoinedGatheringID,
    u32 matchmakeMode,
    u16 matchmakeMaxParticipants,
    const char* matchmakeDescription)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_CreateMatchmakeSession:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    // Create a new session.
    nn::nex::MatchmakeSession* pMatchmakeSession = qNew nn::nex::MatchmakeSession();

    // Matchmaking session settings.

    // The description string setting of MatchmakeSession is optional.
    pMatchmakeSession->SetDescription(matchmakeDescription);

    // The minimum and maximum number of participants.
    pMatchmakeSession->SetMinParticipants(1);
    pMatchmakeSession->SetMaxParticipants(matchmakeMaxParticipants);

    // Application-specific game mode.
    pMatchmakeSession->SetGameMode(matchmakeMode);

    // Search attributes that the application can freely define.
    // As a sample, the attribute values are randomly set within the specification range.
    pMatchmakeSession->SetAttribute(MATCHMAKE_ATTR_INDEX, nn::nex::Platform::GetRandomNumber(MATCHMAKE_ATTR_MAX + 1));

    // Matchmaking system type.
    pMatchmakeSession->SetMatchmakeSystemType(nn::nex::MATCHMAKE_SYSTEM_TYPE_ANYBODY);

    // Binary data that the application can freely set.
    // As a sample, a suitable value is set.
    nn::nex::qVector<nn::nex::qUnsignedInt8> applicationBuffer;
    applicationBuffer.push_back(1);
    applicationBuffer.push_back(2);
    applicationBuffer.push_back(3);
    pMatchmakeSession->SetApplicationBuffer(applicationBuffer);

    // Transfer ownership when the owner logs out?
    pMatchmakeSession->SetFlag(nn::nex::GatheringFlags::MigrateOwner);

    // Whether to set to joinable from the moment the session was created.
    // Set to false, and use OpenParticipation after the Session (P2P communication) preparations are finished.
    pMatchmakeSession->SetOpenParticipation(false);

    // The MatchmakeSession instance is destroyed by the gatheringHolder destructor.
    nn::nex::GatheringHolder gatheringHolder(pMatchmakeSession);
    nn::nex::ProtocolCallContext createContext;

    s_pMatchmakeSessionKey->clear();
    // Start the asynchronous matchmaking session creation process.
    if (s_pMatchMakingClient->CreateMatchmakeSession(&createContext, gatheringHolder, pJoinedGatheringID, s_pMatchmakeSessionKey))
    {
        // Wait for the asynchronous matchmaking session creation process to end.
        createContext.Wait();
        if (!createContext.GetOutcome())
        {
            // The asynchronous matchmaking session creation process failed.
            return SampleFailedCreateMatchmakeSession();
        }
    }
    else
    {
        // The asynchronous matchmaking session creation process failed to start.
        return SampleFailedCreateMatchmakeSession();
    }

    PIA_SAMPLE_LOG(
        "[PID Info >> JoinedGID:%d OwnerPID:%d HostPID:%d MyPID:%d]\n",
        gatheringHolder->GetID(),
        gatheringHolder->GetOwnerPID(),
        gatheringHolder->GetHostPID(),
        s_pMatchMakingClient->GetCredentials()->GetPrincipalID());

    return nn::ResultSuccess();
}

//-----------------------------------------------------------------------------
// Join a matchmaking session.
//-----------------------------------------------------------------------------
nn::Result Nex_JoinMatchmakeSession(
    nn::nex::GatheringID gatheringID)
{
    if (!s_pMatchMakingClient)
    {
        PIA_SAMPLE_LOG("Nex_JoinMatchmakeSession:MatchmakeExtensionClient is not created.\n");
        return SampleNexInvalidState();
    }

    nn::nex::ProtocolCallContext joinContext;

    s_pMatchmakeSessionKey->clear();
    // Start the asynchronous matchmaking session participation process.
    if (s_pMatchMakingClient->JoinMatchmakeSession(&joinContext, gatheringID, s_pMatchmakeSessionKey))
    {
        // Wait for the asynchronous matchmaking session participation process to end.
        joinContext.Wait();
        if (!joinContext.GetOutcome())
        {
            // The asynchronous matchmaking session participation process failed.
            // Handling of the reason for failure:
            if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionFull))
            {
                PIA_SAMPLE_LOG("JoinMatchmakeSession Error (SessionFull)\n");
            }
            else if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, SessionVoid))
            {
                PIA_SAMPLE_LOG("JoinMatchmakeSession Error (SessionVoid)\n");
            }
            else
            {
                if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, DeniedByParticipants))
                {
                    PIA_SAMPLE_LOG("JoinMatchmakeSession Error (DeniedByParticipants)\n");
                }
                else if (joinContext.GetOutcome() == QRESULT_ERROR(RendezVous, ParticipantInBlackList))
                {
                    PIA_SAMPLE_LOG("JoinMatchmakeSession Error (ParticipantInBlackList)\n");
                }
                else
                {
                    PIA_SAMPLE_LOG("JoinMatchmakeSession Error (Unknown)\n");
                }
                s_MatchmakeNetworkErrorCode = nn::nex::ErrorCodeConverter::ConvertToNetworkErrorCode(joinContext.GetOutcome());
            }
            return SampleFailedJoinMatchmakeSession();
        }
    }
    else
    {
        PIA_SAMPLE_LOG("JoinMatchmakeSession Error (Startup)\n");
        // The asynchronous matchmaking session participation process failed to start.
        return SampleFailedJoinMatchmakeSession();
    }

    return nn::ResultSuccess();
}

u32 Nex_GetMatchmakeNetworkErrorCode()
{
    return s_MatchmakeNetworkErrorCode;
}

void Nex_ResetMatchmakeNetworkErrorCode()
{
    s_MatchmakeNetworkErrorCode = 0;
}

nn::nex::qVector<nn::nex::qByte>* Nex_GetMatchmakeSessionSignatureKey()
{
    return s_pMatchmakeSessionKey;
}

nn::pia::common::SignatureSetting* Nex_GetMatchmakeSessionSignatureSetting()
{
    nn::Result result = nn::pia::inet::NexFacade::ConvertNexSignatureKeyToSignatureSetting(
        *s_pMatchmakeSessionKey,
        &s_SignatureSetting);
    if (result.IsFailure())
    {
        return NULL;
    }
    return &s_SignatureSetting;
}

nn::nex::PrincipalID Nex_GetOwnerPID()
{
    return s_OwnerPID;
}

void Nex_SetOwnerPID(nn::nex::PrincipalID id)
{
    s_OwnerPID = id;
}
#endif
