﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Session_Mesh.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>
#include <pia/session.h>
#include <pia/transport/transport_NetworkFactory.h>

// <tt>nn::Result</tt> definitions for broadly categorizing errors that are not expressed by <tt>nn::Result</tt> in the sample.
enum SAMPLE_RESULT_MESH_DESCRIPTION
{
    MESH_STATE_DISCONNECTED = 20  // Disconnected from the session.
};

PIA_DEFINE_RESULT(
    SampleMeshStateDisconnected,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    MESH_STATE_DISCONNECTED);

// Initializes <tt>PiaSession</tt> when directly operating mesh.
nn::Result Session_InitializePiaSession_ForMesh(nn::pia::transport::NetworkFactory*,
                                                nn::pia::session::NetworkTopology topology = nn::pia::session::NetworkTopology_FullMesh);
nn::Result Session_InitializePiaSession_ForMesh(const nn::pia::session::Mesh::Setting& setting);
nn::Result Session_InitializePiaSession_ForMesh(
    const nn::pia::session::Mesh::Setting& setting,
    bool isEnableRelayEmulation);

// Finalizes <tt>PiaSession</tt> when directly operating mesh.
void Session_FinalizePiaSession_ForMesh();

// Starts the mesh.
nn::Result Session_StartupMesh(nn::pia::session::Mesh::StartupSetting startupSetting);

// End the mesh.
void Session_CleanupMesh();

void Session_GetLocalPlayerName_ForMesh(nn::pia::transport::Station::PlayerName* pPlayerName);

// The hosts makes this call. Creates a mesh.
bool Session_CreateMesh();

// The hosts makes this call. Destroys the mesh.
void Session_DestroyMesh();

// The hosts makes this call. Issues instruction to begin host migration and leaves the mesh.
void Session_LeaveMeshWithHostMigration();

// The client makes this call. Joins a mesh.
bool Session_JoinMesh(const nn::pia::transport::StationConnectionInfo& stationConnectionInfo);

// The client makes this call. Leaves the mesh.
void Session_LeaveMesh();
