﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Local_Setup.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/transport.h>
#include <pia/local.h>

#include "SampleLib/SampleLib.h"

// Initialize PiaLocal.
nn::Result Local_InitializePiaLocal(u32 titleId, u8 subId, const nn::pia::local::UdsNetworkSetting& networkSetting);
nn::Result Local_InitializePiaLocal(u32 titleId, u8 subId, bool enableHostMigration = false, bool enableAroundNetworkSearch = false);

// Finalize PiaLocal.
void Local_FinalizePiaLocal();

// Build the UDS network with the local station acting as the host.
bool Local_CreateNetwork(size_t maxStations);
bool Local_CreateNetwork(nn::pia::local::LocalCreateNetworkSetting* pCreateNetworkSetting);

// Join the UDS network with the local station acting as the client.
bool Local_SearchAndConnectToNetwork(nn::pia::transport::StationConnectionInfo* pStationConnectionInfo);

bool Local_ScanNetwork(u32 localCommunicationId, u8 gatheringId);
bool Local_ConnectNetwork(nn::pia::local::LocalConnectNetworkSetting* pConnectNetworkSetting);
bool Local_DisconnectNetwork(bool isMigration = false);
nn::Result Local_StartupPiaLocal();
void Local_CleanupPiaLocal();

// End the UDS network.
void Local_CleanupNetwork();

// Get the NetworkFactory required for Transport initialization.
nn::pia::transport::NetworkFactory* Local_GetNetworkFactory();

// Gets whether connected to the local network.
bool Local_IsConnected();

// Sets a title ID.
void Local_SetTitleId(u32 id);

// Sets a sub ID.
void Local_SetSubId(u8 id);
