﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Session_Setup.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#pragma once

#include <pia/platform.h>
#include <pia/session.h>
#include <pia/transport/transport_NetworkFactory.h>

// <tt>nn::Result</tt> definitions for broadly categorizing errors that are not expressed by <tt>nn::Result</tt> in the sample.
enum SAMPLE_RESULT_SESSION_DESCRIPTION
{
    SESSION_STATE_DISCONNECTED = 20  // Disconnected from the session.
};

PIA_DEFINE_RESULT(
    SampleSessionStateDisconnected,
    nn::Result::LEVEL_PERMANENT,
    nn::Result::SUMMARY_INVALID_STATE,
    nn::Result::MODULE_APPLICATION,
    SESSION_STATE_DISCONNECTED);


// Results of functions that use asynchronous processes.
enum SampleSessionAsyncStatus
{
    SAMPLE_SESSION_ASYNC_SUCCESS = 0,           // Asynchronous process succeeded.
    SAMPLE_SESSION_ASYNC_FAILED,                // Asynchronous process failed. (Handle the error according to the <tt>result</tt>.)
    SAMPLE_SESSION_ASYNC_DISCONNECTED_SESSION,  // Disconnected from P2P session. (Clean up.)
    SAMPLE_SESSION_ASYNC_DISCONNECTED_NETWORK,   // Disconnected from network. (Reconnect to Internet.)
    SAMPLE_SESSION_ASYNC_CANCELED               // Cancel asynchronous process.
};

// (Function for Pia Developers)
// Callback types run every frame during session creation and while participating, and their corresponding registration functions.
// The registered callback is called after <tt>SampleFramework::EndFrame</tt>.
typedef void (*PiaTestCallback)();
PiaTestCallback RegisterPiaTestCallback(PiaTestCallback cb);

// Initialize PiaSession.
nn::Result Session_InitializePiaSession(
    nn::pia::transport::NetworkFactory*,
    nn::pia::session::NetworkTopology topology = nn::pia::session::NetworkTopology_FullMesh);
nn::Result Session_InitializePiaSession(
    const nn::pia::session::Session::Setting& setting);
nn::Result Session_InitializePiaSession(
    const nn::pia::session::Session::Setting& setting,
    bool isEnableRelayEmulation,
    bool isEnableMatchmakeAnalysis = false,
    bool isDisableCheckMaxSilenceTime = false);

// Finalize PiaSession.
void Session_FinalizePiaSession();

nn::Result Session_StartupSession(nn::pia::session::Session::StartupSetting startupSetting);


// End the session.
void Session_CleanupSession();

void Session_GetLocalPlayerName(nn::pia::transport::Station::PlayerName* pPlayerName);

// Performs random matchmaking for session.
SampleSessionAsyncStatus Session_JoinRandomSession(
    nn::pia::session::CreateSessionSetting* pCreateSessionSetting,
    nn::pia::session::SessionSearchCriteria searchCriteriaArray[],
    size_t searchCriteriaNum,
    nn::Result* pAsyncResult,
    bool bRender = true);

// Create a session.
SampleSessionAsyncStatus Session_CreateSession(
    nn::pia::session::CreateSessionSetting* pCreateSessionSetting,
    nn::Result* pAsyncResult);

// Searches for sessions.
SampleSessionAsyncStatus Session_BrowseSession(
    nn::pia::session::SessionSearchCriteria* pSearchCriteria,
    nn::Result* pAsyncResult);

// Joins a session.
SampleSessionAsyncStatus Session_JoinSession(
    nn::pia::session::JoinSessionSetting* pJoinSessionSetting,
    nn::Result* pAsyncResult);

// Check whether it is necessary to log out and log back in for the <tt>Result</tt> when the asynchronous processes for session random matchmaking, creating, searching and joining have failed.
// 
bool Session_CheckWhetherLogoutIsNecessaryInAsyncErrorResult(nn::Result result);

// Leaves the session.
SampleSessionAsyncStatus Session_LeaveSession();
