﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     Shared_Sync_Setup.cpp

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#include "SampleLib/SampleLib.h"
#include "Shared_Sync_Setup.h"

namespace
{
u32 s_hSyncProtocol = 0;
}

void Sync_InitializePiaSync(const nn::pia::sync::SyncProtocol::Setting& setting)
{
    // Initializes the sync module.
    // Complete the setup of the common module and the transport module in advance.
    nn::Result r;
    r = nn::pia::sync::Initialize();
    PIA_SAMPLE_ASSERT(r.IsSuccess());

    r = nn::pia::sync::BeginSetup();
    PIA_SAMPLE_ASSERT(r.IsSuccess());

    // Get the Transport singleton.
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // Creation and initialization of SyncProtocol (key data synchronization).
    {
        s_hSyncProtocol = pTransport->CreateProtocol<nn::pia::sync::SyncProtocol>();
        PIA_SAMPLE_ASSERT(s_hSyncProtocol != 0);  // The CreateProtocol function returns 0 if the same protocol ID already exists. Confirm that CreateProtocol has not returned 0.

        // Check the size of data sent as a single unit.
        size_t dataUnitSizeMax = Sync_GetSyncProtocol()->GetDataUnitSizeMax();
        NN_PIA_DUMMY_PARAM_IN_RELEASE(dataUnitSizeMax);
        for (u32 i = 0; i < nn::pia::sync::SyncProtocol::DATA_ID_NUM; i++)
        {
            PIA_SAMPLE_ASSERT(setting.m_DataUnitSize[i] <= dataUnitSizeMax);
        }

        r = Sync_GetSyncProtocol()->Initialize(setting);
        PIA_SAMPLE_ASSERT(r.IsSuccess());
    }

    r = nn::pia::sync::EndSetup();
    PIA_SAMPLE_ASSERT(r.IsSuccess());
}


void Sync_FinalizePiaSync()
{
    // Get the Transport singleton.
    nn::pia::transport::Transport* pTransport = nn::pia::transport::Transport::GetInstance();
    PIA_SAMPLE_ASSERT(PIA_IS_VALID_POINTER(pTransport));

    // end SyncProtocol
    nn::pia::sync::SyncProtocol* pSyncProtocol = pTransport->GetProtocol<nn::pia::sync::SyncProtocol>(s_hSyncProtocol);
    if (PIA_IS_VALID_POINTER(pSyncProtocol))
    {
        pSyncProtocol->Finalize();
        pTransport->DestroyProtocol(s_hSyncProtocol);
        s_hSyncProtocol = 0;
    }

    nn::pia::sync::Finalize();
}
nn::pia::sync::SyncProtocol* Sync_GetSyncProtocol()
{
    return nn::pia::transport::Transport::GetInstance()->GetProtocol<nn::pia::sync::SyncProtocol>(s_hSyncProtocol);
}
