﻿/*---------------------------------------------------------------------------*
  Project:  Pia
  File:     controller.h

  Copyright Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
 *---------------------------------------------------------------------------*/


#ifndef PIA_SAMPLE_CONTROLLER_H_
#define PIA_SAMPLE_CONTROLLER_H_

#include <pia/platform.h>

// IDs for synchronous communication data.
enum DataId
{
    DATA_ID_KEY = 0,
    DATA_ID_TOUCH,
    DATA_ID_STICK,
    DATA_ID_NUM
};


// Key ID.
enum Key
{
    KEY_UP = 0,
    KEY_DOWN,
    KEY_LEFT,
    KEY_RIGHT,
    KEY_A,
    KEY_B,
    KEY_X,
    KEY_Y,
    KEY_NUM
};


// Key data for synchronous communication.
struct KeyData
{
    bit32 m_HoldFlag;
    bit32 m_TriggerFlag;
    bit32 m_ReleaseFlag;
};

// Touch panel data to synchronize.
struct TouchData
{
    u16 m_X;
    u16 m_Y;
    u8 m_Touch;
};

// Circle Pad/Control Stick data to synchronize.
struct StickData
{
    f32 m_X;
    f32 m_Y;
};


// Controller data.
class ControllerData
{
public:
    void Clear();

protected:
    static bit32 KeyToButton(Key key);

protected:
    KeyData m_KeyData;
    TouchData m_TouchData;
    StickData m_StickData;
};


// Manage controller data for sending.
class ControllerReader : public ControllerData
{
public:
    const void* GetData(DataId dataID) const;
    void Read();
};


// Manage received controller data.
class ControllerAccessor : public ControllerData
{
public:
    void* GetDataPtr(DataId dataID);

    bool IsHold(Key key) const
    {
        return (m_KeyData.m_HoldFlag & (0x1 << key)) != 0;
    }
    bool IsTrigger(Key key) const
    {
        return (m_KeyData.m_TriggerFlag & (0x1 << key)) != 0;
    }
    bool IsRelease(Key key) const
    {
        return (m_KeyData.m_ReleaseFlag & (0x1 << key)) != 0;
    }

    u32 GetHoldFlag() const
    {
        return m_KeyData.m_HoldFlag;
    }
    u32 GetTriggerFlag() const
    {
        return m_KeyData.m_TriggerFlag;
    }
    u32 GetReleaseFlag() const
    {
        return m_KeyData.m_ReleaseFlag;
    }

    bool IsTouch() const
    {
        return (m_TouchData.m_Touch > 0);
    }
    u16 GetTouchX() const
    {
        return m_TouchData.m_X;
    }
    u16 GetTouchY() const
    {
        return m_TouchData.m_Y;
    }

    f32 GetStickX() const
    {
        return m_StickData.m_X;
    }
    f32 GetStickY() const
    {
        return m_StickData.m_Y;
    }
};


#endif  // PIA_SAMPLE_CONTROLLER_H_
